********************************************************************
      program RacoonWW 
c********************************************************************
c
c     Monte Carlo program for e+ e- -> 4 fermions (+ gamma) 
c
c Authors:  A.Denner, S.Dittmaier, M.Roth, D.Wackeroth
c
c published in Comput. Phys. Commun. 153, 462 (2003) (hep-ph/0209330) 
c
c additional publications:
c Nucl. Phys. B587 (2000) (hep-ph/0006307)
c Nucl. Phys. B560 (1999) (hep-ph/9904472) 
c
c      Main program
c     --------------
c
c contacts: roth@mppmu.mpg.de,
c           dow@ubpheno.physics.buffalo.edu, 
c           ansgar.denner@psi.ch, dittmair@mppmu.mpg.de
c********************************************************************
c
c  VERSION 1.3.1 
c
c  released on March 19, 2004
c********************************************************************
      implicit none
c local variables
      real*8 avprel,siprel,atotsqr,a1sqr,a2sqr
      integer neventsunw,unweighting,i1,istrat,irad,itau,icom
c hepevt
      integer nmxhep
      parameter(nmxhep=4000)
      real*8 phep(5,nmxhep),vhep(4,nmxhep)
      integer nevhep,nhep,isthep(nmxhep),idhep(nmxhep)
      integer jmohep(2,nmxhep),jdahep(2,nmxhep)
      common/hepevt/nevhep,nhep,isthep,idhep,jmohep,jdahep,phep,vhep
c read input file
      call readinput
c set parameter and initialize numerical integration
      call initialize
c numerical integration, generation of weighted events 
c initialization for generation of unweighted event
      call integration(avprel,siprel)
c generation of 'neventsunw' unweighted events
      unweighting=1
      neventsunw=0
      do i1=1,neventsunw
        call eventgeneration(atotsqr,a1sqr,a2sqr,unweighting)
c interface to PYTHIA, no warranty!
c remarks:
c exclude photon radiation in pythia (irad=0) in order to
c avoid double counting of radiative corrections in rc-mode (src=1) !
c problems with tau-decay since taus are massless in RacoonWW !
        istrat=0  
        irad=0    
        itau=0
        icom=0    
c to activate the interface, please uncomment the following line:
c        call py4frm(atotsqr,a1sqr,a2sqr,istrat,irad,itau,icom)
      enddo
c write output
      call writeoutput
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     generates a massive two-particle phasespace                 c
c                                                                 c
c input:                                                          c 
c m1,m2: masses of the two particles                              c
c phep1(1:5),phep2(1:5): four momenta of the two particles        c
c                        with zero masses                         c
c output:                                                         c 
c phep1(1:5),phep2(1:5): four momenta of the two particles        c 
c                        with masses m1,m2                        c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine massive(m1,m2,phep1,phep2)
      implicit none 
c local variables
      real*8 m1,m2,s,s1,s2,k(4),k1(4),k2(4),q(4),phep1(5),phep2(5)
      real*8 lambda,lambda2
      integer i1
      do i1=1,4
        k1(i1)=phep1(i1)
        k2(i1)=phep2(i1)
        k(i1)=k1(i1)+k2(i1)
        q(i1)=k1(i1)-k2(i1)
      enddo
      s1=m1**2
      s2=m2**2
      s=k(4)**2-k(1)**2-k(2)**2-k(3)**2
      lambda2=(s1+s2-s)**2-4d0*s1*s2
      if(lambda2.lt.0d0.or.s.eq.0d0)then
        write(*,'(a)')' massive: inconsistent input!'
        return
      endif
      lambda=dsqrt(lambda2)
      do i1=1,4
        phep1(i1)=k(i1)*0.5d0*(s+s1-s2)/s+q(i1)*0.5d0*lambda/s
        phep2(i1)=k(i1)*0.5d0*(s-s1+s2)/s-q(i1)*0.5d0*lambda/s
      enddo
      phep1(5)=m1
      phep2(5)=m2
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     read input file                                             c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine readinput
      implicit none 
c local variables
      integer maxe
      parameter(maxe=29)
      integer i1,i2
c mainprog
      real*8 energy,average,averagemax,sigma,avb,sigb
      real*8 weightmax(maxe),weighttotmax,nqcdfac
      integer nevents,neventsw,nweight(maxe),nunweighted(-1:1)
      integer novershoot,nout,iout,steps,particlenum(0:1,-1:1,3)
      character*90 outputfile
c pol 
      real*8 pm,pp
c processpar
      real*8 mv(3),mv2(3),gv(3),qcdfac,qcdfac2,q(6)
      complex*16 cvf(-1:1,3,6),cvw(2)
      integer qprop,quark(6),iso(6),gen(6),dir(7),pol(7),color
      character*6 fname(6),dfname(6)
c inputcuts
      real*8 ecutp,ecutl,ecutq,scutqq,ccutpb,ccutpl,ccutpq,ccutll
      real*8 ccutqq,ccutlq,ccutlb,ccutqb
c cuts
      real*8 ecut(7),scut(7,7),ccut(7,7),precomb(4),precombg(2)
      integer scuts,srecomb,srecombg
c options
      integer smc,sborn4,sborn5,sborng5,sisr,src,son,sqcdepem
      integer ssigepem4,ssigepem5,ssigepemg5,scoultree,sunw,spaw,satgc
c rcoptions
      integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
c rscheme
      real*8 dalpz,drbos,drferm
      integer qalp,qgw
c aqgc
      real*8 alam2(0:4)
      integer qaqgc
c atgc
      real*8 gtgc(11,2)
      integer qatgc
c common blocks
      common/mainprog/energy,average,averagemax,sigma,avb,sigb,
     *  weightmax,weighttotmax,nqcdfac,nevents,neventsw,nweight,
     *  nunweighted,novershoot,nout,iout,steps,particlenum,outputfile
      common/pol/pm,pp
      common/processpar/cvf,cvw,mv,mv2,gv,qcdfac,qcdfac2,
     *  q,qprop,quark,iso,gen,dir,pol,color,fname,dfname
      common/inputcuts/ecutp,ecutl,ecutq,scutqq,ccutpb,ccutpl,ccutpq,
     *  ccutll,ccutqq,ccutlq,ccutlb,ccutqb
      common/cuts/ecut,scut,ccut,precomb,precombg,scuts,
     *  srecomb,srecombg
      common/options/smc,sborn4,sborn5,sborng5,sisr,src,son,sqcdepem,
     *  ssigepem4,ssigepem5,ssigepemg5,scoultree,sunw,spaw,satgc
      common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
      common/rscheme/dalpz,drbos,drferm,qalp,qgw
      common/aqgc/alam2,qaqgc          
      common/atgc/gtgc,qatgc
c initialize arrays for cuts 
      do i1=1,7
        ecut(i1)=0d0
        do i2=1,7
          scut(i1,i2)=0d0
          ccut(i1,i2)=0d0
        enddo
      enddo
      do i1=1,4
        precomb(i1)=0d0
      enddo
      do i1=1,2
        precombg(i1)=0d0
      enddo
      ecutp=0d0
      ecutl=0d0
      ecutq=0d0
      scutqq=0d0
      ccutpb=0d0
      ccutpl=0d0
      ccutpq=0d0
      ccutll=0d0
      ccutqq=0d0
      ccutlq=0d0
      ccutlb=0d0
      ccutqb=0d0
c QGC
      do i1=0,4
         alam2(i1)=0d0
      enddo
c initialize array for final-state polarizations
      do i1=3,7
         pol(i1)=0d0
      enddo
c read input file
      read'(a)',outputfile     ! name of output file
      read*,energy             ! CMF energy
      read*,neventsw           ! number of weighted events
      read*,smc                ! choice of MC branch 
      read*,sborn4             ! include Born ee->4f
      read*,sborn5             ! include Born ee->4fa
      read*,sborng5            ! include Born ee->4fg
      read*,sisr               ! include higher-order ISR
      read*,src                ! include radiative corrections
      read*,scoultree          ! choice of Coul. sing. in ee->4f,4fa
      read*,qnf                ! choice of Coulomb singularity in DPA
      read*,qreal              ! neglect imaginary part of virtual corr.
      read*,qalp               ! choice of input-parameter scheme
      read*,qgw                ! calculate W-boson width
      read*,qprop              ! choice of width scheme
      read*,ssigepem4          ! choice of diagrams for Born ee->4f
      read*,ssigepem5          ! choice of diagrams for Born ee->4fa
      read*,ssigepemg5         ! choice of diagrams for Born ee->4fg
      read*,qqcd               ! include QCD radiative corrections
      read*,sqcdepem           ! include gluon-exch. diagrams in Born
      do i1=3,6
        read'(a)',fname(i1)    ! particle names
      enddo
      read*,pp                 ! degree of positron beam polarization
      read*,pm                 ! degree of electron beam polarization
      read*,srecomb            ! recombination cuts
      read*,precomb(1)         ! angular rec. cut between photon and beam
      read*,precomb(2)         ! rec. cut on photon energy
      read*,precomb(3)         ! invariant-mass rec. (TH) or
c                              ! angular rec. cut for leptons (EXP)
      read*,precomb(4)         ! angular rec. cut for quarks (EXP)
      read*,srecombg           ! gluon recombination cuts
      read*,precombg(1)        ! rec. cut on gluon energy
      read*,precombg(2)        ! invariant-mass (TH) or angular (EXP) rec.
c TGC
      read*,satgc              ! flag for TGCs
      read*,gtgc(1,1)          ! TGC Delta g_1^A
      read*,gtgc(1,2)          ! TGC Delta g_1^Z
      read*,gtgc(2,1)          ! TGC Delta kappa^A
      read*,gtgc(2,2)          ! TGC Delta kappa^Z
      read*,gtgc(3,1)          ! TGC lambda^A
      read*,gtgc(3,2)          ! TGC lambda^Z
      read*,gtgc(4,1)          ! TGC g_4^A
      read*,gtgc(4,2)          ! TGC g_4^Z
      read*,gtgc(5,1)          ! TGC g_5^A
      read*,gtgc(5,2)          ! TGC g_5^Z
      read*,gtgc(6,1)          ! TGC tilde kappa^A
      read*,gtgc(6,2)          ! TGC tilde kappa^Z
      read*,gtgc(7,1)          ! TGC tilde lambda^A
      read*,gtgc(7,2)          ! TGC tilde lambda^Z
      read*,gtgc(8,1)          ! TGC f_4^A
      read*,gtgc(8,2)          ! TGC f_4^Z
      read*,gtgc(9,1)          ! TGC f_5^A
      read*,gtgc(9,2)          ! TGC f_5^Z
      read*,gtgc(10,1)         ! TGC h_1^A
      read*,gtgc(10,2)         ! TGC h_1^Z
      read*,gtgc(11,1)         ! TGC h_3^A
      read*,gtgc(11,2)         ! TGC h_3^Z
c QGC
      read*,qaqgc              ! flag for QGCs
      read*,alam2(0)           ! QGC a_0/Lambda^2
      read*,alam2(1)           ! QGC a_c/Lambda^2  
      read*,alam2(2)           ! QGC a_n/Lambda^2  
      read*,alam2(4)           ! QGC tilde a_0/Lambda^2
      read*,alam2(3)           ! QGC tilde a_n/Lambda^2
c cuts
      read*,scuts              ! separation cuts
      if(scuts.eq.10)then
c simplified input
        read*,ecutp            ! photon(gluon) energy cut
        read*,ecutl            ! charged-lepton energy cut
        read*,ecutq            ! quark energy cut
        read*,scutqq           ! quark-quark invariant mass cut
        read*,ccutpb           ! angular cut between photon and beam
        read*,ccutpl           ! angular cut between photon and charged lepton
        read*,ccutpq           ! angular cut between photon(gluon) and quark
        read*,ccutll           ! angular cut between charged leptons
        read*,ccutqq           ! angular cut between quarks
        read*,ccutlq           ! angular cut between charged lepton and quark
        read*,ccutlb           ! angular cut between charged lepton and beam
        read*,ccutqb           ! angular cut between quark and beam
      elseif(scuts.eq.11)then
c most general input
        do i1=3,7
          read*,ecut(i1)       ! energy cuts
        enddo
        do i1=1,7
        do i2=max(3,i1+1),7
          read*,ccut(i1,i2)    ! angular cuts
        enddo
        enddo
        do i1=3,7
        do i2=i1+1,7
          read*,scut(i1,i2)    ! invariant-mass cuts
        enddo
        enddo
      endif
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     main initialization routine                                 c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine initialize
      implicit none
c local variables
      integer maxe
      parameter(maxe=29)
      real*8 energy2,conv,alphahelp,scale,pwwidth,brlep,brhad,brtot
      real*8 wwidth,gwl,gwq
      character*6 gfname(0:1,-1:1,3),dgfname(-1:1)
      character*90 process
      integer i1,i2,i3,i4,ng,nl
c mainprog
      real*8 energy,average,averagemax,sigma,avb,sigb
      real*8 weightmax(maxe),weighttotmax,nqcdfac
      integer nevents,neventsw,nweight(maxe),nunweighted(-1:1)
      integer novershoot,nout,iout,steps,particlenum(0:1,-1:1,3)
      character*90 outputfile
c pol 
      real*8 pm,pp
c processpar
      real*8 mv(3),mv2(3),gv(3),qcdfac,qcdfac2,q(6)
      complex*16 cvf(-1:1,3,6),cvw(2)
      integer qprop,quark(6),iso(6),gen(6),dir(7),pol(7),color
      character*6 fname(6),dfname(6)
c param
      real*8 pi,el,alpha,alpha0,alphaz,alphas,gf,cw,cw2,sw,sw2,mw,mw2
      real*8 gw,mz,mz2,gz,mh,mh2,mxe,mxe2,ml(3),ml2(3),mqp(3)
      real*8 mqp2(3),mqm(3),mqm2(3)
c processtype
      integer type,nc,cc,sym,perm
c inputcuts
      real*8 ecutp,ecutl,ecutq,scutqq,ccutpb,ccutpl,ccutpq,ccutll
      real*8 ccutqq,ccutlq,ccutlb,ccutqb
c cuts
      real*8 ecut(7),scut(7,7),ccut(7,7),precomb(4),precombg(2)
      integer scuts,srecomb,srecombg
c options
      integer smc,sborn4,sborn5,sborng5,sisr,src,son,sqcdepem
      integer ssigepem4,ssigepem5,ssigepemg5,scoultree,sunw,spaw,satgc
c event 
      real*8 weight(maxe),pbeam(2,0:3),p(7,0:3,maxe)
      real*8 weightborn,pborn(7,0:3)
      integer nsubevent
c uv
      real*8 mudim2
c ir
      real*8 lambda,lambda2
c rcoptions
      integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
c gwrcoptions
      integer qgwborn,qgwqcd,qgwbos,qgwferm
c rscheme
      real*8 dalpz,drbos,drferm
      integer qalp,qgw
c wswitches
      integer wflag(maxe)
c agqc
      real*8 alam2(0:4)
      integer qaqgc
c atgc
      real*8 gtgc(11,2)
      integer qatgc
c colamps
      real*8 atotsq(maxe),a1sq(maxe),a2sq(maxe)
c common blocks
      common/mainprog/energy,average,averagemax,sigma,avb,sigb,
     *  weightmax,weighttotmax,nqcdfac,nevents,neventsw,nweight,
     *  nunweighted,novershoot,nout,iout,steps,particlenum,outputfile
      common/pol/pm,pp
      common/processpar/cvf,cvw,mv,mv2,gv,qcdfac,qcdfac2,
     *  q,qprop,quark,iso,gen,dir,pol,color,fname,dfname
      common/param/pi,el,alpha,alpha0,alphaz,alphas,gf,cw,cw2,sw,sw2,
     *  mw,mw2,gw,mz,mz2,gz,mh,mh2,mxe,mxe2,ml,ml2,mqp,mqp2,mqm,mqm2
      common/processtype/type,nc,cc,sym,perm
      common/inputcuts/ecutp,ecutl,ecutq,scutqq,ccutpb,ccutpl,ccutpq,
     *  ccutll,ccutqq,ccutlq,ccutlb,ccutqb
      common/cuts/ecut,scut,ccut,precomb,precombg,scuts,
     *  srecomb,srecombg
      common/options/smc,sborn4,sborn5,sborng5,sisr,src,son,sqcdepem,
     *  ssigepem4,ssigepem5,ssigepemg5,scoultree,sunw,spaw,satgc
      common/event/weight,pbeam,p,weightborn,pborn,nsubevent
      common/uv/mudim2
      common/ir/lambda,lambda2
      common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
      common/gwrcoptions/qgwborn,qgwqcd,qgwbos,qgwferm
      common/rscheme/dalpz,drbos,drferm,qalp,qgw
      common/wswitches/wflag
      common/aqgc/alam2,qaqgc          
      common/atgc/gtgc,qatgc
      common/colamps/atotsq,a1sq,a2sq
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     setting parameters                                          c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      i2=0
      do i1=1,90
        if(outputfile(i1:i1).eq.' ')i2=1
        if(i2.eq.1)outputfile(i1:i1)=' '
      enddo
      if(outputfile(1:1).eq.' ')then
        nout=6 
      else
        nout=22 
        open(unit=nout,file=outputfile,status='unknown')
      endif
c iout=0: no output with info about multi-channel integration
      iout=0
      if(iout.ne.0)then
        open(unit=iout,file='optimization.info',status='unknown')
      endif
c setting input parameters
      pi=4d0*datan(1d0)
      call parameter
c further parameters
      conv=3.8937966d11       ! conversion factor GeV^-2 to fbarns
      mxe=1d-15               ! small electron mass for tensor reduction
      lambda=1d0              ! photon mass
      mudim2=1d0              ! renormalization scale
      scale=energy            ! scale in structure functions for ISR
c alternatively, scale^2=-t_min(on-shell):
c      scale=dsqrt(dabs(mw**2-energy**2/2d0*
c     *  (1d0-dsqrt(1d0-4d0*mw**2/energy**2))))
c calculating parameters
      cw=mw/mz                ! weak mixing angle
      cw2=cw**2
      sw2=1d0-cw2
      sw=sqrt(sw2)
      mw2=mw**2
      mz2=mz**2
      mh2=mh**2
      mxe2=mxe**2
      lambda2=lambda**2
      do i1=1,3
        ml2(i1)=ml(i1)**2
        mqp2(i1)=mqp(i1)**2
        mqm2(i1)=mqm(i1)**2
      enddo
c initialize array for wswitches 
c a priori all sub-contributions are included (1)
c (not included: 0)
c wflag(maxe) are only relevant for smc=3 !
      do i1=1,maxe
         wflag(i1)=1
      enddo
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c                                                                 c
c     Warnings                                                    c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(energy.eq.0d0)then
        write(*,'(a)')' Zero CMF energy is not allowed!'
        stop
      endif
      if(src.eq.1)then
        if(energy.eq.2d0*mz)then
          write(*,'(a)')' Energy = 2 M_Z: change energy!'
          stop
        endif
        if(energy.eq.2d0*mqp(3))then
          write(*,'(a)')' Energy = 2 m_top: change energy!'
          stop
        endif
        if(energy.eq.mh+mz)then
          write(*,'(a)')' Energy = M_H + M_Z: change energy!'
          stop
        endif
        if(energy.eq.2d0*mw)then
          write(*,'(a)')' Energy <= 2 M_W: change energy!'
          stop
        endif
      endif
      if(neventsw.eq.0)then
        write(*,'(a)')' Zero number of events is not allowed!'
        stop
      endif
      if(smc.eq.0.or.smc.gt.3)then
        write(*,'(a)')' Please specify MC branch: smc=1,2 or 3!'
        stop
      endif
      if(src.ne.1.and.smc.eq.3)then
        write(*,'(a)')' Please choose smc=1,2 for src=0,2,3!'
        stop
      endif
      if(src.eq.0.and.sborn4.eq.0.and.sborn5.eq.0.and.sborng5.eq.0)then
        write(*,'(a)')
     *    ' Please choose sborn4>0 OR sborn5=1 OR sborng5=1!'
        stop
      endif
      if(((src.eq.0.and.sborn5.eq.1).or.src.eq.3).and.scuts.eq.0)then
        write(*,'(a)')' Please apply separation cuts on the photon!'
        stop
      endif
      if(src.eq.0.and.sborng5.eq.1.and.scuts.eq.0)then
        write(*,'(a)')' Please apply separation cuts on the gluon!'
        stop
      endif
      if(src.eq.1.and.srecomb.ne.0.and.srecomb.ne.3.and.
     *  precomb(1).eq.0d0)then
        write(*,'(a)')' precomb(1)=0d0 is not allowed !'
        stop
      endif
      if(src.eq.1.and.srecomb.ne.0.and.precomb(2).eq.0d0)then
        write(*,'(a)')' precomb(2)=0d0 is not allowed !'
        stop
      endif
      if(src.eq.1.and.srecomb.ne.0.and.precomb(3).eq.0d0)then
        write(*,'(a)')' precomb(3)=0d0 is not allowed !'
        stop
      endif
      if(src.eq.1.and.srecomb.eq.2.and.precomb(4).eq.0d0)then
        write(*,'(a)')' precomb(4)=0d0 is not allowed !'
        stop
      endif
      if(qqcd.eq.1.or.qqcd.eq.3)then
        write(*,'(a)')' qqcd=1,3 is not yet supported !'
        stop
      endif
      if(sborng5.ne.0)then
        write(*,'(a)')' e+e- ->4f+gluon is not yet supported !'
        stop
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c                                                                 c
c     tree-level mode ee->4f                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(src.eq.0.and.sborn4.ne.0)then
        if(smc.eq.3.or.sborn5.ne.0.or.sborng5.ne.0.or.
     *    (qqcd.ne.0.and.qqcd.ne.2).or.scoultree.eq.2.or.
     *    (scoultree.eq.1.and.sborn4.ne.1))then
          write(*,'(a)')'Warning for tree-level mode ee->4f:' 
          write(*,'(a)')'RacoonWW overwrites some of your choices,'
          write(*,'(a)')'since not allowed. See TABLE.1 in README.'
        endif
        if(smc.eq.3)smc=1
        sborn5=0
        sborng5=0
        if(qqcd.ne.0)qqcd=2
        if(scoultree.eq.2)scoultree=1
        if(scoultree.ne.0.and.sborn4.ne.1)scoultree=0
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c                                                                 c
c     tree-level mode ee->4f+photon                               c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(src.eq.0.and.sborn5.eq.1)then
        if(smc.eq.3.or.sborn4.ne.0.or.sborng5.ne.0.or.
     *    (qqcd.ne.0.and.qqcd.ne.2))then
          write(*,'(a)')'Warning for tree-level mode ee->4f+photon:' 
          write(*,'(a)')'RacoonWW overwrites some of your choices,'
          write(*,'(a)')'since not allowed. See TABLE.1 in README.'
        endif
        if(smc.eq.3)smc=1
        sborn4=0
        sborng5=0
        if(qqcd.ne.0)qqcd=2
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c                                                                 c
c     tree-level mode ee->4f+gluon                                c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(src.eq.0.and.sborng5.eq.1)then
        if(smc.eq.3.or.sborn4.ne.0.or.sborn5.ne.0.or.qqcd.ne.0.or.
     *    scoultree.ne.0)then
          write(*,'(a)')'Warning for tree-level mode ee->4f+gluon:' 
          write(*,'(a)')'RacoonWW overwrites some of your choices,'
          write(*,'(a)')'since not allowed. See TABLE.1 in README.'
        endif
        if(smc.eq.3)smc=1
        sborn4=0
        sborn5=0
        qqcd=0
        scoultree=0
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c                                                                 c
c     radiative correction mode                                   c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(src.eq.1)then
        if(qprop.ne.1.or.ssigepem5.ne.5.or.qaqgc.ne.0.or.
     *    sqcdepem.eq.2.or.sborn5.ne.1.or.scoultree.ne.0)then
          write(*,'(a)')'Warning for RC mode:' 
 12       write(*,'(a)')'RacoonWW overwrites some of your choices,'
          write(*,'(a)')'since not allowed. See TABLE.1 in README.'
        endif
        qprop=1
        ssigepem5=5
        if(sqcdepem.eq.2)sqcdepem=1
        qaqgc=0
        sborn5=1
        scoultree=0
        if(smc.eq.3.and.sborn4.ne.1)then
          write(*,'(a)')'Warning for smc=3:' 
          write(*,'(a)')'RacoonWW overwrites some of your choices,'
          write(*,'(a)')'since not allowed. See NOTE.1 in README.'
          sborn4=1
        endif
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c                                                                 c
c     IBA for ee->WW->4f                                          c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(src.eq.2)then
        if(smc.eq.3.or.sborn4.ne.0.or.sborn5.ne.0.or.
     *    sborng5.ne.0.or.sisr.ne.1.or.qalp.ne.2.or.(qgw.ne.0.and.
     *    qgw.ne.2).or.qprop.ne.1.or.ssigepem4.ne.1.or.(qqcd.ne.0.and.
     *    qqcd.ne.2))then
          write(*,'(a)')'Warning for IBA to ee->WW->4f:' 
          write(*,'(a)')'RacoonWW overwrites some of your choices,'
          write(*,'(a)')'since not allowed. See TABLE.1 in README.'
        endif
        if(smc.eq.3)smc=1
        sborn4=0
        sborn5=0
        sborng5=0
        sisr=1
        qalp=2
        if(qgw.ne.0.and.qgw.ne.2)qgw=2
        qprop=1
        ssigepem4=1
        if(qqcd.ne.0)qqcd=2
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c                                                                 c
c     IBA for ee->4f+photon                                       c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(src.eq.3)then
        if(smc.eq.3.or.sborn4.ne.0.or.sborn5.ne.1.or.
     *    sborng5.ne.0.or.sisr.ne.1.or.qalp.ne.2.or.(qgw.ne.0.and.
     *    qgw.ne.2).or.qprop.ne.1.or.(qqcd.ne.0.and.qqcd.ne.2).or.
     *    sqcdepem.eq.2)then
          write(*,'(a)')'Warning for IBA to ee->4f+photon:' 
          write(*,'(a)')'RacoonWW overwrites some of your choices,'
          write(*,'(a)')'since not allowed. See TABLE.1 in README.'
        endif
        if(smc.eq.3)smc=1
        sborn4=0
        sborn5=1
        sborng5=0
        sisr=1
        qalp=2
        if(qgw.ne.0.and.qgw.ne.2)qgw=2
        qprop=1
        if(qqcd.ne.0)qqcd=2
        if(sqcdepem.eq.2)sqcdepem=1
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     parametrization scheme                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(qalp.eq.0)then
	alpha=alpha0
      elseif(qalp.eq.1)then
	alpha=alphaz
      elseif(qalp.eq.2)then
	alpha=sqrt(2d0)*gf/pi*mw2*sw2
      endif
      el=sqrt(4d0*pi*alpha)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     calculating the total W-boson width                         c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(qgw.ne.0)then
        call calcrenc
        call calcrenc_ferm
        qgwborn=1
        alphahelp=alpha
c tree-level W width
        if(qgw.eq.1)then
          qgwqcd=0
          qgwbos=0
          qgwferm=0
c IBA W width
        elseif(qgw.eq.2)then
          alpha=sqrt(2d0)*gf/pi*mw2*sw2
          qgwqcd=1
          qgwbos=0
          qgwferm=0
c one-loop W width without QCD corrections
        elseif(qgw.eq.3)then
          qgwqcd=0
          qgwbos=1
          qgwferm=1
c one-loop W width with QCD corrections
        elseif(qgw.eq.4)then
          qgwqcd=1
          qgwbos=1
          qgwferm=1
        endif
	gw=wwidth()
        alpha=alphahelp
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     setting options                                             c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c qfast=0: form factors for virtual corr. from full formulas (slow !)
c qfast=1: expansion into Legendre polynomials (fast !) 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      qfast=1
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c qsoft=1,2,3: definition of the DPA part of radiative corrections,            
c              terms subtracted from the virtual corrections in DPA     
c              are fixed by:                                    
c qsoft=1: endpoint part as defined in eq. (4.29) of hep-ph/0006307 
c          (default!)    
c qsoft=2: only logarithms as defined in eq. (4.2.53) of 
c          hep-ph/0008033 
c qsoft=3: YFS factor for virtual photons as defined in eq. (4.31) 
c          of hep-ph/0006307
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      qsoft=1
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c son=1,2,3,4: choice of on-shell projection 
c              for virtual corrections in DPA
c RacoonWW provides two different versions for the projection 
c of off-shell momenta to on-shell momenta for               
c e^+(p1)+e^-(p2) -> f1(p3)+ fbar2(p4) + f3(p5)+ fbar4(p6):              
c ( CMS of e^+e^- is assumed )
c son = 1: directions of momenta of W^+, f1 and f3 are fixed
c son = 2: directions of momenta of W^+, f2 and f3 are fixed
c son = 3: directions of momenta of W^+, f1 and f4 are fixed
c son = 4: directions of momenta of W^+, f2 and f4 are fixed
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      son=1
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c radiative corrections                     
c qbos=1 : with virtual bosonic radiative corrections 
c qferm=1: with fermion-loop radiative corrections 
c qnf>0  : non-factorizable corrections are included where
c          qnf=1, 2, or 3 defines how the Coulomb 
c          singularity is implemented.
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(src.eq.1)then
        if(qnf.eq.0)then
          write(*,'(a)')'Warning for RC:' 
          write(*,'(a)')'non-factorizable corrections'
          write(*,'(a)')'should be included. See NOTE.2 in README.'
          qnf=3
        endif
        qborn=0
        qbos=1 
        qferm=1       
        if((qqcd.eq.1.and.(ssigepemg5.ne.1.or.sborng5.ne.1)).or.
     *     (qqcd.eq.3.and.(ssigepemg5.ne.5.or.sborng5.ne.1)))then
          write(*,'(a)')'Warning for RC:' 
          write(*,'(a)')'RacoonWW overwrites some of your choices,'
          write(*,'(a)')'since not allowed. See NOTE.3 in README.'
        endif
        if(qqcd.eq.1.or.qqcd.eq.3)sborng5=1
        if(qqcd.eq.1)ssigepemg5=1
        if(qqcd.eq.3)ssigepemg5=5
      else
        qborn=0
        qbos=0
        qferm=0
        qnf=0
      endif
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c partial W widths (calculated with the same options as chosen for
c the calculation of the amplitude):
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      qgwborn=1
      if(sborn4.eq.0.and.(src.eq.0.or.src.eq.1).and.
     *  sborn5.eq.0)qgwborn=0
      qgwqcd=0
      if(qqcd.ne.0)qgwqcd=1
      qgwbos=qbos
      qgwferm=qferm
c W->l nu_l:
      gwl=pWwidth(-1d0,0d0)
c W->q q':
      gwq=pWwidth(-1d0/3d0,2d0/3d0)
c effective branching ratios:
      brlep=3d0*gwl/gw
      brhad=2d0*gwq/gw
      brtot=(3d0*gwl+2d0*gwq)/gw
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     setting particle parameters                                 c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c fermion names
      gfname(0,-1,1)='e'
      gfname(0,-1,2)='mu'
      gfname(0,-1,3)='tau' 
      gfname(0,+1,1)='nu_e' 
      gfname(0,+1,2)='nu_mu'
      gfname(0,+1,3)='nu_tau'
      gfname(1,-1,1)='d'
      gfname(1,-1,2)='s'
      gfname(1,-1,3)='b'
      gfname(1,+1,1)='u'
      gfname(1,+1,2)='c'
      gfname(1,+1,3)='t'
      dgfname(+1)='      '
      dgfname(-1)=' anti-'
c particle number for hepevt commonblock
      particlenum(0,-1,1)=11
      particlenum(0,-1,2)=13
      particlenum(0,-1,3)=15 
      particlenum(0,+1,1)=12 
      particlenum(0,+1,2)=14
      particlenum(0,+1,3)=16
      particlenum(1,-1,1)=1
      particlenum(1,-1,2)=3
      particlenum(1,-1,3)=5
      particlenum(1,+1,1)=2
      particlenum(1,+1,2)=4
      particlenum(1,+1,3)=6
c beam momenta               
      pbeam(1,0)=0.5d0*energy
      pbeam(1,1)=0d0
      pbeam(1,2)=0d0
      pbeam(1,3)=0.5d0*energy
      pbeam(2,0)=0.5d0*energy
      pbeam(2,1)=0d0
      pbeam(2,2)=0d0
      pbeam(2,3)=-0.5d0*energy
c particle parameters are fixed
      dir(1)=1 
      dir(2)=1
      do i1=3,7
        dir(i1)=-1
      enddo
      fname(1)=gfname(0,-1,1)
      fname(2)=gfname(0,-1,1)
      do i1=1,6
        dfname(i1)=dgfname(dir(i1)*(-1)**i1)      
      do i2=0,1
      do i3=-1,+1,2
      do i4=1,3
        if(fname(i1).eq.gfname(i2,i3,i4))then
          quark(i1)=i2
          iso(i1)=i3
          gen(i1)=i4
        endif
      enddo
      enddo
      enddo
      if(iso(i1).eq.0)then
        write(*,'(a)')' Wrong particle name: ',fname(i1)
        stop
      endif
      enddo
      if(src.ne.1.and.sborn5.eq.0)then
        process=dfname(1)//fname(1)//dfname(2)//fname(2)//' -> '//
     *   dfname(3)//fname(3)//dfname(4)//fname(4)//
     *   dfname(5)//fname(5)//dfname(6)//fname(6)//' '
      elseif(src.ne.1.and.sborn5.ne.0)then
        process=dfname(1)//fname(1)//dfname(2)//fname(2)//' -> '//
     *   dfname(3)//fname(3)//dfname(4)//fname(4)//
     *   dfname(5)//fname(5)//dfname(6)//fname(6)//' + photon '
      elseif(src.eq.1)then
        process=dfname(1)//fname(1)//dfname(2)//fname(2)//' -> '//
     *   dfname(3)//fname(3)//dfname(4)//fname(4)//
     *   dfname(5)//fname(5)//dfname(6)//fname(6)//' (+ photon) '
      endif
      do i1=88,1,-1
        if(process(i1:i1+1).eq.'  ')then
          process=process(1:i1)//process(i1+2:90)//' '
        endif
      enddo
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     setting cut parameters                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c ADLO/TH cuts
      if(scuts.eq.1)then   
        ecutp=0.1d0            ! photon energy cut
        ecutl=1d0              ! charged-lepton energy cut
        ecutq=3d0              ! quark energy cut
        scutqq=5d0             ! quark-quark invariant mass cut
        ccutpb=1d0             ! angular cut between photon and beam
        ccutpl=5d0             ! angular cut between photon and charged lepton
        ccutpq=5d0             ! angular cut between photon(gluon) and quark
        ccutll=5d0             ! angular cut between charged leptons 
        ccutqq=0d0             ! angular cut between quarks 
        ccutlq=5d0             ! angular cut between charged lepton and quark
        ccutlb=10d0            ! angular cut between charged lepton and beam 
        ccutqb=0d0             ! angular cut between quark and beam  
c NLC/TH cuts
      elseif(scuts.eq.2)then
        ecutp=0.1d0            ! photon(gluon) energy cut
        ecutl=1d0              ! charged-lepton energy cut
        ecutq=3d0              ! quark energy cut
        scutqq=0.1d0           ! quark-quark invariant mass cut
        ccutpb=1d0             ! angular cut between photon and beam
        ccutpl=5d0             ! angular cut between photon and charged lepton
        ccutpq=5d0             ! angular cut between photon(gluon) and quark
        ccutll=5d0             ! angular cut between charged leptons
        ccutqq=0d0             ! angular cut between quarks
        ccutlq=5d0             ! angular cut between charged lepton and quark
        ccutlb=10d0            ! angular cut between charged lepton and beam
        ccutqb=5d0             ! angular cut between quark and beam
      endif
c filling cut parameters
      if(scuts.eq.1.or.scuts.eq.2.or.scuts.eq.10)then     
        ecut(7)=ecutp
        ccut(1,7)=ccutpb
        ccut(2,7)=ccutpb
        do i1=3,6
          if(quark(i1).eq.0.and.iso(i1).lt.0)then     ! charged leptons
            ecut(i1)=ecutl
            ccut(1,i1)=ccutlb
            ccut(2,i1)=ccut(1,i1)
            ccut(i1,7)=ccutpl
          elseif(quark(i1).eq.1)then                  ! quarks
            ecut(i1)=ecutq
            ccut(1,i1)=ccutqb
            ccut(2,i1)=ccut(1,i1)
            ccut(i1,7)=ccutpq
          endif
        do i2=i1+1,6
          if(quark(i1).eq.1.and.quark(i2).eq.1)then
            scut(i1,i2)=scutqq
            ccut(i1,i2)=ccutqq
          elseif(quark(i1).eq.0.and.iso(i1).lt.0.and.
     *           quark(i2).eq.0.and.iso(i2).lt.0)then
            ccut(i1,i2)=ccutll
          elseif((quark(i1).eq.0.and.iso(i1).lt.0.and.quark(i2).eq.1)
     *       .or.(quark(i1).eq.1.and.quark(i2).eq.0.and.iso(i2).lt.0)
     *          )then
            ccut(i1,i2)=ccutlq
          endif          
        enddo
        enddo
      endif
c calculating cut parameters
c angle -> cos(angle)
      do i1=1,7
      do i2=1,7
        ccut(i1,i2)=dcos(ccut(i1,i2)*pi/180d0)
      enddo
      enddo
c mass -> mass squared
      do i1=1,7
      do i2=1,7
        scut(i1,i2)=scut(i1,i2)**2
      enddo
      enddo
c filling whole common block
      do i1=1,7
      do i2=i1+1,7
        ccut(i1,i2)=min(ccut(i1,i2),ccut(i2,i1))
        scut(i1,i2)=max(scut(i1,i2),scut(i2,i1))
      enddo
      enddo
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     initialize subroutines                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c initializing amplitude
      call initparam
      call init_epem
      if(sborn4.eq.2.or.sborn4.eq.3)then
        if((nc.eq.1.and.cc.eq.0).or.
     *    (nc.eq.1.and.cc.eq.1.and.(sym.eq.1.or.sym.eq.2)))then
          write(*,'(a)')' Not a valid final-state for DPA Born!'
          stop
        endif
        if(sisr.ne.0)then
          if(src.eq.1)
     $      write(*,'(a)')' No higher-order LL ISR for DPA Born!'
          if(src.ne.1)
     $      write(*,'(a)')' No LL ISR for DPA Born!'
          stop
        endif
      endif
      if(sborng5.ne.0)then
        if((nc.eq.1.and.cc.eq.0).or.
     *    (nc.eq.1.and.cc.eq.1.and.sym.ne.0).or.
     *    (quark(3).eq.0.and.quark(4).eq.0.and.quark(5).eq.0.and.
     *    quark(6).eq.0))then
          if(src.eq.0)
     *      write(*,'(a)')' Not a valid final-state for 4f+gluon!'
          if(src.eq.1)
     *      write(*,'(a)')' Not a valid final-state for qqcd=1 or 3!'
          stop
        endif
      endif
c initializing virtual corrections
      if(src.ne.0.or.sborn4.ge.2)then
        if(cc.eq.1.and.nc.eq.0)then
          if(quark(3).eq.0.and.quark(5).eq.0)i1=1
          if(quark(3).eq.0.and.quark(5).eq.1)i1=2
          if(quark(3).eq.1.and.quark(5).eq.0)i1=3
          if(quark(3).eq.1.and.quark(5).eq.1)i1=4
        elseif(cc.eq.1.and.nc.eq.1.and.(sym.eq.0.or.sym.eq.3))then
          if(quark(3).eq.0)i1=1
          if(quark(3).eq.1)i1=4
        elseif(src.eq.1)then
          write(*,'(a)')
     *         ' Not a valid final-state for virtual corrections!'
          stop
        elseif(src.eq.2)then 
          write(*,'(a)')' Not a valid final-state for IBA ee->WW->4f!'
          stop
        endif
        if(src.ne.3)then
          call init_4f(i1,gen(3),gen(5))
c calculate renormalization constants
          call calcrenc
          call calcrenc_bgf
          call calcrenc_ferm
        endif
      endif
      if(((cc.eq.0.and.nc.eq.1).or.(cc.eq.1.and.nc.eq.1.and.
     *  (sym.eq.1.or.sym.eq.2))).and.scoultree.ne.0)then
        write(*,'(a)')'Warning for Coulomb singularity:' 
        write(*,'(a)')'Coulomb singularity not reasonable for this'
        write(*,'(a)')'process. Changing option to scoultree=0.'
        scoultree=0
      endif
c initialize expansion of formfactors in Legendre polynomials
      if(qfast.eq.1.and.src.eq.1)then
        energy2=energy**2
        ng=20 
        nl=2*ng-1
        call Cfourier(energy2,nl,ng)
        qfast=1
      endif
c naive QCD factor:
      nqcdfac=1d0
      if(qqcd.eq.2)then
        do i1=3,6
          if(quark(i1).ne.0)nqcdfac=nqcdfac*dsqrt(1d0+alphas/pi)
        enddo
      endif
c initializing phase-space slicing
      write(nout,*)' '
      if(smc.ne.2)then
        write(nout,'(a10,i2,(a))')
     *    'smc=',smc,': Phase-space-slicing branch of RacoonWW' 
        write(nout,'(14x,(a))')'======================================'
        write(nout,'(a)')' '
        if(iout.ne.0)then
          write(iout,'(a)')' '
          write(iout,'(a)')' Phase-space-slicing branch of RacoonWW'
          write(iout,'(a)')' ======================================'
          write(iout,'(a)')' '
        endif
        call initslicing(energy,conv,scale,neventsw,nout,iout)
      endif
c initializing subtraction method
      if(smc.eq.2)then
        write(nout,'(a10,i2,(a))')
     *    'smc=',smc,': Subtraction-method branch of RacoonWW'
        write(nout,'(14x,(a))')'====================================='
        if(iout.ne.0)then
          write(iout,'(a)')' '
          write(iout,'(a)')' Subtraction-method branch of RacoonWW'
          write(iout,'(a)')' ====================================='
          write(iout,'(a)')' '
        endif
        call initsubtraction(energy,conv,scale,nout)
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c switch for PAW/HBOOK:                                            
c spaw=1: PAW/HBOOK is used for generating histograms 
c         in addition to default histogram routine; 
c         the output is written in pawplot.paw !
c spaw=1 is only an allowed choice when generating weighted events. 
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      spaw=0
c      if(spaw.eq.1)call setup_graphs(smc,energy)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     output                                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c input parameters
 998  format(/,' Input parameters:',/,' -----------------',/,
     *     4x,'CMF energy = ',f9.5,' GeV',',',4x,
     *     'Number of events = ',i10,',',/,/,
     *     2x,'alpha(0) = 1/', f12.7,',',
     *     1x,'alpha(MZ) = 1/',f9.5,',',
     *     3x,'alpha_s = ',f7.5,',',/,
     *     8x,'GF = ',g12.7,',',/,
     *     8x,'MW = ',f9.5,',',
     *     13x,'MZ = ',f9.5,',',
     *     10x,'MH = ',f9.5,',',/,
     *     8x,'GW = ',f9.5,',',
     *     13x,'GZ = ',f9.5,',',/,
     *     8x,'me = ',g13.8,',',8x,'mmu = ',f13.9,',',
     *     4x,'mtau = ',f9.5,',',/,
     *     8x,'mu = ',f9.5,',',13x,'mc = ',f9.5,',',
     *     10x,'mt = ',f9.5,',',/,
     *     8x,'md = ',f9.5,',',13x,'ms = ',f9.5,',',
     *     10x,'mb = ',f9.5,'.',/)      
      write(nout,998) energy,neventsw,
     *     1d0/alpha0,1d0/alphaz,alphas,GF,
     *     mw,mz,mh,gw,gz,ml(1),ml(2),ml(3),
     *     mqp(1),mqp(2),mqp(3),mqm(1),mqm(2),mqm(3)
      write(nout,'(a)')' Effective branching ratios: '
 999  format(' leptonic BR =',f9.5,',',' hadronic BR =',f9.5,',',
     *     ' total BR =',f9.5)
      write(nout,999)brlep,brhad,brtot
      write(nout,*)' '
      write(nout,'(" Process: ",a60)')process
      write(nout,*)' '
c choice of polarization
      write(nout,'(a10,f4.1,(a))')'pp=',pp,
     *     ': degree of positron beam polarization.'
      write(nout,'(a10,f4.1,(a))')'pm=',pm,
     *     ': degree of electron beam polarization.'
c input-parameter scheme
      if(qalp.eq.0)then
        write(nout,'(a10,i2,(a))')'qalp=',qalp,
     *    ': alpha(0)-parametrization scheme.'
      elseif(qalp.eq.1)then
        write(nout,'(a10,i2,(a))')'qalp=',qalp,
     *    ': alpha(MZ)-parametrization scheme.'
      elseif(qalp.eq.2)then
        write(nout,'(a10,i2,(a))')'qalp=',qalp,
     *    ': GF-parametrization scheme.'
      endif
c calculation of W width
      if(qgw.eq.1)then
        write(nout,'(a10,i2,(a))')'qgw=',qgw,
     *    ': tree-level W-boson width calculated.'
      elseif(qgw.eq.2)then
        write(nout,'(a10,i2,(a))')'qgw=',qgw,
     *    ': IBA W-boson width calculated.'
      elseif(qgw.eq.3)then
        write(nout,'(a10,i2,(a))')'qgw=',qgw,
     *    ': one-loop W-boson width calculated (without QCD corr.).'
      elseif(qgw.eq.4)then
        write(nout,'(a10,i2,(a))')'qgw=',qgw,
     *    ': one-loop W-boson width calculated (with QCD corr.).'
      endif
c treatment of W width in propagator
      if(qprop.eq.0)then
        write(nout,'(a10,i2,(a))')'qprop=',qprop,
     *    ': width = 0.'
      elseif(qprop.eq.1)then
        write(nout,'(a10,i2,(a))')'qprop=',qprop,
     *    ': constant width.'
      elseif(qprop.eq.2)then
        write(nout,'(a10,i2,(a))')'qprop=',qprop,
     *    ': constant width * theta(p^2).'
      elseif(qprop.eq.3)then
        write(nout,'(a10,i2,(a))')'qprop=',qprop,
     *    ': running  width.'
      elseif(qprop.eq.4)then
        write(nout,'(a10,i2,(a))')'qprop=',qprop,
     *    ': constant width and complex sw,cw.'
      endif
c tree-level processes ee -> 4f
      if(sborn4.eq.0)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'sborn4=',sborn4,
     *    ': tree-level process ee -> 4f NOT included.'
      elseif(sborn4.ne.0.and.src.ne.2)then
        write(nout,*)' '
        if(sborn4.eq.1)then
          write(nout,'(a10,i2,(a))')'sborn4=',sborn4,
     *      ': tree-level process ee -> 4f.'
          if(scoultree.ne.0)write(nout,'(a10,i2,(a))')'scoultree=',
     $      scoultree,': Coulomb sing. to ee->4f included.'
        elseif(sborn4.eq.2)then
          write(nout,'(a10,i2,(a))')'sborn4=',sborn4,
     *      ': tree-level ee -> 4f in DPA (off-shell phase space).'
        elseif(sborn4.eq.3)then
          write(nout,'(a10,i2,(a))')'sborn4=',sborn4,
     *      ': tree-level ee -> 4f in DPA.'
        endif
        if(sqcdepem.eq.1)then
          write(nout,'(a10,i2,(a))')'sqcdepem=',sqcdepem,
     *      ': gluon-exchange diagrams included.'
        endif
        if(ssigepem4.eq.0)then
          write(nout,'(a10,i2,(a))')'ssigepem4=',ssigepem4,
     *      ': all electroweak diagrams included.'
        elseif(ssigepem4.eq.1)then
          write(nout,'(a10,i2,(a))')'ssigepem4=',ssigepem4,
     *      ': only electroweak WW-signal diagrams.'
        elseif(ssigepem4.eq.2)then
          write(nout,'(a10,i2,(a))')'ssigepem4=',ssigepem4,
     *      ': only electroweak ZZ-signal diagrams.'
        elseif(ssigepem4.eq.3)then
          write(nout,'(a10,i2,(a))')'ssigepem4=',ssigepem4,
     *      ': only electroweak ZZ,ZA,AA-signal diagrams.'
        elseif(ssigepem4.eq.4)then
          write(nout,'(a10,i2,(a))')'ssigepem4=',ssigepem4,
     *      ': only electroweak WW,ZZ,ZA,AA-signal diagrams.'
        elseif(ssigepem4.eq.5)then
          write(nout,'(a10,i2,(a))')'ssigepem4=',ssigepem4,
     *      ': only diagrams of CC11 class.'
        endif
        if(qqcd.eq.2)then
          write(nout,'(a10,i2,(a))')'qqcd=',qqcd,
     *      ': naive QCD corrections included. '
        endif
      endif
c improved born approximation ee->WW->4f
      if(src.eq.2)then
        write(nout,'(a10,i2,(a))')'src=',src,
     *    ': improved Born approximation ee->WW->4f.'
        if(qqcd.eq.2)then
          write(nout,'(a10,i2,(a))')'qqcd=',qqcd,
     *      ': naive QCD corrections included. '
        endif
      endif
c improved born approximation ee -> 4f+photon
      if(src.eq.3)then
        write(nout,'(a10,i2,(a))')'src=',src,
     *    ': improved Born approximation ee->4f+photon.'
      endif
c tree-level processes ee -> 4f+photon
      if(sborn5.ne.0.and.src.eq.0)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'sborn5=',sborn5,
     *    ': tree-level process ee -> 4f+gamma.'
      endif
c IBA and tree-level mode for ee -> 4f+photon
      if((sborn5.ne.0.and.src.eq.0).or.src.eq.3)then
        if(scoultree.ne.0)write(nout,'(a10,i2,(a))')'scoultree=',
     $    scoultree,': Coulomb sing. to ee->4f+photon included.'
        if(sqcdepem.eq.1)then
          write(nout,'(a10,i2,(a))')'sqcdepem=',sqcdepem,
     *      ': gluon-exchange diagrams included.'
        elseif(sqcdepem.eq.2)then
          write(nout,'(a10,i2,(a))')'sqcdepem=',sqcdepem,
     *      ': only gluon-exchange diagrams.'
        endif
        if(ssigepem5.eq.0)then
          write(nout,'(a10,i2,(a))')'ssigepem5=',ssigepem5,
     *      ': all electroweak diagrams included.'
        elseif(ssigepem5.eq.1)then
          write(nout,'(a10,i2,(a))')'ssigepem5=',ssigepem5,
     *      ': only electroweak WW-signal diagrams.'
        elseif(ssigepem5.eq.2)then
          write(nout,'(a10,i2,(a))')'ssigepem5=',ssigepem5,
     *      ': only electroweak ZZ-signal diagrams.'
        elseif(ssigepem5.eq.3)then
          write(nout,'(a10,i2,(a))')'ssigepem5=',ssigepem5,
     *      ': only electroweak ZZ,ZA,AA-signal diagrams.'
        elseif(ssigepem5.eq.4)then
          write(nout,'(a10,i2,(a))')'ssigepem5=',ssigepem5,
     *      ': only electroweak WW,ZZ,ZA,AA-signal diagrams.'
        elseif(ssigepem5.eq.5)then
          write(nout,'(a10,i2,(a))')'ssigepem5=',ssigepem5,
     *      ': only diagrams of CC11 class.'
        endif
        if(qqcd.eq.2)then
          write(nout,'(a10,i2,(a))')'qqcd=',qqcd,
     *      ': naive QCD corrections included. '
        endif
      endif
c tree-level processes ee -> 4f+gluon
      if(sborng5.ne.0.and.src.eq.0)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'sborng5=',sborng5,
     *    ': tree-level process ee -> 4f+gluon.'
        if(ssigepemg5.eq.1)then
          write(nout,'(a10,i2,(a))')'ssigepemg5=',ssigepemg5,
     *      ': only electroweak WW-signal diagrams.'
        elseif(ssigepemg5.eq.5)then
          write(nout,'(a10,i2,(a))')'ssigepemg5=',ssigepemg5,
     *      ': only diagrams of CC11 class.'
        endif
      endif
c radiative-correction mode:
      if(src.eq.1)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'src=',src,
     *    ': virtual corrections in DPA and real corrections included.'
        write(nout,'(a10,i2,(a))')'ssigepem5=',ssigepem5, 
     *    ': real photon corr. : only CC11 class of diagrams included.'
        if(qqcd.eq.1)then
          write(nout,'(a10,i2,(a))')'qqcd=',qqcd,
     *      ': QCD corrections to CC03 included (full matrix element)'
        else if(qqcd.eq.2)then
          write(nout,'(a10,i2,(a))')'qqcd=',qqcd,
     *      ': naive QCD corrections included '
        else if(qqcd.eq.3)then
          write(nout,'(a10,i2,(a))')'qqcd=',qqcd,
     *      ': QCD corrections to CC11 included (full matrix element)'
        endif
        if(qreal.eq.1)then
          write(nout,'(a10,i2,(a))')'qreal=',qreal,
     *      ': imaginary part of virtual corrections neglected.'
        else
          write(nout,'(a10,i2,(a))')'qreal=',qreal,
     *      ': imaginary part of virtual corrections included.'
        endif
        if(qnf.eq.1)then
          write(nout,'(a10,i2,(a))')'qnf=',qnf,
     *      ': on-shell Coulomb singularity included.'
        elseif(qnf.eq.2)then
          write(nout,'(a10,i2,(a))')'qnf=',qnf,
     *      ': off-shell Coulomb singularity included.'
        elseif(qnf.eq.3)then
          write(nout,'(a10,i2,(a))')'qnf=',qnf,
     *      ': off-shell Coulomb singularity '//
     *      'with off-shell Born included.'
        endif 
      endif
c higher-order ISR
      if(sisr.eq.0)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'sisr=',sisr,
     *    ': higher-order initial-state radiation NOT included.'
      else
        if(smc.ne.3)then 
          write(nout,'(a10,i2,(a))')'sisr=',sisr,
     *      ': initial-state radiation up to order alpha^3 included.'
        elseif(smc.eq.3)then 
          write(nout,'(a10,i2,(a))')'sisr=',sisr,
     *      ': initial-state radiation up to order alpha^3 included.'
        endif
      endif      
c recombination cuts
      if(srecomb.eq.1.or.srecomb.eq.2.or.srecomb.eq.3)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'srecomb=',srecomb,
     *    ': with photon recombination:'
        do i2=1,4
          write(nout,'(" precomb(",i1,")"," = ",f9.5)')i2,precomb(i2)
        enddo
      endif
      if(srecombg.eq.1.or.srecombg.eq.2)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'srecombg=',srecombg,
     *    ': with gluon recombination:'
        do i2=1,2
          write(nout,'(" precombg(",i1,")"," = ",f9.5)')i2,precombg(i2)
        enddo
      endif
c separation cuts
      write(nout,*)' '
      if(scuts.eq.0)then
        write(nout,'(a10,i2,(a))')'scuts=',scuts,
     *    ': no separation cuts !'
      else
        write(nout,'(a10,i2,(a))')'scuts=',scuts,
     *    ': with separation cuts:'
        do i1=1,7
          if(ecut(i1).gt.0d0)then
            write(nout,'(" energy(",i1,")  > ",f9.5," GeV")')
     *        i1,ecut(i1)
          endif
        enddo
        do i1=1,7
        do i2=i1+1,7 
          if(ccut(i1,i2).lt.1d0)then
            write(nout,'(" angle(",i1,",",i1,") > ",f9.5," deg")')
     *        i1,i2,dacos(ccut(i1,i2))/pi*180d0
          endif
        enddo
        enddo
        do i1=1,7
        do i2=i1+1,7 
          if(scut(i1,i2).gt.0d0)then
            write(nout,'(" mass(",i1,",",i1,")  > ",f9.5," GeV")')
     *        i1,i2,dsqrt(scut(i1,i2))
          endif
        enddo
        enddo
      endif
c TGC
      if(satgc.eq.1)then
        write(nout,'(a)')' '
        write(nout,'(a10,i2,(a))')'satgc=',satgc,
     *    ': anomalous triple gauge-boson couplings included.'
        write(nout,'(36x,"V=A           V=Z")')
        write(nout,'(14x,"Delta g_1^V    = ",2(1f10.5,4x))')
     *    gtgc(1,1),gtgc(1,2)
        write(nout,'(14x,"Delta kappa^V  = ",2(1f10.5,4x))')
     *    gtgc(2,1),gtgc(2,2)
        write(nout,'(14x,"lambda^V       = ",2(1f10.5,4x))')
     *    gtgc(3,1),gtgc(3,2)
        write(nout,'(14x,"g_4^V          = ",2(1f10.5,4x))')
     *    gtgc(4,1),gtgc(4,2)
        write(nout,'(14x,"g_5^V          = ",2(1f10.5,4x))')
     *    gtgc(5,1),gtgc(5,2)
        write(nout,'(14x,"tilde kappa^V  = ",2(1f10.5,4x))')
     *    gtgc(6,1),gtgc(6,2)
        write(nout,'(14x,"tilde lambda^V = ",2(1f10.5,4x))')
     *    gtgc(7,1),gtgc(7,2)
        write(nout,'(14x,"f_4^V          = ",2(1f10.5,4x))')
     *    gtgc(8,1),gtgc(8,2)
        write(nout,'(14x,"f_5^V          = ",2(1f10.5,4x))')
     *    gtgc(9,1),gtgc(9,2)
        write(nout,'(14x,"h_4^V          = ",2(1f10.5,4x))')
     *    gtgc(10,1),gtgc(10,2)
        write(nout,'(14x,"h_5^V          = ",2(1f10.5,4x))')
     *    gtgc(11,1),gtgc(11,2)
      endif
c QGC
      if(qaqgc.eq.1)then
        write(nout,'(a)')' '
        write(nout,'(a10,i2,(a))')'qaqgc=',qaqgc,
     *    ': anomalous quartic gauge-boson couplings included.'
        write(nout,'(14x,"a_0/Lambda^2       = ",1f10.5)')alam2(0)
        write(nout,'(14x,"a_c/Lambda^2       = ",1f10.5)')alam2(1)
        write(nout,'(14x,"a_n/Lambda^2       = ",1f10.5)')alam2(2)
        write(nout,'(14x,"tilde a_0/Lambda^2 = ",1f10.5)')alam2(4)
        write(nout,'(14x,"tilde a_n/Lambda^2 = ",1f10.5)')alam2(3)
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     initializing numerical integration                          c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c initialization:
      steps=1000000    ! output all 'steps' times
      if(smc.ne.2)nsubevent=8
      if(smc.eq.2)nsubevent=maxe
c initialization of weights and four-momenta:
      nevents=0 
      sunw=0     
      do i1=1,maxe
        nweight(i1)=0
        weight(i1)=0d0
        weightmax(i1)=0d0
        atotsq(i1)=0d0
        a1sq(i1)=0d0
        a2sq(i1)=0d0
        do i2=1,7
        do i3=0,3   
          p(i2,i3,i1)=0d0
        enddo
        enddo
      enddo
      weightborn=0d0
      do i1=1,7
      do i2=0,3   
        pborn(i1,i2)=0d0
      enddo
      enddo
      weighttotmax=0d0
      nunweighted(-1)=0
      nunweighted(1)=0
      novershoot=0
      averagemax=0d0
      average=0d0
      sigma=0d0
      avb=0d0
      sigb=0d0
      call settings(1,pbeam,p(1,0,1),0d0,energy,1,1)
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     generation of weighted events                               c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine integration(avprel,siprel)
      implicit none 
c local variables
      integer maxe
      parameter(maxe=29)
      real*8 avprel,siprel,atotsqr,a1sqr,a2sqr
      integer unweighting,i1
c mainprog
      real*8 energy,average,averagemax,sigma,avb,sigb
      real*8 weightmax(maxe),weighttotmax,nqcdfac
      integer nevents,neventsw,nweight(maxe),nunweighted(-1:1)
      integer novershoot,nout,iout,steps,particlenum(0:1,-1:1,3)
      character*90 outputfile
      common/mainprog/energy,average,averagemax,sigma,avb,sigb,
     *  weightmax,weighttotmax,nqcdfac,nevents,neventsw,nweight,
     *  nunweighted,novershoot,nout,iout,steps,particlenum,outputfile
      unweighting=0
      do i1=1,neventsw
        call eventgeneration(atotsqr,a1sqr,a2sqr,unweighting)
      enddo
c calculation of preliminary results  
      avprel=average/dble(nevents)
      siprel=sigma/dble(nevents)**2
      siprel=dsqrt(dabs(siprel-avprel**2/dble(nevents)))
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     generation of events                                        c
c     unweighting=0 : generating weighted events                  c
c                    initialization of unweighting                c
c     unweighting=1 : generation of a single unweighted event     c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine eventgeneration(atotsqr,a1sqr,a2sqr,unweighting)
      implicit none 
c local variables
      integer maxe
      parameter(maxe=29)
      real*8 rand,si,av,y,kern,random(15),atotsqr,a1sqr,a2sqr
      integer i1,i2,i3,ns,unweighting
c mainprog
      real*8 energy,average,averagemax,sigma,avb,sigb
      real*8 weightmax(maxe),weighttotmax,nqcdfac
      integer nevents,neventsw,nweight(maxe),nunweighted(-1:1)
      integer novershoot,nout,iout,steps,particlenum(0:1,-1:1,3)
      character*90 outputfile
c processpar
      real*8 mv(3),mv2(3),gv(3),qcdfac,qcdfac2,q(6)
      complex*16 cvf(-1:1,3,6),cvw(2)
      integer qprop,quark(6),iso(6),gen(6),dir(7),pol(7),color
      character*6 fname(6),dfname(6)
c options
      integer smc,sborn4,sborn5,sborng5,sisr,src,son,sqcdepem
      integer ssigepem4,ssigepem5,ssigepemg5,scoultree,sunw,spaw,satgc
c event 
      real*8 weight(maxe),pbeam(2,0:3),p(7,0:3,maxe)
      real*8 weightborn,pborn(7,0:3)
      integer nsubevent
c wswitches
      integer wflag(maxe)
c colamps
      real*8 atotsq(maxe),a1sq(maxe),a2sq(maxe)
c hepevt
      integer nmxhep
      parameter(nmxhep=4000)
      real*8 phep(5,nmxhep),vhep(4,nmxhep)
      integer nevhep,nhep,isthep(nmxhep),idhep(nmxhep)
      integer jmohep(2,nmxhep),jdahep(2,nmxhep)
c common blocks
      common/mainprog/energy,average,averagemax,sigma,avb,sigb,
     *  weightmax,weighttotmax,nqcdfac,nevents,neventsw,nweight,
     *  nunweighted,novershoot,nout,iout,steps,particlenum,outputfile
      common/processpar/cvf,cvw,mv,mv2,gv,qcdfac,qcdfac2,
     *  q,qprop,quark,iso,gen,dir,pol,color,fname,dfname
      common/options/smc,sborn4,sborn5,sborng5,sisr,src,son,sqcdepem,
     *  ssigepem4,ssigepem5,ssigepemg5,scoultree,sunw,spaw,satgc
      common/event/weight,pbeam,p,weightborn,pborn,nsubevent
      common/wswitches/wflag
      common/colamps/atotsq,a1sq,a2sq
      common/hepevt/nevhep,nhep,isthep,idhep,jmohep,jdahep,phep,vhep
      save av,si,ns
      if(ns.lt.nsubevent.and.sunw.eq.1)goto 300      
 100  nevents=nevents+1
c generation of random numbers
      do i1=1,15
        call rans(rand)
        random(i1)=rand
      enddo
c calculating not all weights for each event in unweighting procedure
      if(unweighting.eq.1)then
        if(sunw.eq.0)then
          if(smc.ne.3.and.src.eq.1)then
            write(*,'(a)')
     *        ' Sorry, unweighting not possible for these options!'
            stop
          endif
          do i1=1,nsubevent
            if(weightmax(i1).ne.0d0)then
              nweight(i1)=int(weighttotmax/weightmax(i1))
            else
              nweight(i1)=0
            endif
          enddo
          sunw=1
        endif
        do i1=1,nsubevent
          if(nweight(i1).ne.0)then 
            if(mod(nevents,nweight(i1)).eq.0)then
              wflag(i1)=1
            else
              wflag(i1)=0
            endif
          endif
        enddo 
      endif
c calls for MC kernel
      if(smc.eq.1)then
        y=kern(random,1d0)
      elseif(smc.eq.2)then
        call subtraction(random)
      elseif(smc.eq.3)then
        y=kern(random,1d0) 
      endif
      if(unweighting.eq.1)then
        weightborn=weightborn*dble(nweight(1)*wflag(1))
        do i1=1,nsubevent
          weight(i1)=weight(i1)*dble(nweight(i1)*wflag(i1))
        enddo
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     summation of sub-contributions and filling of histograms    c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      y=0d0
      do i1=1,nsubevent
        y=y+weight(i1)*nqcdfac
        call settings(i1,pbeam,p(1,0,i1),weight(i1)*nqcdfac,energy,
     *    nevents,2)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c When using paw for histogram plotting, please
c uncomment the following lines:
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c        if(spaw.eq.1.and.weight(i1).ne.0d0)then
c          call graphs(smc,i1,q,weight(i1)*nqcdfac,1d0/neventsw)
c          if(smc.le.2.and.i1.ne.1)then
c            if((smc.eq.2.and.i1.le.5).or.(smc.eq.1.and.i1.le.6))  
c     $        call graphs_k(smc,i1,weight(i1)*nqcdfac,1d0/neventsw)
c          endif
c          if(smc.eq.3.and.i1.le.6)then
c            if(i1.eq.1)then  
c              call graphs_k(smc,i1,
c     $            (weight(i1)-weightborn)*nqcdfac,1d0/neventsw)
c            else
c              call graphs_k(smc,i1,weight(i1)*nqcdfac,1d0/neventsw)
c            endif
c          endif
c          if((smc.eq.2.and.i1.gt.5).or.(smc.ne.2.and.i1.gt.6)) 
c     $      call graphs_r(smc,i1,weight(i1)*nqcdfac,1d0/neventsw)
c        endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      enddo
c      if(spaw.eq.1.and.weightborn.ne.0d0)
c     *  call graphs_b(smc,1,weightborn,1d0/neventsw)
      average=average+y
      sigma=sigma+y**2
c Born
      avb=avb+weightborn
      sigb=sigb+weightborn**2
c preliminary results
      av=av+y
      si=si+y**2
      if(mod(nevents,steps).eq.0)then
        if(nevents.eq.steps)then
          write(nout,*)' '
          write(nout,'(a)')'    events  :  intermediary results   :'//
     *      '  preliminary results '
        endif
        write(nout,'(i10,2("  : ",f10.5," +-",f9.5))')
     *    nevents,av/dble(steps),
     *    dsqrt(dabs(si-av**2/dble(steps)))/dble(steps),
     *    average/dble(nevents),
     *    dsqrt(dabs(sigma-average**2/dble(nevents)))/dble(nevents)
        av=0d0
        si=0d0
      endif
c maximal weight
      if(y.gt.averagemax)averagemax=y
      call settings(1,pbeam,pborn,weightborn,energy,nevents,3)
c initialization for unweighting=0: determine overall maximal weight and 
c maximal weight for each subcontribution
      if(unweighting.eq.0)then
        atotsqr=0d0
        a1sqr=0d0
        a2sqr=0d0
        do i1=1,nsubevent
          if(weightmax(i1).lt.weight(i1))then
            weightmax(i1)=weight(i1)
            weighttotmax=max(weighttotmax,weightmax(i1))
          endif
        enddo
        return          
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     unweighting procedure                                       c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c generating unweighted events for unweighting=1
      call rans(rand)
      ns=0
 200  ns=ns+1
      if(nweight(ns).ne.0)then
        if(mod(nevents,nweight(ns)).eq.0)then
          if(weight(ns).gt.2d0*weighttotmax)then
c events with weight > weightmax
            novershoot=novershoot+1
            write(nout,'("Warning: weight>weighttotmax",2i12)')
     *        novershoot,nunweighted(1)
            write(nout,'(9x,"weight/weighttotmax=",1d10.5)')
     *        weight(ns)/weighttotmax
            write(nout,'(9x,"Redefining weighttotmax=weight")')
              weighttotmax=weight(ns)
          endif
          if(rand*weighttotmax.lt.dabs(weight(ns)))then
c unweighted events with weight=1
            if(weight(ns).gt.0d0)then
              nunweighted(1)=nunweighted(1)+1
              do i2=1,6
                idhep(i2)=dir(i2)*(-1)**i2
     *            *particlenum(quark(i2),iso(i2),gen(i2))
              enddo
              do i2=1,2
                isthep(i2)=3
                jmohep(1,i2)=0
                jmohep(2,i2)=0
              enddo
              do i2=3,6
                isthep(i2)=1
                jmohep(1,i2)=1
                jmohep(2,i2)=2
              enddo
              do i2=1,8
                jdahep(1,i2)=0      
                jdahep(2,i2)=0      
                phep(5,i2)=0d0
                do i3=1,4
                  vhep(i3,i2)=0d0  
                enddo                     
              enddo       
              if((ns.ne.7.and.smc.ne.2).or.(ns.ne.6.and.smc.eq.2))then
                nhep=6
                isthep(7)=0
                idhep(7)=0
                jmohep(1,7)=0
                jmohep(2,7)=0
                do i2=1,4
                  phep(i2,7)=0d0
                enddo
              else
                nhep=7
                isthep(7)=1
                idhep(7)=22
                jmohep(1,7)=1
                jmohep(2,7)=2
              endif
              nevhep=nunweighted(1)
c setting momenta
              do i2=1,2
                phep(4,i2)=pbeam(i2,0)
                phep(1,i2)=pbeam(i2,1)
                phep(2,i2)=pbeam(i2,2)
                phep(3,i2)=pbeam(i2,3)
              enddo
              do i2=3,nhep
                phep(4,i2)=p(i2,0,ns)
                phep(1,i2)=p(i2,1,ns)
                phep(2,i2)=p(i2,2,ns)
                phep(3,i2)=p(i2,3,ns)
              enddo
c photon from initial-state radiation added
              do i1=1,2
                if(abs(p(i1,0,ns)-pbeam(i1,0)).gt.1d-10)then
                  nhep=nhep+1
                  isthep(nhep)=1
                  idhep(nhep)=22
                  jmohep(1,nhep)=1
                  jmohep(2,nhep)=2
                  phep(4,nhep)=pbeam(i1,0)-p(i1,0,ns)
                  phep(1,nhep)=pbeam(i1,1)-p(i1,1,ns)
                  phep(2,nhep)=pbeam(i1,2)-p(i1,2,ns)
                  phep(3,nhep)=pbeam(i1,3)-p(i1,3,ns)
                  phep(5,nhep)=0d0
                endif
              enddo
c color amplitudes required for Pythia
              atotsqr=atotsq(ns)
              a1sqr=a1sq(ns)
              a2sqr=a2sq(ns)
c warning
              if(sqcdepem.eq.1.and.src.eq.1.and.nunweighted(1).eq.1)then
                write(nout,'(a)')'Warning: use of '//
     *            'gluon-exchange diagrams and Pythia is inconsistent!'
              endif
              return
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                     c
c Unweighted events                                                   c
c                                                                     c
c The four-momenta of the external particles can be read out here !   c 
c beam (e+)                      phep(1:4,1)                          c
c beam (e-)                      phep(1:4,2)                          c
c final-state fermion 3          phep(1:4,3)                          c
c final-state anti-fermion 4     phep(1:4,4)                          c
c final-state fermion 5          phep(1:4,5)                          c
c final-state anti-fermion 6     phep(1:4,6)                          c
c photon                         phep(1:4,7)                          c
c photon                         phep(1:4,8)                          c
c photon                         phep(1:4,9)                          c
c The photons collinear to the beam result from ISR (structure        c
c functions)                                                          c
c                                                                     c
c Remark:                                                             c
c CC-processes: W+ -> f3 + f4, W- -> f5 + f6                          c
c MC-processes: W+ -> f3 + f6, W- -> f4 + f5                          c
c                                                                     c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
            else
c "unweighted" events with weight=-1
              nunweighted(-1)=nunweighted(-1)+1
              if(nunweighted(-1).le.10000)then
                write(nout,'(" Warning: weight=-1 ",2i12)')
     *            nunweighted(-1),nunweighted(1)
              endif
            endif
          endif
        endif
      endif
 300  if(ns.lt.nsubevent)goto 200
      goto 100
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     writeoutput                                                 c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine writeoutput
      implicit none
c local variables
      integer maxe
      parameter(maxe=29)
      real*8 rej(maxe),dscinti(maxe)
      integer i1
c mainprog
      real*8 energy,average,averagemax,sigma,avb,sigb
      real*8 weightmax(maxe),weighttotmax,nqcdfac
      integer nevents,neventsw,nweight(maxe),nunweighted(-1:1)
      integer novershoot,nout,iout,steps,particlenum(0:1,-1:1,3)
      character*90 outputfile
c options
      integer smc,sborn4,sborn5,sborng5,sisr,src,son,sqcdepem
      integer ssigepem4,ssigepem5,ssigepemg5,scoultree,sunw,spaw,satgc
c rcoptions
      integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
c event 
      real*8 weight(maxe),pbeam(2,0:3),p(7,0:3,maxe)
      real*8 weightborn,pborn(7,0:3)
      integer nsubevent
c maxweight
      real*8 wmax(7),wgmax,wqmax
c par_conti
      real*8 conti(maxe),conti_tot(maxe),dconti_tot(maxe)
c par_n
      integer ntot(2),ncut(2),ncut1_k(2),ncut2_k(2),ncutq_r(2)
      integer ncut3_k(2),ncut4_k(2),ncut5_k(2),ncut6_k(2),ncut_r(2)
c subcontr
      real*8 subav(4),subsi(4),submax(4)
      integer subch(4),subn(4),subrej(maxe)
c common blocks
      common/mainprog/energy,average,averagemax,sigma,avb,sigb,
     *  weightmax,weighttotmax,nqcdfac,nevents,neventsw,nweight,
     *  nunweighted,novershoot,nout,iout,steps,particlenum,outputfile
      common/options/smc,sborn4,sborn5,sborng5,sisr,src,son,sqcdepem,
     *  ssigepem4,ssigepem5,ssigepemg5,scoultree,sunw,spaw,satgc
      common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
      common/event/weight,pbeam,p,weightborn,pborn,nsubevent
      common/maxweight/wmax,wgmax,wqmax
      common/par_conti/conti,conti_tot,dconti_tot
      common/par_n/ntot,ncut,ncut1_k,ncut2_k,ncut3_k,ncut4_k,
     *  ncut5_k,ncut6_k,ncut_r,ncutq_r
      common/subcontr/subav,subsi,submax,subch,subn,subrej
      call settings(1,pbeam,p(1,0,1),0d0,energy,nevents,4)
c calculation of total cross section and statistical error
      average=average/dble(nevents)
      sigma=sigma/dble(nevents)**2
      sigma=dsqrt(dabs(sigma-average**2/dble(nevents)))
      averagemax=averagemax/dble(nevents)
c Born
      avb=avb/dble(nevents)
      sigb=sigb/dble(nevents)**2
      sigb=dsqrt(dabs(sigb-avb**2/dble(nevents)))
c calculation of results for subcontributions:
      if(smc.ne.2)then
        do i1=1,maxe
          conti_tot(i1)=conti_tot(i1)/dble(nevents)
          dconti_tot(i1)=dconti_tot(i1)/dble(nevents)**2
          dscinti(i1)=dsqrt(dabs(dconti_tot(i1)-
     *      conti_tot(i1)**2/dble(nevents)))
        enddo
        do i1=1,7
          wmax(i1)=wmax(i1)/dble(nevents) 
        enddo
        wgmax=wgmax/dble(nevents) 
        wqmax=wqmax/dble(nevents) 
      elseif(smc.eq.2)then
        do i1=1,4
          subav(i1)=subav(i1)/dble(nevents)
          subsi(i1)=subsi(i1)/dble(nevents)**2
          subsi(i1)=dsqrt(dabs(subsi(i1)-subav(i1)**2/dble(nevents)))
          submax(i1)=submax(i1)/dble(nevents)
        enddo
      endif
c output
 1001 format((a),f4.1,"%")
 1002 format(a22,f16.10," fb")
 1003 format(" (event = ",i9,", channel = ",i3,")")
 1004 format(" % events rejected in calculation of weight(",i2,"):"
     *  ,f4.1,"%")
      write(nout,'(a)')' '
      write(nout,'(a)')' Result:'
      write(nout,'(a)')' ------- '
      write(nout,'(" Number of weighted events   = ",i16)')nevents
      write(nout,1002)' Average            = ',average
      write(nout,1002)' Standard deviation = ',sigma
      write(nout,1002)' Maximal weight     = ',averagemax
      write(nout,'(a)')' '
      if(smc.eq.3.and.sunw.eq.1)then
        write(nout,'(a)')' Tree-level four-fermion cross section:'
        write(nout,1002)' Average            = ',avb
        write(nout,1002)' Standard deviation = ',sigb
        write(nout,'(a)')' '
      endif
c additional output for slicing branch
      if(smc.eq.1.or.(smc.eq.3.and.sunw.eq.0))then
        if(qqcd.eq.2)then 
          write(nout,'(a)')' Subcontributions without naive QCD:'
          write(nout,'(a)')' -----------------------------------'
        else
          write(nout,'(a)')' Subcontributions :'
          write(nout,'(a)')' ------------------'
        endif
c write subcontributions 
        write(nout,'(a)')' Tree-level four-fermion cross section:'
        write(nout,1002)' Average            = ',conti_tot(1)
        write(nout,1002)' Standard deviation = ',dscinti(1)
        write(nout,1002)' Maximal weight     = ',wmax(1)
        write(nout,*)' '
        write(nout,'(a)')' 2->4 DPA part :'
        write(nout,1002)' Average            = ',conti_tot(2)
        write(nout,1002)' Standard deviation = ',dscinti(2)
        write(nout,1002)' Maximal weight     = ',wmax(2)
        write(nout,*)' '
        write(nout,'(a)')
     *    ' 2->4 virtual+soft+final-state collinear photon part:'
        if(sisr.ne.0)then
          if(smc.ne.3)write(nout,'(a)')' without LL O(alpha) ISR'
          if(smc.eq.3)write(nout,'(a)')' with LL O(alpha^3) ISR'
        endif
        write(nout,1002)' Average            = ',conti_tot(3)
        write(nout,1002)' Standard deviation = ',dscinti(3)
        write(nout,1002)' Maximal weight     = ',wmax(3)
        write(nout,*)' '
        write(nout,'(a)')
     *    ' 2->4 initial-state collinear photon part (e+):'
        if(sisr.ne.0)then
          if(smc.ne.3)write(nout,'(a)')' without LL O(alpha) ISR'
          if(smc.eq.3)write(nout,'(a)')' with LL O(alpha^3) ISR'
        endif
        write(nout,1002)' Average            = ',conti_tot(4)
        write(nout,1002)' Standard deviation = ',dscinti(4)
        write(nout,1002)' Maximal weight     = ',wmax(4)
        write(nout,*)' '
        write(nout,'(a)')
     *    ' 2->4 initial-state collinear photon part (e-):'
        if(sisr.ne.0)then
          if(smc.ne.3)write(nout,'(a)')' without LL O(alpha) ISR'
          if(smc.eq.3)write(nout,'(a)')' with LL O(alpha^3) ISR'
        endif
        write(nout,1002)' Average            = ',conti_tot(5)
        write(nout,1002)' Standard deviation = ',dscinti(5)
        write(nout,1002)' Maximal weight     = ',wmax(5)
        write(nout,*)' '
        if(src.eq.1)then 
          if(smc.ne.3)write(nout,'(a)')
     *      ' 2->4 LL initial-state radiation up to O(alpha^3):'
          if(smc.eq.3)write(nout,'(a)')
     *      ' 2->4 LL O(alpha^3) initial-state radiation:'
        elseif(src.eq.2)then
          write(nout,'(a)')' Improved Born Approximation ee->WW->4f:'
        elseif(src.eq.0)then
          write(nout,'(a)')
     $      ' 2->4 Born + LL initial-state radiation up to O(alpha^3):'
        endif
        write(nout,1002)' Average            = ',conti_tot(6)
        write(nout,1002)' Standard deviation = ',dscinti(6)
        write(nout,1002)' Maximal weight     = ',wmax(6)
        write(nout,*)' '
        if(src.eq.1)then
          write(nout,'(a)')' 2->5 Bremsstrahlung contribution (QED):'
        elseif(src.eq.0)then 
          write(nout,'(a)')' Tree-level process e+ e- -> 4f + photon:'
        else
          write(nout,'(a)')' Improved Born Approximation ee->4f+photon:'
        endif
        write(nout,1002)' Average            = ',conti_tot(7)
        write(nout,1002)' Standard deviation = ',dscinti(7)
        write(nout,1002)' Maximal weight     = ',wgmax
        write(nout,*)' '
        if(src.eq.1)then
          write(nout,'(a)')' 2->5 Bremsstrahlung contribution (QCD):'
        else 
          write(nout,'(a)')' Tree-level process e+ e- -> 4f + gluon:'
        endif
        write(nout,1002)' Average            = ',conti_tot(8)
        write(nout,1002)' Standard deviation = ',dscinti(8)
        write(nout,1002)' Maximal weight     = ',wqmax
        rej(1)=100d0*dble(ncut1_k(2))/dble(nevents)
        rej(2)=100d0*dble(ncut2_k(2))/dble(nevents)
        rej(3)=100d0*dble(ncut3_k(2))/dble(nevents)
        rej(4)=100d0*dble(ncut4_k(2))/dble(nevents)
        rej(5)=100d0*dble(ncut5_k(2))/dble(nevents)
        rej(6)=100d0*dble(ncut6_k(2))/dble(nevents)
        rej(7)=100d0*dble(ncut_r(2))/dble(nevents)
        rej(8)=100d0*dble(ncutq_r(2))/dble(nevents)
        write(nout,*)' '
        do i1=1,nsubevent
          write(nout,1004)i1,rej(i1)
       enddo
       write(nout,*) ' '
      endif
c additional output for subtraction branch
      if(smc.eq.2)then
        if(qqcd.eq.2)then 
          write(nout,'(a)')' Subcontributions without naive QCD:'
          write(nout,'(a)')' -----------------------------------'
        else
          write(nout,'(a)')' Subcontributions :'
          write(nout,'(a)')' ------------------'
        endif
        write(nout,'(a)')' Tree-level four-fermion cross section:'
        write(nout,1002)' Average            = ',subav(1)
        write(nout,1002)' Standard deviation = ',subsi(1)
        write(nout,1002)' Maximal weight     = ',submax(1)
        write(nout,1003)subn(1),subch(1)
        write(nout,*)' '
        if(sisr.ne.0)then
          write(nout,'(a)')' 2->4 part of radiative corrections'//
     *      ' without O(alpha) LL ISR:'
        else
          write(nout,'(a)')' 2->4 part of radiative corrections:'
        endif
        write(nout,1002)' Average            = ',subav(3)
        write(nout,1002)' Standard deviation = ',subsi(3)
        write(nout,1002)' Maximal weight     = ',submax(3)
        write(nout,1003)subn(3),subch(3)
        write(nout,*)' '
        if(src.ne.2)then
          write(nout,'(a)')
     *      ' Initial-state radiation up to O(alpha^3):'
        else
          write(nout,'(a)')' Improved Born Approximation ee->WW->4f:'
        endif
        write(nout,1002)' Average            = ',subav(2)
        write(nout,1002)' Standard deviation = ',subsi(2)
        write(nout,1002)' Maximal weight     = ',submax(2)
        write(nout,1003)subn(2),subch(2)
        write(nout,*)' '
        if(src.ne.3)then
          write(nout,'(a)')' 2->5 part of radiative corrections:'
        else
          write(nout,'(a)')' Improved Born Approximation ee->4f+photon:'
        endif
        write(nout,1002)' Average            = ',subav(4)
        write(nout,1002)' Standard deviation = ',subsi(4)
        write(nout,1002)' Maximal weight     = ',submax(4)
        write(nout,1003)subn(4),subch(4)
        write(nout,*)' '
        do i1=1,nsubevent
          rej(i1)=100d0*dble(subrej(i1))/dble(nevents)
          write(nout,1004)i1,rej(i1)
        enddo
        write(nout,*)' '
        if(iout.ne.0)then
          call outputphasespace(iout)
          call outputadaptive(iout)
        endif
      endif
c additional output for unweighted event generator
      if((smc.eq.3.or.src.ne.1).and.sunw.eq.1)then
        write(nout,'(a)')' '
        write(nout,'(a)')' Number of events'
        write(nout,'(a)')' ----------------'
        write(nout,'(" Unweighted events            = ",i9)')
     *    nunweighted(1)
        write(nout,'(" Events with weight=-1        = ",i9)')
     *    nunweighted(-1)
        write(nout,'(" Events with weight>weightmax = ",i9)')
     *    novershoot
      endif
c setup of paw
c      if(spaw.eq.1)call setup_paw
      if(iout.ne.0)close(iout)
c close output file
      if(outputfile(1:2).ne.' ')close(nout)	
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     pseudo-random-number generator                              c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine rans(ran)
      implicit none 
c local variables
      real*8 s1,s2,s3,ran
      integer init
      save init,s1,s2,s3
      data init/0/,s1/0d0/,s2/0d0/,s3/0d0/
      if (init.eq.0) then
        init=1
        s1=dsqrt(2d0)-1d0
        s2=dsqrt(3d0)-1d0
        s3=dsqrt(5d0)-2d0
      endif
      s1=dmod(s1+s2+s3,1d0)
      s2=dmod(s1+s2+s3,1d0)
      s3=dmod(s1+s2+s3,1d0)
      ran=s1
      end  
