************************************************************************
        function dM2(F)
************************************************************************
* 	calculate correction to |M|^2 from form factors F
*-----------------------------------------------------------------------
*       25.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        complex*16 F(10,-1:1)
	complex*16 mat(10,-1:1),matcmb(10,-1:1),matb(-1:1),fb(10,-1:1)
        integer    i,j,qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/sme/mat,matcmb,matb,fb,mbcmb

	dM2 = 0d0
        do 100 i=1,10
        do 100 j=-1,1,2
	  if (qreal.eq.1) then
            dM2 = dM2 + 2d0*dreal(F(i,j))*matcmb(i,j)
	  else
            dM2 = dM2 + 2d0*F(i,j)*matcmb(i,j)
	  endif
100     continue

	end

************************************************************************
        subroutine setFnull(F)
************************************************************************
* 	initialize form factors
*-----------------------------------------------------------------------
*       25.5.99 Stefan Dittmaier
************************************************************************
        complex*16 F(10,-1:1)
        integer    i1,i2

        do 100 i1=1,10
        do 100 i2=-1,1,2
          F(i1,i2) = 0d0
100     continue

        end

************************************************************************
        subroutine calcform(F)
************************************************************************
*	calculate formfactors for e+e- --> WW --> 4 f 
*-----------------------------------------------------------------------
*       23.6.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 F(10,-1:1)
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	integer i,j

	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

	call setFnull(F)

	if (qbos.eq.1) then
	  call add_Vself(F)
	  call add_nself(F)
	  call add_VVVvert(F)
	  call add_eeVvert(F)
	  call add_enWvert(F)
	  call add_box(F)
	  call add_Wdec(F)
	elseif (qbos.eq.2) then
	  call add_Vself_bgf(F)
	  call add_nself_bgf(F)
	  call add_VVVvert_bgf(F)
	  call add_eeVvert_bgf(F)
	  call add_enWvert_bgf(F)
	  call add_box_bgf(F)
	  call add_Wdec_bgf(F)
	endif

	if (qferm.eq.1) then
	  call add_Vself_ferm(F)
	  call add_VVVvert_ferm(F)
	  call add_eeVvert_ferm(F)
	  call add_enWvert_ferm(F)
	  call add_Wdec_ferm(F)
	endif

c*** set EW coupling in relative virtual correction to alpha0 !
	do 100 i=1,10
	do 100 j=-1,1,2
	  F(i,j) = F(i,j)*alpha0/alpha
100	continue

	end

************************************************************************
        subroutine calcsme(p,pon)
************************************************************************
*	calculate standard-matrix elements for e+e- --> WW --> 4 f 
*-----------------------------------------------------------------------
*       20.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        complex*16 sp(7,7),csp(7,7)
	complex*16 cmw2,pre
	complex*16 mat(10,-1:1),matcmb(10,-1:1),matb(-1:1),fb(10,-1:1)
        real*8 vp(7,7),p(7,0:3),pon(7,0:3)
	integer i,j,dir(7)
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
        common/offshell/kp2off,km2off,cmw2
	common/sme/mat,matcmb,matb,fb,mbcmb
	common/pol/pm,pp
	common/fstate/qf(4),mf(4),nc4f,dir
        common/prods/sp,csp,vp
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C definitions from off-shell kinematics
	call setprods(p,dir,6)
	s       = vp(1,2)
	kp2off  = vp(3,4)
	km2off  = vp(5,6)
	cmw2    = dcmplx(mw2,-mw*gw)

C on-shell projection
	call setprods(pon,dir,6)
	kp2 = mw2
	km2 = mw2

	t = vp(1,3)+vp(1,4)+vp(3,4)
	u = vp(2,3)+vp(2,4)+vp(3,4)

        MAT(1,1) = -CSP(1,4)*SP(2,5)*(CSP(1,6)*SP(1,3)-CSP(4,6)*
     &  SP(3,4))
        MAT(2,1) = -(CSP(4,6)*SP(3,5)*(CSP(1,3)*SP(2,3)+CSP(1,4)*
     &  SP(2,4)))/2D0
        MAT(3,1) = -(CSP(1,4)*SP(2,3)*(CSP(3,6)*SP(3,5)+CSP(4,6)*
     &  SP(4,5)))/2D0
        MAT(4,1) = CSP(1,6)*SP(2,5)*(CSP(4,5)*SP(3,5)+CSP(4,6)*
     &  SP(3,6))/2D0
        MAT(5,1) = CSP(1,4)*CSP(2,6)*SP(2,3)*SP(2,5)/2D0
        MAT(6,1) = -CSP(1,4)*CSP(1,6)*SP(1,3)*SP(2,5)/2D0
        MAT(7,1) = -(CSP(4,5)*SP(3,5)+CSP(4,6)*SP(3,6))*(CSP(3,6)*
     &  SP(3,5)+CSP(4,6)*SP(4,5))*(CSP(1,3)*SP(2,3)+CSP(1,4)*SP(2,
     &  4))/4D0
        MAT(8,1) = -CSP(1,4)*CSP(2,6)*SP(1,3)*SP(2,5)*(CSP(1,3)*
     &  SP(2,3)+CSP(1,4)*SP(2,4))/4D0
        MAT(9,1) = CSP(2,6)*SP(2,5)*(CSP(4,5)*SP(3,5)+CSP(4,6)*
     &  SP(3,6))*(CSP(1,3)*SP(2,3)+CSP(1,4)*SP(2,4))/4D0
        MAT(10,1) = CSP(1,4)*SP(1,3)*(CSP(3,6)*SP(3,5)+CSP(4,6)*
     &  SP(4,5))*(CSP(1,3)*SP(2,3)+CSP(1,4)*SP(2,4))/4D0

        MAT(1,-1) = -(CSP(2,6)*SP(1,3)*(CSP(1,4)*SP(1,5)+CSP(3,4)*
     &  SP(3,5)))
        MAT(2,-1) = -CSP(4,6)*SP(3,5)*(CSP(2,3)*SP(1,3)+CSP(2,4)*
     &  SP(1,4))/2D0
        MAT(3,-1) = -((CSP(2,4)*SP(1,3)*(CSP(3,6)*SP(3,5)+CSP(4,6)*
     &  SP(4,5)))/2D0)
        MAT(4,-1) = (CSP(2,6)*SP(1,5)*(CSP(4,5)*SP(3,5)+CSP(4,6)*
     &  SP(3,6)))/2D0
        MAT(5,-1) = (CSP(2,4)*CSP(2,6)*SP(1,3)*SP(2,5))/2D0
        MAT(6,-1) = -((CSP(1,4)*CSP(2,6)*SP(1,3)*SP(1,5))/2D0)
        MAT(7,-1) = -(CSP(4,5)*SP(3,5)+CSP(4,6)*SP(3,6))*(CSP(3,
     &  6)*SP(3,5)+CSP(4,6)*SP(4,5))*(CSP(2,3)*SP(1,3)+CSP(2,4)*
     &  SP(1,4))/4D0
        MAT(8,-1) = -CSP(1,4)*CSP(2,6)*SP(1,3)*SP(2,5)*(CSP(2,3)*
     &  SP(1,3)+CSP(2,4)*SP(1,4))/4D0
        MAT(9,-1) = CSP(2,6)*SP(2,5)*(CSP(4,5)*SP(3,5)+CSP(4,6)*
     &  SP(3,6))*(CSP(2,3)*SP(1,3)+CSP(2,4)*SP(1,4))/4D0
        MAT(10,-1) = CSP(1,4)*SP(1,3)*(CSP(3,6)*SP(3,5)+CSP(4,6)*
     &  SP(4,5))*(CSP(2,3)*SP(1,3)+CSP(2,4)*SP(1,4))/4D0

	do 100 i=1,10
	do 100 j=-1,1,2
	  fb(i,j) = 0d0
100	continue

	fb(1,-1) = -1d0/2d0/sw2/t
	fb(2,-1) = (s-2d0*mz2*sw2)/sw2/s/(s-mz2)
	fb(3,-1) = -fb(2,-1)
	fb(4,-1) = -fb(2,-1)
	fb(2,+1) = -2d0*mz2/s/(s-mz2)
	fb(3,+1) = -fb(2,+1)
	fb(4,+1) = -fb(2,+1)

	pre      = 2d0*el**4/sw2/(kp2off-cmw2)/(km2off-cmw2)

	matb(-1) = pre*( fb(1,-1)*mat(1,-1)+fb(2,-1)*mat(2,-1)
     &		        +fb(3,-1)*mat(3,-1)+fb(4,-1)*mat(4,-1) )
	matb(+1) = pre*(                    fb(2,+1)*mat(2,+1)
     &		        +fb(3,+1)*mat(3,+1)+fb(4,+1)*mat(4,+1) )
	
	mbcmb    = nc4f/4d0*( (1d0+pp)*(1d0-pm)*abs(matb(-1))**2
     &		  	     +(1d0-pp)*(1d0+pm)*abs(matb(+1))**2 )

	do 200 i=1,10
	do 200 j=-1,1,2
	  matcmb(i,j) = nc4f*pre/4d0*(1d0-j*pp)*(1d0+j*pm)
     &			*mat(i,j)*dconjg(matb(j))
200	continue

	end

************************************************************************
        function M2_Born_offsh(p,qcc03,qgz)
************************************************************************
*	Off-shell |M_Born|^2 for e+e- --> WW --> 4 f 
*	qcc03 = 0:  lowest-order CC03 cross section
*	qcc03 = 1:  IBA for CC03 cross section 
*	qgz   = 0:  Z-boson width GZ = 0    in Z propagator
*	qgz   = 1:  Z-boson width GZ finite in Z propagator
*-----------------------------------------------------------------------
*       12.6.00 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        complex*16 sp(7,7),csp(7,7)
	complex*16 kp,km,cmw2,cmz2,pre
	complex*16 mat(10,-1:1),matb(-1:1),fb(10,-1:1)
	complex*16 matiba(-1:1),fiba(10,-1:1)
        real*8 vp(7,7),p(7,0:3)
	integer dir(7),qcc03,qgz

	common/pol/pm,pp
        common/prods/sp,csp,vp
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
	common/fstate/qf(4),mf(4),nc4f,dir

C definitions from off-shell kinematics
	call setprods(p,dir,6)
	s       = vp(1,2)
	kp2off  = vp(3,4)
	km2off  = vp(5,6)
	t       = vp(1,3)+vp(1,4)+vp(3,4)
	u       = vp(2,3)+vp(2,4)+vp(3,4)
	cmw2    = dcmplx(mw2,-mw*gw)
	if (qgz.eq.0) then
	  cmz2  = mz2
	else
	  cmz2  = dcmplx(mz2,-mz*gz)
	endif
        kp      = kp2off-cmw2
        km      = km2off-cmw2

c off-shell SMEs
        MAT(1,1) = -CSP(1,4)*SP(2,5)*(CSP(1,6)*SP(1,3)-CSP(4,6)*
     &  SP(3,4))
        MAT(2,1) = -(CSP(4,6)*SP(3,5)*(CSP(1,3)*SP(2,3)+CSP(1,4)*
     &  SP(2,4)))/2D0
        MAT(3,1) = -(CSP(1,4)*SP(2,3)*(CSP(3,6)*SP(3,5)+CSP(4,6)*
     &  SP(4,5)))/2D0
        MAT(4,1) = CSP(1,6)*SP(2,5)*(CSP(4,5)*SP(3,5)+CSP(4,6)*
     &  SP(3,6))/2D0

        MAT(1,-1) = -(CSP(2,6)*SP(1,3)*(CSP(1,4)*SP(1,5)+CSP(3,4)*
     &  SP(3,5)))
        MAT(2,-1) = -CSP(4,6)*SP(3,5)*(CSP(2,3)*SP(1,3)+CSP(2,4)*
     &  SP(1,4))/2D0
        MAT(3,-1) = -((CSP(2,4)*SP(1,3)*(CSP(3,6)*SP(3,5)+CSP(4,6)*
     &  SP(4,5)))/2D0)
        MAT(4,-1) = (CSP(2,6)*SP(1,5)*(CSP(4,5)*SP(3,5)+CSP(4,6)*
     &  SP(3,6)))/2D0

c*** lowest-order CC03 cross section 
	if (qcc03.eq.0) then
	  pre = 2d0*el**4/sw2/kp/km

	  fb(1,-1) = -1d0/2d0/sw2/t
	  fb(2,-1) = (s-2d0*cmz2*sw2)/sw2/s/(s-cmz2)
	  fb(3,-1) = -fb(2,-1)
	  fb(4,-1) = -fb(2,-1)
	  fb(2,+1) = -2d0*cmz2/s/(s-cmz2)
	  fb(3,+1) = -fb(2,+1)
	  fb(4,+1) = -fb(2,+1)

	  matb(-1) = pre*( fb(1,-1)*mat(1,-1)+fb(2,-1)*mat(2,-1)
     &	  	          +fb(3,-1)*mat(3,-1)+fb(4,-1)*mat(4,-1) )
	  matb(+1) = pre*(                    fb(2,+1)*mat(2,+1)
     &		          +fb(3,+1)*mat(3,+1)+fb(4,+1)*mat(4,+1) )
	
	  M2_Born_offsh = nc4f/4d0*( (1d0+pp)*(1d0-pm)*abs(matb(-1))**2
     &		  	            +(1d0-pp)*(1d0+pm)*abs(matb(+1))**2)

c*** IBA for CC03 cross section 
	elseif (qcc03.eq.1) then
	  dalpq   = alphaz/3d0/pi*log(s/mz2)*20d0/3d0
	  alphaq  = alphaz/(1d0-dalpq)      / alpha
	  alphagf = sqrt(2d0)*gf/pi*mw2*sw2 / alpha
	  pre     = alphagf*2d0*el**4/sw2/kp/km

	  fiba(1,-1) = -alphagf/2d0/sw2/t
	  fiba(2,-1) =  alphagf/sw2/(s-cmz2)
     &		       -alphaq*2d0*cmz2/s/(s-cmz2)
  	  fiba(3,-1) = -fiba(2,-1)
	  fiba(4,-1) = -fiba(2,-1)
	  fiba(2,+1) = -alphaq*2d0*cmz2/s/(s-cmz2)
	  fiba(3,+1) = -fiba(2,+1)
	  fiba(4,+1) = -fiba(2,+1)

	  matiba(-1) = pre*( fiba(1,-1)*mat(1,-1)+fiba(2,-1)*mat(2,-1)
     &		            +fiba(3,-1)*mat(3,-1)+fiba(4,-1)*mat(4,-1) )
	  matiba(+1) = pre*(                      fiba(2,+1)*mat(2,+1)
     &		            +fiba(3,+1)*mat(3,+1)+fiba(4,+1)*mat(4,+1) )
	
	  coul   = coulsing(s,kp2off,km2off,1) 
          bb     = sqrt((s-kp2off-km2off)**2-4d0*kp2off*km2off)/s
	  weight = (1d0-bb**2)**2

	  M2_Born_offsh = nc4f/4d0*(1d0+coul*weight)
     &			  *( (1d0+pp)*(1d0-pm)*abs(matiba(-1))**2
     &		 	    +(1d0-pp)*(1d0+pm)*abs(matiba(+1))**2 )

	endif

	end

************************************************************************
        function coulsing(s,kp2off,km2off,qcoul)
************************************************************************
*	Correction factor for Coulomb singularity
*	qcoul = 0:   on-shell singularity
*	qcoul = 1:  off-shell singularity according to
*			D.Bardin, W.Beenakker, A.Denner, 
*			Phys.Lett.B317(1993)213.
*-----------------------------------------------------------------------
*       12.7.00 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 b
	integer qcoul

	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

        d  = abs(kp2off-km2off)/s

	if (qcoul.eq.0) then
	  bw = sqrt(1d0-4d0*mw2/s)
          coulsing = alpha0/bw*dimag( 
     &		       log((kp2off+km2off-2d0*dcmplx(mw2,-mw*gw)+s*bw*d)
     &           	   /2d0/bw**2/s) )
	else
          b  = sqrt(1d0-4d0*dcmplx(mw2,-mw*gw)/s)
          bb = sqrt((s-kp2off-km2off)**2-4d0*kp2off*km2off)/s
          coulsing = alpha0/bb*dimag( log((b-bb+d)/(b+bb+d)) )
	endif

	end

************************************************************************
        subroutine calcrenc
************************************************************************
*	calculate renormalization constants
*	conventional formalism
*-----------------------------------------------------------------------
*       26.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0_,B00H,B00W,B00Z,BW0W,BWWZ,BWHW,BZWW,BZHZ
	complex*16 DB0_,DBW0W,DBWWZ,DBWHW,DBZWW,DBZHZ,DB0WW
	complex*16 DB00W,DB0HW,DB0WZ
	complex*16 seww0
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qalp,qgw

	common/renc/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	            dZeer,dZeel,dZnnl
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
        COMMON/UV/MUDIM2
        COMMON/IR/LAMBDA,LAMBDA2
	common/rscheme/dalpz,drbos,drferm,qalp,qgw

	B00H  = B0_(0d0,0d0,mh,0)
	B00W  = B0_(0d0,0d0,mw,0)
	B00Z  = B0_(0d0,0d0,mz,0)
	BW0W  = B0_(mw2,0d0,mw,0)
	BWWZ  = B0_(mw2, mw,mz,0)
	BWHW  = B0_(mw2, mh,mw,0)
	BZWW  = B0_(mz2, mw,mw,0)
	BZHZ  = B0_(mz2, mh,mz,0)
	DBW0W = DB0_(mw2,0d0,mw,0)
	DBWHW = DB0_(mw2, mh,mw,0)
	DBWWZ = DB0_(mw2, mw,mz,0)
	DBZHZ = DB0_(mz2, mh,mz,0)
	DBZWW = DB0_(mz2, mw,mw,0)
	DB0WW = DB0_(0d0, mw,mw,0)
	DB00W = DB0_(0d0,0d0,mw,0)
	DB0HW = DB0_(0d0, mh,mw,0)
	DB0WZ = DB0_(0d0, mw,mz,0)

	dmw2 = ALPHA/PI*(
     &	   -BW0W*MW2
     &	   +B00Z*(-1D0+3D0*CW2)*(1D0+8D0*CW2)*MZ2/(48D0*CW2*SW2)
     &	   +BWWZ*(1D0-4D0*CW2)*(1D0+20D0*CW2+12D0*CW2**2)*MZ2
     &	    /(48D0*CW2*SW2)
     &	   +B00H*MH2*(-MH2+3D0*MW2)/(48D0*CW2*MZ2*SW2)
     &	   +BWHW*(MH2**2-4D0*MH2*MW2+12D0*MW2**2)/(48D0*CW2*MZ2*SW2)
     &	   +B00W*(MH2+18D0*MW2+MZ2)/(48D0*SW2)
     &	   -(3D0*MH2+56D0*MW2+3D0*MZ2)/(72D0*SW2)	)

	dmz2 = ALPHA/PI*(
     &	    BZWW*(1D0-4D0*CW2)*(1D0+20D0*CW2+12D0*CW2**2)*MZ2
     &      /(48D0*CW2*SW2)
     &	   +B00W*(1D0-4D0*CW2+12D0*CW2**2)*MZ2/(12D0*SW2)
     &	   +B00Z*(MH2+MZ2)/(48D0*CW2*SW2)
     & 	   +B00H*MH2*(-MH2+3D0*MZ2)/(48D0*CW2*MZ2*SW2)
     &	   +BZHZ*(MH2**2-4D0*MH2*MZ2+12D0*MZ2**2)/(48D0*CW2*MZ2*SW2)
     &	   +(-3D0*MH2-10D0*MW2-MZ2+24D0*CW2**2*MZ2-72D0*CW2**3*MZ2)
     &	    /(72D0*CW2*SW2)	)

	dZw = - ALPHA/PI*(
     &	   -BW0W+B00Z/CW2**2*(1D0+8D0*CW2)/48D0
     &	   -1D0/(36D0*SW2)
     &	   +BWWZ/CW2**2*(-1D0-6D0*CW2+16D0*CW2**2-48D0*CW2**3)
     &	    /(48D0*SW2)
     &	   +B00H/CW2**2*MH2*(MH2-MW2)/MZ2**2/(48D0*SW2)
     &	   +BWHW/CW2**2*MH2*(-MH2+2D0*MW2)/MZ2**2/(48D0*SW2)
     &	   +B00W*(-MH2+2D0*MW2-MZ2)/(48D0*CW2*MZ2*SW2)
     &	   -DBW0W*MW2
     &	   +DBWHW*(MH2**2-4D0*MH2*MW2+12D0*MW2**2)
     &	    /(48D0*CW2*MZ2*SW2)
     &	   +DBWWZ*(1D0-4D0*CW2)*(1D0+20D0*CW2+12D0*CW2**2)*MZ2
     &	    /(48D0*CW2*SW2)		)
	dZzz = -ALPHA/PI*(
     &	    (1D0-2D0*CW2)/(36D0*CW2*SW2)
     &	   +BZWW*(1D0-4D0*CW2-36D0*CW2**2)/(48D0*CW2*SW2)
     &	   +B00Z*(-MH2+MZ2)/(48D0*CW2*MZ2*SW2)
     &	   +B00H*MH2/MZ2**2*(MH2-MZ2)/(48D0*CW2*SW2)
     &	   +BZHZ*MH2/MZ2**2*(-MH2+2D0*MZ2)/(48D0*CW2*SW2)
     &	   +DBZHZ*(MH2**2-4D0*MH2*MZ2+12D0*MZ2**2)/(48D0*CW2*MZ2*SW2)
     &	   +DBZWW*(1D0-4D0*CW2)*(1D0+20D0*CW2+12D0*CW2**2)*MZ2
     &	    /(48D0*CW2*SW2)		)
	dZaa = -ALPHA/(4D0*PI)*(3D0-3D0*B00W-4D0*MW2*DB0WW)
	dZza = ALPHA/(PI*SW)*CW*(-1D0+B00W)
	dZaz = ALPHA/(CW*PI*SW)*(
     &	   B00W*CW2*(-1D0+6D0*CW2)/3D0
     &	  -BZWW*(1D0+26D0*CW2+24D0*CW2**2)/12D0
     &	  +(-1D0+6D0*CW2-36D0*CW2**2)/18D0	)

	seww0 =  ALPHA/(48D0*PI)*(
     &	-2D0/CW2/SW2*(MH2+19D0*MW2-MH2*SW2-18D0*MW2*SW2)
     &  +8D0*MH2*MW2/SW2*B00H/(MH2-MW2)
     &	+4D0*MW2/SW**4*(15D0*MH2-15D0*MW2-16D0*MH2*SW2+14D0*MW2*SW2)
     &   *B00W/(MH2-MW2)
     &	-4D0/CW2*MW2/SW**4*(5D0-2D0*SW2)*(3D0-4D0*SW2)*B00Z
     & 	+8D0*MW**4*DB00W+(MH2-MW2)**2/SW2*DB0HW
     &	+MW**4/CW**4*SW2*(9D0-8D0*SW2)*DB0WZ	)

	dsw = cw2/2d0/sw*(dmz2/mz2-dmw2/mw2)

	drbos = -dZaa-2d0*dsw/sw+(seww0-dmw2)/mw2+cw/sw*dZza
     &		+alpha/4d0/pi/sw2*(6d0+(7d0-4d0*sw2)/2d0/sw2*log(cw2))

	if ((qalp.eq.0).or.(qalp.eq.1)) then
	  dZe = -dZaa/2d0-dZza*sw/2d0/cw
	elseif (qalp.eq.2) then
	  dZe = -dZaa/2d0-dZza*sw/2d0/cw - drbos/2d0
	endif

	dZeer = ALPHA/4d0/PI*( 
     &	  -4d0-2d0*LOG(lambda2/ml2(1))+LOG(ml2(1)/MUDIM2) 
     &	  +SW2/CW2*( log(mz2/mudim2)+1d0/2d0 )			)
	dZeel = ALPHA/4d0/PI*(
     &	  -4d0-2d0*LOG(lambda2/ml2(1))+LOG(ml2(1)/MUDIM2) 
     &	  +(SW/CW-1d0/2d0/SW/CW)**2*( log(mz2/mudim2)+1d0/2d0 )
     &	  +1d0/2d0/SW2*( log(mw2/mudim2)+1d0/2d0 )		)
	dZnnl = ALPHA/4d0/PI*(
     &	  +1d0/4d0/SW2/CW2*( log(mz2/mudim2)+1d0/2d0 )
     &	  +1d0/2d0/SW2*( log(mw2/mudim2)+1d0/2d0 )		)

	dmw2  = dreal( dmw2  )
	dmz2  = dreal( dmz2  )
	dsw   = dreal( dsw   )
	dZe   = dreal( dZe   )
	dZaa  = dreal( dZaa  )
	dZaz  = dreal( dZaz  )
	dZza  = dreal( dZza  )
	dZzz  = dreal( dZzz  )
	dZw   = dreal( dZw   )
	dZeer = dreal( dZeer )
	dZeel = dreal( dZeel )
	dZnnl = dreal( dZnnl )

	end

************************************************************************
        subroutine calcrenc_bgf
************************************************************************
*	calculate renormalization constants
*	background-field formalism
*-----------------------------------------------------------------------
*       26.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0_,B00H,B00W,B00Z,BW0W,BWWZ,BWHW,BZWW,BZHZ
	complex*16 DB0_,DBW0W,DBWWZ,DBWHW,DBZWW,DBZHZ,DB0WW
	complex*16 DB00W,DB0HW,DB0WZ
	complex*16 seww0conv
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer	qalp,qgw

	common/rscheme/dalpz,drbos,drferm,qalp,qgw
	common/renc_bgf/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                dZeer,dZeel,dZnnl
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
        COMMON/UV/MUDIM2
        COMMON/IR/LAMBDA,LAMBDA2

	B00H  = B0_(0d0,0d0,mh,0)
	B00W  = B0_(0d0,0d0,mw,0)
	B00Z  = B0_(0d0,0d0,mz,0)
	BW0W  = B0_(mw2,0d0,mw,0)
	BWWZ  = B0_(mw2, mw,mz,0)
	BWHW  = B0_(mw2, mh,mw,0)
	BZWW  = B0_(mz2, mw,mw,0)
	BZHZ  = B0_(mz2, mh,mz,0)
	DBW0W = DB0_(mw2,0d0,mw,0)
	DBWHW = DB0_(mw2, mh,mw,0)
	DBWWZ = DB0_(mw2, mw,mz,0)
	DBZHZ = DB0_(mz2, mh,mz,0)
	DBZWW = DB0_(mz2, mw,mw,0)
	DB0WW = DB0_(0d0, mw,mw,0)
	DB00W = DB0_(0d0,0d0,mw,0)
	DB0HW = DB0_(0d0, mh,mw,0)
	DB0WZ = DB0_(0d0, mw,mz,0)

	dmw2 = ALPHA/PI*(
     &	   -BW0W*MW2
     &	   +B00Z*(-1D0+3D0*CW2)*(1D0+8D0*CW2)*MZ2/(48D0*CW2*SW2)
     &	   +BWWZ*(1D0-4D0*CW2)*(1D0+20D0*CW2+12D0*CW2**2)*MZ2
     &	    /(48D0*CW2*SW2)
     &	   +B00H*MH2*(-MH2+3D0*MW2)/(48D0*CW2*MZ2*SW2)
     &	   +BWHW*(MH2**2-4D0*MH2*MW2+12D0*MW2**2)/(48D0*CW2*MZ2*SW2)
     &	   +B00W*(MH2+18D0*MW2+MZ2)/(48D0*SW2)
     &	   -(3D0*MH2+56D0*MW2+3D0*MZ2)/(72D0*SW2)	)

	dmz2 = ALPHA/PI*(
     &	    BZWW*(1D0-4D0*CW2)*(1D0+20D0*CW2+12D0*CW2**2)*MZ2
     &      /(48D0*CW2*SW2)
     &	   +B00W*(1D0-4D0*CW2+12D0*CW2**2)*MZ2/(12D0*SW2)
     &	   +B00Z*(MH2+MZ2)/(48D0*CW2*SW2)
     & 	   +B00H*MH2*(-MH2+3D0*MZ2)/(48D0*CW2*MZ2*SW2)
     &	   +BZHZ*(MH2**2-4D0*MH2*MZ2+12D0*MZ2**2)/(48D0*CW2*MZ2*SW2)
     &	   +(-3D0*MH2-10D0*MW2-MZ2+24D0*CW2**2*MZ2-72D0*CW2**3*MZ2)
     &	    /(72D0*CW2*SW2)	)

	dZw = - ALPHA/PI*(
     &	   -2D0*BW0W
     &	   +B00Z/CW2**2*(1D0+8D0*CW2)/48D0
     &	   -1D0/(36D0*SW2)
     &	   +BWWZ/CW2**2*(-1D0-6D0*CW2+16D0*CW2**2-96D0*CW2**3)
     &	    /(48D0*SW2)
     &	   +B00H/CW2**2*MH2*(MH2-MW2)/MZ2**2/(48D0*SW2)
     &	   +BWHW/CW2**2*MH2*(-MH2+2D0*MW2)/MZ2**2/(48D0*SW2)
     &	   +B00W*(-MH2+2D0*MW2-MZ2)/(48D0*CW2*MZ2*SW2)
     &	   -MW2*DBW0W
     &	   +DBWHW*(MH2**2-4D0*MH2*MW2+12D0*MW2**2)
     &      /(48D0*CW2*MZ2*SW2)
     &	   +DBWWZ*(1D0-4D0*CW2)*(1D0+20D0*CW2+12D0*CW2**2)*MZ2
     &	    /(48D0*CW2*SW2)		)
	dZzz = -ALPHA/PI*(
     &	    (1D0-2D0*CW2)/(36D0*CW2*SW2)
     &	   +BZWW*(1D0-4D0*CW2-84D0*CW2**2)/(48D0*CW2*SW2)
     &	   +B00Z*(-MH2+MZ2)/(48D0*CW2*MZ2*SW2)
     &	   +B00H*MH2/MZ2**2*(MH2-MZ2)/(48D0*CW2*SW2)
     & 	   +BZHZ*MH2/MZ2**2*(-MH2+2D0*MZ2)/(48D0*CW2*SW2)
     &	   +DBZHZ*(MH2**2-4D0*MH2*MZ2+12D0*MZ2**2)/(48D0*CW2*MZ2*SW2)
     &	   +DBZWW*(1D0-4D0*CW2)*(1D0+20D0*CW2+12D0*CW2**2)*MZ2
     &	    /(48D0*CW2*SW2)		)
	dZaa = -ALPHA/(4D0*PI)*(7D0-7D0*B00W-4D0*MW2*DB0WW)
	dZza = 0d0
	dZaz = ALPHA/(CW*PI*SW)*(
     &	   B00W*CW2*(-1D0+6D0*CW2)/3D0
     &	  -BZWW*(1D0+38D0*CW2+24D0*CW2**2)/12D0
     &	  +(-1D0+6D0*CW2-36D0*CW2**2)/18D0	)

	seww0conv =  ALPHA/(48D0*PI)*(
     &	-2D0/CW2/SW2*(MH2+19D0*MW2-MH2*SW2-18D0*MW2*SW2)
     &  +8D0*MH2*MW2/SW2*B00H/(MH2-MW2)
     &	+4D0*MW2/SW**4*(15D0*MH2-15D0*MW2-16D0*MH2*SW2+14D0*MW2*SW2)
     &   *B00W/(MH2-MW2)
     &	-4D0/CW2*MW2/SW**4*(5D0-2D0*SW2)*(3D0-4D0*SW2)*B00Z
     & 	+8D0*MW**4*DB00W+(MH2-MW2)**2/SW2*DB0HW
     &	+MW**4/CW**4*SW2*(9D0-8D0*SW2)*DB0WZ	)

	dsw = cw2/2d0/sw*(dmz2/mz2-dmw2/mw2)

	drbos = -dZaa-2d0*dsw/sw+(seww0conv-dmw2)/mw2
     &		+alpha/pi/sw2*(-1d0+B00w)
     &		+alpha/4d0/pi/sw2*(6d0+(7d0-4d0*sw2)/2d0/sw2*log(cw2))

	if ((qalp.eq.0).or.(qalp.eq.1)) then
	  dZe = -dZaa/2d0
	elseif (qalp.eq.2) then
	  dZe = -dZaa/2d0 - drbos/2d0
	endif

	dZeer = ALPHA/4d0/PI*( 
     &	  -4d0-2d0*LOG(lambda2/ml2(1))+LOG(ml2(1)/MUDIM2) 
     &	  +SW2/CW2*( log(mz2/mudim2)+1d0/2d0 )			)
	dZeel = ALPHA/4d0/PI*(
     &	  -4d0-2d0*LOG(lambda2/ml2(1))+LOG(ml2(1)/MUDIM2) 
     &	  +(SW/CW-1d0/2d0/SW/CW)**2*( log(mz2/mudim2)+1d0/2d0 )
     &	  +1d0/2d0/SW2*( log(mw2/mudim2)+1d0/2d0 )		)
	dZnnl = ALPHA/4d0/PI*(
     &	  +1d0/4d0/SW2/CW2*( log(mz2/mudim2)+1d0/2d0 )
     &	  +1d0/2d0/SW2*( log(mw2/mudim2)+1d0/2d0 )		)

	dmw2  = dreal( dmw2  )
	dmz2  = dreal( dmz2  )
	dsw   = dreal( dsw   )
	dZe   = dreal( dZe   )
	dZaa  = dreal( dZaa  )
	dZaz  = dreal( dZaz  )
	dZza  = dreal( dZza  )
	dZzz  = dreal( dZzz  )
	dZw   = dreal( dZw   )
	dZeer = dreal( dZeer )
	dZeel = dreal( dZeel )
	dZnnl = dreal( dZnnl )

	end

************************************************************************
        subroutine calcrenc_ferm
************************************************************************
*	calculate renormalization constants
*	fermion-loop contributions only
*-----------------------------------------------------------------------
*       2.6.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0_,B00l,B00u,B00d,B0du,BWnl,BWud,BZ00,BZll,BZdd,BZuu  
	complex*16 DB0_,DBWnl,DBWud,DBZ00,DBZll,DBZdd,DBZuu
	complex*16 DB0ll,DB0dd,DB0uu,DB00l,DB0du,seww0 
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer i,qalp,qgw

	common/renc_ferm/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                 dZeer,dZeel,dZnnl
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
        COMMON/UV/MUDIM2
        COMMON/IR/LAMBDA,LAMBDA2
	common/rscheme/dalpz,drbos,drferm,qalp,qgw

	dmw2  = 0d0
	dmz2  = 0d0
	dZw   = 0d0
	dZzz  = 0d0
	dZaa  = 0d0
	dZza  = 0d0
	dZaz  = 0d0
	dalpz = 0d0
	seww0 = 0d0

	do 10 i=1,3

	B00l  =  B0_(0D0,   0D0, ML(I),0)
	B00u  =  B0_(0D0,   0D0,MQP(I),0)
	B00d  =  B0_(0D0,   0D0,MQM(I),0)
	B0du  =  B0_(0D0,MQM(I),MQP(I),0)
	BWnl  =  B0_(MW2,   0D0, ML(I),0)
	BWud  =  B0_(MW2,MQP(I),MQM(I),0)
	DBWnl = DB0_(MW2,   0D0, ML(I),0)
	DBWud = DB0_(MW2,MQP(I),MQM(I),0)
	BZ00  =  B0_(MZ2,0D0,0D0,0)
	BZll  =  B0_(MZ2,ML(I),ML(I),0)
	BZdd  =  B0_(MZ2,MQM(I),MQM(I),0)
	BZuu  =  B0_(MZ2,MQP(I),MQP(I),0)
	DBZ00 = DB0_(MZ2,0D0,0D0,0)
	DBZll = DB0_(MZ2,ML(I),ML(I),0)
	DBZdd = DB0_(MZ2,MQM(I),MQM(I),0)
	DBZuu = DB0_(MZ2,MQP(I),MQP(I),0)
	DB0ll = DB0_(0d0,ML(I),ML(I),0)
	DB0dd = DB0_(0d0,MQM(I),MQM(I),0)
	DB0uu = DB0_(0d0,MQP(I),MQP(I),0)
	DB00l = DB0_(0d0,0d0,ML(I),0)
	DB0du = DB0_(0d0,MQM(I),MQP(I),0)

	dmw2  = dmw2 - ALPHA/(72D0*PI*SW2)*(
     &	8D0*MW2-6D0*ML2(I)-18D0*MQM2(I)-18D0*MQP2(I)
     &  -3D0*B00l*ML2(I)*(-2D0*MW2+ML2(I))/MW2
     &	+3D0*BWnl*(-MW2+ML2(I))*(2D0*MW2+ML2(I))/MW2
     &  -9D0*B00d*MQM2(I)*(-2D0*MW2+MQM2(I)-MQP2(I))/MW2
     &	+9D0*B00u*MQP2(I)*(2D0*MW2+MQM2(I)-MQP2(I))/MW2
     &	+9D0*BWud*(-2D0*MW2**2+MW2*MQM2(I)+MQM2(I)**2+MW2*MQP2(I)
     &		   -2D0*MQM2(I)*MQP2(I)+MQP2(I)**2)/MW2		  )

	dmz2  = dmz2 + ALPHA/(216D0*CW2*PI)*(
     &	48D0*MZ2-64D0*MZ2*SW2-24D0*MZ2/SW2+18D0*ML2(I)/SW2
     &	-72D0*ML2(I)+144D0*SW2*ML2(I)
     &	-72D0*MQM2(I)+48D0*SW2*MQM2(I)+54D0*MQM2(I)/SW2
     &	-144D0*MQP2(I)+192D0*SW2*MQP2(I)+54D0*MQP2(I)/SW2
     &	+9D0*MZ2*BZ00/SW2
     &	-18D0*(1D0-4D0*SW2+8D0*SW2**2)*B00l*ML2(I)/SW2
     &	-6D0*(9D0-12D0*SW2+8D0*SW2**2)*B00d*MQM2(I)/SW2
     &	-6D0*(9D0-24D0*SW2+32D0*SW2**2)*B00u*MQP2(I)/SW2
     &	-9D0*BZll*(-MZ2+4D0*MZ2*SW2-8D0*MZ2*SW2**2+ML2(I)
     &		   +8D0*SW2*ML2(I)-16D0*SW2**2*ML2(I))/SW2
     &	+3D0*BZdd*(9D0*MZ2-12D0*MZ2*SW2+8D0*MZ2*SW2**2-9D0*MQM2(I)
     &		   -24D0*SW2*MQM2(I)+16D0*SW2**2*MQM2(I))/SW2
     &	+3D0*BZuu*(9D0*MZ2-24D0*MZ2*SW2+32D0*MZ2*SW2**2-9D0*MQP2(I)
     &	    	   -48D0*SW2*MQP2(I)+64D0*SW2**2*MQP2(I))/SW2	    )

	dZw   = dZw  + ALPHA/(72D0*PI*SW2)*(
     &	8D0 + 3D0*MW2**(-2)*B00l*ML2(I)**2
     & 	+3D0*DBWnl*(-MW2+ML2(I))*(2D0*MW2+ML2(I))/MW2
     &  -3D0*MW2**(-2)*BWnl*(2D0*MW2**2+ML2(I)**2)
     & 	+9D0*MW2**(-2)*B00d*MQM2(I)*(MQM2(I)-MQP2(I))
     & 	-9D0*MW2**(-2)*B00u*MQP2(I)*(MQM2(I)-MQP2(I))
     &	-9D0*MW2**(-2)*BWud*(2D0*MW2**2+MQM2(I)**2-2D0*MQM2(I)*MQP2(I)
     &			     +MQP2(I)**2)
     &	+9D0*DBWud*(-2D0*MW2**2+MW2*MQM2(I)+MQM2(I)**2+MW2*MQP2(I)
     &  	    -2D0*MQM2(I)*MQP2(I)+MQP2(I)**2)/MW2	    )

	dZzz  = dZzz - ALPHA/(216D0*CW2*PI)*(
     &	48D0-64D0*SW2-24D0/SW2
     &	+9D0*BZ00/SW2
     &  +9D0*(1D0-4D0*SW2+8D0*SW2**2)*BZll/SW2
     &  +3D0*(9D0-12D0*SW2+8D0*SW2**2)*BZdd/SW2
     &  +3D0*(9D0-24D0*SW2+32D0*SW2**2)*BZuu/SW2
     &	+9D0*MZ2*DBZ00/SW2
     &	-9D0*DBZll*(-MZ2+4D0*MZ2*SW2-8D0*MZ2*SW2**2+ML2(I)
     &  	    +8D0*SW2*ML2(I)-16D0*SW2**2*ML2(I))/SW2
     &	+3D0*DBZdd*(9D0*MZ2-12D0*MZ2*SW2+8D0*MZ2*SW2**2-9D0*MQM2(I)
     &  	    -24D0*SW2*MQM2(I)+16D0*SW2**2*MQM2(I))/SW2
     &	+3D0*DBZuu*(9D0*MZ2-24D0*MZ2*SW2+32D0*MZ2*SW2**2-9D0*MQP2(I)
     &		    -48D0*SW2*MQP2(I)+64D0*SW2**2*MQP2(I))/SW2	     )

	dZaa  = dZaa - ALPHA/(27D0*PI)*(
     &	-32D0+9D0*B00l+3D0*B00d+12D0*B00u
     &	+18D0*DB0ll*ML2(I)+6D0*DB0dd*MQM2(I)+24D0*DB0uu*MQP2(I)	)

	dZaz  = dZaz + ALPHA/(54D0*CW*MZ2*PI)*(
     &	32D0*MZ2*SW-12D0*MZ2/SW-72D0*SW*ML2(I)+18D0*ML2(I)/SW
     &	-24D0*SW*MQM2(I)+18D0*MQM2(I)/SW
     &	-96D0*SW*MQP2(I)+36D0*MQP2(I)/SW
     &	-18D0*(1D0-4D0*SW2)*B00l*ML2(I)/SW
     &	+9D0*(1D0-4D0*SW2)*BZll*(MZ2+2D0*ML2(I))/SW
     &	-6D0*(3D0-4D0*SW2)*B00d*MQM2(I)/SW
     &	+3D0*(3D0-4D0*SW2)*BZdd*(MZ2+2D0*MQM2(I))/SW
     &	-12D0*(3D0-8D0*SW2)*B00u*MQP2(I)/SW
     &	+6D0*(3D0-8D0*SW2)*BZuu*(MZ2+2D0*MQP2(I))/SW		)

	dalpz = dalpz + alpha/3d0/pi*dreal(
     & 	     ( 1d0/3d0+(1d0+2d0* ml2(i)/mz2)*(B00l-1d0-BZll) )
     & 	    +( 1d0/3d0+(1d0+2d0*mqm2(i)/mz2)*(B00d-1d0-BZdd) )/3d0 )
	if (i.ne.3)
     &  dalpz = dalpz + alpha/3d0/pi*dreal(
     &       ( 1d0/3d0+(1d0+2d0*mqp2(i)/mz2)*(B00u-1d0-BZuu) )*4d0/3d0 )

	seww0 = seww0 - ALPHA/(24D0*PI*SW2)*(
     & 	-6D0*mqm2(i)-2D0*ml2(i)-6D0*mqp2(i)
     &	+6D0*mqm2(i)*B00d+3D0*ml2(i)*B00l+6D0*mqp2(i)*B00u
     &	+3D0*(mqm2(i)+mqp2(i))*B0du
     &	+ml(i)**4*DB00l+3D0*(mqm2(i)-mqp2(i))**2*DB0du	)

10	continue

	dsw  = cw2/2d0/sw*(dmz2/mz2-dmw2/mw2)

	drferm = -dZaa-2d0*dsw/sw+(seww0-dmw2)/mw2

	if (qalp.eq.0) then
	  dZe = -dZaa/2d0
	elseif (qalp.eq.1) then
	  dZe = -dZaa/2d0 - dalpz/2d0
	elseif (qalp.eq.2) then
	  dZe = -dZaa/2d0 - drferm/2d0
	endif

	dZeer = 0d0
	dZeel = 0d0
	dZnnl = 0d0

	dmw2  = dreal( dmw2  )
	dmz2  = dreal( dmz2  )
	dsw   = dreal( dsw   )
	dZe   = dreal( dZe   )
	dZaa  = dreal( dZaa  )
	dZaz  = dreal( dZaz  )
	dZza  = dreal( dZza  )
	dZzz  = dreal( dZzz  )
	dZw   = dreal( dZw   )

	end

************************************************************************
        subroutine add_Vself(F)
************************************************************************
*	contributions of AA, AZ, and ZZ self-energies
*	conventional formalism
*-----------------------------------------------------------------------
*       26.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 A0_,A0
	complex*16 B0,B1(1),B2(0:2,0:2) 
        COMPLEX*16 K(7)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	            dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N8)

        A0 = A0_(MH)

        DF(2,-1) = (1D0-2D0*SW2)/SW2
        DF(2,1) = -2D0
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = -((ALPHA*A0*(MZ2-S)**(-2))/(16D0*CW2*PI*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N1)+G(T1,N10)+G(T1,N2)+G(T1,N3)+G(T1,N4)+G(T1,N5)+G(T1,
C       N6)+G(T1,N7)

        A0 = A0_(MW)

        K(1) = CW2*S-MZ2*SW2+S*SW2
        K(2) = -(14D0*CW2**2*MZ2*S)+CW2**2*S**2+16D0*CW2*MZ2**2*SW2-
     &  16D0*CW2*MZ2*S*SW2+2D0*CW2*S**2*SW2-2D0*MZ2*S*SW2**2+S**2*
     &  SW2**2
        K(3) = 13D0*CW2**2*S**2-14D0*CW2*MZ2*S*SW2-28D0*CW2**2*MZ2*
     &  S*SW2+12D0*CW2*S**2*SW2+2D0*CW2**2*S**2*SW2+32D0*CW2*MZ2**2*
     &  SW2**2+2D0*MZ2*S*SW2**2-32D0*CW2*MZ2*S*SW2**2-S**2*SW2**2+
     &  4D0*CW2*S**2*SW2**2-4D0*MZ2*S*SW2**3+2D0*S**2*SW2**3

        DF(2,-1) = (K(1)*8D0*MW2*(S-2D0*MZ2*SW2)+(A0*(-(13D0*CW2**2*
     &  S**2)+14D0*CW2*MZ2*S*SW2+28D0*CW2**2*MZ2*S*SW2-12D0*CW2*S**
     &  2*SW2-2D0*CW2**2*S**2*SW2-32D0*CW2*MZ2**2*SW2**2-2D0*MZ2*S*
     &  SW2**2+32D0*CW2*MZ2*S*SW2**2+S**2*SW2**2-4D0*CW2*S**2*SW2**
     &  2+4D0*MZ2*S*SW2**3-2D0*S**2*SW2**3))/CW2)/SW2
        DF(2,1) = 2D0*(-(K(1)*8D0*MW2*MZ2)+(A0*(14D0*CW2**2*MZ2*S-
     &  CW2**2*S**2-16D0*CW2*MZ2**2*SW2+16D0*CW2*MZ2*S*SW2-2D0*CW2*
     &  S**2*SW2+2D0*MZ2*S*SW2**2-S**2*SW2**2))/CW2)
        DF(3,-1) = ((K(3)*A0)/CW2-K(1)*8D0*MW2*(S-2D0*MZ2*SW2))/SW2
        DF(3,1) = 2D0*((K(2)*A0)/CW2+K(1)*8D0*MW2*MZ2)
        DF(4,-1) = DF(3,-1)
        DF(4,1) = DF(3,1)

        PRE = (ALPHA*S**(-2)*(MZ2-S)**(-2))/(8D0*PI*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N9)

        A0 = A0_(MZ)

        DF(2,-1) = (1D0-2D0*SW2)/SW2
        DF(2,1) = -2D0
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = -((ALPHA*A0*(MZ2-S)**(-2))/(16D0*CW2*PI*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T2,N32)+G(T2,N33)

        CALL B012_(S,MH,MZ,B0,B1,B2,2,0)

        DF(2,-1) = (1D0-2D0*SW2)/SW2
        DF(2,1) = -2D0
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = -((ALPHA*(-(B2(0,0))+(B0*MW2)/CW2)*(MZ2-S)**(-2))/
     &  (4D0*CW2*PI*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T2,N11)+G(T2,N12)+G(T2,N13)+G(T2,N14)+G(T2,N15)+G(T2,N16)+
C       G(T2,N17)+G(T2,N18)+G(T2,N19)+G(T2,N20)+G(T2,N21)+G(T2,N22)+
C       G(T2,N23)+G(T2,N24)+G(T2,N25)+G(T2,N26)+G(T2,N27)+G(T2,N28)+
C       G(T2,N29)+G(T2,N30)+G(T2,N31)+G(T2,N34)+G(T2,N35)+G(T2,N36)

        CALL B012_(S,MW,MW,B0,B1,B2,2,0)

        K(1) = 6D0*MW2-S
        K(2) = S-2D0*MZ2*SW2
        K(3) = CW2*S-MZ2*SW2+S*SW2
        K(4) = 5D0*CW2**2*S**2+2D0*CW2*MW2*MZ2*SW2-2D0*CW2*MW2*S*
     &  SW2-5D0*CW2*MZ2*S*SW2+5D0*CW2*S**2*SW2-2D0*MW2*S*SW2**2
        K(5) = 18D0*CW2**2*MZ2*S-CW2**2*S**2-20D0*CW2*MZ2**2*SW2+
     &  20D0*CW2*MZ2*S*SW2-2D0*CW2*S**2*SW2+2D0*MZ2*S*SW2**2-S**2*
     &  SW2**2
        K(6) = -(5D0*CW2**2*S**3)+5D0*CW2*MZ2*S**2*SW2+10D0*CW2**2*
     &  MZ2*S**2*SW2-5D0*CW2*S**3*SW2+4D0*CW2*MW2*MZ2**2*SW2**2+2D0*
     &  MW2*MZ2*S*SW2**2-4D0*CW2*MW2*MZ2*S*SW2**2-10D0*CW2*MZ2**2*S*
     &  SW2**2+10D0*CW2*MZ2*S**2*SW2**2-4D0*MW2*MZ2*S*SW2**3
        K(7) = -(17D0*CW2**2*S**2)+18D0*CW2*MZ2*S*SW2+36D0*CW2**2*
     &  MZ2*S*SW2-16D0*CW2*S**2*SW2-2D0*CW2**2*S**2*SW2-40D0*CW2*
     &  MZ2**2*SW2**2-2D0*MZ2*S*SW2**2+40D0*CW2*MZ2*S*SW2**2+S**2*
     &  SW2**2-4D0*CW2*S**2*SW2**2+4D0*MZ2*S*SW2**3-2D0*S**2*SW2**3

        DF(2,-1) = (-(B1(1)*K(2)*K(3)*2D0)-B2(1,1)*K(2)*K(3)*2D0+
     &  (K(1)*K(2)*K(3))/S-(B0*(5D0*CW2**2*S**3-5D0*CW2*MZ2*S**2*
     &  SW2-10D0*CW2**2*MZ2*S**2*SW2+5D0*CW2*S**3*SW2-4D0*CW2*MW2*
     &  MZ2**2*SW2**2-2D0*MW2*MZ2*S*SW2**2+4D0*CW2*MW2*MZ2*S*SW2**2+
     &  10D0*CW2*MZ2**2*S*SW2**2-10D0*CW2*MZ2*S**2*SW2**2+4D0*MW2*
     &  MZ2*S*SW2**3))/(CW2*S)-(B2(0,0)*(17D0*CW2**2*S**2-18D0*CW2*
     &  MZ2*S*SW2-36D0*CW2**2*MZ2*S*SW2+16D0*CW2*S**2*SW2+2D0*CW2**
     &  2*S**2*SW2+40D0*CW2*MZ2**2*SW2**2+2D0*MZ2*S*SW2**2-40D0*CW2*
     &  MZ2*S*SW2**2-S**2*SW2**2+4D0*CW2*S**2*SW2**2-4D0*MZ2*S*SW2**
     &  3+2D0*S**2*SW2**3))/(CW2*S))/SW2
        DF(2,1) = -(2D0*(-(B1(1)*K(3)*2D0*MZ2)-B2(1,1)*K(3)*2D0*MZ2+
     &  (K(1)*K(3)*MZ2)/S+(B0*MZ2*(-(5D0*CW2**2*S**2)-2D0*CW2*MW2*
     &  MZ2*SW2+2D0*CW2*MW2*S*SW2+5D0*CW2*MZ2*S*SW2-5D0*CW2*S**2*
     &  SW2+2D0*MW2*S*SW2**2))/(CW2*S)+(B2(0,0)*(-(18D0*CW2**2*MZ2*
     &  S)+CW2**2*S**2+20D0*CW2*MZ2**2*SW2-20D0*CW2*MZ2*S*SW2+2D0*
     &  CW2*S**2*SW2-2D0*MZ2*S*SW2**2+S**2*SW2**2))/(CW2*S)))
        DF(3,-1) = -(((-(B1(1)*K(2)*K(3)*2D0)-B2(1,1)*K(2)*K(3)*2D0+
     &  (K(1)*K(2)*K(3))/S+(B2(0,0)*K(7))/(CW2*S)+(K(6)*B0)/(CW2*
     &  S)))/SW2)
        DF(3,1) = 2D0*(-(B1(1)*K(3)*2D0*MZ2)-B2(1,1)*K(3)*2D0*MZ2-
     &  (B2(0,0)*K(5))/(CW2*S)+(K(1)*K(3)*MZ2)/S-(K(4)*B0*MZ2)/(CW2*
     &  S))
        DF(4,-1) = DF(3,-1)
        DF(4,1) = DF(3,1)

        PRE = -((ALPHA*(MZ2-S)**(-2))/(4D0*PI*S*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       2D0*G(T2,N2)+2D0*G(T2,N3)

        K(1) = DMZ2+DZZZ*(MZ2-S)
        K(2) = DZAZ+DZZA
        K(3) = MZ2-S
        K(4) = -((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0
        K(5) = CW2*MZ2*S*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)-
     &  CW2*S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)+CW*DZAA*
     &  MZ2**2*S*SW-2D0*CW*DZAA*MZ2*S**2*SW+CW*DZAA*S**3*SW+CW*S**2*
     &  (DMZ2+DZZZ*(MZ2-S))*SW-MZ2*S*(-((DZZA*MZ2)/2D0)+((DZAZ+
     &  DZZA)*S)/2D0)*SW2+S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/
     &  2D0)*SW2
        K(6) = -(CW*S**2*(DMZ2+DZZZ*(MZ2-S)))+MZ2*S*(-((DZZA*MZ2)/
     &  2D0)+((DZAZ+DZZA)*S)/2D0)*SW+2D0*CW2*MZ2*S*(-((DZZA*MZ2)/
     &  2D0)+((DZAZ+DZZA)*S)/2D0)*SW-S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+
     &  DZZA)*S)/2D0)*SW-2D0*CW2*S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+
     &  DZZA)*S)/2D0)*SW+2D0*CW*DZAA*MZ2**2*S*SW2-4D0*CW*DZAA*MZ2*
     &  S**2*SW2+2D0*CW*DZAA*S**3*SW2+2D0*CW*S**2*(DMZ2+DZZZ*(MZ2-
     &  S))*SW2-2D0*MZ2*S*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)*
     &  SW*SW2+2D0*S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)*SW*
     &  SW2

        DF(2,-1) = -((K(6))/SW)
        DF(2,1) = -(K(5)*2D0)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = (S**(-2)*(MZ2-S)**(-2))/(CW*SW)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

	end

************************************************************************
        subroutine add_Vself_bgf(F)
************************************************************************
*	contributions of AA, AZ, and ZZ self-energies
*	background-field formalism
*-----------------------------------------------------------------------
*       26.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 A0_,A0
	complex*16 B0,B1(1),B2(0:2,0:2) 
        COMPLEX*16 K(6)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc_bgf/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV30V30,   Model: SMbgf

C       G(T1,N14)

        A0 = A0_(MH)

        DF(2,-1) = (1D0-2D0*SW2)/SW2
        DF(2,1) = -2D0
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = -((ALPHA*A0*(MZ2-S)**(-2))/(16D0*CW2*PI*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N1)+G(T1,N10)+G(T1,N11)+G(T1,N12)+G(T1,N13)+G(T1,N16)+
C       G(T1,N17)+G(T1,N18)+G(T1,N2)+G(T1,N3)+G(T1,N4)+G(T1,N5)+
C       G(T1,N6)+G(T1,N7)+G(T1,N8)+G(T1,N9)

        A0 = A0_(MW)

        K(1) = CW2*S-MZ2*SW2+S*SW2
        K(2) = -(10D0*CW2**2*MZ2*S)+CW2**2*S**2+12D0*CW2*MZ2**2*SW2-
     &  12D0*CW2*MZ2*S*SW2+2D0*CW2*S**2*SW2-2D0*MZ2*S*SW2**2+S**2*
     &  SW2**2
        K(3) = 9D0*CW2**2*S**2-10D0*CW2*MZ2*S*SW2-20D0*CW2**2*MZ2*S*
     &  SW2+8D0*CW2*S**2*SW2+2D0*CW2**2*S**2*SW2+24D0*CW2*MZ2**2*
     &  SW2**2+2D0*MZ2*S*SW2**2-24D0*CW2*MZ2*S*SW2**2-S**2*SW2**2+
     &  4D0*CW2*S**2*SW2**2-4D0*MZ2*S*SW2**3+2D0*S**2*SW2**3

        DF(2,-1) = (K(1)*8D0*MW2*(S-2D0*MZ2*SW2)+(A0*(-(9D0*CW2**2*
     &  S**2)+10D0*CW2*MZ2*S*SW2+20D0*CW2**2*MZ2*S*SW2-8D0*CW2*S**2*
     &  SW2-2D0*CW2**2*S**2*SW2-24D0*CW2*MZ2**2*SW2**2-2D0*MZ2*S*
     &  SW2**2+24D0*CW2*MZ2*S*SW2**2+S**2*SW2**2-4D0*CW2*S**2*SW2**
     &  2+4D0*MZ2*S*SW2**3-2D0*S**2*SW2**3))/CW2)/SW2
        DF(2,1) = 2D0*(-(K(1)*8D0*MW2*MZ2)+(A0*(10D0*CW2**2*MZ2*S-
     &  CW2**2*S**2-12D0*CW2*MZ2**2*SW2+12D0*CW2*MZ2*S*SW2-2D0*CW2*
     &  S**2*SW2+2D0*MZ2*S*SW2**2-S**2*SW2**2))/CW2)
        DF(3,-1) = ((K(3)*A0)/CW2-K(1)*8D0*MW2*(S-2D0*MZ2*SW2))/SW2
        DF(3,1) = 2D0*((K(2)*A0)/CW2+K(1)*8D0*MW2*MZ2)
        DF(4,-1) = DF(3,-1)
        DF(4,1) = DF(3,1)

        PRE = (ALPHA*S**(-2)*(MZ2-S)**(-2))/(8D0*PI*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N15)

        A0 = A0_(MZ)

        DF(2,-1) = (1D0-2D0*SW2)/SW2
        DF(2,1) = -2D0
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = -((ALPHA*A0*(MZ2-S)**(-2))/(16D0*CW2*PI*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T2,N34)+G(T2,N35)

        CALL B012_(S,MH,MZ,B0,B1,B2,2,0)

        DF(2,-1) = (1D0-2D0*SW2)/SW2
        DF(2,1) = -2D0
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = -((ALPHA*(-(B2(0,0))+(B0*MW2)/CW2)*(MZ2-S)**(-2))/
     &  (4D0*CW2*PI*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T2,N19)+G(T2,N20)+G(T2,N21)+G(T2,N22)+G(T2,N23)+G(T2,N24)+
C       G(T2,N25)+G(T2,N26)+G(T2,N27)+G(T2,N28)+G(T2,N29)+G(T2,N30)+
C       G(T2,N31)+G(T2,N32)+G(T2,N33)+G(T2,N36)+G(T2,N37)+G(T2,N38)

        CALL B012_(S,MW,MW,B0,B1,B2,2,0)

        K(1) = CW2*S-MZ2*SW2+S*SW2
        K(2) = MW2*S-4D0*CW2**2*MZ2*S+4D0*CW2*MZ2**2*SW2-4D0*CW2*
     &  MZ2*S*SW2
        K(3) = 10D0*CW2**2*MZ2*S-CW2**2*S**2-12D0*CW2*MZ2**2*SW2+
     &  12D0*CW2*MZ2*S*SW2-2D0*CW2*S**2*SW2+2D0*MZ2*S*SW2**2-S**2*
     &  SW2**2
        K(4) = MW2*S-4D0*CW2**2*S**2-2D0*MW2*S*SW2+4D0*CW2*MZ2*S*
     &  SW2+8D0*CW2**2*MZ2*S*SW2-4D0*CW2*S**2*SW2-8D0*CW2*MZ2**2*
     &  SW2**2+8D0*CW2*MZ2*S*SW2**2
        K(5) = -(9D0*CW2**2*S**2)+10D0*CW2*MZ2*S*SW2+20D0*CW2**2*
     &  MZ2*S*SW2-8D0*CW2*S**2*SW2-2D0*CW2**2*S**2*SW2-24D0*CW2*
     &  MZ2**2*SW2**2-2D0*MZ2*S*SW2**2+24D0*CW2*MZ2*S*SW2**2+S**2*
     &  SW2**2-4D0*CW2*S**2*SW2**2+4D0*MZ2*S*SW2**3-2D0*S**2*SW2**3

        DF(2,-1) = ((K(4)*6D0*B0)/CW2-(K(1)*2D0*(-(6D0*MW2)+S)*(S-
     &  2D0*MZ2*SW2))/S-(B2(0,0)*3D0*(9D0*CW2**2*S**2-10D0*CW2*MZ2*
     &  S*SW2-20D0*CW2**2*MZ2*S*SW2+8D0*CW2*S**2*SW2+2D0*CW2**2*S**
     &  2*SW2+24D0*CW2*MZ2**2*SW2**2+2D0*MZ2*S*SW2**2-24D0*CW2*MZ2*
     &  S*SW2**2-S**2*SW2**2+4D0*CW2*S**2*SW2**2-4D0*MZ2*S*SW2**3+
     &  2D0*S**2*SW2**3))/(CW2*S))/SW2
        DF(2,1) = -(2D0*((K(2)*6D0*B0)/CW2-(K(1)*2D0*MZ2*(-(6D0*
     &  MW2)+S))/S+(B2(0,0)*3D0*(-(10D0*CW2**2*MZ2*S)+CW2**2*S**2+
     &  12D0*CW2*MZ2**2*SW2-12D0*CW2*MZ2*S*SW2+2D0*CW2*S**2*SW2-2D0*
     &  MZ2*S*SW2**2+S**2*SW2**2))/(CW2*S)))
        DF(3,-1) = -((((K(4)*6D0*B0)/CW2+(B2(0,0)*K(5)*3D0)/(CW2*S)+
     &  (K(1)*2D0*(6D0*MW2-S)*(S-2D0*MZ2*SW2))/S))/SW2)
        DF(3,1) = 2D0*((K(2)*6D0*B0)/CW2-(B2(0,0)*K(3)*3D0)/(CW2*S)+
     &  (K(1)*2D0*MZ2*(6D0*MW2-S))/S)
        DF(4,-1) = DF(3,-1)
        DF(4,1) = DF(3,1)

        PRE = -((ALPHA*(MZ2-S)**(-2))/(12D0*PI*S*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       2D0*G(T2,N2)+2D0*G(T2,N3)

        K(1) = DMZ2+DZZZ*(MZ2-S)
        K(2) = DZAZ+DZZA
        K(3) = MZ2-S
        K(4) = -((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0
        K(5) = CW2*MZ2*S*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)-
     &  CW2*S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)+CW*DZAA*
     &  MZ2**2*S*SW-2D0*CW*DZAA*MZ2*S**2*SW+CW*DZAA*S**3*SW+CW*S**2*
     &  (DMZ2+DZZZ*(MZ2-S))*SW-MZ2*S*(-((DZZA*MZ2)/2D0)+((DZAZ+
     &  DZZA)*S)/2D0)*SW2+S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/
     &  2D0)*SW2
        K(6) = -(CW*S**2*(DMZ2+DZZZ*(MZ2-S)))+MZ2*S*(-((DZZA*MZ2)/
     &  2D0)+((DZAZ+DZZA)*S)/2D0)*SW+2D0*CW2*MZ2*S*(-((DZZA*MZ2)/
     &  2D0)+((DZAZ+DZZA)*S)/2D0)*SW-S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+
     &  DZZA)*S)/2D0)*SW-2D0*CW2*S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+
     &  DZZA)*S)/2D0)*SW+2D0*CW*DZAA*MZ2**2*S*SW2-4D0*CW*DZAA*MZ2*
     &  S**2*SW2+2D0*CW*DZAA*S**3*SW2+2D0*CW*S**2*(DMZ2+DZZZ*(MZ2-
     &  S))*SW2-2D0*MZ2*S*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)*
     &  SW*SW2+2D0*S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)*SW*
     &  SW2

        DF(2,-1) = -((K(6))/SW)
        DF(2,1) = -(K(5)*2D0)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = (S**(-2)*(MZ2-S)**(-2))/(CW*SW)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

	end

************************************************************************
        subroutine add_VVVvert(F)
************************************************************************
*	contributions of AWW and ZWW vertex corrections
*	conventional formalism
*-----------------------------------------------------------------------
*       27.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        COMPLEX*16 K(4)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	            dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV3V3,   Model: SM

C       G(T3,N47)+G(T3,N51)

        CALL B012_(KM2,MW,0D0,B0,B1,B2,1,0)

        DF(2,-1) = ((-(B1(1))+4D0*B0)*(S-2D0*MZ2*SW2))/SW2
        DF(2,1) = -(2D0*(-(B1(1))+4D0*B0)*MZ2)
        DF(4,-1) = -(((B1(1)+5D0*B0)*(S-2D0*MZ2*SW2))/SW2)
        DF(4,1) = 2D0*(B1(1)+5D0*B0)*MZ2

        PRE = (ALPHA)/(8D0*PI*S*(MZ2-S))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T3,N45)+G(T3,N49)
C	no contribution

C       G(T3,N46)+G(T3,N48)+G(T3,N50)+G(T3,N52)

        CALL B012_(KM2,MW,MZ,B0,B1,B2,1,0)

        DF(2,-1) = ((-(B1(1))+4D0*B0)*(S-2D0*MZ2*SW2))/SW2
        DF(2,1) = -(2D0*(-(B1(1))+4D0*B0)*MZ2)
        DF(4,-1) = -(((B1(1)+5D0*B0)*(S-2D0*MZ2*SW2))/SW2)
        DF(4,1) = 2D0*(B1(1)+5D0*B0)*MZ2

        PRE = (ALPHA*CW2)/(8D0*PI*S*(MZ2-S)*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T4,N55)+G(T4,N59)

        CALL B012_(KP2,MW,0D0,B0,B1,B2,1,0)

        DF(2,-1) = ((-(B1(1))+4D0*B0)*(S-2D0*MZ2*SW2))/SW2
        DF(2,1) = -(2D0*(-(B1(1))+4D0*B0)*MZ2)
        DF(3,-1) = -(((B1(1)+5D0*B0)*(S-2D0*MZ2*SW2))/SW2)
        DF(3,1) = 2D0*(B1(1)+5D0*B0)*MZ2

        PRE = (ALPHA)/(8D0*PI*S*(MZ2-S))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE

C       G(T4,N53)+G(T4,N57)
C	no contribution

C       G(T4,N54)+G(T4,N56)+G(T4,N58)+G(T4,N60)

        CALL B012_(KP2,MW,MZ,B0,B1,B2,1,0)

        DF(2,-1) = ((-(B1(1))+4D0*B0)*(S-2D0*MZ2*SW2))/SW2
        DF(2,1) = -(2D0*(-(B1(1))+4D0*B0)*MZ2)
        DF(3,-1) = -(((B1(1)+5D0*B0)*(S-2D0*MZ2*SW2))/SW2)
        DF(3,1) = 2D0*(B1(1)+5D0*B0)*MZ2

        PRE = (ALPHA*CW2)/(8D0*PI*S*(MZ2-S)*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE

C       G(T2,N41)+G(T2,N42)+G(T2,N43)+G(T2,N44)

        CALL B012_(S,MW,MW,B0,B1,B2,1,0)

        DF(3,-1) = ((-(B1(1))+4D0*B0)*(S-2D0*MZ2*SW2))/SW2
        DF(3,1) = -(2D0*(-(B1(1))+4D0*B0)*MZ2)
        DF(4,-1) = ((B1(1)+5D0*B0)*(S-2D0*MZ2*SW2))/SW2
        DF(4,1) = -(2D0*(B1(1)+5D0*B0)*MZ2)

        PRE = -((ALPHA)/(8D0*PI*S*(MZ2-S)*SW2))

        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N10)+G(T1,N11)+G(T1,N16)+G(T1,N19)+G(T1,N22)+G(T1,N25)+
C       G(T1,N28)+G(T1,N31)+G(T1,N35)+G(T1,N39)+G(T1,N4)+G(T1,N5)

        CALL C0123_(KP2,S,KM2,0D0,MW,MW,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = S-2D0*MZ2*SW2
        K(2) = T+U

        DF(2,-1) = (K(1)*11D0-C2(0,0)*K(1)*132D0-C3(0,0,1)*K(1)*
     &  84D0-C3(0,0,2)*K(1)*84D0-C3(2,2,2)*K(1)*6D0*KM2-C3(1,2,2)*
     &  K(1)*6D0*(K(2)+KM2)-C2(2,2)*K(1)*3D0*(K(2)+8D0*KM2)-C3(1,1,
     &  1)*K(1)*6D0*KP2-C3(1,1,2)*K(1)*6D0*(K(2)+KP2)-C2(1,1)*K(1)*
     &  3D0*(K(2)+8D0*KP2)+(3D0*C0*(-(4D0*CW2*KM2*S)-4D0*CW2*KP2*S+
     &  CW2*MW2*S+3D0*CW2*S**2+8D0*CW2*KM2*MZ2*SW2+8D0*CW2*KP2*MZ2*
     &  SW2+MW2*S*SW2-2D0*CW2*MW2*S*SW2-6D0*CW2*MZ2*S*SW2-2D0*MW2*S*
     &  SW2**2))/CW2-(C1(2)*3D0*(9D0*CW2*KM2*S+7D0*CW2*KP2*S-CW2*
     &  MW2*S-8D0*CW2*S**2-18D0*CW2*KM2*MZ2*SW2-14D0*CW2*KP2*MZ2*
     &  SW2+8D0*CW2*MW2*MZ2*SW2+3D0*MW2*S*SW2-6D0*CW2*MW2*S*SW2+
     &  16D0*CW2*MZ2*S*SW2-6D0*MW2*S*SW2**2))/CW2-(C1(1)*3D0*(7D0*
     &  CW2*KM2*S+9D0*CW2*KP2*S-CW2*MW2*S-8D0*CW2*S**2-14D0*CW2*KM2*
     &  MZ2*SW2-18D0*CW2*KP2*MZ2*SW2+8D0*CW2*MW2*MZ2*SW2+3D0*MW2*S*
     &  SW2-6D0*CW2*MW2*S*SW2+16D0*CW2*MZ2*S*SW2-6D0*MW2*S*SW2**2))/
     &  CW2-C2(1,2)*K(1)*6D0*(KM2+KP2+4D0*T+4D0*U))/SW2
        DF(2,1) = 2D0*(-(11D0*MZ2)+C2(0,0)*132D0*MZ2+C3(0,0,1)*84D0*
     &  MZ2+C3(0,0,2)*84D0*MZ2+C3(2,2,2)*6D0*KM2*MZ2+C3(1,2,2)*6D0*
     &  (K(2)+KM2)*MZ2+C2(2,2)*3D0*(K(2)+8D0*KM2)*MZ2+C3(1,1,1)*6D0*
     &  KP2*MZ2+C3(1,1,2)*6D0*(K(2)+KP2)*MZ2+C2(1,1)*3D0*(K(2)+8D0*
     &  KP2)*MZ2+(3D0*C0*(4D0*CW2*KM2*MZ2+4D0*CW2*KP2*MZ2-CW2*MW2*S-
     &  3D0*CW2*MZ2*S-MW2*S*SW2))/CW2-(C1(2)*3D0*(-(9D0*CW2*KM2*
     &  MZ2)-7D0*CW2*KP2*MZ2+4D0*CW2*MW2*MZ2-3D0*CW2*MW2*S+8D0*CW2*
     &  MZ2*S-3D0*MW2*S*SW2))/CW2-(C1(1)*3D0*(-(7D0*CW2*KM2*MZ2)-
     &  9D0*CW2*KP2*MZ2+4D0*CW2*MW2*MZ2-3D0*CW2*MW2*S+8D0*CW2*MZ2*S-
     &  3D0*MW2*S*SW2))/CW2+C2(1,2)*6D0*MZ2*(KM2+KP2+4D0*T+4D0*U))
        DF(3,-1) = -(((K(1)*11D0-C1(2)*K(1)*K(2)*12D0-C2(0,0)*K(1)*
     &  48D0+C3(1,1,2)*K(1)*K(2)*6D0+C3(0,0,1)*K(1)*84D0-C2(2,2)*
     &  K(1)*12D0*KM2+C3(1,2,2)*K(1)*6D0*KM2+C3(1,1,1)*K(1)*6D0*KP2+
     &  C2(1,1)*K(1)*3D0*(K(2)-4D0*KP2)-C2(1,2)*K(1)*6D0*(KM2+2D0*
     &  KP2-2D0*S)-(C1(1)*3D0*(6D0*CW2*KM2*S+3D0*CW2*KP2*S-5D0*CW2*
     &  S**2-12D0*CW2*KM2*MZ2*SW2-6D0*CW2*KP2*MZ2*SW2+2D0*CW2*MW2*
     &  MZ2*SW2+MW2*S*SW2-2D0*CW2*MW2*S*SW2+10D0*CW2*MZ2*S*SW2-2D0*
     &  MW2*S*SW2**2))/CW2-(6D0*C0*(2D0*CW2*KM2*S+CW2*KP2*S-CW2*S**
     &  2-4D0*CW2*KM2*MZ2*SW2-2D0*CW2*KP2*MZ2*SW2-2D0*CW2*MW2*MZ2*
     &  SW2-MW2*S*SW2+2D0*CW2*MW2*S*SW2+2D0*CW2*MZ2*S*SW2+2D0*MW2*S*
     &  SW2**2))/CW2))/SW2)
        DF(3,1) = 2D0*(11D0*MZ2-C1(2)*K(2)*12D0*MZ2-C2(0,0)*48D0*
     &  MZ2+C3(1,1,2)*K(2)*6D0*MZ2+C3(0,0,1)*84D0*MZ2-C2(2,2)*12D0*
     &  KM2*MZ2+C3(1,2,2)*6D0*KM2*MZ2+C3(1,1,1)*6D0*KP2*MZ2+C2(1,1)*
     &  3D0*(K(2)-4D0*KP2)*MZ2-C2(1,2)*6D0*MZ2*(KM2+2D0*KP2-2D0*S)-
     &  (6D0*C0*(2D0*CW2*KM2*MZ2+CW2*KP2*MZ2+CW2*MW2*MZ2-CW2*MW2*S-
     &  CW2*MZ2*S-MW2*S*SW2))/CW2+(C1(1)*3D0*(-(6D0*CW2*KM2*MZ2)-
     &  3D0*CW2*KP2*MZ2+CW2*MW2*MZ2-CW2*MW2*S+5D0*CW2*MZ2*S-MW2*S*
     &  SW2))/CW2)
        DF(4,-1) = -(((K(1)*11D0-C1(1)*K(1)*K(2)*12D0-C2(0,0)*K(1)*
     &  48D0+C3(1,2,2)*K(1)*K(2)*6D0+C3(0,0,2)*K(1)*84D0+C3(2,2,2)*
     &  K(1)*6D0*KM2+C2(2,2)*K(1)*3D0*(K(2)-4D0*KM2)-C2(1,1)*K(1)*
     &  12D0*KP2+C3(1,1,2)*K(1)*6D0*KP2-C2(1,2)*K(1)*6D0*(2D0*KM2+
     &  KP2-2D0*S)-(C1(2)*3D0*(3D0*CW2*KM2*S+6D0*CW2*KP2*S-5D0*CW2*
     &  S**2-6D0*CW2*KM2*MZ2*SW2-12D0*CW2*KP2*MZ2*SW2+2D0*CW2*MW2*
     &  MZ2*SW2+MW2*S*SW2-2D0*CW2*MW2*S*SW2+10D0*CW2*MZ2*S*SW2-2D0*
     &  MW2*S*SW2**2))/CW2-(6D0*C0*(CW2*KM2*S+2D0*CW2*KP2*S-CW2*S**
     &  2-2D0*CW2*KM2*MZ2*SW2-4D0*CW2*KP2*MZ2*SW2-2D0*CW2*MW2*MZ2*
     &  SW2-MW2*S*SW2+2D0*CW2*MW2*S*SW2+2D0*CW2*MZ2*S*SW2+2D0*MW2*S*
     &  SW2**2))/CW2))/SW2)
        DF(4,1) = 2D0*(11D0*MZ2-C1(1)*K(2)*12D0*MZ2-C2(0,0)*48D0*
     &  MZ2+C3(1,2,2)*K(2)*6D0*MZ2+C3(0,0,2)*84D0*MZ2+C3(2,2,2)*6D0*
     &  KM2*MZ2+C2(2,2)*3D0*(K(2)-4D0*KM2)*MZ2-C2(1,1)*12D0*KP2*MZ2+
     &  C3(1,1,2)*6D0*KP2*MZ2-C2(1,2)*6D0*MZ2*(2D0*KM2+KP2-2D0*S)-
     &  (6D0*C0*(CW2*KM2*MZ2+2D0*CW2*KP2*MZ2+CW2*MW2*MZ2-CW2*MW2*S-
     &  CW2*MZ2*S-MW2*S*SW2))/CW2+(C1(2)*3D0*(-(3D0*CW2*KM2*MZ2)-
     &  6D0*CW2*KP2*MZ2+CW2*MW2*MZ2-CW2*MW2*S+5D0*CW2*MZ2*S-MW2*S*
     &  SW2))/CW2)
        DF(7,-1) = ((C2(1,2)+C3(1,1,2)+C3(1,2,2))*K(1)*48D0)/SW2
        DF(7,1) = -((C2(1,2)+C3(1,1,2)+C3(1,2,2))*96D0*MZ2)

        PRE = (ALPHA)/(24D0*PI*S*(MZ2-S))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE

C       G(T1,N15)+G(T1,N18)+G(T1,N21)+G(T1,N24)+G(T1,N27)+G(T1,N30)+
C       G(T1,N33)+G(T1,N37)

        CALL C0123_(KP2,S,KM2,MH,MW,MW,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = 2D0*CW2*MZ2-CW2*S-S*SW2
        K(2) = CW2*S-4D0*CW2*MZ2*SW2-S*SW2+2D0*CW2*S*SW2+2D0*S*SW2**
     &  2

        DF(2,-1) = ((C2(0,0)*K(2))/CW2+(C3(0,0,1)*K(2))/CW2+(C3(0,0,
     &  2)*K(2))/CW2-C1(1)*2D0*MW2*(S-2D0*MZ2*SW2)-C1(2)*2D0*MW2*(S-
     &  2D0*MZ2*SW2)-2D0*C0*MW2*(S-2D0*MZ2*SW2))/(S*SW2)
        DF(2,1) = -((2D0*((C2(0,0)*K(1))/CW2+(C3(0,0,1)*K(1))/CW2+
     &  (C3(0,0,2)*K(1))/CW2-C1(1)*2D0*MW2*MZ2-C1(2)*2D0*MW2*MZ2-
     &  2D0*C0*MW2*MZ2))/S)
        DF(3,-1) = ((C3(0,0,1)*K(2))/(CW2*S)+(C1(1)*MW2*(CW2+SW2)*
     &  (1D0-2D0*SW2))/CW2+(2D0*C0*MW2*(S-2D0*MZ2*SW2))/S)/SW2
        DF(3,1) = -(2D0*((C3(0,0,1)*K(1))/(CW2*S)+(2D0*C0*MW2*MZ2)/
     &  S+(C1(1)*MW2*(CW2+SW2))/CW2))
        DF(4,-1) = ((C3(0,0,2)*K(2))/(CW2*S)+(C1(2)*MW2*(CW2+SW2)*
     &  (1D0-2D0*SW2))/CW2+(2D0*C0*MW2*(S-2D0*MZ2*SW2))/S)/SW2
        DF(4,1) = -(2D0*((C3(0,0,2)*K(1))/(CW2*S)+(2D0*C0*MW2*MZ2)/
     &  S+(C1(2)*MW2*(CW2+SW2))/CW2))
        DF(7,-1) = -(((C2(1,2)+C3(1,1,2)+C3(1,2,2))*K(2))/(CW2*S*
     &  SW2))
        DF(7,1) = ((C2(1,2)+C3(1,1,2)+C3(1,2,2))*K(1)*2D0)/(CW2*S)

        PRE = -((ALPHA)/(8D0*PI*(MZ2-S)*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE

C       G(T1,N1)+G(T1,N7)+G(T1,N8)

        CALL C0123_(KP2,S,KM2,MW,MH,MZ,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = -(C2(0,0))-C3(0,0,1)-C3(0,0,2)-C1(1)*MW2-C1(2)*MW2+
     &  C0*MW2

        DF(2,-1) = (K(1)*(1D0-2D0*SW2))/SW2
        DF(2,1) = -(K(1)*2D0)
        DF(3,-1) = -(((C3(0,0,1)-C1(1)*2D0*MW2)*(1D0-2D0*SW2))/SW2)
        DF(3,1) = 2D0*(C3(0,0,1)-C1(1)*2D0*MW2)
        DF(4,-1) = -(((C3(0,0,2)+2D0*C0*MW2+(C1(2)*MW2*(CW+SW)*(CW-
     &  SW))/CW2)*(1D0-2D0*SW2))/SW2)
        DF(4,1) = 2D0*(C3(0,0,2)+2D0*C0*MW2+(C1(2)*MW2*(CW+SW)*(CW-
     &  SW))/CW2)
        DF(7,-1) = ((C2(1,2)+C3(1,1,2)+C3(1,2,2))*(1D0-2D0*SW2))/SW2
        DF(7,1) = -((C2(1,2)+C3(1,1,2)+C3(1,2,2))*2D0)

        PRE = (ALPHA)/(8D0*CW2*PI*(MZ2-S)*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE

C       G(T1,N13)+G(T1,N14)+G(T1,N2)

        CALL C0123_(KP2,S,KM2,MW,MZ,MH,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = -(C2(0,0))-C3(0,0,1)-C3(0,0,2)-C1(1)*MW2-C1(2)*MW2+
     &  C0*MW2

        DF(2,-1) = (K(1)*(1D0-2D0*SW2))/SW2
        DF(2,1) = -(K(1)*2D0)
        DF(3,-1) = -(((C3(0,0,1)+2D0*C0*MW2+(C1(1)*MW2*(CW+SW)*(CW-
     &  SW))/CW2)*(1D0-2D0*SW2))/SW2)
        DF(3,1) = 2D0*(C3(0,0,1)+2D0*C0*MW2+(C1(1)*MW2*(CW+SW)*(CW-
     &  SW))/CW2)
        DF(4,-1) = -(((C3(0,0,2)-C1(2)*2D0*MW2)*(1D0-2D0*SW2))/SW2)
        DF(4,1) = 2D0*(C3(0,0,2)-C1(2)*2D0*MW2)
        DF(7,-1) = ((C2(1,2)+C3(1,1,2)+C3(1,2,2))*(1D0-2D0*SW2))/SW2
        DF(7,1) = -((C2(1,2)+C3(1,1,2)+C3(1,2,2))*2D0)

        PRE = (ALPHA)/(8D0*CW2*PI*(MZ2-S)*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE

C       G(T1,N12)+G(T1,N17)+G(T1,N20)+G(T1,N23)+G(T1,N26)+G(T1,N29)+
C       G(T1,N3)+G(T1,N32)+G(T1,N34)+G(T1,N36)+G(T1,N38)+G(T1,N40)+
C       G(T1,N6)+G(T1,N9)

        CALL C0123_(KP2,S,KM2,MZ,MW,MW,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = S-2D0*MZ2*SW2
        K(2) = T+U
        K(3) = 2D0*CW2*MZ2+28D0*CW2**2*MZ2-CW2*S-S*SW2
        K(4) = CW2*S+28D0*CW2**2*S-4D0*CW2*MZ2*SW2-56D0*CW2**2*MZ2*
     &  SW2-S*SW2+2D0*CW2*S*SW2+2D0*S*SW2**2

        DF(2,-1) = (-(K(1)*11D0*CW2)+(C3(0,0,1)*K(4)*3D0)/CW2+(C3(0,
     &  0,2)*K(4)*3D0)/CW2+C3(2,2,2)*K(1)*6D0*CW2*KM2+C3(1,2,2)*
     &  K(1)*6D0*CW2*(K(2)+KM2)+C2(2,2)*K(1)*3D0*CW2*(K(2)+8D0*KM2)+
     &  C3(1,1,1)*K(1)*6D0*CW2*KP2+C3(1,1,2)*K(1)*6D0*CW2*(K(2)+
     &  KP2)+C2(1,1)*K(1)*3D0*CW2*(K(2)+8D0*KP2)+(C2(0,0)*3D0*(CW2*
     &  S+44D0*CW2**2*S-4D0*CW2*MZ2*SW2-88D0*CW2**2*MZ2*SW2-S*SW2+
     &  2D0*CW2*S*SW2+2D0*S*SW2**2))/CW2-C1(2)*3D0*CW2**(-2)*(-(9D0*
     &  CW2**3*KM2*S)-7D0*CW2**3*KP2*S+8D0*CW2**3*S**2+18D0*CW2**3*
     &  KM2*MZ2*SW2+14D0*CW2**3*KP2*MZ2*SW2-16D0*CW2**3*MZ2*S*SW2+
     &  4D0*CW2**2*MW2*MZ2*SW2**2+3D0*CW2*MW2*S*SW2**2-4D0*CW2**2*
     &  MW2*S*SW2**2-4D0*CW2*MW2*MZ2*SW2**3-MW2*S*SW2**3-2D0*CW2*
     &  MW2*S*SW2**3+2D0*MW2*S*SW2**4)-C1(1)*3D0*CW2**(-2)*(-(7D0*
     &  CW2**3*KM2*S)-9D0*CW2**3*KP2*S+8D0*CW2**3*S**2+14D0*CW2**3*
     &  KM2*MZ2*SW2+18D0*CW2**3*KP2*MZ2*SW2-16D0*CW2**3*MZ2*S*SW2+
     &  4D0*CW2**2*MW2*MZ2*SW2**2+3D0*CW2*MW2*S*SW2**2-4D0*CW2**2*
     &  MW2*S*SW2**2-4D0*CW2*MW2*MZ2*SW2**3-MW2*S*SW2**3-2D0*CW2*
     &  MW2*S*SW2**3+2D0*MW2*S*SW2**4)-3D0*CW2**(-2)*C0*(-(4D0*CW2**
     &  3*KM2*S)-4D0*CW2**3*KP2*S+3D0*CW2**3*S**2+8D0*CW2**3*KM2*
     &  MZ2*SW2+8D0*CW2**3*KP2*MZ2*SW2-6D0*CW2**3*MZ2*S*SW2-4D0*
     &  CW2**2*MW2*MZ2*SW2**2-CW2*MW2*S*SW2**2+4D0*CW2**2*MW2*S*
     &  SW2**2-4D0*CW2*MW2*MZ2*SW2**3-MW2*S*SW2**3+6D0*CW2*MW2*S*
     &  SW2**3+2D0*MW2*S*SW2**4)+C2(1,2)*K(1)*6D0*CW2*(KM2+KP2+4D0*
     &  T+4D0*U))/SW2
        DF(2,1) = -(2D0*((C3(0,0,1)*K(3)*3D0)/CW2+(C3(0,0,2)*K(3)*
     &  3D0)/CW2-11D0*CW2*MZ2+C3(2,2,2)*6D0*CW2*KM2*MZ2+C3(1,2,2)*
     &  6D0*CW2*(K(2)+KM2)*MZ2+C2(2,2)*3D0*CW2*(K(2)+8D0*KM2)*MZ2+
     &  C3(1,1,1)*6D0*CW2*KP2*MZ2+C3(1,1,2)*6D0*CW2*(K(2)+KP2)*MZ2+
     &  C2(1,1)*3D0*CW2*(K(2)+8D0*KP2)*MZ2+(C2(0,0)*3D0*(2D0*CW2*
     &  MZ2+44D0*CW2**2*MZ2-CW2*S-S*SW2))/CW2+C1(2)*3D0*CW2**(-2)*
     &  (9D0*CW2**3*KM2*MZ2+7D0*CW2**3*KP2*MZ2-8D0*CW2**3*MZ2*S+2D0*
     &  CW2**2*MW2*MZ2*SW2-2D0*CW2**2*MW2*S*SW2-2D0*CW2*MW2*MZ2*
     &  SW2**2-CW2*MW2*S*SW2**2+MW2*S*SW2**3)+C1(1)*3D0*CW2**(-2)*
     &  (7D0*CW2**3*KM2*MZ2+9D0*CW2**3*KP2*MZ2-8D0*CW2**3*MZ2*S+2D0*
     &  CW2**2*MW2*MZ2*SW2-2D0*CW2**2*MW2*S*SW2-2D0*CW2*MW2*MZ2*
     &  SW2**2-CW2*MW2*S*SW2**2+MW2*S*SW2**3)+3D0*CW2**(-2)*C0*(4D0*
     &  CW2**3*KM2*MZ2+4D0*CW2**3*KP2*MZ2-3D0*CW2**3*MZ2*S-2D0*CW2**
     &  2*MW2*MZ2*SW2+2D0*CW2**2*MW2*S*SW2-2D0*CW2*MW2*MZ2*SW2**2+
     &  3D0*CW2*MW2*S*SW2**2+MW2*S*SW2**3)+C2(1,2)*6D0*CW2*MZ2*(KM2+
     &  KP2+4D0*T+4D0*U)))
        DF(3,-1) = (K(1)*11D0*CW2-C1(2)*K(1)*K(2)*12D0*CW2-C2(0,0)*
     &  K(1)*48D0*CW2+C3(1,1,2)*K(1)*K(2)*6D0*CW2+(C3(0,0,1)*K(4)*
     &  3D0)/CW2-C2(2,2)*K(1)*12D0*CW2*KM2+C3(1,2,2)*K(1)*6D0*CW2*
     &  KM2+C3(1,1,1)*K(1)*6D0*CW2*KP2+C2(1,1)*K(1)*3D0*CW2*(K(2)-
     &  4D0*KP2)-C2(1,2)*K(1)*6D0*CW2*(KM2+2D0*KP2-2D0*S)+(C1(1)*
     &  3D0*(-(6D0*CW2**2*KM2*S)-3D0*CW2**2*KP2*S+5D0*CW2**2*S**2+
     &  12D0*CW2**2*KM2*MZ2*SW2+6D0*CW2**2*KP2*MZ2*SW2-10D0*CW2**2*
     &  MZ2*S*SW2+2D0*CW2*MW2*MZ2*SW2**2+MW2*S*SW2**2-2D0*CW2*MW2*S*
     &  SW2**2-2D0*MW2*S*SW2**3))/CW2-(6D0*C0*(2D0*CW2**2*KM2*S+
     &  CW2**2*KP2*S-CW2**2*S**2-4D0*CW2**2*KM2*MZ2*SW2-2D0*CW2**2*
     &  KP2*MZ2*SW2+2D0*CW2**2*MZ2*S*SW2+2D0*CW2*MW2*MZ2*SW2**2+MW2*
     &  S*SW2**2-2D0*CW2*MW2*S*SW2**2-2D0*MW2*S*SW2**3))/CW2)/SW2
        DF(3,1) = -(2D0*((C3(0,0,1)*K(3)*3D0)/CW2+11D0*CW2*MZ2-
     &  C1(2)*K(2)*12D0*CW2*MZ2-C2(0,0)*48D0*CW2*MZ2+C3(1,1,2)*K(2)*
     &  6D0*CW2*MZ2-C2(2,2)*12D0*CW2*KM2*MZ2+C3(1,2,2)*6D0*CW2*KM2*
     &  MZ2+C3(1,1,1)*6D0*CW2*KP2*MZ2+C2(1,1)*3D0*CW2*(K(2)-4D0*
     &  KP2)*MZ2-C2(1,2)*6D0*CW2*MZ2*(KM2+2D0*KP2-2D0*S)-(6D0*C0*
     &  (2D0*CW2**2*KM2*MZ2+CW2**2*KP2*MZ2-CW2**2*MZ2*S-CW2*MW2*MZ2*
     &  SW2+CW2*MW2*S*SW2+MW2*S*SW2**2))/CW2-(C1(1)*3D0*(6D0*CW2**2*
     &  KM2*MZ2+3D0*CW2**2*KP2*MZ2-5D0*CW2**2*MZ2*S+CW2*MW2*MZ2*SW2-
     &  CW2*MW2*S*SW2-MW2*S*SW2**2))/CW2))
        DF(4,-1) = (K(1)*11D0*CW2-C1(1)*K(1)*K(2)*12D0*CW2-C2(0,0)*
     &  K(1)*48D0*CW2+C3(1,2,2)*K(1)*K(2)*6D0*CW2+(C3(0,0,2)*K(4)*
     &  3D0)/CW2+C3(2,2,2)*K(1)*6D0*CW2*KM2+C2(2,2)*K(1)*3D0*CW2*
     &  (K(2)-4D0*KM2)-C2(1,1)*K(1)*12D0*CW2*KP2+C3(1,1,2)*K(1)*6D0*
     &  CW2*KP2-C2(1,2)*K(1)*6D0*CW2*(2D0*KM2+KP2-2D0*S)+(C1(2)*3D0*
     &  (-(3D0*CW2**2*KM2*S)-6D0*CW2**2*KP2*S+5D0*CW2**2*S**2+6D0*
     &  CW2**2*KM2*MZ2*SW2+12D0*CW2**2*KP2*MZ2*SW2-10D0*CW2**2*MZ2*
     &  S*SW2+2D0*CW2*MW2*MZ2*SW2**2+MW2*S*SW2**2-2D0*CW2*MW2*S*
     &  SW2**2-2D0*MW2*S*SW2**3))/CW2-(6D0*C0*(CW2**2*KM2*S+2D0*
     &  CW2**2*KP2*S-CW2**2*S**2-2D0*CW2**2*KM2*MZ2*SW2-4D0*CW2**2*
     &  KP2*MZ2*SW2+2D0*CW2**2*MZ2*S*SW2+2D0*CW2*MW2*MZ2*SW2**2+MW2*
     &  S*SW2**2-2D0*CW2*MW2*S*SW2**2-2D0*MW2*S*SW2**3))/CW2)/SW2
        DF(4,1) = -(2D0*((C3(0,0,2)*K(3)*3D0)/CW2+11D0*CW2*MZ2-
     &  C1(1)*K(2)*12D0*CW2*MZ2-C2(0,0)*48D0*CW2*MZ2+C3(1,2,2)*K(2)*
     &  6D0*CW2*MZ2+C3(2,2,2)*6D0*CW2*KM2*MZ2+C2(2,2)*3D0*CW2*(K(2)-
     &  4D0*KM2)*MZ2-C2(1,1)*12D0*CW2*KP2*MZ2+C3(1,1,2)*6D0*CW2*KP2*
     &  MZ2-C2(1,2)*6D0*CW2*MZ2*(2D0*KM2+KP2-2D0*S)-(6D0*C0*(CW2**2*
     &  KM2*MZ2+2D0*CW2**2*KP2*MZ2-CW2**2*MZ2*S-CW2*MW2*MZ2*SW2+CW2*
     &  MW2*S*SW2+MW2*S*SW2**2))/CW2-(C1(2)*3D0*(3D0*CW2**2*KM2*MZ2+
     &  6D0*CW2**2*KP2*MZ2-5D0*CW2**2*MZ2*S+CW2*MW2*MZ2*SW2-CW2*MW2*
     &  S*SW2-MW2*S*SW2**2))/CW2))
        DF(7,-1) = -(((C2(1,2)+C3(1,1,2)+C3(1,2,2))*3D0*(CW2*S+16D0*
     &  CW2**2*S-4D0*CW2*MZ2*SW2-32D0*CW2**2*MZ2*SW2-S*SW2+2D0*CW2*
     &  S*SW2+2D0*S*SW2**2))/(CW2*SW2))
        DF(7,1) = ((C2(1,2)+C3(1,1,2)+C3(1,2,2))*6D0*(2D0*CW2*MZ2+
     &  16D0*CW2**2*MZ2-CW2*S-S*SW2))/CW2

        PRE = -((ALPHA)/(24D0*PI*S*(MZ2-S)*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T2,N2)+G(T2,N3)

        K(1) = CW*CW2*DZZA*MZ2+2D0*DSW*S-CW*CW2*DZZA*S-CW2*DZAA*MZ2*
     &  SW-2D0*CW2*DZE*MZ2*SW-2D0*CW2*DZW*MZ2*SW+CW2*DZAA*S*SW-CW2*
     &  DZZZ*S*SW+CW*DZAZ*S*SW2
        K(2) = 2D0*DSW*S-2D0*CW2*DZE*S*SW-2D0*CW2*DZW*S*SW-CW2*DZZZ*
     &  S*SW-2D0*CW*CW2*DZZA*MZ2*SW2-4D0*DSW*S*SW2+CW*DZAZ*S*SW2+
     &  2D0*CW*CW2*DZZA*S*SW2+2D0*CW2*DZAA*MZ2*SW*SW2+4D0*CW2*DZE*
     &  MZ2*SW*SW2+4D0*CW2*DZW*MZ2*SW*SW2-2D0*CW2*DZAA*S*SW*SW2+2D0*
     &  CW2*DZZZ*S*SW*SW2-2D0*CW*DZAZ*S*SW2**2

        DF(2,-1) = K(2)/SW2
        DF(2,1) = -(K(1)*2D0)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = 1D0/(2D0*CW2*S*(MZ2-S)*SW)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

	end

************************************************************************
        subroutine add_VVVvert_bgf(F)
************************************************************************
*	contributions of AWW and ZWW vertex corrections
*	background-field formalism
*-----------------------------------------------------------------------
*       27.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        COMPLEX*16 K(4)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc_bgf/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV30V30,   Model: SMbgf

C       G(T3,N52)+G(T3,N54)+G(T3,N55)+G(T3,N60)+G(T3,N62)+G(T3,N63)

        CALL B012_(KM2,MW,0D0,B0,B1,B2,0,0)

        DF(2,-1) = (S-2D0*MZ2*SW2)/SW2
        DF(2,1) = -(2D0*MZ2)
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = (ALPHA*B0)/(PI*S*(MZ2-S))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T3,N49)+G(T3,N57)
C	no contribution

C       G(T3,N50)+G(T3,N51)+G(T3,N53)+G(T3,N56)+G(T3,N58)+G(T3,N59)+
C       G(T3,N61)+G(T3,N64)

        CALL B012_(KM2,MW,MZ,B0,B1,B2,0,0)

        DF(2,-1) = (S-2D0*MZ2*SW2)/SW2
        DF(2,1) = -(2D0*MZ2)
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = (ALPHA*B0*CW2)/(PI*S*(MZ2-S)*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T4,N68)+G(T4,N70)+G(T4,N71)+G(T4,N76)+G(T4,N78)+G(T4,N79)

        CALL B012_(KP2,MW,0D0,B0,B1,B2,0,0)

        DF(2,-1) = (S-2D0*MZ2*SW2)/SW2
        DF(2,1) = -(2D0*MZ2)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))

        PRE = (ALPHA*B0)/(PI*S*(MZ2-S))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE

C       G(T4,N65)+G(T4,N73)
C	no contribution

C       G(T4,N66)+G(T4,N67)+G(T4,N69)+G(T4,N72)+G(T4,N74)+G(T4,N75)+
C       G(T4,N77)+G(T4,N80)

        CALL B012_(KP2,MW,MZ,B0,B1,B2,0,0)

        DF(2,-1) = (S-2D0*MZ2*SW2)/SW2
        DF(2,1) = -(2D0*MZ2)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))

        PRE = (ALPHA*B0*CW2)/(PI*S*(MZ2-S)*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE

C       G(T2,N41)+G(T2,N42)+G(T2,N43)+G(T2,N44)+G(T2,N45)+G(T2,N46)+
C       G(T2,N47)+G(T2,N48)

        CALL B012_(S,MW,MW,B0,B1,B2,0,0)

        DF(3,-1) = (S-2D0*MZ2*SW2)/SW2
        DF(3,1) = -(2D0*MZ2)
        DF(4,-1) = DF(3,-1)
        DF(4,1) = DF(3,1)

        PRE = -((ALPHA*B0)/(PI*S*(MZ2-S)*SW2))

        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N12)+G(T1,N13)+G(T1,N19)+G(T1,N2)+G(T1,N23)+G(T1,N27)+
C       G(T1,N3)+G(T1,N31)+G(T1,N35)+G(T1,N39)

        CALL C0123_(KP2,S,KM2,0D0,MW,MW,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = S-2D0*MZ2*SW2
        K(2) = -(2D0*CW2*KM2*MZ2)-2D0*CW2*KP2*MZ2+2D0*CW2*MW2*MZ2-
     &  CW2*MW2*S+2D0*CW2*MZ2*S-MW2*S*SW2
        K(3) = 2D0*CW2*KM2*S+2D0*CW2*KP2*S-CW2*MW2*S-2D0*CW2*S**2-
     &  4D0*CW2*KM2*MZ2*SW2-4D0*CW2*KP2*MZ2*SW2+4D0*CW2*MW2*MZ2*SW2+
     &  MW2*S*SW2-2D0*CW2*MW2*S*SW2+4D0*CW2*MZ2*S*SW2-2D0*MW2*S*
     &  SW2**2

        DF(2,-1) = (K(1)-C2(0,0)*12D0*(S-2D0*MZ2*SW2)-C3(0,0,1)*
     &  12D0*(S-2D0*MZ2*SW2)-C3(0,0,2)*12D0*(S-2D0*MZ2*SW2)-(C1(1)*
     &  3D0*(2D0*CW2*KM2*S+2D0*CW2*KP2*S-CW2*MW2*S-2D0*CW2*S**2-4D0*
     &  CW2*KM2*MZ2*SW2-4D0*CW2*KP2*MZ2*SW2+4D0*CW2*MW2*MZ2*SW2+MW2*
     &  S*SW2-2D0*CW2*MW2*S*SW2+4D0*CW2*MZ2*S*SW2-2D0*MW2*S*SW2**
     &  2))/CW2-(C1(2)*3D0*(2D0*CW2*KM2*S+2D0*CW2*KP2*S-CW2*MW2*S-
     &  2D0*CW2*S**2-4D0*CW2*KM2*MZ2*SW2-4D0*CW2*KP2*MZ2*SW2+4D0*
     &  CW2*MW2*MZ2*SW2+MW2*S*SW2-2D0*CW2*MW2*S*SW2+4D0*CW2*MZ2*S*
     &  SW2-2D0*MW2*S*SW2**2))/CW2+(3D0*C0*(-(2D0*CW2*KM2*S)-2D0*
     &  CW2*KP2*S+2D0*MW2*S+CW2*MW2*S+4D0*CW2*KM2*MZ2*SW2+4D0*CW2*
     &  KP2*MZ2*SW2-4D0*CW2*MW2*MZ2*SW2-5D0*MW2*S*SW2+2D0*CW2*MW2*S*
     &  SW2+2D0*MW2*S*SW2**2))/CW2)/SW2
        DF(2,1) = -(2D0*((C1(1)*K(2)*3D0)/CW2+(C1(2)*K(2)*3D0)/CW2+
     &  MZ2-C2(0,0)*12D0*MZ2-C3(0,0,1)*12D0*MZ2-C3(0,0,2)*12D0*MZ2+
     &  (3D0*C0*(-(2D0*CW2*KM2*MZ2)-2D0*CW2*KP2*MZ2+2D0*CW2*MW2*MZ2+
     &  2D0*MW2*S-CW2*MW2*S-MW2*S*SW2))/CW2))
        DF(3,-1) = -(((K(1)+C3(0,0,1)*12D0*(S-2D0*MZ2*SW2)+(3D0*C0*
     &  (-(2D0*CW2*KM2*S)+MW2*S+4D0*CW2*KM2*MZ2*SW2-2D0*MW2*S*SW2))/
     &  CW2+(C1(1)*3D0*(-(2D0*CW2*KM2*S)+2D0*CW2*KP2*S-MW2*S+2D0*
     &  CW2*S**2+4D0*CW2*KM2*MZ2*SW2-4D0*CW2*KP2*MZ2*SW2+2D0*MW2*S*
     &  SW2-4D0*CW2*MZ2*S*SW2))/CW2))/SW2)
        DF(3,1) = 2D0*(MZ2+C3(0,0,1)*12D0*MZ2-(3D0*C0*(2D0*CW2*KM2*
     &  MZ2-MW2*S))/CW2-(C1(1)*3D0*(2D0*CW2*KM2*MZ2-2D0*CW2*KP2*MZ2+
     &  MW2*S-2D0*CW2*MZ2*S))/CW2)
        DF(4,-1) = -(((K(1)+C3(0,0,2)*12D0*(S-2D0*MZ2*SW2)+(3D0*C0*
     &  (-(2D0*CW2*KP2*S)+MW2*S+4D0*CW2*KP2*MZ2*SW2-2D0*MW2*S*SW2))/
     &  CW2+(C1(2)*3D0*(2D0*CW2*KM2*S-2D0*CW2*KP2*S-MW2*S+2D0*CW2*
     &  S**2-4D0*CW2*KM2*MZ2*SW2+4D0*CW2*KP2*MZ2*SW2+2D0*MW2*S*SW2-
     &  4D0*CW2*MZ2*S*SW2))/CW2))/SW2)
        DF(4,1) = 2D0*(MZ2+C3(0,0,2)*12D0*MZ2-(3D0*C0*(2D0*CW2*KP2*
     &  MZ2-MW2*S))/CW2-(C1(2)*3D0*(-(2D0*CW2*KM2*MZ2)+2D0*CW2*KP2*
     &  MZ2+MW2*S-2D0*CW2*MZ2*S))/CW2)
        DF(7,-1) = ((C2(1,2)+C3(1,1,2)+C3(1,2,2))*K(1)*12D0)/SW2
        DF(7,1) = -((C2(1,2)+C3(1,1,2)+C3(1,2,2))*24D0*MZ2)

        PRE = (ALPHA)/(6D0*PI*S*(MZ2-S))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE

C       G(T1,N17)+G(T1,N21)+G(T1,N25)+G(T1,N29)+G(T1,N33)+G(T1,N37)

        CALL C0123_(KP2,S,KM2,MH,MW,MW,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = 2D0*CW2*MZ2-CW2*S-S*SW2
        K(2) = CW2*S-4D0*CW2*MZ2*SW2-S*SW2+2D0*CW2*S*SW2+2D0*S*SW2**
     &  2

        DF(2,-1) = ((C2(0,0)*K(2))/CW2+(C3(0,0,1)*K(2))/CW2+(C3(0,0,
     &  2)*K(2))/CW2-C1(1)*2D0*MW2*(S-2D0*MZ2*SW2)-C1(2)*2D0*MW2*(S-
     &  2D0*MZ2*SW2)-2D0*C0*MW2*(S-2D0*MZ2*SW2))/(S*SW2)
        DF(2,1) = -((2D0*((C2(0,0)*K(1))/CW2+(C3(0,0,1)*K(1))/CW2+
     &  (C3(0,0,2)*K(1))/CW2-C1(1)*2D0*MW2*MZ2-C1(2)*2D0*MW2*MZ2-
     &  2D0*C0*MW2*MZ2))/S)
        DF(3,-1) = ((C3(0,0,1)*K(2))/(CW2*S)+(C1(1)*MW2*(1D0-2D0*
     &  SW2))/CW2+(2D0*C0*MW2*(S-2D0*MZ2*SW2))/S)/SW2
        DF(3,1) = -(2D0*((C1(1)*MW2)/CW2+(C3(0,0,1)*K(1))/(CW2*S)+
     &  (2D0*C0*MW2*MZ2)/S))
        DF(4,-1) = ((C3(0,0,2)*K(2))/(CW2*S)+(C1(2)*MW2*(1D0-2D0*
     &  SW2))/CW2+(2D0*C0*MW2*(S-2D0*MZ2*SW2))/S)/SW2
        DF(4,1) = -(2D0*((C1(2)*MW2)/CW2+(C3(0,0,2)*K(1))/(CW2*S)+
     &  (2D0*C0*MW2*MZ2)/S))
        DF(7,-1) = -(((C2(1,2)+C3(1,1,2)+C3(1,2,2))*K(2))/(CW2*S*
     &  SW2))
        DF(7,1) = ((C2(1,2)+C3(1,1,2)+C3(1,2,2))*K(1)*2D0)/(CW2*S)

        PRE = -((ALPHA)/(8D0*PI*(MZ2-S)*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE

C       G(T1,N5)+G(T1,N6)+G(T1,N7)+G(T1,N8)

        CALL C0123_(KP2,S,KM2,MW,MH,MZ,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = -(C2(0,0))-C3(0,0,1)-C3(0,0,2)-C1(1)*MW2-C1(2)*MW2+
     &  C0*MW2

        DF(2,-1) = (K(1)*(1D0-2D0*SW2))/SW2
        DF(2,1) = -(K(1)*2D0)
        DF(3,-1) = -(((C3(0,0,1)-C1(1)*2D0*MW2)*(1D0-2D0*SW2))/SW2)
        DF(3,1) = 2D0*(C3(0,0,1)-C1(1)*2D0*MW2)
        DF(4,-1) = -(((C3(0,0,2)+2D0*C0*MW2+(C1(2)*MW2*(CW+SW)*(CW-
     &  SW))/CW2)*(1D0-2D0*SW2))/SW2)
        DF(4,1) = 2D0*(C3(0,0,2)+2D0*C0*MW2+(C1(2)*MW2*(CW+SW)*(CW-
     &  SW))/CW2)
        DF(7,-1) = ((C2(1,2)+C3(1,1,2)+C3(1,2,2))*(1D0-2D0*SW2))/SW2
        DF(7,1) = -((C2(1,2)+C3(1,1,2)+C3(1,2,2))*2D0)

        PRE = (ALPHA)/(8D0*CW2*PI*(MZ2-S)*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE

C       G(T1,N10)+G(T1,N15)+G(T1,N16)+G(T1,N9)

        CALL C0123_(KP2,S,KM2,MW,MZ,MH,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = -(C2(0,0))-C3(0,0,1)-C3(0,0,2)-C1(1)*MW2-C1(2)*MW2+
     &  C0*MW2

        DF(2,-1) = (K(1)*(1D0-2D0*SW2))/SW2
        DF(2,1) = -(K(1)*2D0)
        DF(3,-1) = -(((C3(0,0,1)+2D0*C0*MW2+(C1(1)*MW2*(CW+SW)*(CW-
     &  SW))/CW2)*(1D0-2D0*SW2))/SW2)
        DF(3,1) = 2D0*(C3(0,0,1)+2D0*C0*MW2+(C1(1)*MW2*(CW+SW)*(CW-
     &  SW))/CW2)
        DF(4,-1) = -(((C3(0,0,2)-C1(2)*2D0*MW2)*(1D0-2D0*SW2))/SW2)
        DF(4,1) = 2D0*(C3(0,0,2)-C1(2)*2D0*MW2)
        DF(7,-1) = ((C2(1,2)+C3(1,1,2)+C3(1,2,2))*(1D0-2D0*SW2))/SW2
        DF(7,1) = -((C2(1,2)+C3(1,1,2)+C3(1,2,2))*2D0)

        PRE = (ALPHA)/(8D0*CW2*PI*(MZ2-S)*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE

C       G(T1,N1)+G(T1,N11)+G(T1,N14)+G(T1,N18)+G(T1,N20)+G(T1,N22)+
C       G(T1,N24)+G(T1,N26)+G(T1,N28)+G(T1,N30)+G(T1,N32)+G(T1,N34)+
C       G(T1,N36)+G(T1,N38)+G(T1,N4)+G(T1,N40)

        CALL C0123_(KP2,S,KM2,MZ,MW,MW,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = 2D0*CW2*MZ2+16D0*CW2**2*MZ2-CW2*S-S*SW2
        K(2) = CW2*S+16D0*CW2**2*S-4D0*CW2*MZ2*SW2-32D0*CW2**2*MZ2*
     &  SW2-S*SW2+2D0*CW2*S*SW2+2D0*S*SW2**2
        K(3) = 8D0*CW2**3*KM2*MZ2+8D0*CW2**3*KP2*MZ2-2D0*CW2**2*MW2*
     &  MZ2-2D0*CW2**3*MW2*MZ2+CW2**3*MW2*S-8D0*CW2**3*MZ2*S+4D0*
     &  CW2**2*MW2*MZ2*SW2-CW2**2*MW2*S*SW2-2D0*CW2*MW2*MZ2*SW2**2-
     &  CW2*MW2*S*SW2**2+MW2*S*SW2**3
        K(4) = 8D0*CW2**3*KM2*S+8D0*CW2**3*KP2*S-2D0*CW2**2*MW2*S-
     &  CW2**3*MW2*S-8D0*CW2**3*S**2-16D0*CW2**3*KM2*MZ2*SW2-16D0*
     &  CW2**3*KP2*MZ2*SW2+4D0*CW2**2*MW2*MZ2*SW2+4D0*CW2**3*MW2*
     &  MZ2*SW2+3D0*CW2**2*MW2*S*SW2-2D0*CW2**3*MW2*S*SW2+16D0*CW2**
     &  3*MZ2*S*SW2-8D0*CW2**2*MW2*MZ2*SW2**2-3D0*CW2*MW2*S*SW2**2+
     &  2D0*CW2**2*MW2*S*SW2**2+4D0*CW2*MW2*MZ2*SW2**3+MW2*S*SW2**3+
     &  2D0*CW2*MW2*S*SW2**3-2D0*MW2*S*SW2**4

        DF(2,-1) = ((C2(0,0)*K(2)*3D0)/CW2+(C3(0,0,1)*K(2)*3D0)/CW2+
     &  (C3(0,0,2)*K(2)*3D0)/CW2+C1(1)*K(4)*3D0*CW2**(-2)+C1(2)*
     &  K(4)*3D0*CW2**(-2)-4D0*CW2*(S-2D0*MZ2*SW2)-3D0*CW2**(-2)*C0*
     &  (-(8D0*CW2**3*KM2*S)-8D0*CW2**3*KP2*S+6D0*CW2**2*MW2*S+CW2**
     &  3*MW2*S+16D0*CW2**3*KM2*MZ2*SW2+16D0*CW2**3*KP2*MZ2*SW2-4D0*
     &  CW2**2*MW2*MZ2*SW2-4D0*CW2**3*MW2*MZ2*SW2-4D0*CW2*MW2*S*SW2-
     &  11D0*CW2**2*MW2*S*SW2+2D0*CW2**3*MW2*S*SW2+8D0*CW2**2*MW2*
     &  MZ2*SW2**2+11D0*CW2*MW2*S*SW2**2-2D0*CW2**2*MW2*S*SW2**2-
     &  4D0*CW2*MW2*MZ2*SW2**3-MW2*S*SW2**3-2D0*CW2*MW2*S*SW2**3+
     &  2D0*MW2*S*SW2**4))/SW2
        DF(2,1) = -(2D0*((C2(0,0)*K(1)*3D0)/CW2+(C3(0,0,1)*K(1)*
     &  3D0)/CW2+(C3(0,0,2)*K(1)*3D0)/CW2+C1(1)*K(3)*3D0*CW2**(-2)+
     &  C1(2)*K(3)*3D0*CW2**(-2)-4D0*CW2*MZ2-3D0*CW2**(-2)*C0*(-
     &  (8D0*CW2**3*KM2*MZ2)-8D0*CW2**3*KP2*MZ2+2D0*CW2**2*MW2*MZ2+
     &  2D0*CW2**3*MW2*MZ2+4D0*CW2**2*MW2*S-CW2**3*MW2*S-4D0*CW2**2*
     &  MW2*MZ2*SW2-4D0*CW2*MW2*S*SW2+CW2**2*MW2*S*SW2+2D0*CW2*MW2*
     &  MZ2*SW2**2+CW2*MW2*S*SW2**2-MW2*S*SW2**3)))
        DF(3,-1) = ((C3(0,0,1)*K(2)*3D0)/CW2+4D0*CW2*(S-2D0*MZ2*
     &  SW2)+(6D0*C0*(-(4D0*CW2**2*KM2*S)+2D0*CW2*MW2*S+8D0*CW2**2*
     &  KM2*MZ2*SW2-2D0*CW2*MW2*MZ2*SW2-MW2*S*SW2-2D0*CW2*MW2*S*SW2+
     &  2D0*MW2*S*SW2**2))/CW2-(C1(1)*3D0*(8D0*CW2**2*KM2*S-8D0*
     &  CW2**2*KP2*S+MW2*S+2D0*CW2*MW2*S-8D0*CW2**2*S**2-16D0*CW2**
     &  2*KM2*MZ2*SW2+16D0*CW2**2*KP2*MZ2*SW2-4D0*MW2*S*SW2-4D0*CW2*
     &  MW2*S*SW2+16D0*CW2**2*MZ2*S*SW2+4D0*MW2*S*SW2**2))/CW2)/SW2
        DF(3,1) = -(2D0*((C3(0,0,1)*K(1)*3D0)/CW2+4D0*CW2*MZ2+(6D0*
     &  C0*(-(4D0*CW2**2*KM2*MZ2)+CW2*MW2*MZ2+CW2*MW2*S-MW2*S*SW2))/
     &  CW2-(C1(1)*3D0*(8D0*CW2**2*KM2*MZ2-8D0*CW2**2*KP2*MZ2+MW2*S+
     &  2D0*CW2*MW2*S-8D0*CW2**2*MZ2*S-2D0*MW2*S*SW2))/CW2))
        DF(4,-1) = ((C3(0,0,2)*K(2)*3D0)/CW2+4D0*CW2*(S-2D0*MZ2*
     &  SW2)+(6D0*C0*(-(4D0*CW2**2*KP2*S)+2D0*CW2*MW2*S+8D0*CW2**2*
     &  KP2*MZ2*SW2-2D0*CW2*MW2*MZ2*SW2-MW2*S*SW2-2D0*CW2*MW2*S*SW2+
     &  2D0*MW2*S*SW2**2))/CW2-(C1(2)*3D0*(-(8D0*CW2**2*KM2*S)+8D0*
     &  CW2**2*KP2*S+MW2*S+2D0*CW2*MW2*S-8D0*CW2**2*S**2+16D0*CW2**
     &  2*KM2*MZ2*SW2-16D0*CW2**2*KP2*MZ2*SW2-4D0*MW2*S*SW2-4D0*CW2*
     &  MW2*S*SW2+16D0*CW2**2*MZ2*S*SW2+4D0*MW2*S*SW2**2))/CW2)/SW2
        DF(4,1) = -(2D0*((C3(0,0,2)*K(1)*3D0)/CW2+4D0*CW2*MZ2+(6D0*
     &  C0*(-(4D0*CW2**2*KP2*MZ2)+CW2*MW2*MZ2+CW2*MW2*S-MW2*S*SW2))/
     &  CW2-(C1(2)*3D0*(-(8D0*CW2**2*KM2*MZ2)+8D0*CW2**2*KP2*MZ2+
     &  MW2*S+2D0*CW2*MW2*S-8D0*CW2**2*MZ2*S-2D0*MW2*S*SW2))/CW2))
        DF(7,-1) = -(((C2(1,2)+C3(1,1,2)+C3(1,2,2))*K(2)*3D0)/(CW2*
     &  SW2))
        DF(7,1) = ((C2(1,2)+C3(1,1,2)+C3(1,2,2))*K(1)*6D0)/CW2

        PRE = -((ALPHA)/(24D0*PI*S*(MZ2-S)*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T2,N2)+G(T2,N3)

        K(1) = CW*CW2*DZZA*MZ2+2D0*DSW*S-CW*CW2*DZZA*S-CW2*DZAA*MZ2*
     &  SW-2D0*CW2*DZE*MZ2*SW-2D0*CW2*DZW*MZ2*SW+CW2*DZAA*S*SW-CW2*
     &  DZZZ*S*SW+CW*DZAZ*S*SW2
        K(2) = 2D0*DSW*S-2D0*CW2*DZE*S*SW-2D0*CW2*DZW*S*SW-CW2*DZZZ*
     &  S*SW-2D0*CW*CW2*DZZA*MZ2*SW2-4D0*DSW*S*SW2+CW*DZAZ*S*SW2+
     &  2D0*CW*CW2*DZZA*S*SW2+2D0*CW2*DZAA*MZ2*SW*SW2+4D0*CW2*DZE*
     &  MZ2*SW*SW2+4D0*CW2*DZW*MZ2*SW*SW2-2D0*CW2*DZAA*S*SW*SW2+2D0*
     &  CW2*DZZZ*S*SW*SW2-2D0*CW*DZAZ*S*SW2**2

        DF(2,-1) = K(2)/SW2
        DF(2,1) = -(K(1)*2D0)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = 1D0/(2D0*CW2*S*(MZ2-S)*SW)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

	end

************************************************************************
        subroutine add_nself(F)
************************************************************************
*	contribution of nu_e self-energy
*	conventional formalism
*-----------------------------------------------------------------------
*       27.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	            dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N2)

        CALL B012_(T,MW,0d0,B0,B1,B2,1,0)

        DF(1,-1) = 1D0

        PRE = -((ALPHA*(1D0-B1(1)*2D0-2D0*B0)*SW2**(-2))/(16D0*PI*
     &  T))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE

C       G(T1,N1)

        CALL B012_(T,MZ,0D0,B0,B1,B2,1,0)

        DF(1,-1) = 1D0

        PRE = -((ALPHA*(1D0-B1(1)*2D0-2D0*B0)*SW2**(-2))/(32D0*CW2*
     &  PI*T))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N1)

        DF(1,-1) = 1D0

        PRE = (DZNNL)/(2D0*SW2*T)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE

	end

************************************************************************
        subroutine add_nself_bgf(F)
************************************************************************
*	contribution of nu_e self-energy
*	background-field formalism
*-----------------------------------------------------------------------
*       27.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc_bgf/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV30V30,   Model: SMbgf

C       G(T1,N1)

        CALL B012_(T,0D0,MZ,B0,B1,B2,1,0)

        DF(1,-1) = 1D0

        PRE = -(((1D0+B1(1)*2D0)*ALPHA*SW2**(-2))/(32D0*CW2*PI*T))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE

C       G(T1,N2)

        CALL B012_(T,0d0,MW,B0,B1,B2,1,0)

        DF(1,-1) = 1D0

        PRE = -(((1D0+B1(1)*2D0)*ALPHA*SW2**(-2))/(16D0*PI*T))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N1)

        DF(1,-1) = 1D0

        PRE = (DZNNL)/(2D0*SW2*T)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE

	end

************************************************************************
        subroutine add_eeVvert(F)
************************************************************************
*	contributions of eeA and eeZ vertex corrections
*	conventional formalism
*-----------------------------------------------------------------------
*       27.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        COMPLEX*16 K(2)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	            dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N5)

        CALL C0123_(0D0,0D0,S,0D0,MW,0D0,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(2,-1) = 1D0
        DF(3,-1) = -(DF(2,-1))
        DF(4,-1) = -(DF(2,-1))

        PRE = -((ALPHA*(1D0-C2(0,0)*2D0-C1(1)*S-C1(2)*S-C2(1,2)*S-
     &  C2(2,2)*S)*SW2**(-2))/(4D0*PI*(MZ2-S)))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE

C       G(T1,N2)+G(T1,N6)

        CALL C0123_(mxe2,mxe2,S,mxe,0D0,mxe,C0,C1,C2,C3,B0,B1,B2,2,301)

        DF(2,-1) = (S-2D0*MZ2*SW2)/SW2
        DF(2,1) = -(2D0*MZ2)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = (ALPHA*(-(C1(1))-C1(2)-C2(1,2)-C2(2,2)+S**(-1)-
     &  (C2(0,0)*2D0)/S))/(2D0*PI*(MZ2-S))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N3)+G(T1,N7)

        CALL C0123_(0D0,0D0,S,0D0,MZ,0D0,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(2,-1) = SW2**(-2)*(1D0-2D0*SW2)**2*(S-2D0*MZ2*SW2)
        DF(2,1) = -(8D0*MZ2*SW2)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = (ALPHA*(-(C1(1))-C1(2)-C2(1,2)-C2(2,2)+S**(-1)-
     &  (C2(0,0)*2D0)/S))/(8D0*CW2*PI*(MZ2-S))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N1)+G(T1,N4)

        CALL C0123_(0D0,0D0,S,MW,0D0,MW,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(2,-1) = 1D0
        DF(3,-1) = -(DF(2,-1))
        DF(4,-1) = -(DF(2,-1))

        PRE = -((ALPHA*(C1(1)+C1(2)+C2(1,2)+C2(2,2)+C0-1D0/S+(C2(0,
     &  0)*6D0)/S)*SW2**(-2)*(CW2*S-MZ2*SW2+S*SW2))/(2D0*PI*(MZ2-
     &  S)))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T2,N2)+G(T2,N3)

        K(1) = -(2D0*DSW*S)-CW*CW2*DZAZ*S-CW2*DZAA*MZ2*SW-2D0*CW2*
     &  DZE*MZ2*SW-2D0*CW2*DZEER*MZ2*SW+CW2*DZAA*S*SW-CW2*DZZZ*S*SW-
     &  CW*DZZA*MZ2*SW2+CW*DZZA*S*SW2
        K(2) = -(2D0*DSW*S)+2D0*CW2*DZE*S*SW+2D0*CW2*DZEEL*S*SW+CW2*
     &  DZZZ*S*SW+CW*DZZA*MZ2*SW2-2D0*CW*CW2*DZAZ*S*SW2-CW*DZZA*S*
     &  SW2-2D0*CW2*DZAA*MZ2*SW*SW2-4D0*CW2*DZE*MZ2*SW*SW2-4D0*CW2*
     &  DZEEL*MZ2*SW*SW2+2D0*CW2*DZAA*S*SW*SW2-2D0*CW2*DZZZ*S*SW*
     &  SW2-2D0*CW*DZZA*MZ2*SW2**2+2D0*CW*DZZA*S*SW2**2

        DF(2,-1) = K(2)/SW2
        DF(2,1) = K(1)*2D0
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = -(1D0/(2D0*CW2*S*(MZ2-S)*SW))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

	end

************************************************************************
        subroutine add_eeVvert_bgf(F)
************************************************************************
*	contributions of eeA and eeZ vertex corrections
*	background-field formalism
*-----------------------------------------------------------------------
*       27.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        COMPLEX*16 K(2)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc_bgf/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV30V30,   Model: SMbgf

C       G(T1,N5)

        CALL C0123_(0D0,0D0,S,0D0,MW,0D0,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(2,-1) = 1D0
        DF(3,-1) = -(DF(2,-1))
        DF(4,-1) = -(DF(2,-1))

        PRE = -((ALPHA*(1D0-C2(0,0)*2D0-C1(1)*S-C1(2)*S-C2(1,2)*S-
     &  C2(2,2)*S)*SW2**(-2))/(4D0*PI*(MZ2-S)))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE

C       G(T1,N2)+G(T1,N6)

        CALL C0123_(mxe2,mxe2,S,mxe,0D0,mxe,C0,C1,C2,C3,B0,B1,B2,2,301)

        DF(2,-1) = (S-2D0*MZ2*SW2)/SW2
        DF(2,1) = -(2D0*MZ2)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = (ALPHA*(-(C1(1))-C1(2)-C2(1,2)-C2(2,2)+S**(-1)-
     &  (C2(0,0)*2D0)/S))/(2D0*PI*(MZ2-S))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N3)+G(T1,N7)

        CALL C0123_(0D0,0D0,S,0D0,MZ,0D0,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(2,-1) = SW2**(-2)*(1D0-2D0*SW2)**2*(S-2D0*MZ2*SW2)
        DF(2,1) = -(8D0*MZ2*SW2)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = (ALPHA*(-(C1(1))-C1(2)-C2(1,2)-C2(2,2)+S**(-1)-
     &  (C2(0,0)*2D0)/S))/(8D0*CW2*PI*(MZ2-S))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N1)+G(T1,N4)

        CALL C0123_(0D0,0D0,S,MW,0D0,MW,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(2,-1) = 1D0
        DF(3,-1) = -(DF(2,-1))
        DF(4,-1) = -(DF(2,-1))

        PRE = -((ALPHA*(C1(1)*2D0+2D0*C0-1D0/S+(C2(0,0)*4D0)/S)*
     &  SW2**(-2)*(CW2*S-MZ2*SW2+S*SW2))/(4D0*PI*(MZ2-S)))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T2,N2)+G(T2,N3)

        K(1) = -(2D0*DSW*S)-CW*CW2*DZAZ*S-CW2*DZAA*MZ2*SW-2D0*CW2*
     &  DZE*MZ2*SW-2D0*CW2*DZEER*MZ2*SW+CW2*DZAA*S*SW-CW2*DZZZ*S*SW-
     &  CW*DZZA*MZ2*SW2+CW*DZZA*S*SW2
        K(2) = -(2D0*DSW*S)+2D0*CW2*DZE*S*SW+2D0*CW2*DZEEL*S*SW+CW2*
     &  DZZZ*S*SW+CW*DZZA*MZ2*SW2-2D0*CW*CW2*DZAZ*S*SW2-CW*DZZA*S*
     &  SW2-2D0*CW2*DZAA*MZ2*SW*SW2-4D0*CW2*DZE*MZ2*SW*SW2-4D0*CW2*
     &  DZEEL*MZ2*SW*SW2+2D0*CW2*DZAA*S*SW*SW2-2D0*CW2*DZZZ*S*SW*
     &  SW2-2D0*CW*DZZA*MZ2*SW2**2+2D0*CW*DZZA*S*SW2**2

        DF(2,-1) = K(2)/SW2
        DF(2,1) = K(1)*2D0
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = -(1D0/(2D0*CW2*S*(MZ2-S)*SW))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

	end

************************************************************************
        subroutine add_enWvert(F)
************************************************************************
*	contributions of enW vertex corrections
*	conventional formalism
*-----------------------------------------------------------------------
*       27.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	            dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV3V3,   Model: SM

C       G(T2,N7)

        CALL C0123_(mxe2,T,KM2,0D0,mxe,MW,C0,C1,C2,C3,B0,B1,B2,2,302)

        DF(1,-1) = (-(C2(0,0)*12D0)+2D0-C2(2,2)*2D0*KM2+C1(2)*(-
     &  (2D0*KM2)+T)-C1(1)*2D0*(KM2-T)-C2(1,2)*2D0*(KM2-T)-2D0*C0*
     &  (KM2-T))/T
        DF(5,-1) = -(2D0*(C1(1)-C1(2)-C2(1,2)*2D0+C0))

        PRE = (ALPHA)/(8D0*PI*SW2)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE

C       G(T2,N6)

        CALL C0123_(0D0,T,KM2,0d0,MZ,0D0,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = (1D0-C2(0,0)*2D0-C1(2)*KM2-C2(2,2)*KM2-C1(1)*
     &  (KM2-T)-C2(1,2)*(KM2-T))/T
        DF(5,-1) = -((C1(1)+C2(1,2))*2D0)

        PRE = -((ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(16D0*CW2*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE

C       G(T2,N5)

        CALL C0123_(0D0,T,KM2,MW,0D0,MZ,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = (-(C2(0,0)*12D0)+2D0-C2(2,2)*2D0*KM2+C1(2)*(-
     &  (2D0*KM2)+T)-C1(1)*2D0*(KM2-T)-C2(1,2)*2D0*(KM2-T)-2D0*C0*
     &  (KM2-T))/T
        DF(5,-1) = -(2D0*(C1(1)-C1(2)-C2(1,2)*2D0+C0))

        PRE = (ALPHA*SW2**(-2))/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE

C       G(T2,N8)

        CALL C0123_(0D0,T,KM2,MZ,0d0,MW,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = (-(C2(0,0)*12D0)+2D0-C2(2,2)*2D0*KM2-C1(1)*2D0*
     &  (KM2-T)-C2(1,2)*2D0*(KM2-T)-2D0*C0*(KM2-T)-C1(2)*(2D0*KM2-
     &  T))/T
        DF(5,-1) = -(2D0*(C1(1)-C1(2)-C2(1,2)*2D0+C0))

        PRE = (ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE

C       G(T1,N4)

        CALL C0123_(KP2,0D0,T,0D0,0d0,MZ,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = -(C2(2,2))+T**(-1)-(C2(0,0)*2D0)/T-(C1(1)*KP2)/
     &  T-(C1(2)*KP2)/T-(C2(1,1)*KP2)/T-(C2(1,2)*(KP2+T))/T

        DF(6,-1) = (C2(1,2)+C2(2,2))*2D0

        PRE = -((ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(16D0*CW2*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

C       G(T1,N2)

        CALL C0123_(KP2,mxe2,T,MW,0D0,mxe,C0,C1,C2,C3,B0,B1,B2,2,302)

        DF(1,-1) = C2(2,2)*2D0+(C2(0,0)*12D0)/T-2D0/T+(C2(1,1)*2D0*
     &  KP2)/T+(C2(1,2)*2D0*(KP2+T))/T-(C0*(-(2D0*KP2)+T))/T+(C1(1)*
     &  (2D0*KP2+T))/T+(C1(2)*(2D0*KP2+T))/T
        DF(6,-1) = 2D0*(C1(1)+C2(1,2)*2D0+C2(2,2)*2D0+C1(2)*4D0+2D0*
     &  C0)

        PRE = -((ALPHA)/(8D0*PI*SW2))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

C       G(T1,N3)

        CALL C0123_(KP2,0D0,T,MW,MZ,0d0,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = C2(2,2)*2D0+(C2(0,0)*12D0)/T-2D0/T+(C2(1,1)*2D0*
     &  KP2)/T+(C2(1,2)*2D0*(KP2+T))/T+(C1(1)*(2D0*KP2+T))/T+(C1(2)*
     &  (2D0*KP2+T))/T+(C0*(2D0*KP2-T))/T
        DF(6,-1) = 2D0*(C1(1)+C2(1,2)*2D0+C2(2,2)*2D0+C1(2)*4D0+2D0*
     &  C0)

        PRE = -((ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(16D0*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

C       G(T1,N1)

        CALL C0123_(KP2,0D0,T,MZ,MW,0D0,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = C2(2,2)*2D0+(C2(0,0)*12D0)/T-2D0/T+(C2(1,1)*2D0*
     &  KP2)/T+(C2(1,2)*2D0*(KP2+T))/T-(C0*(-(2D0*KP2)+T))/T+(C1(1)*
     &  (2D0*KP2+T))/T+(C1(2)*(2D0*KP2+T))/T
        DF(6,-1) = 2D0*(C1(1)+C2(1,2)*2D0+C2(2,2)*2D0+C1(2)*4D0+2D0*
     &  C0)

        PRE = -((ALPHA*SW2**(-2))/(16D0*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N1)


        DF(1,-1) = 1D0

        PRE = ((2D0*DSW-2D0*DZE*SW-DZEEL*SW-DZNNL*SW-DZW*SW))/(2D0*
     &  SW*SW2*T)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE

	end

************************************************************************
        subroutine add_enWvert_bgf(F)
************************************************************************
*	contributions of enW vertex corrections
*	background-field formalism
*-----------------------------------------------------------------------
*       27.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc_bgf/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV30V30,   Model: SMbgf

C       G(T2,N7)

        CALL C0123_(mxe2,T,KM2,0D0,mxe,MW,C0,C1,C2,C3,B0,B1,B2,2,302)

        DF(1,-1) = (1D0-C2(0,0)*4D0-C1(1)*2D0*(KM2-T)-2D0*C0*(KM2-
     &  T))/T
        DF(5,-1) = -(4D0*(C1(1)-C2(1,2)+C0))

        PRE = (ALPHA)/(8D0*PI*SW2)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE

C       G(T2,N6)

        CALL C0123_(0D0,T,KM2,0d0,MZ,0D0,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = (1D0-C2(0,0)*2D0-C1(2)*KM2-C2(2,2)*KM2-C1(1)*
     &  (KM2-T)-C2(1,2)*(KM2-T))/T
        DF(5,-1) = -((C1(1)+C2(1,2))*2D0)

        PRE = -((ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(16D0*CW2*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE

C       G(T2,N5)

        CALL C0123_(0D0,T,KM2,MW,0D0,MZ,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = (1D0-C2(0,0)*4D0-C1(1)*2D0*(KM2-T)-2D0*C0*(KM2-
     &  T))/T
        DF(5,-1) = -(4D0*(C1(1)-C2(1,2)+C0))

        PRE = (ALPHA*SW2**(-2))/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE

C       G(T2,N8)

        CALL C0123_(0D0,T,KM2,MZ,0d0,MW,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = (1D0-C2(0,0)*4D0-C1(1)*2D0*(KM2-T)-2D0*C0*(KM2-
     &  T))/T
        DF(5,-1) = -(4D0*(C1(1)-C2(1,2)+C0))

        PRE = (ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE

C       G(T1,N4)

        CALL C0123_(KP2,0D0,T,0D0,0d0,MZ,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = -(C2(2,2))+T**(-1)-(C2(0,0)*2D0)/T-(C1(1)*KP2)/
     &  T-(C1(2)*KP2)/T-(C2(1,1)*KP2)/T-(C2(1,2)*(KP2+T))/T
        DF(6,-1) = (C2(1,2)+C2(2,2))*2D0

        PRE = -((ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(16D0*CW2*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

C       G(T1,N2)

        CALL C0123_(KP2,mxe2,T,MW,0D0,mxe,C0,C1,C2,C3,B0,B1,B2,2,302)

        DF(1,-1) = (1D0-C2(0,0)*4D0-C1(2)*2D0*(KP2-T)-2D0*C0*(KP2-
     &  T))/T
        DF(6,-1) = -(4D0*(C2(1,2)+C2(2,2)+C1(2)*2D0+C0))

        PRE = (ALPHA)/(8D0*PI*SW2)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

C       G(T1,N3)

        CALL C0123_(KP2,0D0,T,MW,MZ,0d0,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = (1D0-C2(0,0)*4D0-C1(2)*2D0*(KP2-T)-2D0*C0*(KP2-
     &  T))/T
        DF(6,-1) = -(4D0*(C2(1,2)+C2(2,2)+C1(2)*2D0+C0))

        PRE = (ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

C       G(T1,N1)

        CALL C0123_(KP2,0D0,T,MZ,MW,0D0,C0,C1,C2,C3,B0,B1,B2,2,0)

        DF(1,-1) = (1D0-C2(0,0)*4D0-C1(2)*2D0*(KP2-T)-2D0*C0*(KP2-
     &  T))/T
        DF(6,-1) = -(4D0*(C2(1,2)+C2(2,2)+C1(2)*2D0+C0))

        PRE = (ALPHA*SW2**(-2))/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N1)


        DF(1,-1) = 1D0

        PRE = ((2D0*DSW-2D0*DZE*SW-DZEEL*SW-DZNNL*SW-DZW*SW))/(2D0*
     &  SW*SW2*T)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE

	end

************************************************************************
        subroutine add_box(F)
************************************************************************
*	contributions of box corrections
*	conventional formalism
*-----------------------------------------------------------------------
*       27.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        complex*16 D0,D1(3),D2(0:3,0:3),D3(0:3,0:3,3)
        complex*16 D4(0:3,0:3,0:3,0:3)
        COMPLEX*16 K(27)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N2)

        CALL C0123_(mxe2,mxe2,S,0D0,mxe,0D0,C0,C1,C2,C3,B0,B1,B2,1,304)

        DF(3,-1) = C1(2)
        DF(3,1) = DF(3,-1)
        DF(4,-1) = -(C1(1))-C1(2)-C0
        DF(4,1) = DF(4,-1)
        DF(5,-1) = C1(1)+C0
        DF(5,1) = DF(5,-1)
        DF(6,-1) = DF(5,-1)
        DF(6,1) = DF(5,-1)

        PRE = -((ALPHA)/(2D0*PI))

        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE

C       G(T1,N4)

        CALL C0123_(mxe2,mxe2,S,0D0,mxe,MZ,C0,C1,C2,C3,B0,B1,B2,1,303)

        DF(3,-1) = (C1(2)*(1D0-2D0*SW2))/SW2
        DF(3,1) = -(2D0*C1(2))
        DF(4,-1) = -(((C1(1)+C1(2)+C0)*(1D0-2D0*SW2))/SW2)
        DF(4,1) = 2D0*(C1(1)+C1(2)+C0)
        DF(5,-1) = ((C1(1)+C0)*(1D0-2D0*SW2))/SW2
        DF(5,1) = -(2D0*(C1(1)+C0))
        DF(6,-1) = DF(5,-1)
        DF(6,1) = DF(5,1)

        PRE = -((ALPHA)/(4D0*PI))

        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE

C       G(T1,N1)

        CALL C0123_(0D0,0D0,S,MW,0D0,MW,C0,C1,C2,C3,B0,B1,B2,1,0)

        DF(3,-1) = 2D0*C1(2)
        DF(4,-1) = -(2D0*(C1(1)+C1(2)+C0))
        DF(5,-1) = -(C1(1))-3D0*C1(2)-C0
        DF(6,-1) = 2D0*C1(1)+3D0*C1(2)+2D0*C0

        PRE = -((ALPHA*SW2**(-2))/(4D0*PI))

        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

C       G(T1,N3)

        CALL C0123_(mxe2,mxe2,S,MZ,mxe,0D0,C0,C1,C2,C3,B0,B1,B2,1,303)

        DF(3,-1) = (C1(2)*(1D0-2D0*SW2))/SW2
        DF(3,1) = -(2D0*C1(2))
        DF(4,-1) = -(((C1(1)+C1(2)+C0)*(1D0-2D0*SW2))/SW2)
        DF(4,1) = 2D0*(C1(1)+C1(2)+C0)
        DF(5,-1) = ((C1(1)+C0)*(1D0-2D0*SW2))/SW2
        DF(5,1) = -(2D0*(C1(1)+C0))
        DF(6,-1) = DF(5,-1)
        DF(6,1) = DF(5,1)

        PRE = -((ALPHA)/(4D0*PI))

        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE

C       G(T1,N5)

        CALL C0123_(0d0,0d0,S,MZ,0d0,MZ,C0,C1,C2,C3,B0,B1,B2,1,0)

        DF(3,-1) = C1(2)*SW2**(-2)*(1D0-2D0*SW2)**2
        DF(3,1) = 4D0*C1(2)
        DF(4,-1) = -((C1(1)+C1(2)+C0)*SW2**(-2)*(1D0-2D0*SW2)**2)
        DF(4,1) = -(4D0*(C1(1)+C1(2)+C0))
        DF(5,-1) = (C1(1)+C0)*SW2**(-2)*(1D0-2D0*SW2)**2
        DF(5,1) = 4D0*(C1(1)+C0)
        DF(6,-1) = DF(5,-1)
        DF(6,1) = DF(5,1)

        PRE = -((ALPHA)/(8D0*PI))

        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE

C       G(T2,N8)

        CALL D01234_(mxe2,KP2,mxe2,KM2,T,U,mxe,0D0,MW,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,401)

        K(1) = KM2-T
        K(2) = KP2-U

        DF(1,-1) = 6D0*D2(0,0)+4D0*D2(1,3)*K(1)-2D0*D1(1)*K(2)+3D0*
     &  D1(3)*KM2+3D0*D2(3,3)*KM2+2D0*D2(2,3)*(KM2+2D0*U)+D1(2)*
     &  (KM2-2D0*S+2D0*U)-D2(2,2)*(KM2-4D0*U)+D2(1,2)*(-(4D0*KP2)-S+
     &  4D0*U)
        DF(2,-1) = 2D0*(2D0*D2(0,0)-2D0*D3(0,0,2)-2D0*D3(0,0,3)+
     &  D2(1,3)*K(1)-D3(1,3,3)*K(1)+D3(1,2,2)*K(2)+D1(3)*KM2-D3(3,3,
     &  3)*KM2-D2(1,2)*(K(2)+S)+D1(2)*(KM2-2D0*S)+D3(1,2,3)*(-(2D0*
     &  KM2)+S+2D0*T)-D3(2,2,2)*U-D3(2,3,3)*(2D0*KM2+U)-D2(2,2)*
     &  (KM2-U)-D2(2,3)*(KM2-U)-D3(2,2,3)*(KM2+2D0*U))
        DF(3,-1) = 2D0*(-(4D0*D2(0,0))+10D0*D3(0,0,2)+D2(1,2)*K(1)-
     &  2D0*D2(1,3)*K(1)+D3(1,2,3)*K(1)-D3(1,2,2)*K(2)-2D0*D1(3)*
     &  KM2-2D0*D2(3,3)*KM2+D3(2,3,3)*KM2-2D0*D2(2,3)*T+D1(2)*(KM2-
     &  2D0*T)+D3(2,2,2)*U+D3(2,2,3)*(KM2+U)+D2(2,2)*(KM2-2D0*KP2+
     &  U))
        DF(4,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,1)+2D0*D3(0,0,2)+
     &  2D0*D3(0,0,3)+D2(1,1)*K(1)+3D0*D2(1,3)*K(1)+D3(1,1,3)*K(1)-
     &  D3(1,1,2)*K(2)+D1(1)*KM2+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,
     &  3)*KM2-D3(1,3,3)*(-(2D0*KM2)+T)+D1(2)*(KM2-2D0*KP2+2D0*T)-
     &  D2(1,2)*(-(3D0*KM2)+KP2+S+2D0*T)+D3(2,2,2)*U+D2(2,2)*(KM2+
     &  U)+D3(2,3,3)*(2D0*KM2+U)+D2(2,3)*(3D0*KM2+U)-D3(1,2,2)*(KP2-
     &  2D0*U)-D3(1,2,3)*(-(2D0*KM2)+KP2+T-2D0*U)+D3(2,2,3)*(KM2+
     &  2D0*U)))
        DF(5,-1) = 2D0*(2D0*D2(0,0)-10D0*D3(0,0,1)-10D0*D3(0,0,2)-
     &  D3(1,1,3)*K(1)+2D0*D1(1)*K(2)+2D0*D1(2)*K(2)+2D0*D2(1,1)*
     &  K(2)+5D0*D2(1,2)*K(2)+D3(1,1,2)*K(2)+D1(3)*KM2+D2(3,3)*KM2-
     &  D3(1,3,3)*KM2-D3(2,3,3)*KM2-D3(2,2,2)*U-D3(2,2,3)*(KM2+U)+
     &  2D0*D2(1,3)*(T-U)+2D0*D2(2,3)*(T-U)+D3(1,2,3)*(-(2D0*KM2)+T-
     &  U)+D3(1,2,2)*(KP2-2D0*U)+D2(2,2)*(2D0*KP2-3D0*U))
        DF(6,-1) = 2D0*(-(4D0*D2(0,0))+2D0*D3(0,0,1)+2D0*D3(0,0,2)+
     &  D2(1,1)*K(1)-2D0*D2(1,3)*K(1)+D3(1,1,3)*K(1)-D3(1,1,2)*K(2)+
     &  D1(1)*KM2-2D0*D1(3)*KM2-2D0*D2(3,3)*KM2+D3(1,3,3)*KM2+D3(2,
     &  3,3)*KM2-2D0*D2(2,3)*U+D3(2,2,2)*U+D3(2,2,3)*(KM2+U)+D3(1,2,
     &  3)*(2D0*KM2-T+U)+2D0*D1(2)*(KM2-U)+2D0*D2(2,2)*(KM2-U)+D2(1,
     &  2)*(KM2+2D0*S-U)-D3(1,2,2)*(KP2-2D0*U))
        DF(7,-1) = 8D0*(D1(2)+D2(1,2)-D3(1,2,2)-D3(1,2,3)-D3(2,2,2)-
     &  2D0*D3(2,2,3)-D3(2,3,3))
        DF(8,-1) = 8D0*(D1(2)+D2(1,3)+D2(2,3)-D3(1,1,2)-D3(1,1,3)-
     &  2D0*D3(1,2,2)-2D0*D3(1,2,3)-D3(2,2,2)-D3(2,2,3))
        DF(9,-1) = -(8D0*(D1(2)-D2(1,3)-D3(1,1,2)-D3(1,1,3)-2D0*
     &  D3(1,2,2)-3D0*D3(1,2,3)-D3(1,3,3)-D3(2,2,2)-2D0*D3(2,2,3)-
     &  D3(2,3,3)))
        DF(10,-1) = -(8D0*(D1(2)+D2(1,2)+D2(2,3)-D3(1,2,2)-D3(1,2,
     &  3)-D3(2,2,2)-D3(2,2,3)))

        PRE = (ALPHA)/(8D0*PI*SW2)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T2,N9)

        CALL D01234_(0d0,KP2,0d0,KM2,T,U,0d0,MZ,MW,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = KM2-T
        K(2) = KP2-U

        DF(1,-1) = 6D0*D2(0,0)+4D0*D2(1,3)*K(1)-2D0*D1(1)*K(2)+3D0*
     &  D1(3)*KM2+3D0*D2(3,3)*KM2+2D0*D2(2,3)*(KM2+2D0*U)+D1(2)*
     &  (KM2-2D0*S+2D0*U)-D2(2,2)*(KM2-4D0*U)-D2(1,2)*(4D0*KP2+S-
     &  4D0*U)
        DF(2,-1) = 2D0*(2D0*D2(0,0)-2D0*D3(0,0,2)-2D0*D3(0,0,3)+
     &  D2(1,3)*K(1)-D3(1,3,3)*K(1)+D3(1,2,2)*K(2)+D1(3)*KM2-D3(3,3,
     &  3)*KM2-D2(1,2)*(K(2)+S)+D1(2)*(KM2-2D0*S)-D3(2,2,2)*U-D3(2,
     &  3,3)*(2D0*KM2+U)-D2(2,2)*(KM2-U)-D2(2,3)*(KM2-U)-D3(2,2,3)*
     &  (KM2+2D0*U)-D3(1,2,3)*(-(2D0*KP2)+S+2D0*U))
        DF(3,-1) = 2D0*(-(4D0*D2(0,0))+10D0*D3(0,0,2)+D2(1,2)*K(1)-
     &  2D0*D2(1,3)*K(1)+D3(1,2,3)*K(1)-D3(1,2,2)*K(2)-2D0*D1(3)*
     &  KM2-2D0*D2(3,3)*KM2+D3(2,3,3)*KM2-2D0*D2(2,3)*T-D2(2,2)*(-
     &  (2D0*KM2)+KP2+S+T)+D1(2)*(KM2-2D0*T)+D3(2,2,2)*U+D3(2,2,3)*
     &  (KM2+U))
        DF(4,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,1)+2D0*D3(0,0,2)+
     &  2D0*D3(0,0,3)+D2(1,1)*K(1)+3D0*D2(1,3)*K(1)+D3(1,1,3)*K(1)-
     &  D3(1,1,2)*K(2)+D1(1)*KM2+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,
     &  3)*KM2+D3(1,3,3)*(2D0*KM2-T)+D1(2)*(KM2-2D0*KP2+2D0*T)+D3(2,
     &  2,2)*U+D2(2,2)*(KM2+U)+D3(2,3,3)*(2D0*KM2+U)+D2(2,3)*(3D0*
     &  KM2+U)-D3(1,2,2)*(KP2-2D0*U)+D3(2,2,3)*(KM2+2D0*U)+D2(1,2)*
     &  (KM2-3D0*KP2+S+2D0*U)+D3(1,2,3)*(KM2-2D0*KP2+S+3D0*U)))
        DF(5,-1) = 2D0*(2D0*D2(0,0)-10D0*D3(0,0,1)-10D0*D3(0,0,2)-
     &  D3(1,1,3)*K(1)+2D0*D1(1)*K(2)+2D0*D1(2)*K(2)+2D0*D2(1,1)*
     &  K(2)+5D0*D2(1,2)*K(2)+D3(1,1,2)*K(2)+D1(3)*KM2+D2(3,3)*KM2-
     &  D3(1,3,3)*KM2-D3(2,3,3)*KM2-D3(2,2,2)*U-D3(2,2,3)*(KM2+U)-
     &  D3(1,2,3)*(2D0*KM2-T+U)+2D0*D2(1,3)*(T-U)+2D0*D2(2,3)*(T-U)+
     &  D3(1,2,2)*(KP2-2D0*U)-D2(2,2)*(-(2D0*KP2)+3D0*U))
        DF(6,-1) = 2D0*(-(4D0*D2(0,0))+2D0*D3(0,0,1)+2D0*D3(0,0,2)+
     &  D2(1,1)*K(1)-2D0*D2(1,3)*K(1)+D3(1,1,3)*K(1)-D3(1,1,2)*K(2)+
     &  D1(1)*KM2-2D0*D1(3)*KM2-2D0*D2(3,3)*KM2+D3(1,3,3)*KM2+D3(2,
     &  3,3)*KM2-2D0*D2(2,3)*U+D3(2,2,2)*U+D3(2,2,3)*(KM2+U)+2D0*
     &  D1(2)*(KM2-U)+2D0*D2(2,2)*(KM2-U)+D2(1,2)*(KM2+2D0*S-U)-
     &  D3(1,2,3)*(-(2D0*KM2)+T-U)-D3(1,2,2)*(KP2-2D0*U))
        DF(7,-1) = 8D0*(D1(2)+D2(1,2)-D3(1,2,2)-D3(1,2,3)-D3(2,2,2)-
     &  2D0*D3(2,2,3)-D3(2,3,3))
        DF(8,-1) = 8D0*(D1(2)+D2(1,3)+D2(2,3)-D3(1,1,2)-D3(1,1,3)-
     &  2D0*D3(1,2,2)-2D0*D3(1,2,3)-D3(2,2,2)-D3(2,2,3))
        DF(9,-1) = -(8D0*(D1(2)-D2(1,3)-D3(1,1,2)-D3(1,1,3)-2D0*
     &  D3(1,2,2)-3D0*D3(1,2,3)-D3(1,3,3)-D3(2,2,2)-2D0*D3(2,2,3)-
     &  D3(2,3,3)))
        DF(10,-1) = -(8D0*(D1(2)+D2(1,2)+D2(2,3)-D3(1,2,2)-D3(1,2,
     &  3)-D3(2,2,2)-D3(2,2,3)))

        PRE = (ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T2,N6)

        CALL D01234_(mxe2,KP2,mxe2,KM2,T,U,MW,0D0,mxe,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,401)

        K(1) = KM2-T
        K(2) = KP2-U

        DF(1,-1) = 6D0*D2(0,0)+2D0*D1(1)*K(1)-4D0*D2(1,2)*K(2)+2D0*
     &  K(1)*D0+D2(3,3)*(4D0*KM2-KP2+S)+D2(1,3)*(4D0*KM2+S-4D0*T)+
     &  D1(3)*(5D0*KM2-3D0*T+U)-D2(2,2)*(KP2-4D0*U)+D2(2,3)*(4D0*
     &  KM2-2D0*KP2+S+4D0*U)+D1(2)*(-(3D0*KP2)+2D0*S+6D0*U)
        DF(2,-1) = 2D0*(4D0*D2(0,0)+2D0*D3(0,0,2)+2D0*D3(0,0,3)+2D0*
     &  D1(1)*K(1)+3D0*D2(1,3)*K(1)+D3(1,3,3)*K(1)-3D0*D2(1,2)*K(2)-
     &  D3(1,2,2)*K(2)+2D0*K(1)*D0+D3(3,3,3)*KM2+D2(3,3)*(4D0*KM2-
     &  T)+D1(3)*(5D0*KM2-3D0*T)+D3(2,2,2)*U+D3(2,3,3)*(2D0*KM2+U)+
     &  D3(2,2,3)*(KM2+2D0*U)+D3(1,2,3)*(-(2D0*KP2)+S+2D0*U)-D2(2,
     &  2)*(KP2-4D0*U)+D2(2,3)*(3D0*KM2-2D0*KP2+S+5D0*U)+D1(2)*(-
     &  (3D0*KP2)+2D0*S+5D0*U))
        DF(3,-1) = -(2D0*(-(2D0*D3(0,0,2))+2D0*D1(1)*K(1)+D2(1,3)*
     &  K(1)-D3(1,2,3)*K(1)+D2(1,2)*K(2)+D3(1,2,2)*K(2)+2D0*K(1)*D0+
     &  D2(3,3)*KM2-D3(2,3,3)*KM2-D1(2)*(K(2)-2D0*S)-D1(3)*(-(3D0*
     &  KM2)+T)-D3(2,2,2)*U-D3(2,2,3)*(KM2+U)+D2(2,3)*(T-U)+D2(2,2)*
     &  (KP2-2D0*U)))
        DF(4,-1) = -(2D0*(14D0*D2(0,0)+10D0*D3(0,0,1)+10D0*D3(0,0,
     &  2)+10D0*D3(0,0,3)+4D0*D1(1)*K(1)+2D0*D2(1,1)*K(1)+6D0*D2(1,
     &  3)*K(1)+D3(1,1,3)*K(1)-D3(1,1,2)*K(2)+2D0*K(1)*D0+D3(3,3,3)*
     &  KM2-D3(1,3,3)*(-(2D0*KM2)+T)-D2(3,3)*(-(4D0*KM2)+T)-2D0*
     &  D2(1,2)*(-(3D0*KM2)+KP2+S+2D0*T)-D1(3)*(-(5D0*KM2)+3D0*T)-
     &  D1(2)*(-(5D0*KM2)+S+3D0*T)+D3(2,2,2)*U+D3(2,3,3)*(2D0*KM2+
     &  U)-D3(1,2,2)*(KP2-2D0*U)-D2(2,2)*(-(2D0*KM2)+KP2-2D0*U)-
     &  D3(1,2,3)*(-(2D0*KM2)+KP2+T-2D0*U)-D2(2,3)*(-(6D0*KM2)+KP2+
     &  T-2D0*U)+D3(2,2,3)*(KM2+2D0*U)))
        DF(5,-1) = -(2D0*(6D0*D2(0,0)+2D0*D3(0,0,1)+2D0*D3(0,0,2)+
     &  D3(1,1,3)*K(1)-D2(1,1)*K(2)-6D0*D2(1,2)*K(2)-D3(1,1,2)*K(2)+
     &  D3(1,3,3)*KM2+D3(2,3,3)*KM2-D1(3)*(-(3D0*KM2)+2D0*T)-D2(1,
     &  3)*(-(4D0*KM2)-KP2+5D0*T)+D3(2,2,2)*U+D0*U+D3(2,2,3)*(KM2+
     &  U)-D2(3,3)*(-(3D0*KM2)+T+U)-D3(1,2,3)*(-(2D0*KM2)+T-U)-
     &  D1(1)*(KP2-2D0*U)-D3(1,2,2)*(KP2-2D0*U)-D2(2,3)*(-(4D0*KM2)+
     &  KP2+2D0*T-3D0*U)-D1(2)*(2D0*KP2-5D0*U)-D2(2,2)*(2D0*KP2-5D0*
     &  U)))
        DF(6,-1) = 2D0*(12D0*D2(0,0)+10D0*D3(0,0,1)+10D0*D3(0,0,2)+
     &  2D0*D1(1)*K(1)+2D0*D2(1,1)*K(1)+D2(1,3)*K(1)+D3(1,1,3)*K(1)-
     &  D3(1,1,2)*K(2)+2D0*D2(2,2)*KM2+D3(1,3,3)*KM2+D3(2,3,3)*KM2+
     &  D3(2,2,2)*U+D2(2,3)*(KM2+U)+D3(2,2,3)*(KM2+U)+2D0*D2(1,2)*
     &  (KM2-2D0*KP2+S+U)+D3(1,2,3)*(2D0*KM2-T+U)+D1(2)*(2D0*KM2-U)-
     &  D3(1,2,2)*(KP2-2D0*U))
        DF(7,-1) = 8D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+3D0*D2(2,
     &  3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3))
        DF(8,-1) = 8D0*(2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,
     &  2)+2D0*D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+2D0*D3(1,2,
     &  3)+D3(2,2,2)+D3(2,2,3))
        DF(9,-1) = -(8D0*(2D0*D1(2)+3D0*D2(1,2)+D2(1,3)+3D0*D2(2,2)+
     &  3D0*D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)))
        DF(10,-1) = -(8D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,3)))

        PRE = (ALPHA)/(8D0*PI*SW2)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T2,N7)

        CALL D01234_(0d0,KP2,0d0,KM2,T,U,MW,0D0,0d0,MZ,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = KM2-T
        K(2) = KP2-U

        DF(1,-1) = 6D0*D2(0,0)+2D0*D1(1)*K(1)-4D0*D2(1,2)*K(2)+2D0*
     &  K(1)*D0-D2(3,3)*(-(5D0*KM2)+T+U)-D1(3)*(-(5D0*KM2)+3D0*T-U)-
     &  D2(2,3)*(-(5D0*KM2)+KP2+T-3D0*U)-D2(2,2)*(KP2-4D0*U)-D2(1,
     &  3)*(4D0*KP2-5D0*S-4D0*U)-D1(2)*(3D0*KP2-2D0*S-6D0*U)
        DF(2,-1) = 2D0*(4D0*D2(0,0)+2D0*D3(0,0,2)+2D0*D3(0,0,3)+2D0*
     &  D1(1)*K(1)+3D0*D2(1,3)*K(1)+D3(1,3,3)*K(1)-3D0*D2(1,2)*K(2)-
     &  D3(1,2,2)*K(2)+2D0*K(1)*D0+D3(3,3,3)*KM2-D2(3,3)*(-(4D0*
     &  KM2)+T)-D3(1,2,3)*(-(2D0*KM2)+S+2D0*T)-D1(3)*(-(5D0*KM2)+
     &  3D0*T)+D3(2,2,2)*U+D3(2,3,3)*(2D0*KM2+U)+D3(2,2,3)*(KM2+2D0*
     &  U)-D2(2,2)*(KP2-4D0*U)-D2(2,3)*(-(4D0*KM2)+KP2+T-4D0*U)-
     &  D1(2)*(3D0*KP2-2D0*S-5D0*U))
        DF(3,-1) = -(2D0*(-(2D0*D3(0,0,2))+2D0*D1(1)*K(1)+D2(1,3)*
     &  K(1)-D3(1,2,3)*K(1)+D2(1,2)*K(2)+D3(1,2,2)*K(2)+2D0*K(1)*D0+
     &  D2(3,3)*KM2-D3(2,3,3)*KM2+D1(2)*(K(1)+S)+D1(3)*(3D0*KM2-T)-
     &  D3(2,2,2)*U-D3(2,2,3)*(KM2+U)+D2(2,3)*(T-U)+D2(2,2)*(KP2-
     &  2D0*U)))
        DF(4,-1) = -(2D0*(14D0*D2(0,0)+10D0*D3(0,0,1)+10D0*D3(0,0,
     &  2)+10D0*D3(0,0,3)+4D0*D1(1)*K(1)+2D0*D2(1,1)*K(1)+6D0*D2(1,
     &  3)*K(1)+D3(1,1,3)*K(1)-D3(1,1,2)*K(2)+2D0*K(1)*D0+D3(3,3,3)*
     &  KM2+D3(1,3,3)*(2D0*KM2-T)+D2(3,3)*(4D0*KM2-T)+D1(3)*(5D0*
     &  KM2-3D0*T)+D1(2)*(5D0*KM2-S-3D0*T)+D3(2,2,2)*U+D3(2,3,3)*
     &  (2D0*KM2+U)-D3(1,2,2)*(KP2-2D0*U)+D3(2,2,3)*(KM2+2D0*U)+
     &  D2(2,2)*(2D0*KM2-KP2+2D0*U)+2D0*D2(1,2)*(KM2-3D0*KP2+S+2D0*
     &  U)+D3(1,2,3)*(KM2-2D0*KP2+S+3D0*U)+D2(2,3)*(5D0*KM2-2D0*KP2+
     &  S+3D0*U)))
        DF(5,-1) = -(2D0*(6D0*D2(0,0)+2D0*D3(0,0,1)+2D0*D3(0,0,2)+
     &  D3(1,1,3)*K(1)-D2(1,1)*K(2)-6D0*D2(1,2)*K(2)-D3(1,1,2)*K(2)+
     &  D3(1,3,3)*KM2+D3(2,3,3)*KM2+D2(3,3)*(2D0*KM2-KP2+S)+D1(3)*
     &  (3D0*KM2-2D0*T)+D2(1,3)*(4D0*KM2+KP2-5D0*T)+D3(2,2,2)*U+D0*
     &  U+D3(2,2,3)*(KM2+U)+D3(1,2,3)*(2D0*KM2-T+U)-D1(1)*(KP2-2D0*
     &  U)-D3(1,2,2)*(KP2-2D0*U)+D2(2,3)*(4D0*KM2-KP2-2D0*T+3D0*U)+
     &  D1(2)*(-(2D0*KP2)+5D0*U)+D2(2,2)*(-(2D0*KP2)+5D0*U)))
        DF(6,-1) = 2D0*(12D0*D2(0,0)+10D0*D3(0,0,1)+10D0*D3(0,0,2)+
     &  2D0*D1(1)*K(1)+2D0*D2(1,1)*K(1)+D2(1,3)*K(1)+D3(1,1,3)*K(1)-
     &  D3(1,1,2)*K(2)+2D0*D2(2,2)*KM2+D3(1,3,3)*KM2+D3(2,3,3)*KM2-
     &  2D0*D2(1,2)*(-(2D0*KM2)+KP2+T)+D3(2,2,2)*U+D2(2,3)*(KM2+U)+
     &  D3(2,2,3)*(KM2+U)-D1(2)*(-(2D0*KM2)+U)-D3(1,2,3)*(-(2D0*
     &  KM2)+T-U)-D3(1,2,2)*(KP2-2D0*U))
        DF(7,-1) = 8D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+3D0*D2(2,
     &  3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3))
        DF(8,-1) = 8D0*(2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,
     &  2)+2D0*D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+2D0*D3(1,2,
     &  3)+D3(2,2,2)+D3(2,2,3))
        DF(9,-1) = -(8D0*(2D0*D1(2)+3D0*D2(1,2)+D2(1,3)+3D0*D2(2,2)+
     &  3D0*D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)))
        DF(10,-1) = -(8D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,3)))

        PRE = (ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T3,N10)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,T,0D0,mxe,0D0,mxe,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,402)

        K(1) = T+U

        DF(1,-1) = 4D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*D3(0,
     &  0,3)+2D0*D2(1,3)*K(1)+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,3)*
     &  KM2+D3(1,3,3)*(K(1)+KM2)+D1(1)*KP2+2D0*D2(1,1)*KP2+D3(1,1,
     &  1)*KP2+D3(1,1,3)*(K(1)+KP2)+2D0*D2(2,2)*T+D3(2,2,2)*T+2D0*
     &  D2(2,3)*(KM2+T)+D3(2,3,3)*(2D0*KM2+T)+2D0*D2(1,2)*(KP2+T)+
     &  D3(1,1,2)*(2D0*KP2+T)-D1(2)*(S-T)+D3(2,2,3)*(KM2+2D0*T)+
     &  D3(1,2,2)*(KP2+2D0*T)+D3(1,2,3)*(KM2+KP2+3D0*T+U)
        DF(2,-1) = 2D0*(4D0*D2(0,0)+4D0*D3(0,0,1)+4D0*D3(0,0,2)+4D0*
     &  D3(0,0,3)+2D0*D2(1,3)*K(1)+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,
     &  3)*KM2+D3(1,3,3)*(K(1)+KM2)+D1(1)*KP2+2D0*D2(1,1)*KP2+D3(1,
     &  1,1)*KP2+D3(1,1,3)*(K(1)+KP2)+D1(2)*T+2D0*D2(2,2)*T+D3(2,2,
     &  2)*T+2D0*D2(2,3)*(KM2+T)+D3(2,3,3)*(2D0*KM2+T)+2D0*D2(1,2)*
     &  (KP2+T)+D3(1,1,2)*(2D0*KP2+T)+D3(2,2,3)*(KM2+2D0*T)+D3(1,2,
     &  2)*(KP2+2D0*T)+D3(1,2,3)*(KM2+KP2+3D0*T+U))
        DF(3,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,1)+D1(3)*KM2+D2(3,
     &  3)*KM2+D1(2)*T+D2(1,2)*T+D2(1,3)*T+D2(2,2)*T+D2(2,3)*(KM2+
     &  T)))
        DF(4,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,3)+D1(1)*KP2+D2(1,
     &  1)*KP2+D1(2)*T+D2(1,3)*T+D2(2,2)*T+D2(2,3)*T+D2(1,2)*(KP2+
     &  T)))
        DF(5,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,2)+6D0*D3(0,0,3)+
     &  D2(1,3)*K(1)+D3(1,3,3)*K(1)+D2(3,3)*KM2+D3(3,3,3)*KM2+D1(1)*
     &  KP2+D2(1,1)*KP2+D3(1,1,3)*KP2+D2(1,2)*T+D3(2,2,3)*T+D3(2,3,
     &  3)*(KM2+T)+D3(1,2,3)*(KP2+T)-D1(2)*(KP2-T)-D2(2,2)*(KP2-T)+
     &  D2(2,3)*(KM2+T-U)))
        DF(6,-1) = 2D0*(-(2D0*D2(0,0))-6D0*D3(0,0,1)-2D0*D3(0,0,2)-
     &  D2(1,3)*K(1)-D3(1,1,3)*K(1)-D1(3)*KM2-D2(3,3)*KM2-D3(1,3,3)*
     &  KM2-D2(1,1)*KP2-D3(1,1,1)*KP2-D2(2,3)*T-D3(1,2,2)*T-D3(1,2,
     &  3)*(KM2+T)-D3(1,1,2)*(KP2+T)+D1(2)*(KM2-T)+D2(2,2)*(KM2-T)-
     &  D2(1,2)*(KP2+T-U))
        DF(7,-1) = 4D0*(2D0*D2(1,3)+D3(1,1,3)+D3(1,2,3)+D3(1,3,3))
        DF(8,-1) = -(4D0*(D1(2)+D2(1,2)+D2(2,3)-D3(1,2,2)-D3(2,2,2)-
     &  D3(2,2,3)))
        DF(9,-1) = -(4D0*(D2(1,3)-D2(2,3)-D3(1,2,3)-D3(2,2,3)-D3(2,
     &  3,3)))
        DF(10,-1) = 4D0*(D2(1,2)-D2(1,3)+D3(1,1,2)+D3(1,2,2)+D3(1,2,
     &  3))

        PRE = (ALPHA)/(4D0*PI*SW2)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T3,N11)

        CALL D01234_(KP2,0d0,0d0,KM2,S,T,0D0,0d0,MZ,0d0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = T+U

        DF(1,-1) = 4D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*D3(0,
     &  0,3)+2D0*D2(1,3)*K(1)+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,3)*
     &  KM2+D3(1,3,3)*(K(1)+KM2)+D1(1)*KP2+2D0*D2(1,1)*KP2+D3(1,1,
     &  1)*KP2+D3(1,1,3)*(K(1)+KP2)+2D0*D2(2,2)*T+D3(2,2,2)*T+2D0*
     &  D2(2,3)*(KM2+T)+D3(2,3,3)*(2D0*KM2+T)+2D0*D2(1,2)*(KP2+T)+
     &  D3(1,1,2)*(2D0*KP2+T)-D1(2)*(S-T)+D3(2,2,3)*(KM2+2D0*T)+
     &  D3(1,2,2)*(KP2+2D0*T)+D3(1,2,3)*(KM2+KP2+3D0*T+U)
        DF(2,-1) = 2D0*(4D0*D2(0,0)+4D0*D3(0,0,1)+4D0*D3(0,0,2)+4D0*
     &  D3(0,0,3)+2D0*D2(1,3)*K(1)+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,
     &  3)*KM2+D3(1,3,3)*(K(1)+KM2)+D1(1)*KP2+2D0*D2(1,1)*KP2+D3(1,
     &  1,1)*KP2+D3(1,1,3)*(K(1)+KP2)+D1(2)*T+2D0*D2(2,2)*T+D3(2,2,
     &  2)*T+2D0*D2(2,3)*(KM2+T)+D3(2,3,3)*(2D0*KM2+T)+2D0*D2(1,2)*
     &  (KP2+T)+D3(1,1,2)*(2D0*KP2+T)+D3(2,2,3)*(KM2+2D0*T)+D3(1,2,
     &  2)*(KP2+2D0*T)+D3(1,2,3)*(KM2+KP2+3D0*T+U))
        DF(3,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,1)+D1(3)*KM2+D2(3,
     &  3)*KM2+D1(2)*T+D2(1,2)*T+D2(1,3)*T+D2(2,2)*T+D2(2,3)*(KM2+
     &  T)))
        DF(4,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,3)+D1(1)*KP2+D2(1,
     &  1)*KP2+D1(2)*T+D2(1,3)*T+D2(2,2)*T+D2(2,3)*T+D2(1,2)*(KP2+
     &  T)))
        DF(5,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,2)+6D0*D3(0,0,3)+
     &  D2(1,3)*K(1)+D3(1,3,3)*K(1)+D2(3,3)*KM2+D3(3,3,3)*KM2+D1(1)*
     &  KP2+D2(1,1)*KP2+D3(1,1,3)*KP2+D2(1,2)*T+D3(2,2,3)*T+D3(2,3,
     &  3)*(KM2+T)+D3(1,2,3)*(KP2+T)-D1(2)*(KP2-T)-D2(2,2)*(KP2-T)+
     &  D2(2,3)*(KM2+T-U)))
        DF(6,-1) = 2D0*(-(2D0*D2(0,0))-6D0*D3(0,0,1)-2D0*D3(0,0,2)-
     &  D2(1,3)*K(1)-D3(1,1,3)*K(1)-D1(3)*KM2-D2(3,3)*KM2-D3(1,3,3)*
     &  KM2-D2(1,1)*KP2-D3(1,1,1)*KP2-D2(2,3)*T-D3(1,2,2)*T-D3(1,2,
     &  3)*(KM2+T)-D3(1,1,2)*(KP2+T)+D1(2)*(KM2-T)+D2(2,2)*(KM2-T)-
     &  D2(1,2)*(KP2+T-U))
        DF(7,-1) = 4D0*(2D0*D2(1,3)+D3(1,1,3)+D3(1,2,3)+D3(1,3,3))
        DF(8,-1) = -(4D0*(D1(2)+D2(1,2)+D2(2,3)-D3(1,2,2)-D3(2,2,2)-
     &  D3(2,2,3)))
        DF(9,-1) = -(4D0*(D2(1,3)-D2(2,3)-D3(1,2,3)-D3(2,2,3)-D3(2,
     &  3,3)))
        DF(10,-1) = 4D0*(D2(1,2)-D2(1,3)+D3(1,1,2)+D3(1,2,2)+D3(1,2,
     &  3))

        PRE = (ALPHA*SW2**(-2)*(1D0-2D0*SW2)**2)/(16D0*CW2*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T3,N21)

        CALL D01234_(KP2,0D0,0D0,KM2,S,T,0D0,MW,0D0,MW,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = T+U

        DF(1,-1) = 24D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*
     &  D3(0,0,3)+2D0*K(1)*D0+D3(3,3,3)*KM2+D3(1,3,3)*(K(1)+KM2)+
     &  D2(3,3)*(K(1)+3D0*KM2)+D3(1,1,1)*KP2+D3(1,1,3)*(K(1)+KP2)+
     &  D2(1,1)*(K(1)+3D0*KP2)+D2(1,3)*(5D0*KM2+5D0*KP2-3D0*S)+D3(2,
     &  2,2)*T+D3(2,3,3)*(2D0*KM2+T)+D3(1,1,2)*(2D0*KP2+T)+D3(2,2,
     &  3)*(KM2+2D0*T)+D3(1,2,2)*(KP2+2D0*T)+D2(2,2)*(KM2+KP2+3D0*
     &  T)+D3(1,2,3)*(KM2+KP2+3D0*T+U)+D2(2,3)*(4D0*KM2+KP2+4D0*T+
     &  U)+D2(1,2)*(KM2+4D0*KP2+4D0*T+U)+D1(2)*(KM2+KP2+4D0*T+2D0*
     &  U)+D1(3)*(2D0*KM2+3D0*T+3D0*U)+D1(1)*(2D0*KP2+3D0*T+3D0*U)
        DF(2,-1) = -(2D0*D2(0,0))-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*
     &  D3(0,0,3)-2D0*D2(2,3)*KM2+D3(3,3,3)*KM2+D3(1,3,3)*(K(1)+
     &  KM2)-2D0*D2(1,2)*KP2+D3(1,1,1)*KP2+D3(1,1,3)*(K(1)+KP2)-
     &  D1(2)*T-D2(2,2)*T+D3(2,2,2)*T+D0*T+D3(2,3,3)*(2D0*KM2+T)+
     &  D3(1,1,2)*(2D0*KP2+T)-D2(3,3)*(2D0*KM2-T)-D2(1,1)*(2D0*KP2-
     &  T)+D1(3)*(KM2-2D0*T)+D1(1)*(KP2-2D0*T)+D3(2,2,3)*(KM2+2D0*
     &  T)+D3(1,2,2)*(KP2+2D0*T)+D3(1,2,3)*(KM2+KP2+3D0*T+U)+D2(1,
     &  3)*(S-2D0*U)
        DF(3,-1) = -(26D0*D3(0,0,1))-3D0*D3(1,1,3)*K(1)+2D0*D2(3,3)*
     &  KM2-3D0*D3(1,3,3)*KM2-2D0*D2(1,2)*KP2-3D0*D3(1,1,1)*KP2+2D0*
     &  D2(2,2)*T-3D0*D3(1,2,2)*T-2D0*D0*T+2D0*D2(2,3)*(KM2+T)-3D0*
     &  D3(1,2,3)*(KM2+T)-3D0*D3(1,1,2)*(KP2+T)-D1(1)*(2D0*KP2+T)-
     &  2D0*D1(3)*(KM2-T)-D2(1,1)*(KP2+3D0*T)-D2(1,3)*(3D0*KM2-3D0*
     &  T+2D0*U)
        DF(4,-1) = -(26D0*D3(0,0,3))-3D0*D3(1,3,3)*K(1)-2D0*D2(2,3)*
     &  KM2-3D0*D3(3,3,3)*KM2+2D0*D2(1,1)*KP2-3D0*D3(1,1,3)*KP2+2D0*
     &  D2(2,2)*T-3D0*D3(2,2,3)*T-2D0*D0*T-3D0*D3(2,3,3)*(KM2+T)-
     &  D1(3)*(2D0*KM2+T)+2D0*D2(1,2)*(KP2+T)-3D0*D3(1,2,3)*(KP2+T)-
     &  2D0*D1(1)*(KP2-T)-D2(3,3)*(KM2+3D0*T)-D2(1,3)*(3D0*KP2-3D0*
     &  T+2D0*U)
        DF(5,-1) = -(26D0*D2(0,0))-26D0*D3(0,0,2)-12D0*D3(0,0,3)-
     &  4D0*D1(3)*K(1)-2D0*D3(1,3,3)*K(1)-2D0*D3(3,3,3)*KM2-2D0*
     &  D2(3,3)*(K(1)+2D0*KM2)-3D0*D3(1,1,2)*KP2-2D0*D3(1,1,3)*KP2-
     &  2D0*D2(1,3)*(K(1)+2D0*KP2)-3D0*D3(2,2,2)*T-3D0*D2(1,2)*(KP2+
     &  T)-3D0*D3(1,2,2)*(KP2+T)-2D0*D2(2,2)*(2D0*KP2+T)-D1(2)*(4D0*
     &  KP2-T)-D3(2,3,3)*(5D0*KM2+2D0*T)-D3(2,2,3)*(3D0*KM2+5D0*T)-
     &  D2(2,3)*(8D0*KM2+5D0*KP2-3D0*S+U)-D3(1,2,3)*(2D0*KP2+5D0*T+
     &  3D0*U)
        DF(6,-1) = -(26D0*D2(0,0))-12D0*D3(0,0,1)-26D0*D3(0,0,2)-
     &  4D0*D1(1)*K(1)-2D0*D3(1,1,3)*K(1)-2D0*D3(1,3,3)*KM2-3D0*
     &  D3(2,3,3)*KM2-2D0*D2(1,3)*(K(1)+2D0*KM2)-2D0*D3(1,1,1)*KP2-
     &  2D0*D2(1,1)*(K(1)+2D0*KP2)-3D0*D3(2,2,2)*T-3D0*D2(2,3)*(KM2+
     &  T)-3D0*D3(2,2,3)*(KM2+T)-2D0*D2(2,2)*(2D0*KM2+T)-D1(2)*(4D0*
     &  KM2-T)-D3(1,1,2)*(5D0*KP2+2D0*T)-D3(1,2,2)*(3D0*KP2+5D0*T)-
     &  D2(1,2)*(5D0*KM2+8D0*KP2-3D0*S+U)-D3(1,2,3)*(2D0*KM2+5D0*T+
     &  3D0*U)
        DF(7,-1) = -(8D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3)))
        DF(8,-1) = 8D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,
     &  3)+D3(1,2,2)+D3(2,2,2)+D3(2,2,3))
        DF(9,-1) = 8D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3))
        DF(10,-1) = 8D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))

        PRE = -((ALPHA)/(8D0*PI*SW2))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T3,N20)

        CALL D01234_(KP2,0D0,0D0,KM2,S,T,MH,MW,0D0,MW,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,1,0)

        DF(1,-1) = D1(1)+D1(2)+D1(3)+D0
        DF(5,-1) = -(2D0*D1(3))
        DF(6,-1) = -(2D0*D1(1))

        PRE = (ALPHA*MW2*SW2**(-2))/(8D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

C       G(T3,N12)+G(T3,N13)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,T,MW,0D0,mxe,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,403)

        K(1) = KM2+T
        K(2) = 2D0*KM2+T
        K(3) = KP2+T
        K(4) = 2D0*KP2+T
        K(5) = T+U
        K(6) = KM2+T+U
        K(7) = 2D0*KM2+T+U
        K(8) = KP2+T+U
        K(9) = 2D0*KP2+T+U
        K(10) = MW2-2D0*T-2D0*U
        K(11) = KM2+KP2+3D0*T+U
        K(12) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(13) = 26D0*D3(0,0,3)+2D0*D2(2,3)*KM2+3D0*D3(3,3,3)*KM2-
     &  2D0*D2(1,1)*KP2+3D0*D3(1,1,3)*KP2-2D0*D2(2,2)*T+3D0*D3(2,2,
     &  3)*T+2D0*D0*T+3D0*D3(2,3,3)*(KM2+T)+D1(3)*(2D0*KM2+T)-2D0*
     &  D2(1,2)*(KP2+T)+3D0*D3(1,2,3)*(KP2+T)+2D0*D1(1)*(KP2-T)+
     &  D2(3,3)*(KM2+3D0*T)+3D0*D3(1,3,3)*(T+U)+D2(1,3)*(3D0*KP2-
     &  3D0*T+2D0*U)
        K(14) = 26D0*D3(0,0,1)-2D0*D2(3,3)*KM2+3D0*D3(1,3,3)*KM2+
     &  2D0*D2(1,2)*KP2+3D0*D3(1,1,1)*KP2-2D0*D2(2,2)*T+3D0*D3(1,2,
     &  2)*T+2D0*D0*T-2D0*D2(2,3)*(KM2+T)+3D0*D3(1,2,3)*(KM2+T)+3D0*
     &  D3(1,1,2)*(KP2+T)+D1(1)*(2D0*KP2+T)+2D0*D1(3)*(KM2-T)+D2(1,
     &  1)*(KP2+3D0*T)+3D0*D3(1,1,3)*(T+U)+D2(1,3)*(3D0*KM2-3D0*T+
     &  2D0*U)
        K(15) = 26D0*D2(0,0)+26D0*D3(0,0,2)+12D0*D3(0,0,3)+2D0*D3(3,
     &  3,3)*KM2+3D0*D3(1,1,2)*KP2+2D0*D3(1,1,3)*KP2+3D0*D3(2,2,2)*
     &  T+3D0*D2(1,2)*(KP2+T)+3D0*D3(1,2,2)*(KP2+T)+2D0*D2(2,2)*
     &  (2D0*KP2+T)+D1(2)*(4D0*KP2-T)+D3(2,3,3)*(5D0*KM2+2D0*T)+
     &  D3(2,2,3)*(3D0*KM2+5D0*T)+D2(2,3)*(8D0*KM2+5D0*KP2-3D0*S+U)+
     &  2D0*D3(1,3,3)*(T+U)+2D0*D2(3,3)*(2D0*KM2+T+U)+2D0*D2(1,3)*
     &  (2D0*KP2+T+U)-2D0*D1(3)*(MW2-2D0*T-2D0*U)+D3(1,2,3)*(2D0*
     &  KP2+5D0*T+3D0*U)
        K(16) = 26D0*D2(0,0)+12D0*D3(0,0,1)+26D0*D3(0,0,2)+2D0*D3(1,
     &  3,3)*KM2+3D0*D3(2,3,3)*KM2+2D0*D3(1,1,1)*KP2+3D0*D3(2,2,2)*
     &  T+3D0*D2(2,3)*(KM2+T)+3D0*D3(2,2,3)*(KM2+T)+2D0*D2(2,2)*
     &  (2D0*KM2+T)+D1(2)*(4D0*KM2-T)+D3(1,1,2)*(5D0*KP2+2D0*T)+
     &  D3(1,2,2)*(3D0*KP2+5D0*T)+D2(1,2)*(5D0*KM2+8D0*KP2-3D0*S+U)+
     &  2D0*D3(1,1,3)*(T+U)+2D0*D2(1,3)*(2D0*KM2+T+U)+2D0*D2(1,1)*
     &  (2D0*KP2+T+U)-2D0*D1(1)*(MW2-2D0*T-2D0*U)+D3(1,2,3)*(2D0*
     &  KM2+5D0*T+3D0*U)
        K(17) = -(2D0*D2(0,0))-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,
     &  0,3)-2D0*D2(2,3)*KM2+D3(3,3,3)*KM2-2D0*D2(1,2)*KP2+D3(1,1,
     &  1)*KP2-D1(2)*T-D2(2,2)*T+D3(2,2,2)*T+D0*T+D3(2,3,3)*(2D0*
     &  KM2+T)+D3(1,1,2)*(2D0*KP2+T)-D2(3,3)*(2D0*KM2-T)-D2(1,1)*
     &  (2D0*KP2-T)+D1(3)*(KM2-2D0*T)+D1(1)*(KP2-2D0*T)+D3(2,2,3)*
     &  (KM2+2D0*T)+D3(1,2,2)*(KP2+2D0*T)+D3(1,3,3)*(KM2+T+U)+D3(1,
     &  1,3)*(KP2+T+U)+D3(1,2,3)*(KM2+KP2+3D0*T+U)+D2(1,3)*(S-2D0*U)
        K(18) = -(24D0*D2(0,0))-6D0*D3(0,0,1)-6D0*D3(0,0,2)-6D0*
     &  D3(0,0,3)-D3(3,3,3)*KM2-D3(1,1,1)*KP2-D2(1,3)*(5D0*KM2+5D0*
     &  KP2-3D0*S)-D3(2,2,2)*T-D3(2,3,3)*(2D0*KM2+T)-D3(1,1,2)*(2D0*
     &  KP2+T)-D3(2,2,3)*(KM2+2D0*T)-D3(1,2,2)*(KP2+2D0*T)-D2(2,2)*
     &  (KM2+KP2+3D0*T)-D3(1,3,3)*(KM2+T+U)-D2(3,3)*(3D0*KM2+T+U)-
     &  D3(1,1,3)*(KP2+T+U)-D2(1,1)*(3D0*KP2+T+U)-D3(1,2,3)*(KM2+
     &  KP2+3D0*T+U)-D2(2,3)*(4D0*KM2+KP2+4D0*T+U)-D2(1,2)*(KM2+4D0*
     &  KP2+4D0*T+U)+D0*(MW2-2D0*T-2D0*U)-D1(2)*(KM2+KP2-MW2+4D0*T+
     &  2D0*U)+D1(3)*(-(2D0*KM2)+MW2-3D0*T-3D0*U)+D1(1)*(-(2D0*KP2)+
     &  MW2-3D0*T-3D0*U)

        DF(1,-1) = K(18)
        DF(1,1) = DF(1,-1)
        DF(2,-1) = -(K(17))
        DF(2,1) = DF(2,-1)
        DF(3,-1) = K(14)
        DF(3,1) = DF(3,-1)
        DF(4,-1) = K(13)
        DF(4,1) = DF(4,-1)
        DF(5,-1) = K(15)
        DF(5,1) = DF(5,-1)
        DF(6,-1) = K(16)
        DF(6,1) = DF(6,-1)
        DF(7,-1) = 8D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))
        DF(7,1) = DF(7,-1)
        DF(8,-1) = -(8D0*K(12))
        DF(8,1) = DF(8,-1)
        DF(9,-1) = -(8D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3)))
        DF(9,1) = DF(9,-1)
        DF(10,-1) = -(8D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3)))
        DF(10,1) = DF(10,-1)

        PRE = (ALPHA)/(4D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T3,N14)+G(T3,N15)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,T,MW,0D0,mxe,MZ,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,404)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2+T
        K(3) = 2D0*KM2+T
        K(4) = KP2+T
        K(5) = 2D0*KP2+T
        K(6) = T+U
        K(7) = KM2+T+U
        K(8) = 2D0*KM2+T+U
        K(9) = KP2+T+U
        K(10) = 2D0*KP2+T+U
        K(11) = MW2*SW2+2D0*CW2*T+2D0*CW2*U
        K(12) = KM2+KP2+3D0*T+U
        K(13) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(14) = 26D0*D3(0,0,3)+2D0*D2(2,3)*KM2+3D0*D3(3,3,3)*KM2-
     &  2D0*D2(1,1)*KP2+3D0*D3(1,1,3)*KP2-2D0*D2(2,2)*T+3D0*D3(2,2,
     &  3)*T+2D0*D0*T+3D0*D3(2,3,3)*(KM2+T)+D1(3)*(2D0*KM2+T)-2D0*
     &  D2(1,2)*(KP2+T)+3D0*D3(1,2,3)*(KP2+T)+2D0*D1(1)*(KP2-T)+
     &  D2(3,3)*(KM2+3D0*T)+3D0*D3(1,3,3)*(T+U)-D2(1,3)*(-(3D0*KP2)+
     &  3D0*T-2D0*U)
        K(15) = 26D0*D3(0,0,1)-2D0*D2(3,3)*KM2+3D0*D3(1,3,3)*KM2+
     &  2D0*D2(1,2)*KP2+3D0*D3(1,1,1)*KP2-2D0*D2(2,2)*T+3D0*D3(1,2,
     &  2)*T+2D0*D0*T-2D0*D2(2,3)*(KM2+T)+3D0*D3(1,2,3)*(KM2+T)+3D0*
     &  D3(1,1,2)*(KP2+T)+D1(1)*(2D0*KP2+T)+2D0*D1(3)*(KM2-T)+D2(1,
     &  1)*(KP2+3D0*T)+3D0*D3(1,1,3)*(T+U)-D2(1,3)*(-(3D0*KM2)+3D0*
     &  T-2D0*U)
        K(16) = -(26D0*D2(0,0))-26D0*D3(0,0,2)-12D0*D3(0,0,3)-2D0*
     &  D3(3,3,3)*KM2-3D0*D3(1,1,2)*KP2-2D0*D3(1,1,3)*KP2-3D0*D3(2,
     &  2,2)*T-3D0*D2(1,2)*(KP2+T)-3D0*D3(1,2,2)*(KP2+T)-2D0*D2(2,
     &  2)*(2D0*KP2+T)+D1(2)*(-(4D0*KP2)+T)-D3(2,3,3)*(5D0*KM2+2D0*
     &  T)-D3(2,2,3)*(3D0*KM2+5D0*T)-D2(2,3)*(8D0*KM2+5D0*KP2-3D0*S+
     &  U)-2D0*D3(1,3,3)*(T+U)-2D0*D2(3,3)*(2D0*KM2+T+U)-2D0*D2(1,
     &  3)*(2D0*KP2+T+U)-D3(1,2,3)*(2D0*KP2+5D0*T+3D0*U)-(2D0*D1(3)*
     &  (MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2
        K(17) = 26D0*D2(0,0)+12D0*D3(0,0,1)+26D0*D3(0,0,2)+2D0*D3(1,
     &  3,3)*KM2+3D0*D3(2,3,3)*KM2+2D0*D3(1,1,1)*KP2+3D0*D3(2,2,2)*
     &  T+3D0*D2(2,3)*(KM2+T)+3D0*D3(2,2,3)*(KM2+T)+2D0*D2(2,2)*
     &  (2D0*KM2+T)-D1(2)*(-(4D0*KM2)+T)+D3(1,1,2)*(5D0*KP2+2D0*T)+
     &  D3(1,2,2)*(3D0*KP2+5D0*T)+D2(1,2)*(5D0*KM2+8D0*KP2-3D0*S+U)+
     &  2D0*D3(1,1,3)*(T+U)+2D0*D2(1,3)*(2D0*KM2+T+U)+2D0*D2(1,1)*
     &  (2D0*KP2+T+U)+D3(1,2,3)*(2D0*KM2+5D0*T+3D0*U)+(2D0*D1(1)*
     &  (MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2
        K(18) = -(2D0*D2(0,0))-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,
     &  0,3)-2D0*D2(2,3)*KM2+D3(3,3,3)*KM2-2D0*D2(1,2)*KP2+D3(1,1,
     &  1)*KP2-D1(2)*T-D2(2,2)*T+D3(2,2,2)*T+D0*T+D2(3,3)*(-(2D0*
     &  KM2)+T)+D3(2,3,3)*(2D0*KM2+T)+D2(1,1)*(-(2D0*KP2)+T)+D3(1,1,
     &  2)*(2D0*KP2+T)+D1(3)*(KM2-2D0*T)+D1(1)*(KP2-2D0*T)+D3(2,2,
     &  3)*(KM2+2D0*T)+D3(1,2,2)*(KP2+2D0*T)+D3(1,3,3)*(KM2+T+U)+
     &  D3(1,1,3)*(KP2+T+U)+D3(1,2,3)*(KM2+KP2+3D0*T+U)+D2(1,3)*(S-
     &  2D0*U)
        K(19) = 24D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*D3(0,0,
     &  3)+D3(3,3,3)*KM2+D3(1,1,1)*KP2+D2(1,3)*(5D0*KM2+5D0*KP2-3D0*
     &  S)+D3(2,2,2)*T+D3(2,3,3)*(2D0*KM2+T)+D3(1,1,2)*(2D0*KP2+T)+
     &  D3(2,2,3)*(KM2+2D0*T)+D3(1,2,2)*(KP2+2D0*T)+D2(2,2)*(KM2+
     &  KP2+3D0*T)+D3(1,3,3)*(KM2+T+U)+D2(3,3)*(3D0*KM2+T+U)+D3(1,1,
     &  3)*(KP2+T+U)+D2(1,1)*(3D0*KP2+T+U)+D3(1,2,3)*(KM2+KP2+3D0*T+
     &  U)+D2(2,3)*(4D0*KM2+KP2+4D0*T+U)+D2(1,2)*(KM2+4D0*KP2+4D0*T+
     &  U)+(D0*(MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2+(D1(2)*(CW2*KM2+
     &  CW2*KP2+MW2*SW2+4D0*CW2*T+2D0*CW2*U))/CW2+(D1(3)*(2D0*CW2*
     &  KM2+MW2*SW2+3D0*CW2*T+3D0*CW2*U))/CW2+(D1(1)*(2D0*CW2*KP2+
     &  MW2*SW2+3D0*CW2*T+3D0*CW2*U))/CW2

        DF(1,-1) = (K(1)*K(19))/SW2
        DF(1,1) = -(2D0*K(19))
        DF(2,-1) = (K(1)*K(18))/SW2
        DF(2,1) = -(2D0*K(18))
        DF(3,-1) = -((K(1)*K(15))/SW2)
        DF(3,1) = 2D0*K(15)
        DF(4,-1) = -((K(1)*K(14))/SW2)
        DF(4,1) = 2D0*K(14)
        DF(5,-1) = (K(1)*K(16))/SW2
        DF(5,1) = -(2D0*K(16))
        DF(6,-1) = -((K(1)*K(17))/SW2)
        DF(6,1) = 2D0*K(17)
        DF(7,-1) = -((8D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))*
     &  K(1))/SW2)
        DF(7,1) = 16D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))
        DF(8,-1) = (8D0*K(1)*K(13))/SW2
        DF(8,1) = -(16D0*K(13))
        DF(9,-1) = (8D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3))*K(1))/SW2
        DF(9,1) = -(16D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3)))
        DF(10,-1) = (8D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))*K(1))/SW2
        DF(10,1) = -(16D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3)))

        PRE = -((ALPHA)/(8D0*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T3,N16)+G(T3,N17)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,T,MW,MZ,mxe,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,404)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2+T
        K(3) = 2D0*KM2+T
        K(4) = KP2+T
        K(5) = 2D0*KP2+T
        K(6) = T+U
        K(7) = KM2+T+U
        K(8) = 2D0*KM2+T+U
        K(9) = KP2+T+U
        K(10) = 2D0*KP2+T+U
        K(11) = MW2*SW2+2D0*CW2*T+2D0*CW2*U
        K(12) = KM2+KP2+3D0*T+U
        K(13) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(14) = 26D0*D3(0,0,3)+2D0*D2(2,3)*KM2+3D0*D3(3,3,3)*KM2-
     &  2D0*D2(1,1)*KP2+3D0*D3(1,1,3)*KP2-2D0*D2(2,2)*T+3D0*D3(2,2,
     &  3)*T+2D0*D0*T+3D0*D3(2,3,3)*(KM2+T)+D1(3)*(2D0*KM2+T)-2D0*
     &  D2(1,2)*(KP2+T)+3D0*D3(1,2,3)*(KP2+T)+2D0*D1(1)*(KP2-T)+
     &  D2(3,3)*(KM2+3D0*T)+3D0*D3(1,3,3)*(T+U)-D2(1,3)*(-(3D0*KP2)+
     &  3D0*T-2D0*U)
        K(15) = 26D0*D3(0,0,1)-2D0*D2(3,3)*KM2+3D0*D3(1,3,3)*KM2+
     &  2D0*D2(1,2)*KP2+3D0*D3(1,1,1)*KP2-2D0*D2(2,2)*T+3D0*D3(1,2,
     &  2)*T+2D0*D0*T-2D0*D2(2,3)*(KM2+T)+3D0*D3(1,2,3)*(KM2+T)+3D0*
     &  D3(1,1,2)*(KP2+T)+D1(1)*(2D0*KP2+T)+2D0*D1(3)*(KM2-T)+D2(1,
     &  1)*(KP2+3D0*T)+3D0*D3(1,1,3)*(T+U)-D2(1,3)*(-(3D0*KM2)+3D0*
     &  T-2D0*U)
        K(16) = -(26D0*D2(0,0))-26D0*D3(0,0,2)-12D0*D3(0,0,3)-2D0*
     &  D3(3,3,3)*KM2-3D0*D3(1,1,2)*KP2-2D0*D3(1,1,3)*KP2-3D0*D3(2,
     &  2,2)*T-3D0*D2(1,2)*(KP2+T)-3D0*D3(1,2,2)*(KP2+T)-2D0*D2(2,
     &  2)*(2D0*KP2+T)+D1(2)*(-(4D0*KP2)+T)-D3(2,3,3)*(5D0*KM2+2D0*
     &  T)-D3(2,2,3)*(3D0*KM2+5D0*T)-D2(2,3)*(8D0*KM2+5D0*KP2-3D0*S+
     &  U)-2D0*D3(1,3,3)*(T+U)-2D0*D2(3,3)*(2D0*KM2+T+U)-2D0*D2(1,
     &  3)*(2D0*KP2+T+U)-D3(1,2,3)*(2D0*KP2+5D0*T+3D0*U)-(2D0*D1(3)*
     &  (MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2
        K(17) = 26D0*D2(0,0)+12D0*D3(0,0,1)+26D0*D3(0,0,2)+2D0*D3(1,
     &  3,3)*KM2+3D0*D3(2,3,3)*KM2+2D0*D3(1,1,1)*KP2+3D0*D3(2,2,2)*
     &  T+3D0*D2(2,3)*(KM2+T)+3D0*D3(2,2,3)*(KM2+T)+2D0*D2(2,2)*
     &  (2D0*KM2+T)-D1(2)*(-(4D0*KM2)+T)+D3(1,1,2)*(5D0*KP2+2D0*T)+
     &  D3(1,2,2)*(3D0*KP2+5D0*T)+D2(1,2)*(5D0*KM2+8D0*KP2-3D0*S+U)+
     &  2D0*D3(1,1,3)*(T+U)+2D0*D2(1,3)*(2D0*KM2+T+U)+2D0*D2(1,1)*
     &  (2D0*KP2+T+U)+D3(1,2,3)*(2D0*KM2+5D0*T+3D0*U)+(2D0*D1(1)*
     &  (MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2
        K(18) = -(2D0*D2(0,0))-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,
     &  0,3)-2D0*D2(2,3)*KM2+D3(3,3,3)*KM2-2D0*D2(1,2)*KP2+D3(1,1,
     &  1)*KP2-D1(2)*T-D2(2,2)*T+D3(2,2,2)*T+D0*T+D2(3,3)*(-(2D0*
     &  KM2)+T)+D3(2,3,3)*(2D0*KM2+T)+D2(1,1)*(-(2D0*KP2)+T)+D3(1,1,
     &  2)*(2D0*KP2+T)+D1(3)*(KM2-2D0*T)+D1(1)*(KP2-2D0*T)+D3(2,2,
     &  3)*(KM2+2D0*T)+D3(1,2,2)*(KP2+2D0*T)+D3(1,3,3)*(KM2+T+U)+
     &  D3(1,1,3)*(KP2+T+U)+D3(1,2,3)*(KM2+KP2+3D0*T+U)+D2(1,3)*(S-
     &  2D0*U)
        K(19) = 24D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*D3(0,0,
     &  3)+D3(3,3,3)*KM2+D3(1,1,1)*KP2+D2(1,3)*(5D0*KM2+5D0*KP2-3D0*
     &  S)+D3(2,2,2)*T+D3(2,3,3)*(2D0*KM2+T)+D3(1,1,2)*(2D0*KP2+T)+
     &  D3(2,2,3)*(KM2+2D0*T)+D3(1,2,2)*(KP2+2D0*T)+D2(2,2)*(KM2+
     &  KP2+3D0*T)+D3(1,3,3)*(KM2+T+U)+D2(3,3)*(3D0*KM2+T+U)+D3(1,1,
     &  3)*(KP2+T+U)+D2(1,1)*(3D0*KP2+T+U)+D3(1,2,3)*(KM2+KP2+3D0*T+
     &  U)+D2(2,3)*(4D0*KM2+KP2+4D0*T+U)+D2(1,2)*(KM2+4D0*KP2+4D0*T+
     &  U)+(D0*(MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2+(D1(2)*(CW2*KM2+
     &  CW2*KP2+MW2*SW2+4D0*CW2*T+2D0*CW2*U))/CW2+(D1(3)*(2D0*CW2*
     &  KM2+MW2*SW2+3D0*CW2*T+3D0*CW2*U))/CW2+(D1(1)*(2D0*CW2*KP2+
     &  MW2*SW2+3D0*CW2*T+3D0*CW2*U))/CW2

        DF(1,-1) = (K(1)*K(19))/SW2
        DF(1,1) = -(2D0*K(19))
        DF(2,-1) = (K(1)*K(18))/SW2
        DF(2,1) = -(2D0*K(18))
        DF(3,-1) = -((K(1)*K(15))/SW2)
        DF(3,1) = 2D0*K(15)
        DF(4,-1) = -((K(1)*K(14))/SW2)
        DF(4,1) = 2D0*K(14)
        DF(5,-1) = (K(1)*K(16))/SW2
        DF(5,1) = -(2D0*K(16))
        DF(6,-1) = -((K(1)*K(17))/SW2)
        DF(6,1) = 2D0*K(17)
        DF(7,-1) = -((8D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))*
     &  K(1))/SW2)
        DF(7,1) = 16D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))
        DF(8,-1) = (8D0*K(1)*K(13))/SW2
        DF(8,1) = -(16D0*K(13))
        DF(9,-1) = (8D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3))*K(1))/SW2
        DF(9,1) = -(16D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3)))
        DF(10,-1) = (8D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))*K(1))/SW2
        DF(10,1) = -(16D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3)))

        PRE = -((ALPHA)/(8D0*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T3,N18)+G(T3,N19)

        CALL D01234_(KP2,0d0,0d0,KM2,S,T,MW,MZ,0d0,MZ,D0,D1,D2,D3,D4,
     &  C0,C1,C2,C3,3,0)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2+T
        K(3) = 2D0*KM2+T
        K(4) = KP2+T
        K(5) = 2D0*KP2+T
        K(6) = T+U
        K(7) = KM2+T+U
        K(8) = 2D0*KM2+T+U
        K(9) = KP2+T+U
        K(10) = 2D0*KP2+T+U
        K(11) = MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*U
        K(12) = KM2+KP2+3D0*T+U
        K(13) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(14) = 26D0*D3(0,0,3)+2D0*D2(2,3)*KM2+3D0*D3(3,3,3)*KM2-
     &  2D0*D2(1,1)*KP2+3D0*D3(1,1,3)*KP2-2D0*D2(2,2)*T+3D0*D3(2,2,
     &  3)*T+2D0*D0*T+3D0*D3(2,3,3)*(KM2+T)+D1(3)*(2D0*KM2+T)-2D0*
     &  D2(1,2)*(KP2+T)+3D0*D3(1,2,3)*(KP2+T)+2D0*D1(1)*(KP2-T)+
     &  D2(3,3)*(KM2+3D0*T)+3D0*D3(1,3,3)*(T+U)+D2(1,3)*(3D0*KP2-
     &  3D0*T+2D0*U)
        K(15) = 26D0*D3(0,0,1)-2D0*D2(3,3)*KM2+3D0*D3(1,3,3)*KM2+
     &  2D0*D2(1,2)*KP2+3D0*D3(1,1,1)*KP2-2D0*D2(2,2)*T+3D0*D3(1,2,
     &  2)*T+2D0*D0*T-2D0*D2(2,3)*(KM2+T)+3D0*D3(1,2,3)*(KM2+T)+3D0*
     &  D3(1,1,2)*(KP2+T)+D1(1)*(2D0*KP2+T)+2D0*D1(3)*(KM2-T)+D2(1,
     &  1)*(KP2+3D0*T)+3D0*D3(1,1,3)*(T+U)+D2(1,3)*(3D0*KM2-3D0*T+
     &  2D0*U)
        K(16) = 26D0*D2(0,0)+26D0*D3(0,0,2)+12D0*D3(0,0,3)+2D0*D3(3,
     &  3,3)*KM2+3D0*D3(1,1,2)*KP2+2D0*D3(1,1,3)*KP2+3D0*D3(2,2,2)*
     &  T+3D0*D2(1,2)*(KP2+T)+3D0*D3(1,2,2)*(KP2+T)+2D0*D2(2,2)*
     &  (2D0*KP2+T)+D1(2)*(4D0*KP2-T)+D3(2,3,3)*(5D0*KM2+2D0*T)+
     &  D3(2,2,3)*(3D0*KM2+5D0*T)+D2(2,3)*(8D0*KM2+5D0*KP2-3D0*S+U)+
     &  2D0*D3(1,3,3)*(T+U)+2D0*D2(3,3)*(2D0*KM2+T+U)+2D0*D2(1,3)*
     &  (2D0*KP2+T+U)+D3(1,2,3)*(2D0*KP2+5D0*T+3D0*U)-2D0*D1(3)*
     &  CW2**(-2)*(MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*U)
        K(17) = 26D0*D2(0,0)+12D0*D3(0,0,1)+26D0*D3(0,0,2)+2D0*D3(1,
     &  3,3)*KM2+3D0*D3(2,3,3)*KM2+2D0*D3(1,1,1)*KP2+3D0*D3(2,2,2)*
     &  T+3D0*D2(2,3)*(KM2+T)+3D0*D3(2,2,3)*(KM2+T)+2D0*D2(2,2)*
     &  (2D0*KM2+T)+D1(2)*(4D0*KM2-T)+D3(1,1,2)*(5D0*KP2+2D0*T)+
     &  D3(1,2,2)*(3D0*KP2+5D0*T)+D2(1,2)*(5D0*KM2+8D0*KP2-3D0*S+U)+
     &  2D0*D3(1,1,3)*(T+U)+2D0*D2(1,3)*(2D0*KM2+T+U)+2D0*D2(1,1)*
     &  (2D0*KP2+T+U)+D3(1,2,3)*(2D0*KM2+5D0*T+3D0*U)-2D0*D1(1)*
     &  CW2**(-2)*(MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*U)
        K(18) = -(2D0*D2(0,0))-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,
     &  0,3)-2D0*D2(2,3)*KM2+D3(3,3,3)*KM2-2D0*D2(1,2)*KP2+D3(1,1,
     &  1)*KP2-D1(2)*T-D2(2,2)*T+D3(2,2,2)*T+D0*T+D3(2,3,3)*(2D0*
     &  KM2+T)+D3(1,1,2)*(2D0*KP2+T)-D2(3,3)*(2D0*KM2-T)-D2(1,1)*
     &  (2D0*KP2-T)+D1(3)*(KM2-2D0*T)+D1(1)*(KP2-2D0*T)+D3(2,2,3)*
     &  (KM2+2D0*T)+D3(1,2,2)*(KP2+2D0*T)+D3(1,3,3)*(KM2+T+U)+D3(1,
     &  1,3)*(KP2+T+U)+D3(1,2,3)*(KM2+KP2+3D0*T+U)+D2(1,3)*(S-2D0*U)
        K(19) = -(24D0*D2(0,0))-6D0*D3(0,0,1)-6D0*D3(0,0,2)-6D0*
     &  D3(0,0,3)-D3(3,3,3)*KM2-D3(1,1,1)*KP2-D2(1,3)*(5D0*KM2+5D0*
     &  KP2-3D0*S)-D3(2,2,2)*T-D3(2,3,3)*(2D0*KM2+T)-D3(1,1,2)*(2D0*
     &  KP2+T)-D3(2,2,3)*(KM2+2D0*T)-D3(1,2,2)*(KP2+2D0*T)-D2(2,2)*
     &  (KM2+KP2+3D0*T)-D3(1,3,3)*(KM2+T+U)-D2(3,3)*(3D0*KM2+T+U)-
     &  D3(1,1,3)*(KP2+T+U)-D2(1,1)*(3D0*KP2+T+U)-D3(1,2,3)*(KM2+
     &  KP2+3D0*T+U)-D2(2,3)*(4D0*KM2+KP2+4D0*T+U)-D2(1,2)*(KM2+4D0*
     &  KP2+4D0*T+U)+CW2**(-2)*D0*(MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**
     &  2*U)-D1(2)*CW2**(-2)*(CW2**2*KM2+CW2**2*KP2-MW2*SW2**2+4D0*
     &  CW2**2*T+2D0*CW2**2*U)+D1(3)*CW2**(-2)*(-(2D0*CW2**2*KM2)+
     &  MW2*SW2**2-3D0*CW2**2*T-3D0*CW2**2*U)+D1(1)*CW2**(-2)*(-
     &  (2D0*CW2**2*KP2)+MW2*SW2**2-3D0*CW2**2*T-3D0*CW2**2*U)

        DF(1,-1) = K(1)**2*K(19)*SW2**(-2)
        DF(1,1) = 4D0*K(19)
        DF(2,-1) = -(K(1)**2*K(18)*SW2**(-2))
        DF(2,1) = -(4D0*K(18))
        DF(3,-1) = K(1)**2*K(15)*SW2**(-2)
        DF(3,1) = 4D0*K(15)
        DF(4,-1) = K(1)**2*K(14)*SW2**(-2)
        DF(4,1) = 4D0*K(14)
        DF(5,-1) = K(1)**2*K(16)*SW2**(-2)
        DF(5,1) = 4D0*K(16)
        DF(6,-1) = K(1)**2*K(17)*SW2**(-2)
        DF(6,1) = 4D0*K(17)
        DF(7,-1) = 8D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))*
     &  K(1)**2*SW2**(-2)
        DF(7,1) = 32D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))
        DF(8,-1) = -(8D0*K(1)**2*K(13)*SW2**(-2))
        DF(8,1) = -(32D0*K(13))
        DF(9,-1) = -(8D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3))*K(1)**2*SW2**(-2))
        DF(9,1) = -(32D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3)))
        DF(10,-1) = -(8D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))*K(1)**2*SW2**(-2))
        DF(10,1) = -(32D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3)))

        PRE = (ALPHA)/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T3,N22)

        CALL D01234_(KP2,0D0,0D0,KM2,S,T,MZ,MW,0D0,MW,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = T+U

        DF(1,-1) = 24D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*
     &  D3(0,0,3)+2D0*K(1)*D0+D3(3,3,3)*KM2+D3(1,3,3)*(K(1)+KM2)+
     &  D2(3,3)*(K(1)+3D0*KM2)+D3(1,1,1)*KP2+D3(1,1,3)*(K(1)+KP2)+
     &  D2(1,1)*(K(1)+3D0*KP2)+D2(1,3)*(5D0*KM2+5D0*KP2-3D0*S)+D3(2,
     &  2,2)*T+D3(2,3,3)*(2D0*KM2+T)+D3(1,1,2)*(2D0*KP2+T)+D3(2,2,
     &  3)*(KM2+2D0*T)+D3(1,2,2)*(KP2+2D0*T)+D2(2,2)*(KM2+KP2+3D0*
     &  T)+D3(1,2,3)*(KM2+KP2+3D0*T+U)+D2(2,3)*(4D0*KM2+KP2+4D0*T+
     &  U)+D2(1,2)*(KM2+4D0*KP2+4D0*T+U)+D1(2)*(KM2+KP2+4D0*T+2D0*
     &  U)+D1(3)*(2D0*KM2+3D0*T+3D0*U)+D1(1)*(2D0*KP2+3D0*T+3D0*U)
        DF(2,-1) = -(2D0*D2(0,0))-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*
     &  D3(0,0,3)-2D0*D2(2,3)*KM2+D3(3,3,3)*KM2+D3(1,3,3)*(K(1)+
     &  KM2)-2D0*D2(1,2)*KP2+D3(1,1,1)*KP2+D3(1,1,3)*(K(1)+KP2)-
     &  D1(2)*T-D2(2,2)*T+D3(2,2,2)*T+D0*T+D3(2,3,3)*(2D0*KM2+T)+
     &  D3(1,1,2)*(2D0*KP2+T)-D2(3,3)*(2D0*KM2-T)-D2(1,1)*(2D0*KP2-
     &  T)+D1(3)*(KM2-2D0*T)+D1(1)*(KP2-2D0*T)+D3(2,2,3)*(KM2+2D0*
     &  T)+D3(1,2,2)*(KP2+2D0*T)+D3(1,2,3)*(KM2+KP2+3D0*T+U)+D2(1,
     &  3)*(S-2D0*U)
        DF(3,-1) = -(26D0*D3(0,0,1))-3D0*D3(1,1,3)*K(1)+2D0*D2(3,3)*
     &  KM2-3D0*D3(1,3,3)*KM2-2D0*D2(1,2)*KP2-3D0*D3(1,1,1)*KP2+2D0*
     &  D2(2,2)*T-3D0*D3(1,2,2)*T-2D0*D0*T+2D0*D2(2,3)*(KM2+T)-3D0*
     &  D3(1,2,3)*(KM2+T)-3D0*D3(1,1,2)*(KP2+T)-D1(1)*(2D0*KP2+T)-
     &  2D0*D1(3)*(KM2-T)-D2(1,1)*(KP2+3D0*T)-D2(1,3)*(3D0*KM2-3D0*
     &  T+2D0*U)
        DF(4,-1) = -(26D0*D3(0,0,3))-3D0*D3(1,3,3)*K(1)-2D0*D2(2,3)*
     &  KM2-3D0*D3(3,3,3)*KM2+2D0*D2(1,1)*KP2-3D0*D3(1,1,3)*KP2+2D0*
     &  D2(2,2)*T-3D0*D3(2,2,3)*T-2D0*D0*T-3D0*D3(2,3,3)*(KM2+T)-
     &  D1(3)*(2D0*KM2+T)+2D0*D2(1,2)*(KP2+T)-3D0*D3(1,2,3)*(KP2+T)-
     &  2D0*D1(1)*(KP2-T)-D2(3,3)*(KM2+3D0*T)-D2(1,3)*(3D0*KP2-3D0*
     &  T+2D0*U)
        DF(5,-1) = -(26D0*D2(0,0))-26D0*D3(0,0,2)-12D0*D3(0,0,3)-
     &  4D0*D1(3)*K(1)-2D0*D3(1,3,3)*K(1)-2D0*D3(3,3,3)*KM2-2D0*
     &  D2(3,3)*(K(1)+2D0*KM2)-3D0*D3(1,1,2)*KP2-2D0*D3(1,1,3)*KP2-
     &  2D0*D2(1,3)*(K(1)+2D0*KP2)-3D0*D3(2,2,2)*T-3D0*D2(1,2)*(KP2+
     &  T)-3D0*D3(1,2,2)*(KP2+T)-2D0*D2(2,2)*(2D0*KP2+T)-D1(2)*(4D0*
     &  KP2-T)-D3(2,3,3)*(5D0*KM2+2D0*T)-D3(2,2,3)*(3D0*KM2+5D0*T)-
     &  D2(2,3)*(8D0*KM2+5D0*KP2-3D0*S+U)-D3(1,2,3)*(2D0*KP2+5D0*T+
     &  3D0*U)
        DF(6,-1) = -(26D0*D2(0,0))-12D0*D3(0,0,1)-26D0*D3(0,0,2)-
     &  4D0*D1(1)*K(1)-2D0*D3(1,1,3)*K(1)-2D0*D3(1,3,3)*KM2-3D0*
     &  D3(2,3,3)*KM2-2D0*D2(1,3)*(K(1)+2D0*KM2)-2D0*D3(1,1,1)*KP2-
     &  2D0*D2(1,1)*(K(1)+2D0*KP2)-3D0*D3(2,2,2)*T-3D0*D2(2,3)*(KM2+
     &  T)-3D0*D3(2,2,3)*(KM2+T)-2D0*D2(2,2)*(2D0*KM2+T)-D1(2)*(4D0*
     &  KM2-T)-D3(1,1,2)*(5D0*KP2+2D0*T)-D3(1,2,2)*(3D0*KP2+5D0*T)-
     &  D2(1,2)*(5D0*KM2+8D0*KP2-3D0*S+U)-D3(1,2,3)*(2D0*KM2+5D0*T+
     &  3D0*U)
        DF(7,-1) = -(8D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3)))
        DF(8,-1) = 8D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,
     &  3)+D3(1,2,2)+D3(2,2,2)+D3(2,2,3))
        DF(9,-1) = 8D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3))
        DF(10,-1) = 8D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))

        PRE = -((ALPHA*CW2*SW2**(-2))/(8D0*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T4,N23)

        CALL D01234_(KP2,0d0,0d0,KM2,S,U,0d0,0D0,MW,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = T+U

        DF(1,-1) = 4D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*D3(0,
     &  0,3)+2D0*D2(1,3)*K(1)+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,3)*
     &  KM2+D3(1,3,3)*(K(1)+KM2)+D1(1)*KP2+2D0*D2(1,1)*KP2+D3(1,1,
     &  1)*KP2+D3(1,1,3)*(K(1)+KP2)+2D0*D2(2,2)*U+D3(2,2,2)*U+2D0*
     &  D2(2,3)*(KM2+U)+D3(2,3,3)*(2D0*KM2+U)+2D0*D2(1,2)*(KP2+U)+
     &  D3(1,1,2)*(2D0*KP2+U)-D1(2)*(S-U)+D3(2,2,3)*(KM2+2D0*U)+
     &  D3(1,2,2)*(KP2+2D0*U)+D3(1,2,3)*(KM2+KP2+T+3D0*U)
        DF(2,-1) = -(2D0*(-(2D0*D3(0,0,1))-2D0*D3(0,0,2)-2D0*D3(0,0,
     &  3)+D1(2)*S))
        DF(3,-1) = 2D0*(-(4D0*D3(0,0,1))-4D0*D3(0,0,2)-D3(1,1,3)*
     &  K(1)-D3(1,3,3)*KM2-D3(2,3,3)*KM2-D2(1,1)*KP2-2D0*D2(1,2)*
     &  KP2-D2(2,2)*KP2-D3(1,1,1)*KP2-D2(1,3)*T-D2(2,3)*T+D1(2)*
     &  (KM2-T)-D3(2,2,2)*U-D3(2,2,3)*(KM2+U)-D3(1,1,2)*(2D0*KP2+U)-
     &  D3(1,2,2)*(KP2+2D0*U)-D3(1,2,3)*(KM2+T+2D0*U))
        DF(4,-1) = 2D0*(-(4D0*D3(0,0,2))-4D0*D3(0,0,3)-D3(1,3,3)*
     &  K(1)-D2(2,2)*KM2-2D0*D2(2,3)*KM2-D2(3,3)*KM2-D3(3,3,3)*KM2-
     &  D3(1,1,2)*KP2-D3(1,1,3)*KP2-D2(1,2)*T-D2(1,3)*T+D1(2)*(KP2-
     &  T)-D3(2,2,2)*U-D3(2,3,3)*(2D0*KM2+U)-D3(1,2,2)*(KP2+U)-D3(2,
     &  2,3)*(KM2+2D0*U)-D3(1,2,3)*(KP2+T+2D0*U))
        DF(5,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,2)+6D0*D3(0,0,3)+
     &  D2(1,3)*K(1)+D3(1,3,3)*K(1)+D2(3,3)*KM2+D3(3,3,3)*KM2+D1(1)*
     &  KP2+D2(1,1)*KP2+D3(1,1,3)*KP2+D2(1,2)*U+D3(2,2,3)*U+D3(2,3,
     &  3)*(KM2+U)+D3(1,2,3)*(KP2+U)+D2(2,3)*(KM2-T+U)-D1(2)*(KP2-
     &  U)-D2(2,2)*(KP2-U)))
        DF(6,-1) = 2D0*(-(2D0*D2(0,0))-6D0*D3(0,0,1)-2D0*D3(0,0,2)-
     &  D2(1,3)*K(1)-D3(1,1,3)*K(1)-D1(3)*KM2-D2(3,3)*KM2-D3(1,3,3)*
     &  KM2-D2(1,1)*KP2-D3(1,1,1)*KP2-D2(2,3)*U-D3(1,2,2)*U-D3(1,2,
     &  3)*(KM2+U)-D3(1,1,2)*(KP2+U)-D2(1,2)*(KP2-T+U)+D1(2)*(KM2-
     &  U)+D2(2,2)*(KM2-U))
        DF(7,-1) = 4D0*(D1(2)-D3(1,1,2)-D3(1,1,3)-2D0*D3(1,2,2)-3D0*
     &  D3(1,2,3)-D3(1,3,3)-D3(2,2,2)-2D0*D3(2,2,3)-D3(2,3,3))
        DF(8,-1) = 4D0*(D1(2)+D2(1,2)+D2(2,3)-D3(1,2,2)-D3(2,2,2)-
     &  D3(2,2,3))
        DF(9,-1) = -(4D0*(D1(2)+D2(1,2)+D2(1,3)-D3(1,2,2)-D3(1,2,3)-
     &  D3(2,2,2)-2D0*D3(2,2,3)-D3(2,3,3)))
        DF(10,-1) = -(4D0*(D1(2)+D2(1,3)+D2(2,3)-D3(1,1,2)-2D0*D3(1,
     &  2,2)-D3(1,2,3)-D3(2,2,2)-D3(2,2,3)))

        PRE = (ALPHA*SW2**(-2))/(8D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T4,N24)+G(T4,N25)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,U,MW,0D0,mxe,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,403)

        K(1) = KM2+U
        K(2) = KM2+2D0*U
        K(3) = 2D0*KM2+U
        K(4) = KP2+U
        K(5) = KP2+2D0*U
        K(6) = 2D0*KP2+U
        K(7) = T+U
        K(8) = KM2+T+U
        K(9) = KM2+T+2D0*U
        K(10) = 2D0*KM2+T+U
        K(11) = KP2+T+U
        K(12) = KP2+T+2D0*U
        K(13) = 2D0*KP2+T+U
        K(14) = MW2-2D0*T-2D0*U
        K(15) = MW2-2D0*T-U
        K(16) = KM2+KP2+T+3D0*U
        K(17) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(18) = 2D0*D1(2)+2D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,
     &  3)
        K(19) = 2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,1,2)+2D0*D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,
     &  3)
        K(20) = 2D0*D1(2)+3D0*D2(1,2)+3D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)
        K(21) = 26D0*D2(0,0)+26D0*D3(0,0,2)+12D0*D3(0,0,3)+2D0*D3(3,
     &  3,3)*KM2+3D0*D3(1,1,2)*KP2+2D0*D3(1,1,3)*KP2+D2(2,3)*(8D0*
     &  KM2+5D0*KP2-3D0*S+T)+3D0*D3(2,2,2)*U+3D0*D2(1,2)*(KP2+U)+
     &  3D0*D3(1,2,2)*(KP2+U)+2D0*D2(2,2)*(2D0*KP2+U)+2D0*D3(1,3,3)*
     &  (T+U)+2D0*D2(3,3)*(2D0*KM2+T+U)+2D0*D2(1,3)*(2D0*KP2+T+U)+
     &  D1(2)*(4D0*KP2-U)-2D0*D1(3)*(MW2-2D0*T-2D0*U)+D3(2,3,3)*
     &  (5D0*KM2+2D0*U)+D3(2,2,3)*(3D0*KM2+5D0*U)+D3(1,2,3)*(2D0*
     &  KP2+3D0*T+5D0*U)
        K(22) = 26D0*D2(0,0)+12D0*D3(0,0,1)+26D0*D3(0,0,2)+2D0*D3(1,
     &  3,3)*KM2+3D0*D3(2,3,3)*KM2+2D0*D3(1,1,1)*KP2+D2(1,2)*(5D0*
     &  KM2+8D0*KP2-3D0*S+T)+3D0*D3(2,2,2)*U+3D0*D2(2,3)*(KM2+U)+
     &  3D0*D3(2,2,3)*(KM2+U)+2D0*D2(2,2)*(2D0*KM2+U)+2D0*D3(1,1,3)*
     &  (T+U)+2D0*D2(1,3)*(2D0*KM2+T+U)+2D0*D2(1,1)*(2D0*KP2+T+U)+
     &  D1(2)*(4D0*KM2-U)-2D0*D1(1)*(MW2-2D0*T-2D0*U)+D3(1,1,2)*
     &  (5D0*KP2+2D0*U)+D3(1,2,2)*(3D0*KP2+5D0*U)+D3(1,2,3)*(2D0*
     &  KM2+3D0*T+5D0*U)
        K(23) = 22D0*D2(0,0)-14D0*D3(0,0,2)-14D0*D3(0,0,3)-D3(3,3,
     &  3)*KM2+4D0*D2(1,1)*KP2-D3(1,1,2)*KP2-D3(1,1,3)*KP2-D3(2,2,
     &  2)*U-D3(2,3,3)*(2D0*KM2+U)-D3(1,2,2)*(KP2+U)-D3(1,3,3)*(T+
     &  U)-2D0*D0*(MW2-2D0*T-U)+D2(3,3)*(5D0*KM2+2D0*T-U)-D3(2,2,3)*
     &  (KM2+2D0*U)+2D0*D1(1)*(KP2+T+2D0*U)-D3(1,2,3)*(KP2+T+2D0*U)-
     &  2D0*D2(2,2)*(KM2-KP2-3D0*U)+D2(2,3)*(3D0*KM2+2D0*KP2+2D0*T+
     &  5D0*U)+D1(3)*(2D0*KM2-2D0*MW2+6D0*T+5D0*U)+D2(1,2)*(5D0*KP2-
     &  2D0*T+7D0*U)+D2(1,3)*(KP2+2D0*T+7D0*U)+D1(2)*(-(2D0*KM2)+
     &  2D0*KP2-2D0*MW2+4D0*T+9D0*U)
        K(24) = 22D0*D2(0,0)-14D0*D3(0,0,1)-14D0*D3(0,0,2)+4D0*D2(3,
     &  3)*KM2-D3(1,3,3)*KM2-D3(2,3,3)*KM2-D3(1,1,1)*KP2-D3(2,2,2)*
     &  U-D3(2,2,3)*(KM2+U)-D3(1,1,2)*(2D0*KP2+U)-D3(1,1,3)*(T+U)-
     &  2D0*D0*(MW2-2D0*T-U)+D2(1,1)*(5D0*KP2+2D0*T-U)-D3(1,2,2)*
     &  (KP2+2D0*U)+2D0*D1(3)*(KM2+T+2D0*U)-D3(1,2,3)*(KM2+T+2D0*U)+
     &  2D0*D2(2,2)*(KM2-KP2+3D0*U)+D2(1,2)*(2D0*KM2+3D0*KP2+2D0*T+
     &  5D0*U)+D1(1)*(2D0*KP2-2D0*MW2+6D0*T+5D0*U)+D2(2,3)*(5D0*KM2-
     &  2D0*T+7D0*U)+D2(1,3)*(KM2+2D0*T+7D0*U)+D1(2)*(2D0*KM2-2D0*
     &  KP2-2D0*MW2+4D0*T+9D0*U)
        K(25) = -(24D0*D2(0,0))-6D0*D3(0,0,1)-6D0*D3(0,0,2)-6D0*
     &  D3(0,0,3)-D3(3,3,3)*KM2-D3(1,1,1)*KP2-D2(1,3)*(5D0*KM2+5D0*
     &  KP2-3D0*S)-D3(2,2,2)*U-D3(2,3,3)*(2D0*KM2+U)-D3(1,1,2)*(2D0*
     &  KP2+U)-D3(1,3,3)*(KM2+T+U)-D2(3,3)*(3D0*KM2+T+U)-D3(1,1,3)*
     &  (KP2+T+U)-D2(1,1)*(3D0*KP2+T+U)+D0*(MW2-2D0*T-2D0*U)-D3(2,2,
     &  3)*(KM2+2D0*U)-D3(1,2,2)*(KP2+2D0*U)+D1(3)*(-(2D0*KM2)+MW2-
     &  3D0*T-3D0*U)+D1(1)*(-(2D0*KP2)+MW2-3D0*T-3D0*U)-D2(2,2)*
     &  (KM2+KP2+3D0*U)-D3(1,2,3)*(KM2+KP2+T+3D0*U)-D2(2,3)*(4D0*
     &  KM2+KP2+T+4D0*U)-D2(1,2)*(KM2+4D0*KP2+T+4D0*U)-D1(2)*(KM2+
     &  KP2-MW2+2D0*T+4D0*U)
        K(26) = -(50D0*D2(0,0))-14D0*D3(0,0,1)-14D0*D3(0,0,2)-14D0*
     &  D3(0,0,3)-D3(3,3,3)*KM2-D3(1,1,1)*KP2-D3(2,2,2)*U-D3(2,3,3)*
     &  (2D0*KM2+U)-D3(1,1,2)*(2D0*KP2+U)-D3(1,3,3)*(KM2+T+U)-D3(1,
     &  1,3)*(KP2+T+U)-D2(3,3)*(8D0*KM2+2D0*T+U)-D2(1,1)*(8D0*KP2+
     &  2D0*T+U)-D3(2,2,3)*(KM2+2D0*U)-D3(1,2,2)*(KP2+2D0*U)+D0*
     &  (2D0*MW2-4D0*T-3D0*U)-D3(1,2,3)*(KM2+KP2+T+3D0*U)-2D0*D2(2,
     &  3)*(5D0*KM2+KP2+T+4D0*U)-2D0*D2(1,2)*(KM2+5D0*KP2+T+4D0*U)-
     &  D2(2,2)*(2D0*KM2+2D0*KP2+7D0*U)-D2(1,3)*(3D0*KM2+3D0*KP2+
     &  9D0*T+7D0*U)+D1(3)*(-(3D0*KM2)+2D0*MW2-6D0*T-8D0*U)+D1(1)*(-
     &  (3D0*KP2)+2D0*MW2-6D0*T-8D0*U)+D1(2)*(-(2D0*KM2)-2D0*KP2+
     &  2D0*MW2-4D0*T-9D0*U)

        DF(1,-1) = K(25)
        DF(1,1) = DF(1,-1)
        DF(2,-1) = K(26)
        DF(2,1) = DF(2,-1)
        DF(3,-1) = K(24)
        DF(3,1) = DF(3,-1)
        DF(4,-1) = K(23)
        DF(4,1) = DF(4,-1)
        DF(5,-1) = K(21)
        DF(5,1) = DF(5,-1)
        DF(6,-1) = K(22)
        DF(6,1) = DF(6,-1)
        DF(7,-1) = 8D0*K(20)
        DF(7,1) = DF(7,-1)
        DF(8,-1) = 8D0*K(17)
        DF(8,1) = DF(8,-1)
        DF(9,-1) = -(8D0*K(18))
        DF(9,1) = DF(9,-1)
        DF(10,-1) = -(8D0*K(19))
        DF(10,1) = DF(10,-1)

        PRE = (ALPHA)/(4D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T4,N28)+G(T4,N29)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,U,MW,0D0,mxe,MZ,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,404)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2+U
        K(3) = KM2+2D0*U
        K(4) = 2D0*KM2+U
        K(5) = KP2+U
        K(6) = KP2+2D0*U
        K(7) = 2D0*KP2+U
        K(8) = T+U
        K(9) = KM2+T+U
        K(10) = KM2+T+2D0*U
        K(11) = 2D0*KM2+T+U
        K(12) = KP2+T+U
        K(13) = KP2+T+2D0*U
        K(14) = 2D0*KP2+T+U
        K(15) = MW2*SW2+2D0*CW2*T+CW2*U
        K(16) = MW2*SW2+2D0*CW2*T+2D0*CW2*U
        K(17) = KM2+KP2+T+3D0*U
        K(18) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(19) = 2D0*D1(2)+2D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,
     &  3)
        K(20) = 2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,1,2)+2D0*D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,
     &  3)
        K(21) = 2D0*D1(2)+3D0*D2(1,2)+3D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)
        K(22) = -(26D0*D2(0,0))-26D0*D3(0,0,2)-12D0*D3(0,0,3)-2D0*
     &  D3(3,3,3)*KM2-3D0*D3(1,1,2)*KP2-2D0*D3(1,1,3)*KP2-D2(2,3)*
     &  (8D0*KM2+5D0*KP2-3D0*S+T)-3D0*D3(2,2,2)*U-3D0*D2(1,2)*(KP2+
     &  U)-3D0*D3(1,2,2)*(KP2+U)-2D0*D2(2,2)*(2D0*KP2+U)+D1(2)*(-
     &  (4D0*KP2)+U)-2D0*D3(1,3,3)*(T+U)-2D0*D2(3,3)*(2D0*KM2+T+U)-
     &  2D0*D2(1,3)*(2D0*KP2+T+U)-D3(2,3,3)*(5D0*KM2+2D0*U)-D3(2,2,
     &  3)*(3D0*KM2+5D0*U)-D3(1,2,3)*(2D0*KP2+3D0*T+5D0*U)-(2D0*
     &  D1(3)*(MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2
        K(23) = 26D0*D2(0,0)+12D0*D3(0,0,1)+26D0*D3(0,0,2)+2D0*D3(1,
     &  3,3)*KM2+3D0*D3(2,3,3)*KM2+2D0*D3(1,1,1)*KP2+D2(1,2)*(5D0*
     &  KM2+8D0*KP2-3D0*S+T)+3D0*D3(2,2,2)*U+3D0*D2(2,3)*(KM2+U)+
     &  3D0*D3(2,2,3)*(KM2+U)+2D0*D2(2,2)*(2D0*KM2+U)-D1(2)*(-(4D0*
     &  KM2)+U)+2D0*D3(1,1,3)*(T+U)+2D0*D2(1,3)*(2D0*KM2+T+U)+2D0*
     &  D2(1,1)*(2D0*KP2+T+U)+D3(1,1,2)*(5D0*KP2+2D0*U)+D3(1,2,2)*
     &  (3D0*KP2+5D0*U)+D3(1,2,3)*(2D0*KM2+3D0*T+5D0*U)+(2D0*D1(1)*
     &  (MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2
        K(24) = 22D0*D2(0,0)-14D0*D3(0,0,2)-14D0*D3(0,0,3)-D3(3,3,
     &  3)*KM2+4D0*D2(1,1)*KP2-D3(1,1,2)*KP2-D3(1,1,3)*KP2-D3(2,2,
     &  2)*U-D3(2,3,3)*(2D0*KM2+U)-D3(1,2,2)*(KP2+U)-D3(1,3,3)*(T+
     &  U)-D2(3,3)*(-(5D0*KM2)-2D0*T+U)-D3(2,2,3)*(KM2+2D0*U)+2D0*
     &  D1(1)*(KP2+T+2D0*U)-D3(1,2,3)*(KP2+T+2D0*U)-2D0*D2(2,2)*
     &  (KM2-KP2-3D0*U)+D2(2,3)*(3D0*KM2+2D0*KP2+2D0*T+5D0*U)-D2(1,
     &  2)*(-(5D0*KP2)+2D0*T-7D0*U)+D2(1,3)*(KP2+2D0*T+7D0*U)+(2D0*
     &  D0*(MW2*SW2+2D0*CW2*T+CW2*U))/CW2+(D1(3)*(2D0*CW2*KM2+2D0*
     &  MW2*SW2+6D0*CW2*T+5D0*CW2*U))/CW2-(D1(2)*(2D0*CW2*KM2-2D0*
     &  CW2*KP2-2D0*MW2*SW2-4D0*CW2*T-9D0*CW2*U))/CW2
        K(25) = 22D0*D2(0,0)-14D0*D3(0,0,1)-14D0*D3(0,0,2)+4D0*D2(3,
     &  3)*KM2-D3(1,3,3)*KM2-D3(2,3,3)*KM2-D3(1,1,1)*KP2-D3(2,2,2)*
     &  U-D3(2,2,3)*(KM2+U)-D3(1,1,2)*(2D0*KP2+U)-D3(1,1,3)*(T+U)-
     &  D2(1,1)*(-(5D0*KP2)-2D0*T+U)-D3(1,2,2)*(KP2+2D0*U)+2D0*
     &  D1(3)*(KM2+T+2D0*U)-D3(1,2,3)*(KM2+T+2D0*U)+2D0*D2(2,2)*
     &  (KM2-KP2+3D0*U)+D2(1,2)*(2D0*KM2+3D0*KP2+2D0*T+5D0*U)-D2(2,
     &  3)*(-(5D0*KM2)+2D0*T-7D0*U)+D2(1,3)*(KM2+2D0*T+7D0*U)+(2D0*
     &  D0*(MW2*SW2+2D0*CW2*T+CW2*U))/CW2+(D1(1)*(2D0*CW2*KP2+2D0*
     &  MW2*SW2+6D0*CW2*T+5D0*CW2*U))/CW2-(D1(2)*(-(2D0*CW2*KM2)+
     &  2D0*CW2*KP2-2D0*MW2*SW2-4D0*CW2*T-9D0*CW2*U))/CW2
        K(26) = 24D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*D3(0,0,
     &  3)+D3(3,3,3)*KM2+D3(1,1,1)*KP2+D2(1,3)*(5D0*KM2+5D0*KP2-3D0*
     &  S)+D3(2,2,2)*U+D3(2,3,3)*(2D0*KM2+U)+D3(1,1,2)*(2D0*KP2+U)+
     &  D3(1,3,3)*(KM2+T+U)+D2(3,3)*(3D0*KM2+T+U)+D3(1,1,3)*(KP2+T+
     &  U)+D2(1,1)*(3D0*KP2+T+U)+D3(2,2,3)*(KM2+2D0*U)+D3(1,2,2)*
     &  (KP2+2D0*U)+D2(2,2)*(KM2+KP2+3D0*U)+D3(1,2,3)*(KM2+KP2+T+
     &  3D0*U)+D2(2,3)*(4D0*KM2+KP2+T+4D0*U)+D2(1,2)*(KM2+4D0*KP2+T+
     &  4D0*U)+(D0*(MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2+(D1(3)*(2D0*
     &  CW2*KM2+MW2*SW2+3D0*CW2*T+3D0*CW2*U))/CW2+(D1(1)*(2D0*CW2*
     &  KP2+MW2*SW2+3D0*CW2*T+3D0*CW2*U))/CW2+(D1(2)*(CW2*KM2+CW2*
     &  KP2+MW2*SW2+2D0*CW2*T+4D0*CW2*U))/CW2
        K(27) = 50D0*D2(0,0)+14D0*D3(0,0,1)+14D0*D3(0,0,2)+14D0*
     &  D3(0,0,3)+D3(3,3,3)*KM2+D3(1,1,1)*KP2+D3(2,2,2)*U+D3(2,3,3)*
     &  (2D0*KM2+U)+D3(1,1,2)*(2D0*KP2+U)+D3(1,3,3)*(KM2+T+U)+D3(1,
     &  1,3)*(KP2+T+U)+D2(3,3)*(8D0*KM2+2D0*T+U)+D2(1,1)*(8D0*KP2+
     &  2D0*T+U)+D3(2,2,3)*(KM2+2D0*U)+D3(1,2,2)*(KP2+2D0*U)+D3(1,2,
     &  3)*(KM2+KP2+T+3D0*U)+2D0*D2(2,3)*(5D0*KM2+KP2+T+4D0*U)+2D0*
     &  D2(1,2)*(KM2+5D0*KP2+T+4D0*U)+D2(2,2)*(2D0*KM2+2D0*KP2+7D0*
     &  U)+D2(1,3)*(3D0*KM2+3D0*KP2+9D0*T+7D0*U)+(D0*(2D0*MW2*SW2+
     &  4D0*CW2*T+3D0*CW2*U))/CW2+(D1(3)*(3D0*CW2*KM2+2D0*MW2*SW2+
     &  6D0*CW2*T+8D0*CW2*U))/CW2+(D1(1)*(3D0*CW2*KP2+2D0*MW2*SW2+
     &  6D0*CW2*T+8D0*CW2*U))/CW2+(D1(2)*(2D0*CW2*KM2+2D0*CW2*KP2+
     &  2D0*MW2*SW2+4D0*CW2*T+9D0*CW2*U))/CW2

        DF(1,-1) = (K(1)*K(26))/SW2
        DF(1,1) = -(2D0*K(26))
        DF(2,-1) = (K(1)*K(27))/SW2
        DF(2,1) = -(2D0*K(27))
        DF(3,-1) = -((K(1)*K(25))/SW2)
        DF(3,1) = 2D0*K(25)
        DF(4,-1) = -((K(1)*K(24))/SW2)
        DF(4,1) = 2D0*K(24)
        DF(5,-1) = (K(1)*K(22))/SW2
        DF(5,1) = -(2D0*K(22))
        DF(6,-1) = -((K(1)*K(23))/SW2)
        DF(6,1) = 2D0*K(23)
        DF(7,-1) = -((8D0*K(1)*K(21))/SW2)
        DF(7,1) = 16D0*K(21)
        DF(8,-1) = -((8D0*K(1)*K(18))/SW2)
        DF(8,1) = 16D0*K(18)
        DF(9,-1) = (8D0*K(1)*K(19))/SW2
        DF(9,1) = -(16D0*K(19))
        DF(10,-1) = (8D0*K(1)*K(20))/SW2
        DF(10,1) = -(16D0*K(20))

        PRE = -((ALPHA)/(8D0*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T4,N26)+G(T4,N27)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,U,MW,MZ,mxe,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,404)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2+U
        K(3) = KM2+2D0*U
        K(4) = 2D0*KM2+U
        K(5) = KP2+U
        K(6) = KP2+2D0*U
        K(7) = 2D0*KP2+U
        K(8) = T+U
        K(9) = KM2+T+U
        K(10) = KM2+T+2D0*U
        K(11) = 2D0*KM2+T+U
        K(12) = KP2+T+U
        K(13) = KP2+T+2D0*U
        K(14) = 2D0*KP2+T+U
        K(15) = MW2*SW2+2D0*CW2*T+CW2*U
        K(16) = MW2*SW2+2D0*CW2*T+2D0*CW2*U
        K(17) = KM2+KP2+T+3D0*U
        K(18) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(19) = 2D0*D1(2)+2D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,
     &  3)
        K(20) = 2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,1,2)+2D0*D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,
     &  3)
        K(21) = 2D0*D1(2)+3D0*D2(1,2)+3D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)
        K(22) = -(26D0*D2(0,0))-26D0*D3(0,0,2)-12D0*D3(0,0,3)-2D0*
     &  D3(3,3,3)*KM2-3D0*D3(1,1,2)*KP2-2D0*D3(1,1,3)*KP2-D2(2,3)*
     &  (8D0*KM2+5D0*KP2-3D0*S+T)-3D0*D3(2,2,2)*U-3D0*D2(1,2)*(KP2+
     &  U)-3D0*D3(1,2,2)*(KP2+U)-2D0*D2(2,2)*(2D0*KP2+U)+D1(2)*(-
     &  (4D0*KP2)+U)-2D0*D3(1,3,3)*(T+U)-2D0*D2(3,3)*(2D0*KM2+T+U)-
     &  2D0*D2(1,3)*(2D0*KP2+T+U)-D3(2,3,3)*(5D0*KM2+2D0*U)-D3(2,2,
     &  3)*(3D0*KM2+5D0*U)-D3(1,2,3)*(2D0*KP2+3D0*T+5D0*U)-(2D0*
     &  D1(3)*(MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2
        K(23) = 26D0*D2(0,0)+12D0*D3(0,0,1)+26D0*D3(0,0,2)+2D0*D3(1,
     &  3,3)*KM2+3D0*D3(2,3,3)*KM2+2D0*D3(1,1,1)*KP2+D2(1,2)*(5D0*
     &  KM2+8D0*KP2-3D0*S+T)+3D0*D3(2,2,2)*U+3D0*D2(2,3)*(KM2+U)+
     &  3D0*D3(2,2,3)*(KM2+U)+2D0*D2(2,2)*(2D0*KM2+U)-D1(2)*(-(4D0*
     &  KM2)+U)+2D0*D3(1,1,3)*(T+U)+2D0*D2(1,3)*(2D0*KM2+T+U)+2D0*
     &  D2(1,1)*(2D0*KP2+T+U)+D3(1,1,2)*(5D0*KP2+2D0*U)+D3(1,2,2)*
     &  (3D0*KP2+5D0*U)+D3(1,2,3)*(2D0*KM2+3D0*T+5D0*U)+(2D0*D1(1)*
     &  (MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2
        K(24) = 22D0*D2(0,0)-14D0*D3(0,0,2)-14D0*D3(0,0,3)-D3(3,3,
     &  3)*KM2+4D0*D2(1,1)*KP2-D3(1,1,2)*KP2-D3(1,1,3)*KP2-D3(2,2,
     &  2)*U-D3(2,3,3)*(2D0*KM2+U)-D3(1,2,2)*(KP2+U)-D3(1,3,3)*(T+
     &  U)-D2(3,3)*(-(5D0*KM2)-2D0*T+U)-D3(2,2,3)*(KM2+2D0*U)+2D0*
     &  D1(1)*(KP2+T+2D0*U)-D3(1,2,3)*(KP2+T+2D0*U)-2D0*D2(2,2)*
     &  (KM2-KP2-3D0*U)+D2(2,3)*(3D0*KM2+2D0*KP2+2D0*T+5D0*U)-D2(1,
     &  2)*(-(5D0*KP2)+2D0*T-7D0*U)+D2(1,3)*(KP2+2D0*T+7D0*U)+(2D0*
     &  D0*(MW2*SW2+2D0*CW2*T+CW2*U))/CW2+(D1(3)*(2D0*CW2*KM2+2D0*
     &  MW2*SW2+6D0*CW2*T+5D0*CW2*U))/CW2-(D1(2)*(2D0*CW2*KM2-2D0*
     &  CW2*KP2-2D0*MW2*SW2-4D0*CW2*T-9D0*CW2*U))/CW2
        K(25) = 22D0*D2(0,0)-14D0*D3(0,0,1)-14D0*D3(0,0,2)+4D0*D2(3,
     &  3)*KM2-D3(1,3,3)*KM2-D3(2,3,3)*KM2-D3(1,1,1)*KP2-D3(2,2,2)*
     &  U-D3(2,2,3)*(KM2+U)-D3(1,1,2)*(2D0*KP2+U)-D3(1,1,3)*(T+U)-
     &  D2(1,1)*(-(5D0*KP2)-2D0*T+U)-D3(1,2,2)*(KP2+2D0*U)+2D0*
     &  D1(3)*(KM2+T+2D0*U)-D3(1,2,3)*(KM2+T+2D0*U)+2D0*D2(2,2)*
     &  (KM2-KP2+3D0*U)+D2(1,2)*(2D0*KM2+3D0*KP2+2D0*T+5D0*U)-D2(2,
     &  3)*(-(5D0*KM2)+2D0*T-7D0*U)+D2(1,3)*(KM2+2D0*T+7D0*U)+(2D0*
     &  D0*(MW2*SW2+2D0*CW2*T+CW2*U))/CW2+(D1(1)*(2D0*CW2*KP2+2D0*
     &  MW2*SW2+6D0*CW2*T+5D0*CW2*U))/CW2-(D1(2)*(-(2D0*CW2*KM2)+
     &  2D0*CW2*KP2-2D0*MW2*SW2-4D0*CW2*T-9D0*CW2*U))/CW2
        K(26) = 24D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*D3(0,0,
     &  3)+D3(3,3,3)*KM2+D3(1,1,1)*KP2+D2(1,3)*(5D0*KM2+5D0*KP2-3D0*
     &  S)+D3(2,2,2)*U+D3(2,3,3)*(2D0*KM2+U)+D3(1,1,2)*(2D0*KP2+U)+
     &  D3(1,3,3)*(KM2+T+U)+D2(3,3)*(3D0*KM2+T+U)+D3(1,1,3)*(KP2+T+
     &  U)+D2(1,1)*(3D0*KP2+T+U)+D3(2,2,3)*(KM2+2D0*U)+D3(1,2,2)*
     &  (KP2+2D0*U)+D2(2,2)*(KM2+KP2+3D0*U)+D3(1,2,3)*(KM2+KP2+T+
     &  3D0*U)+D2(2,3)*(4D0*KM2+KP2+T+4D0*U)+D2(1,2)*(KM2+4D0*KP2+T+
     &  4D0*U)+(D0*(MW2*SW2+2D0*CW2*T+2D0*CW2*U))/CW2+(D1(3)*(2D0*
     &  CW2*KM2+MW2*SW2+3D0*CW2*T+3D0*CW2*U))/CW2+(D1(1)*(2D0*CW2*
     &  KP2+MW2*SW2+3D0*CW2*T+3D0*CW2*U))/CW2+(D1(2)*(CW2*KM2+CW2*
     &  KP2+MW2*SW2+2D0*CW2*T+4D0*CW2*U))/CW2
        K(27) = 50D0*D2(0,0)+14D0*D3(0,0,1)+14D0*D3(0,0,2)+14D0*
     &  D3(0,0,3)+D3(3,3,3)*KM2+D3(1,1,1)*KP2+D3(2,2,2)*U+D3(2,3,3)*
     &  (2D0*KM2+U)+D3(1,1,2)*(2D0*KP2+U)+D3(1,3,3)*(KM2+T+U)+D3(1,
     &  1,3)*(KP2+T+U)+D2(3,3)*(8D0*KM2+2D0*T+U)+D2(1,1)*(8D0*KP2+
     &  2D0*T+U)+D3(2,2,3)*(KM2+2D0*U)+D3(1,2,2)*(KP2+2D0*U)+D3(1,2,
     &  3)*(KM2+KP2+T+3D0*U)+2D0*D2(2,3)*(5D0*KM2+KP2+T+4D0*U)+2D0*
     &  D2(1,2)*(KM2+5D0*KP2+T+4D0*U)+D2(2,2)*(2D0*KM2+2D0*KP2+7D0*
     &  U)+D2(1,3)*(3D0*KM2+3D0*KP2+9D0*T+7D0*U)+(D0*(2D0*MW2*SW2+
     &  4D0*CW2*T+3D0*CW2*U))/CW2+(D1(3)*(3D0*CW2*KM2+2D0*MW2*SW2+
     &  6D0*CW2*T+8D0*CW2*U))/CW2+(D1(1)*(3D0*CW2*KP2+2D0*MW2*SW2+
     &  6D0*CW2*T+8D0*CW2*U))/CW2+(D1(2)*(2D0*CW2*KM2+2D0*CW2*KP2+
     &  2D0*MW2*SW2+4D0*CW2*T+9D0*CW2*U))/CW2

        DF(1,-1) = (K(1)*K(26))/SW2
        DF(1,1) = -(2D0*K(26))
        DF(2,-1) = (K(1)*K(27))/SW2
        DF(2,1) = -(2D0*K(27))
        DF(3,-1) = -((K(1)*K(25))/SW2)
        DF(3,1) = 2D0*K(25)
        DF(4,-1) = -((K(1)*K(24))/SW2)
        DF(4,1) = 2D0*K(24)
        DF(5,-1) = (K(1)*K(22))/SW2
        DF(5,1) = -(2D0*K(22))
        DF(6,-1) = -((K(1)*K(23))/SW2)
        DF(6,1) = 2D0*K(23)
        DF(7,-1) = -((8D0*K(1)*K(21))/SW2)
        DF(7,1) = 16D0*K(21)
        DF(8,-1) = -((8D0*K(1)*K(18))/SW2)
        DF(8,1) = 16D0*K(18)
        DF(9,-1) = (8D0*K(1)*K(19))/SW2
        DF(9,1) = -(16D0*K(19))
        DF(10,-1) = (8D0*K(1)*K(20))/SW2
        DF(10,1) = -(16D0*K(20))

        PRE = -((ALPHA)/(8D0*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T4,N30)+G(T4,N31)

        CALL D01234_(KP2,0d0,0d0,KM2,S,U,MW,MZ,0d0,MZ,D0,D1,D2,D3,D4,
     &  C0,C1,C2,C3,3,0)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2+U
        K(3) = KM2+2D0*U
        K(4) = 2D0*KM2+U
        K(5) = KP2+U
        K(6) = KP2+2D0*U
        K(7) = 2D0*KP2+U
        K(8) = T+U
        K(9) = KM2+T+U
        K(10) = KM2+T+2D0*U
        K(11) = 2D0*KM2+T+U
        K(12) = KP2+T+U
        K(13) = KP2+T+2D0*U
        K(14) = 2D0*KP2+T+U
        K(15) = MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*U
        K(16) = MW2*SW2**2-2D0*CW2**2*T-CW2**2*U
        K(17) = KM2+KP2+T+3D0*U
        K(18) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(19) = 2D0*D1(2)+2D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,
     &  3)
        K(20) = 2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,1,2)+2D0*D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,
     &  3)
        K(21) = 2D0*D1(2)+3D0*D2(1,2)+3D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)
        K(22) = 26D0*D2(0,0)+26D0*D3(0,0,2)+12D0*D3(0,0,3)+2D0*D3(3,
     &  3,3)*KM2+3D0*D3(1,1,2)*KP2+2D0*D3(1,1,3)*KP2+D2(2,3)*(8D0*
     &  KM2+5D0*KP2-3D0*S+T)+3D0*D3(2,2,2)*U+3D0*D2(1,2)*(KP2+U)+
     &  3D0*D3(1,2,2)*(KP2+U)+2D0*D2(2,2)*(2D0*KP2+U)+2D0*D3(1,3,3)*
     &  (T+U)+2D0*D2(3,3)*(2D0*KM2+T+U)+2D0*D2(1,3)*(2D0*KP2+T+U)+
     &  D1(2)*(4D0*KP2-U)+D3(2,3,3)*(5D0*KM2+2D0*U)+D3(2,2,3)*(3D0*
     &  KM2+5D0*U)+D3(1,2,3)*(2D0*KP2+3D0*T+5D0*U)-2D0*D1(3)*CW2**(-
     &  2)*(MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*U)
        K(23) = 26D0*D2(0,0)+12D0*D3(0,0,1)+26D0*D3(0,0,2)+2D0*D3(1,
     &  3,3)*KM2+3D0*D3(2,3,3)*KM2+2D0*D3(1,1,1)*KP2+D2(1,2)*(5D0*
     &  KM2+8D0*KP2-3D0*S+T)+3D0*D3(2,2,2)*U+3D0*D2(2,3)*(KM2+U)+
     &  3D0*D3(2,2,3)*(KM2+U)+2D0*D2(2,2)*(2D0*KM2+U)+2D0*D3(1,1,3)*
     &  (T+U)+2D0*D2(1,3)*(2D0*KM2+T+U)+2D0*D2(1,1)*(2D0*KP2+T+U)+
     &  D1(2)*(4D0*KM2-U)+D3(1,1,2)*(5D0*KP2+2D0*U)+D3(1,2,2)*(3D0*
     &  KP2+5D0*U)+D3(1,2,3)*(2D0*KM2+3D0*T+5D0*U)-2D0*D1(1)*CW2**(-
     &  2)*(MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*U)
        K(24) = 22D0*D2(0,0)-14D0*D3(0,0,2)-14D0*D3(0,0,3)-D3(3,3,
     &  3)*KM2+4D0*D2(1,1)*KP2-D3(1,1,2)*KP2-D3(1,1,3)*KP2-D3(2,2,
     &  2)*U-D3(2,3,3)*(2D0*KM2+U)-D3(1,2,2)*(KP2+U)-D3(1,3,3)*(T+
     &  U)+D2(3,3)*(5D0*KM2+2D0*T-U)-D3(2,2,3)*(KM2+2D0*U)+2D0*
     &  D1(1)*(KP2+T+2D0*U)-D3(1,2,3)*(KP2+T+2D0*U)-2D0*D2(2,2)*
     &  (KM2-KP2-3D0*U)+D2(2,3)*(3D0*KM2+2D0*KP2+2D0*T+5D0*U)+D2(1,
     &  2)*(5D0*KP2-2D0*T+7D0*U)+D2(1,3)*(KP2+2D0*T+7D0*U)-2D0*CW2**
     &  (-2)*D0*(MW2*SW2**2-2D0*CW2**2*T-CW2**2*U)+D1(3)*CW2**(-2)*
     &  (2D0*CW2**2*KM2-2D0*MW2*SW2**2+6D0*CW2**2*T+5D0*CW2**2*U)+
     &  D1(2)*CW2**(-2)*(-(2D0*CW2**2*KM2)+2D0*CW2**2*KP2-2D0*MW2*
     &  SW2**2+4D0*CW2**2*T+9D0*CW2**2*U)
        K(25) = 22D0*D2(0,0)-14D0*D3(0,0,1)-14D0*D3(0,0,2)+4D0*D2(3,
     &  3)*KM2-D3(1,3,3)*KM2-D3(2,3,3)*KM2-D3(1,1,1)*KP2-D3(2,2,2)*
     &  U-D3(2,2,3)*(KM2+U)-D3(1,1,2)*(2D0*KP2+U)-D3(1,1,3)*(T+U)+
     &  D2(1,1)*(5D0*KP2+2D0*T-U)-D3(1,2,2)*(KP2+2D0*U)+2D0*D1(3)*
     &  (KM2+T+2D0*U)-D3(1,2,3)*(KM2+T+2D0*U)+2D0*D2(2,2)*(KM2-KP2+
     &  3D0*U)+D2(1,2)*(2D0*KM2+3D0*KP2+2D0*T+5D0*U)+D2(2,3)*(5D0*
     &  KM2-2D0*T+7D0*U)+D2(1,3)*(KM2+2D0*T+7D0*U)-2D0*CW2**(-2)*D0*
     &  (MW2*SW2**2-2D0*CW2**2*T-CW2**2*U)+D1(1)*CW2**(-2)*(2D0*
     &  CW2**2*KP2-2D0*MW2*SW2**2+6D0*CW2**2*T+5D0*CW2**2*U)+D1(2)*
     &  CW2**(-2)*(2D0*CW2**2*KM2-2D0*CW2**2*KP2-2D0*MW2*SW2**2+4D0*
     &  CW2**2*T+9D0*CW2**2*U)
        K(26) = -(24D0*D2(0,0))-6D0*D3(0,0,1)-6D0*D3(0,0,2)-6D0*
     &  D3(0,0,3)-D3(3,3,3)*KM2-D3(1,1,1)*KP2-D2(1,3)*(5D0*KM2+5D0*
     &  KP2-3D0*S)-D3(2,2,2)*U-D3(2,3,3)*(2D0*KM2+U)-D3(1,1,2)*(2D0*
     &  KP2+U)-D3(1,3,3)*(KM2+T+U)-D2(3,3)*(3D0*KM2+T+U)-D3(1,1,3)*
     &  (KP2+T+U)-D2(1,1)*(3D0*KP2+T+U)-D3(2,2,3)*(KM2+2D0*U)-D3(1,
     &  2,2)*(KP2+2D0*U)-D2(2,2)*(KM2+KP2+3D0*U)-D3(1,2,3)*(KM2+KP2+
     &  T+3D0*U)-D2(2,3)*(4D0*KM2+KP2+T+4D0*U)-D2(1,2)*(KM2+4D0*KP2+
     &  T+4D0*U)+CW2**(-2)*D0*(MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*
     &  U)+D1(3)*CW2**(-2)*(-(2D0*CW2**2*KM2)+MW2*SW2**2-3D0*CW2**2*
     &  T-3D0*CW2**2*U)+D1(1)*CW2**(-2)*(-(2D0*CW2**2*KP2)+MW2*SW2**
     &  2-3D0*CW2**2*T-3D0*CW2**2*U)-D1(2)*CW2**(-2)*(CW2**2*KM2+
     &  CW2**2*KP2-MW2*SW2**2+2D0*CW2**2*T+4D0*CW2**2*U)
        K(27) = -(50D0*D2(0,0))-14D0*D3(0,0,1)-14D0*D3(0,0,2)-14D0*
     &  D3(0,0,3)-D3(3,3,3)*KM2-D3(1,1,1)*KP2-D3(2,2,2)*U-D3(2,3,3)*
     &  (2D0*KM2+U)-D3(1,1,2)*(2D0*KP2+U)-D3(1,3,3)*(KM2+T+U)-D3(1,
     &  1,3)*(KP2+T+U)-D2(3,3)*(8D0*KM2+2D0*T+U)-D2(1,1)*(8D0*KP2+
     &  2D0*T+U)-D3(2,2,3)*(KM2+2D0*U)-D3(1,2,2)*(KP2+2D0*U)-D3(1,2,
     &  3)*(KM2+KP2+T+3D0*U)-2D0*D2(2,3)*(5D0*KM2+KP2+T+4D0*U)-2D0*
     &  D2(1,2)*(KM2+5D0*KP2+T+4D0*U)-D2(2,2)*(2D0*KM2+2D0*KP2+7D0*
     &  U)-D2(1,3)*(3D0*KM2+3D0*KP2+9D0*T+7D0*U)+CW2**(-2)*D0*(2D0*
     &  MW2*SW2**2-4D0*CW2**2*T-3D0*CW2**2*U)+D1(3)*CW2**(-2)*(-
     &  (3D0*CW2**2*KM2)+2D0*MW2*SW2**2-6D0*CW2**2*T-8D0*CW2**2*U)+
     &  D1(1)*CW2**(-2)*(-(3D0*CW2**2*KP2)+2D0*MW2*SW2**2-6D0*CW2**
     &  2*T-8D0*CW2**2*U)+D1(2)*CW2**(-2)*(-(2D0*CW2**2*KM2)-2D0*
     &  CW2**2*KP2+2D0*MW2*SW2**2-4D0*CW2**2*T-9D0*CW2**2*U)

        DF(1,-1) = K(1)**2*K(26)*SW2**(-2)
        DF(1,1) = 4D0*K(26)
        DF(2,-1) = K(1)**2*K(27)*SW2**(-2)
        DF(2,1) = 4D0*K(27)
        DF(3,-1) = K(1)**2*K(25)*SW2**(-2)
        DF(3,1) = 4D0*K(25)
        DF(4,-1) = K(1)**2*K(24)*SW2**(-2)
        DF(4,1) = 4D0*K(24)
        DF(5,-1) = K(1)**2*K(22)*SW2**(-2)
        DF(5,1) = 4D0*K(22)
        DF(6,-1) = K(1)**2*K(23)*SW2**(-2)
        DF(6,1) = 4D0*K(23)
        DF(7,-1) = 8D0*K(1)**2*K(21)*SW2**(-2)
        DF(7,1) = 32D0*K(21)
        DF(8,-1) = 8D0*K(1)**2*K(18)*SW2**(-2)
        DF(8,1) = 32D0*K(18)
        DF(9,-1) = -(8D0*K(1)**2*K(19)*SW2**(-2))
        DF(9,1) = -(32D0*K(19))
        DF(10,-1) = -(8D0*K(1)**2*K(20)*SW2**(-2))
        DF(10,1) = -(32D0*K(20))

        PRE = (ALPHA)/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

	end

************************************************************************
        subroutine add_box_bgf(F)
************************************************************************
*	contributions of box corrections
*	background-field formalism
*-----------------------------------------------------------------------
*       27.5.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        complex*16 D0,D1(3),D2(0:3,0:3),D3(0:3,0:3,3)
        complex*16 D4(0:3,0:3,0:3,0:3)
        COMPLEX*16 K(17)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

C       Process: aF1F1aV30V30,   Model: SMbgf

C       G(T1,N2)
C        no contribution

C       G(T1,N4)
C        no contribution

C       G(T1,N1)

        CALL C0123_(0D0,0D0,S,MW,0D0,MW,C0,C1,C2,C3,B0,B1,B2,1,0)

        DF(3,-1) = C1(2)
        DF(4,-1) = -(C1(1))-C1(2)-C0
        DF(5,-1) = -(C1(1))-2D0*C1(2)-C0
        DF(6,-1) = -(DF(5,-1))

        PRE = -((ALPHA*SW2**(-2))/(2D0*PI))

        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

C       G(T1,N3)
C        no contribution

C       G(T1,N5)
C        no contribution

C       G(T2,N8)

        CALL D01234_(mxe2,KP2,mxe2,KM2,T,U,mxe,0D0,MW,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,401)

        K(1) = KM2-T
        K(2) = KP2-U

        DF(1,-1) = -(2D0*D2(0,0))-2D0*D2(1,3)*K(1)+D1(1)*K(2)+2D0*
     &  D2(1,2)*K(2)-2D0*D1(3)*KM2-2D0*D2(3,3)*KM2-2D0*D2(2,2)*U-
     &  2D0*D2(2,3)*(KM2+U)+D1(2)*(KP2-T-2D0*U)
        DF(2,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,2)+2D0*D3(0,0,3)+
     &  D2(1,3)*K(1)-D2(1,2)*K(2)+D1(3)*KM2+D2(3,3)*KM2+D1(2)*(KM2-
     &  S)+D2(2,2)*U+D2(2,3)*(KM2+U)))
        DF(3,-1) = 2D0*(2D0*D2(0,0)-2D0*D3(0,0,2)+D2(1,3)*K(1)+
     &  D1(3)*KM2+D2(3,3)*KM2+D2(2,2)*KP2+D1(2)*T+D2(2,3)*(KM2+T))
        DF(4,-1) = 2D0*(2D0*D2(0,0)+4D0*D3(0,0,1)+4D0*D3(0,0,2)+4D0*
     &  D3(0,0,3)+D2(1,1)*K(1)+D3(1,1,3)*K(1)-D3(1,1,2)*K(2)+D1(1)*
     &  KM2+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,3)*KM2+D1(2)*(KM2-KP2+
     &  T)+D3(1,3,3)*(2D0*KM2-T)+D2(1,3)*(3D0*KM2-2D0*T)+D3(2,2,2)*
     &  U+D2(2,2)*(KM2+U)+D3(2,3,3)*(2D0*KM2+U)+D2(2,3)*(3D0*KM2+U)-
     &  D3(1,2,2)*(KP2-2D0*U)+D3(2,2,3)*(KM2+2D0*U)+D2(1,2)*(KM2-
     &  2D0*KP2+S+2D0*U)+D3(1,2,3)*(KM2-2D0*KP2+S+3D0*U))
        DF(5,-1) = -(2D0*(2D0*D2(0,0)-2D0*D3(0,0,1)-2D0*D3(0,0,2)+
     &  D1(1)*K(2)+D1(2)*K(2)+D2(1,1)*K(2)+2D0*D2(1,2)*K(2)+D2(2,2)*
     &  K(2)+D2(1,3)*(T-U)+D2(2,3)*(T-U)))
        DF(6,-1) = -(2D0*(-(2D0*D2(0,0))+4D0*D3(0,0,1)+4D0*D3(0,0,
     &  2)+D2(1,1)*K(1)+D3(1,1,3)*K(1)-D3(1,1,2)*K(2)+D1(1)*KM2-
     &  D1(3)*KM2-D2(3,3)*KM2+D3(1,3,3)*KM2+D3(2,3,3)*KM2+D2(1,2)*
     &  (KM2+S)+D2(1,3)*T-D2(2,3)*U+D3(2,2,2)*U+D3(2,2,3)*(KM2+U)+
     &  D3(1,2,3)*(2D0*KM2-T+U)+D1(2)*(KM2-U)+D2(2,2)*(KM2-U)-D3(1,
     &  2,2)*(KP2-2D0*U)))
        DF(7,-1) = -(4D0*(D1(2)+D2(1,2)-D3(1,2,2)-D3(1,2,3)-D3(2,2,
     &  2)-2D0*D3(2,2,3)-D3(2,3,3)))
        DF(8,-1) = -(4D0*(D1(2)+D2(1,3)+D2(2,3)-D3(1,1,2)-D3(1,1,3)-
     &  2D0*D3(1,2,2)-2D0*D3(1,2,3)-D3(2,2,2)-D3(2,2,3)))
        DF(9,-1) = 4D0*(D1(2)-D2(1,3)-D3(1,1,2)-D3(1,1,3)-2D0*D3(1,
     &  2,2)-3D0*D3(1,2,3)-D3(1,3,3)-D3(2,2,2)-2D0*D3(2,2,3)-D3(2,3,
     &  3))
        DF(10,-1) = 4D0*(D1(2)+D2(1,2)+D2(2,3)-D3(1,2,2)-D3(1,2,3)-
     &  D3(2,2,2)-D3(2,2,3))

        PRE = -((ALPHA)/(4D0*PI*SW2))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T2,N9)

        CALL D01234_(0d0,KP2,0d0,KM2,T,U,0d0,MZ,MW,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = KM2-T
        K(2) = KP2-U

        DF(1,-1) = -(2D0*D2(0,0))-2D0*D2(1,3)*K(1)+D1(1)*K(2)+2D0*
     &  D2(1,2)*K(2)-2D0*D1(3)*KM2-2D0*D2(3,3)*KM2-2D0*D2(2,2)*U-
     &  2D0*D2(2,3)*(KM2+U)+D1(2)*(KP2-T-2D0*U)
        DF(2,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,2)+2D0*D3(0,0,3)+
     &  D2(1,3)*K(1)-D2(1,2)*K(2)+D1(3)*KM2+D2(3,3)*KM2+D1(2)*(KM2-
     &  S)+D2(2,2)*U+D2(2,3)*(KM2+U)))
        DF(3,-1) = 2D0*(2D0*D2(0,0)-2D0*D3(0,0,2)+D2(1,3)*K(1)+
     &  D1(3)*KM2+D2(3,3)*KM2+D2(2,2)*KP2+D1(2)*T+D2(2,3)*(KM2+T))
        DF(4,-1) = 2D0*(2D0*D2(0,0)+4D0*D3(0,0,1)+4D0*D3(0,0,2)+4D0*
     &  D3(0,0,3)+D2(1,1)*K(1)+D3(1,1,3)*K(1)-D3(1,1,2)*K(2)+D1(1)*
     &  KM2+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,3)*KM2+D1(2)*(KM2-KP2+
     &  T)+D3(1,3,3)*(2D0*KM2-T)+D2(1,3)*(3D0*KM2-2D0*T)+D3(2,2,2)*
     &  U+D2(2,2)*(KM2+U)+D3(2,3,3)*(2D0*KM2+U)+D2(2,3)*(3D0*KM2+U)-
     &  D3(1,2,2)*(KP2-2D0*U)+D3(2,2,3)*(KM2+2D0*U)+D2(1,2)*(KM2-
     &  2D0*KP2+S+2D0*U)+D3(1,2,3)*(KM2-2D0*KP2+S+3D0*U))
        DF(5,-1) = -(2D0*(2D0*D2(0,0)-2D0*D3(0,0,1)-2D0*D3(0,0,2)+
     &  D1(1)*K(2)+D1(2)*K(2)+D2(1,1)*K(2)+2D0*D2(1,2)*K(2)+D2(2,2)*
     &  K(2)+D2(1,3)*(T-U)+D2(2,3)*(T-U)))
        DF(6,-1) = -(2D0*(-(2D0*D2(0,0))+4D0*D3(0,0,1)+4D0*D3(0,0,
     &  2)+D2(1,1)*K(1)+D3(1,1,3)*K(1)-D3(1,1,2)*K(2)+D1(1)*KM2-
     &  D1(3)*KM2-D2(3,3)*KM2+D3(1,3,3)*KM2+D3(2,3,3)*KM2+D2(1,2)*
     &  (KM2+S)+D2(1,3)*T-D2(2,3)*U+D3(2,2,2)*U+D3(2,2,3)*(KM2+U)+
     &  D3(1,2,3)*(2D0*KM2-T+U)+D1(2)*(KM2-U)+D2(2,2)*(KM2-U)-D3(1,
     &  2,2)*(KP2-2D0*U)))
        DF(7,-1) = -(4D0*(D1(2)+D2(1,2)-D3(1,2,2)-D3(1,2,3)-D3(2,2,
     &  2)-2D0*D3(2,2,3)-D3(2,3,3)))
        DF(8,-1) = -(4D0*(D1(2)+D2(1,3)+D2(2,3)-D3(1,1,2)-D3(1,1,3)-
     &  2D0*D3(1,2,2)-2D0*D3(1,2,3)-D3(2,2,2)-D3(2,2,3)))
        DF(9,-1) = 4D0*(D1(2)-D2(1,3)-D3(1,1,2)-D3(1,1,3)-2D0*D3(1,
     &  2,2)-3D0*D3(1,2,3)-D3(1,3,3)-D3(2,2,2)-2D0*D3(2,2,3)-D3(2,3,
     &  3))
        DF(10,-1) = 4D0*(D1(2)+D2(1,2)+D2(2,3)-D3(1,2,2)-D3(1,2,3)-
     &  D3(2,2,2)-D3(2,2,3))

        PRE = -((ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(8D0*PI))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T2,N6)

        CALL D01234_(mxe2,KP2,mxe2,KM2,T,U,MW,0D0,mxe,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,401)

        K(1) = KM2-T
        K(2) = KP2-U

        DF(1,-1) = 2D0*D2(0,0)+D1(1)*K(1)+2D0*D2(1,3)*K(1)-2D0*D2(1,
     &  2)*K(2)+K(1)*D0+2D0*D2(3,3)*KM2+2D0*D2(2,2)*U+2D0*D2(2,3)*
     &  (KM2+U)+D1(3)*(2D0*KM2-T+U)+D1(2)*(K(1)+2D0*U)
        DF(2,-1) = 2D0*(-(2D0*D3(0,0,2))-2D0*D3(0,0,3)+D1(1)*K(1)+
     &  D2(1,3)*K(1)-D2(1,2)*K(2)+K(1)*D0+D2(3,3)*KM2+D1(3)*(2D0*
     &  KM2-T)+D2(2,2)*U+D1(2)*(K(1)+U)+D2(2,3)*(KM2+U))
        DF(3,-1) = -(2D0*(-(2D0*D2(0,0))-4D0*D3(0,0,2)+D1(1)*K(1)-
     &  D3(1,2,3)*K(1)+D2(1,2)*K(2)+D3(1,2,2)*K(2)+K(1)*D0+D1(3)*
     &  KM2-D3(2,3,3)*KM2+D1(2)*S-D3(2,2,2)*U-D2(2,3)*(K(1)+U)-D3(2,
     &  2,3)*(KM2+U)+D2(2,2)*(KP2-2D0*U)))
        DF(4,-1) = -(2D0*(4D0*D2(0,0)+2D0*D3(0,0,1)+2D0*D3(0,0,2)+
     &  2D0*D3(0,0,3)+2D0*D1(1)*K(1)+D2(1,1)*K(1)+2D0*D2(1,3)*K(1)+
     &  K(1)*D0+D2(2,2)*KM2+2D0*D2(2,3)*KM2+D2(3,3)*KM2+D1(2)*(2D0*
     &  KM2-T)+D1(3)*(2D0*KM2-T)+D2(1,2)*(KM2-2D0*KP2+S+U)))
        DF(5,-1) = -(2D0*(6D0*D2(0,0)+4D0*D3(0,0,1)+4D0*D3(0,0,2)+
     &  D3(1,1,3)*K(1)-D2(1,1)*K(2)-D3(1,1,2)*K(2)+2D0*D2(3,3)*KM2+
     &  D3(1,3,3)*KM2+D3(2,3,3)*KM2+D3(2,2,2)*U+D0*U+D3(2,2,3)*(KM2+
     &  U)+D1(3)*(2D0*KM2-T+U)+D3(1,2,3)*(2D0*KM2-T+U)+D2(1,3)*(3D0*
     &  KM2-3D0*T+U)-D1(1)*(KP2-2D0*U)-D3(1,2,2)*(KP2-2D0*U)+D2(2,
     &  3)*(3D0*KM2-T+3D0*U)-D1(2)*(KP2-4D0*U)-D2(2,2)*(KP2-4D0*U)-
     &  D2(1,2)*(4D0*KP2-5D0*U)))
        DF(6,-1) = 2D0*(4D0*D2(0,0)+2D0*D3(0,0,1)+2D0*D3(0,0,2)+
     &  D1(1)*K(1)+D2(1,1)*K(1)+D2(1,2)*(KM2-KP2+S)+D1(2)*(KM2-U)+
     &  D2(2,2)*(KM2-U))
        DF(7,-1) = 4D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+3D0*D2(2,
     &  3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3))
        DF(8,-1) = 4D0*(2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,
     &  2)+2D0*D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+2D0*D3(1,2,
     &  3)+D3(2,2,2)+D3(2,2,3))
        DF(9,-1) = -(4D0*(2D0*D1(2)+3D0*D2(1,2)+D2(1,3)+3D0*D2(2,2)+
     &  3D0*D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)))
        DF(10,-1) = -(4D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,3)))

        PRE = (ALPHA)/(4D0*PI*SW2)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T2,N7)

        CALL D01234_(0d0,KP2,0d0,KM2,T,U,MW,0D0,0d0,MZ,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = KM2-T
        K(2) = KP2-U

        DF(1,-1) = 2D0*D2(0,0)+D1(1)*K(1)+2D0*D2(1,3)*K(1)-2D0*D2(1,
     &  2)*K(2)+K(1)*D0+2D0*D2(3,3)*KM2+2D0*D2(2,2)*U+2D0*D2(2,3)*
     &  (KM2+U)+D1(3)*(2D0*KM2-T+U)+D1(2)*(K(1)+2D0*U)
        DF(2,-1) = 2D0*(-(2D0*D3(0,0,2))-2D0*D3(0,0,3)+D1(1)*K(1)+
     &  D2(1,3)*K(1)-D2(1,2)*K(2)+K(1)*D0+D2(3,3)*KM2+D1(3)*(2D0*
     &  KM2-T)+D2(2,2)*U+D1(2)*(K(1)+U)+D2(2,3)*(KM2+U))
        DF(3,-1) = -(2D0*(-(2D0*D2(0,0))-4D0*D3(0,0,2)+D1(1)*K(1)-
     &  D3(1,2,3)*K(1)+D2(1,2)*K(2)+D3(1,2,2)*K(2)+K(1)*D0+D1(3)*
     &  KM2-D3(2,3,3)*KM2+D1(2)*S-D3(2,2,2)*U-D2(2,3)*(K(1)+U)-D3(2,
     &  2,3)*(KM2+U)+D2(2,2)*(KP2-2D0*U)))
        DF(4,-1) = -(2D0*(4D0*D2(0,0)+2D0*D3(0,0,1)+2D0*D3(0,0,2)+
     &  2D0*D3(0,0,3)+2D0*D1(1)*K(1)+D2(1,1)*K(1)+2D0*D2(1,3)*K(1)+
     &  K(1)*D0+D2(2,2)*KM2+2D0*D2(2,3)*KM2+D2(3,3)*KM2+D1(2)*(2D0*
     &  KM2-T)+D1(3)*(2D0*KM2-T)+D2(1,2)*(KM2-2D0*KP2+S+U)))
        DF(5,-1) = -(2D0*(6D0*D2(0,0)+4D0*D3(0,0,1)+4D0*D3(0,0,2)+
     &  D3(1,1,3)*K(1)-D2(1,1)*K(2)-D3(1,1,2)*K(2)+2D0*D2(3,3)*KM2+
     &  D3(1,3,3)*KM2+D3(2,3,3)*KM2+D3(2,2,2)*U+D0*U+D3(2,2,3)*(KM2+
     &  U)+D1(3)*(2D0*KM2-T+U)+D3(1,2,3)*(2D0*KM2-T+U)+D2(1,3)*(3D0*
     &  KM2-3D0*T+U)-D1(1)*(KP2-2D0*U)-D3(1,2,2)*(KP2-2D0*U)+D2(2,
     &  3)*(3D0*KM2-T+3D0*U)-D1(2)*(KP2-4D0*U)-D2(2,2)*(KP2-4D0*U)-
     &  D2(1,2)*(4D0*KP2-5D0*U)))
        DF(6,-1) = 2D0*(4D0*D2(0,0)+2D0*D3(0,0,1)+2D0*D3(0,0,2)+
     &  D1(1)*K(1)+D2(1,1)*K(1)+D2(1,2)*(KM2-KP2+S)+D1(2)*(KM2-U)+
     &  D2(2,2)*(KM2-U))
        DF(7,-1) = 4D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+3D0*D2(2,
     &  3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3))
        DF(8,-1) = 4D0*(2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,
     &  2)+2D0*D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+2D0*D3(1,2,
     &  3)+D3(2,2,2)+D3(2,2,3))
        DF(9,-1) = -(4D0*(2D0*D1(2)+3D0*D2(1,2)+D2(1,3)+3D0*D2(2,2)+
     &  3D0*D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)))
        DF(10,-1) = -(4D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,3)))

        PRE = (ALPHA*SW2**(-2)*(1D0-2D0*SW2))/(8D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T3,N10)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,T,0D0,mxe,0D0,mxe,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,402)

        K(1) = T+U

        DF(1,-1) = 4D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*D3(0,
     &  0,3)+2D0*D2(1,3)*K(1)+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,3)*
     &  KM2+D3(1,3,3)*(K(1)+KM2)+D1(1)*KP2+2D0*D2(1,1)*KP2+D3(1,1,
     &  1)*KP2+D3(1,1,3)*(K(1)+KP2)+2D0*D2(2,2)*T+D3(2,2,2)*T+2D0*
     &  D2(2,3)*(KM2+T)+D3(2,3,3)*(2D0*KM2+T)+2D0*D2(1,2)*(KP2+T)+
     &  D3(1,1,2)*(2D0*KP2+T)-D1(2)*(S-T)+D3(2,2,3)*(KM2+2D0*T)+
     &  D3(1,2,2)*(KP2+2D0*T)+D3(1,2,3)*(KM2+KP2+3D0*T+U)
        DF(2,-1) = 2D0*(4D0*D2(0,0)+4D0*D3(0,0,1)+4D0*D3(0,0,2)+4D0*
     &  D3(0,0,3)+2D0*D2(1,3)*K(1)+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,
     &  3)*KM2+D3(1,3,3)*(K(1)+KM2)+D1(1)*KP2+2D0*D2(1,1)*KP2+D3(1,
     &  1,1)*KP2+D3(1,1,3)*(K(1)+KP2)+D1(2)*T+2D0*D2(2,2)*T+D3(2,2,
     &  2)*T+2D0*D2(2,3)*(KM2+T)+D3(2,3,3)*(2D0*KM2+T)+2D0*D2(1,2)*
     &  (KP2+T)+D3(1,1,2)*(2D0*KP2+T)+D3(2,2,3)*(KM2+2D0*T)+D3(1,2,
     &  2)*(KP2+2D0*T)+D3(1,2,3)*(KM2+KP2+3D0*T+U))
        DF(3,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,1)+D1(3)*KM2+D2(3,
     &  3)*KM2+D1(2)*T+D2(1,2)*T+D2(1,3)*T+D2(2,2)*T+D2(2,3)*(KM2+
     &  T)))
        DF(4,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,3)+D1(1)*KP2+D2(1,
     &  1)*KP2+D1(2)*T+D2(1,3)*T+D2(2,2)*T+D2(2,3)*T+D2(1,2)*(KP2+
     &  T)))
        DF(5,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,2)+6D0*D3(0,0,3)+
     &  D2(1,3)*K(1)+D3(1,3,3)*K(1)+D2(3,3)*KM2+D3(3,3,3)*KM2+D1(1)*
     &  KP2+D2(1,1)*KP2+D3(1,1,3)*KP2+D2(1,2)*T+D3(2,2,3)*T+D3(2,3,
     &  3)*(KM2+T)+D3(1,2,3)*(KP2+T)-D1(2)*(KP2-T)-D2(2,2)*(KP2-T)+
     &  D2(2,3)*(KM2+T-U)))
        DF(6,-1) = 2D0*(-(2D0*D2(0,0))-6D0*D3(0,0,1)-2D0*D3(0,0,2)-
     &  D2(1,3)*K(1)-D3(1,1,3)*K(1)-D1(3)*KM2-D2(3,3)*KM2-D3(1,3,3)*
     &  KM2-D2(1,1)*KP2-D3(1,1,1)*KP2-D2(2,3)*T-D3(1,2,2)*T-D3(1,2,
     &  3)*(KM2+T)-D3(1,1,2)*(KP2+T)+D1(2)*(KM2-T)+D2(2,2)*(KM2-T)-
     &  D2(1,2)*(KP2+T-U))
        DF(7,-1) = 4D0*(2D0*D2(1,3)+D3(1,1,3)+D3(1,2,3)+D3(1,3,3))
        DF(8,-1) = -(4D0*(D1(2)+D2(1,2)+D2(2,3)-D3(1,2,2)-D3(2,2,2)-
     &  D3(2,2,3)))
        DF(9,-1) = -(4D0*(D2(1,3)-D2(2,3)-D3(1,2,3)-D3(2,2,3)-D3(2,
     &  3,3)))
        DF(10,-1) = 4D0*(D2(1,2)-D2(1,3)+D3(1,1,2)+D3(1,2,2)+D3(1,2,
     &  3))

        PRE = (ALPHA)/(4D0*PI*SW2)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T3,N11)

        CALL D01234_(KP2,0d0,0d0,KM2,S,T,0D0,0d0,MZ,0d0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = T+U

        DF(1,-1) = 4D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*D3(0,
     &  0,3)+2D0*D2(1,3)*K(1)+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,3)*
     &  KM2+D3(1,3,3)*(K(1)+KM2)+D1(1)*KP2+2D0*D2(1,1)*KP2+D3(1,1,
     &  1)*KP2+D3(1,1,3)*(K(1)+KP2)+2D0*D2(2,2)*T+D3(2,2,2)*T+2D0*
     &  D2(2,3)*(KM2+T)+D3(2,3,3)*(2D0*KM2+T)+2D0*D2(1,2)*(KP2+T)+
     &  D3(1,1,2)*(2D0*KP2+T)-D1(2)*(S-T)+D3(2,2,3)*(KM2+2D0*T)+
     &  D3(1,2,2)*(KP2+2D0*T)+D3(1,2,3)*(KM2+KP2+3D0*T+U)
        DF(2,-1) = 2D0*(4D0*D2(0,0)+4D0*D3(0,0,1)+4D0*D3(0,0,2)+4D0*
     &  D3(0,0,3)+2D0*D2(1,3)*K(1)+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,
     &  3)*KM2+D3(1,3,3)*(K(1)+KM2)+D1(1)*KP2+2D0*D2(1,1)*KP2+D3(1,
     &  1,1)*KP2+D3(1,1,3)*(K(1)+KP2)+D1(2)*T+2D0*D2(2,2)*T+D3(2,2,
     &  2)*T+2D0*D2(2,3)*(KM2+T)+D3(2,3,3)*(2D0*KM2+T)+2D0*D2(1,2)*
     &  (KP2+T)+D3(1,1,2)*(2D0*KP2+T)+D3(2,2,3)*(KM2+2D0*T)+D3(1,2,
     &  2)*(KP2+2D0*T)+D3(1,2,3)*(KM2+KP2+3D0*T+U))
        DF(3,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,1)+D1(3)*KM2+D2(3,
     &  3)*KM2+D1(2)*T+D2(1,2)*T+D2(1,3)*T+D2(2,2)*T+D2(2,3)*(KM2+
     &  T)))
        DF(4,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,3)+D1(1)*KP2+D2(1,
     &  1)*KP2+D1(2)*T+D2(1,3)*T+D2(2,2)*T+D2(2,3)*T+D2(1,2)*(KP2+
     &  T)))
        DF(5,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,2)+6D0*D3(0,0,3)+
     &  D2(1,3)*K(1)+D3(1,3,3)*K(1)+D2(3,3)*KM2+D3(3,3,3)*KM2+D1(1)*
     &  KP2+D2(1,1)*KP2+D3(1,1,3)*KP2+D2(1,2)*T+D3(2,2,3)*T+D3(2,3,
     &  3)*(KM2+T)+D3(1,2,3)*(KP2+T)-D1(2)*(KP2-T)-D2(2,2)*(KP2-T)+
     &  D2(2,3)*(KM2+T-U)))
        DF(6,-1) = 2D0*(-(2D0*D2(0,0))-6D0*D3(0,0,1)-2D0*D3(0,0,2)-
     &  D2(1,3)*K(1)-D3(1,1,3)*K(1)-D1(3)*KM2-D2(3,3)*KM2-D3(1,3,3)*
     &  KM2-D2(1,1)*KP2-D3(1,1,1)*KP2-D2(2,3)*T-D3(1,2,2)*T-D3(1,2,
     &  3)*(KM2+T)-D3(1,1,2)*(KP2+T)+D1(2)*(KM2-T)+D2(2,2)*(KM2-T)-
     &  D2(1,2)*(KP2+T-U))
        DF(7,-1) = 4D0*(2D0*D2(1,3)+D3(1,1,3)+D3(1,2,3)+D3(1,3,3))
        DF(8,-1) = -(4D0*(D1(2)+D2(1,2)+D2(2,3)-D3(1,2,2)-D3(2,2,2)-
     &  D3(2,2,3)))
        DF(9,-1) = -(4D0*(D2(1,3)-D2(2,3)-D3(1,2,3)-D3(2,2,3)-D3(2,
     &  3,3)))
        DF(10,-1) = 4D0*(D2(1,2)-D2(1,3)+D3(1,1,2)+D3(1,2,2)+D3(1,2,
     &  3))

        PRE = (ALPHA*SW2**(-2)*(1D0-2D0*SW2)**2)/(16D0*CW2*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T3,N22)

        CALL D01234_(KP2,0D0,0D0,KM2,S,T,0D0,MW,0D0,MW,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = T+U

        DF(1,-1) = 8D0*D2(0,0)+D1(1)*K(1)+D1(2)*K(1)+D1(3)*K(1)+
     &  K(1)*D0
        DF(2,-1) = 2D0*(2D0*D2(0,0)-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*
     &  D3(0,0,3)+D1(3)*KM2+D1(1)*KP2+D1(2)*T+D0*T)
        DF(3,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,1)+D1(3)*KM2+D1(1)*
     &  KP2+D1(2)*T+D0*T+D2(1,2)*(KP2-T)-D2(1,3)*(T-U)))
        DF(4,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,3)+D1(3)*KM2+D1(1)*
     &  KP2+D1(2)*T+D0*T+D2(2,3)*(KM2-T)-D2(1,3)*(T-U)))
        DF(5,-1) = -(2D0*(4D0*D2(0,0)+2D0*D3(0,0,2)+D1(3)*K(1)+
     &  D1(2)*(KP2-T)+D2(2,2)*(KP2-T)-D2(2,3)*(T-U)))
        DF(6,-1) = -(2D0*(4D0*D2(0,0)+2D0*D3(0,0,2)+D1(1)*K(1)+
     &  D1(2)*(KM2-T)+D2(2,2)*(KM2-T)-D2(1,2)*(T-U)))
        DF(7,-1) = -(4D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3)))
        DF(8,-1) = 4D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,
     &  3)+D3(1,2,2)+D3(2,2,2)+D3(2,2,3))
        DF(9,-1) = 4D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3))
        DF(10,-1) = 4D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))

        PRE = -((ALPHA)/(4D0*PI*SW2))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T3,N20)

        CALL D01234_(KP2,0D0,0D0,KM2,S,T,MH,MW,0D0,MW,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,1,0)

        DF(1,-1) = D1(1)+D1(2)+D1(3)+D0
        DF(5,-1) = -(2D0*D1(3))
        DF(6,-1) = -(2D0*D1(1))

        PRE = (ALPHA*MW2*SW2**(-2))/(8D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE

C       G(T3,N12)+G(T3,N13)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,T,MW,0D0,mxe,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,403)

        K(1) = KM2-T
        K(2) = KP2-T
        K(3) = T-U
        K(4) = 2D0*MW2-T-U
        K(5) = 4D0*D2(0,0)+2D0*D3(0,0,2)+D1(2)*(KP2-T)+D2(2,2)*(KP2-
     &  T)-D2(2,3)*(T-U)-D1(3)*(2D0*MW2-T-U)
        K(6) = -(4D0*D2(0,0))-2D0*D3(0,0,2)-D1(2)*(KM2-T)-D2(2,2)*
     &  (KM2-T)+D2(1,2)*(T-U)+D1(1)*(2D0*MW2-T-U)
        K(7) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,2,
     &  2)+D3(2,2,2)+D3(2,2,3)
        K(8) = 2D0*D2(0,0)-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,0,
     &  3)+D1(3)*KM2+D1(1)*KP2+D1(2)*T+D0*T
        K(9) = 2D0*D2(0,0)+2D0*D3(0,0,1)+D1(3)*KM2+D1(1)*KP2+D1(2)*
     &  T+D0*T+D2(1,2)*(KP2-T)-D2(1,3)*(T-U)
        K(10) = 2D0*D2(0,0)+2D0*D3(0,0,3)+D1(3)*KM2+D1(1)*KP2+D1(2)*
     &  T+D0*T+D2(2,3)*(KM2-T)-D2(1,3)*(T-U)

        DF(1,-1) = -(8D0*D2(0,0))+D1(1)*K(4)+D1(2)*K(4)+D1(3)*K(4)+
     &  K(4)*D0
        DF(1,1) = DF(1,-1)
        DF(2,-1) = -(2D0*K(8))
        DF(2,1) = DF(2,-1)
        DF(3,-1) = 2D0*K(9)
        DF(3,1) = DF(3,-1)
        DF(4,-1) = 2D0*K(10)
        DF(4,1) = DF(4,-1)
        DF(5,-1) = 2D0*K(5)
        DF(5,1) = DF(5,-1)
        DF(6,-1) = -(2D0*K(6))
        DF(6,1) = DF(6,-1)
        DF(7,-1) = 4D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))
        DF(7,1) = DF(7,-1)
        DF(8,-1) = -(4D0*K(7))
        DF(8,1) = DF(8,-1)
        DF(9,-1) = -(4D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3)))
        DF(9,1) = DF(9,-1)
        DF(10,-1) = -(4D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3)))
        DF(10,1) = DF(10,-1)

        PRE = (ALPHA)/(2D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T3,N14)+G(T3,N15)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,T,MW,0D0,mxe,MZ,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,404)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2-T
        K(3) = KP2-T
        K(4) = T-U
        K(5) = CW2*MW2-MW2*SW2-CW2*T-CW2*U
        K(6) = 4D0*D2(0,0)+2D0*D3(0,0,2)+D1(2)*(KP2-T)+D2(2,2)*(KP2-
     &  T)-D2(2,3)*(T-U)-(D1(3)*(CW2*MW2-MW2*SW2-CW2*T-CW2*U))/CW2
        K(7) = -(4D0*D2(0,0))-2D0*D3(0,0,2)-D1(2)*(KM2-T)-D2(2,2)*
     &  (KM2-T)+D2(1,2)*(T-U)+(D1(1)*(CW2*MW2-MW2*SW2-CW2*T-CW2*U))/
     &  CW2
        K(8) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,2,
     &  2)+D3(2,2,2)+D3(2,2,3)
        K(9) = 2D0*D2(0,0)-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,0,
     &  3)+D1(3)*KM2+D1(1)*KP2+D1(2)*T+D0*T
        K(10) = 2D0*D2(0,0)+2D0*D3(0,0,1)+D1(3)*KM2+D1(1)*KP2+D1(2)*
     &  T+D0*T+D2(1,2)*(KP2-T)-D2(1,3)*(T-U)
        K(11) = 2D0*D2(0,0)+2D0*D3(0,0,3)+D1(3)*KM2+D1(1)*KP2+D1(2)*
     &  T+D0*T+D2(2,3)*(KM2-T)-D2(1,3)*(T-U)

        DF(1,-1) = (K(1)*(-(8D0*D2(0,0))+(D1(1)*K(5))/CW2+(D1(2)*
     &  K(5))/CW2+(D1(3)*K(5))/CW2+(K(5)*D0)/CW2))/SW2
        DF(1,1) = -(2D0*(-(8D0*D2(0,0))+(D1(1)*K(5))/CW2+(D1(2)*
     &  K(5))/CW2+(D1(3)*K(5))/CW2+(K(5)*D0)/CW2))
        DF(2,-1) = -((2D0*K(1)*K(9))/SW2)
        DF(2,1) = 4D0*K(9)
        DF(3,-1) = (2D0*K(1)*K(10))/SW2
        DF(3,1) = -(4D0*K(10))
        DF(4,-1) = (2D0*K(1)*K(11))/SW2
        DF(4,1) = -(4D0*K(11))
        DF(5,-1) = (2D0*K(1)*K(6))/SW2
        DF(5,1) = -(4D0*K(6))
        DF(6,-1) = -((2D0*K(1)*K(7))/SW2)
        DF(6,1) = 4D0*K(7)
        DF(7,-1) = (4D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))*
     &  K(1))/SW2
        DF(7,1) = -(8D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3)))
        DF(8,-1) = -((4D0*K(1)*K(8))/SW2)
        DF(8,1) = 8D0*K(8)
        DF(9,-1) = -((4D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3))*K(1))/SW2)
        DF(9,1) = 8D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+D3(2,
     &  3,3))
        DF(10,-1) = -((4D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))*K(1))/SW2)
        DF(10,1) = 8D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))

        PRE = (ALPHA)/(4D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T3,N16)+G(T3,N17)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,T,MW,MZ,mxe,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,404)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2-T
        K(3) = KP2-T
        K(4) = T-U
        K(5) = CW2*MW2-MW2*SW2-CW2*T-CW2*U
        K(6) = 4D0*D2(0,0)+2D0*D3(0,0,2)+D1(2)*(KP2-T)+D2(2,2)*(KP2-
     &  T)-D2(2,3)*(T-U)-(D1(3)*(CW2*MW2-MW2*SW2-CW2*T-CW2*U))/CW2
        K(7) = -(4D0*D2(0,0))-2D0*D3(0,0,2)-D1(2)*(KM2-T)-D2(2,2)*
     &  (KM2-T)+D2(1,2)*(T-U)+(D1(1)*(CW2*MW2-MW2*SW2-CW2*T-CW2*U))/
     &  CW2
        K(8) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,2,
     &  2)+D3(2,2,2)+D3(2,2,3)
        K(9) = 2D0*D2(0,0)-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,0,
     &  3)+D1(3)*KM2+D1(1)*KP2+D1(2)*T+D0*T
        K(10) = 2D0*D2(0,0)+2D0*D3(0,0,1)+D1(3)*KM2+D1(1)*KP2+D1(2)*
     &  T+D0*T+D2(1,2)*(KP2-T)-D2(1,3)*(T-U)
        K(11) = 2D0*D2(0,0)+2D0*D3(0,0,3)+D1(3)*KM2+D1(1)*KP2+D1(2)*
     &  T+D0*T+D2(2,3)*(KM2-T)-D2(1,3)*(T-U)

        DF(1,-1) = (K(1)*(-(8D0*D2(0,0))+(D1(1)*K(5))/CW2+(D1(2)*
     &  K(5))/CW2+(D1(3)*K(5))/CW2+(K(5)*D0)/CW2))/SW2
        DF(1,1) = -(2D0*(-(8D0*D2(0,0))+(D1(1)*K(5))/CW2+(D1(2)*
     &  K(5))/CW2+(D1(3)*K(5))/CW2+(K(5)*D0)/CW2))
        DF(2,-1) = -((2D0*K(1)*K(9))/SW2)
        DF(2,1) = 4D0*K(9)
        DF(3,-1) = (2D0*K(1)*K(10))/SW2
        DF(3,1) = -(4D0*K(10))
        DF(4,-1) = (2D0*K(1)*K(11))/SW2
        DF(4,1) = -(4D0*K(11))
        DF(5,-1) = (2D0*K(1)*K(6))/SW2
        DF(5,1) = -(4D0*K(6))
        DF(6,-1) = -((2D0*K(1)*K(7))/SW2)
        DF(6,1) = 4D0*K(7)
        DF(7,-1) = (4D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))*
     &  K(1))/SW2
        DF(7,1) = -(8D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3)))
        DF(8,-1) = -((4D0*K(1)*K(8))/SW2)
        DF(8,1) = 8D0*K(8)
        DF(9,-1) = -((4D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3))*K(1))/SW2)
        DF(9,1) = 8D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+D3(2,
     &  3,3))
        DF(10,-1) = -((4D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))*K(1))/SW2)
        DF(10,1) = 8D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))

        PRE = (ALPHA)/(4D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T3,N18)+G(T3,N19)

        CALL D01234_(KP2,0d0,0d0,KM2,S,T,MW,MZ,0d0,MZ,D0,D1,D2,D3,D4,
     &  C0,C1,C2,C3,3,0)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2-T
        K(3) = KP2-T
        K(4) = T-U
        K(5) = CW2**2*MW2-2D0*CW2*MW2*SW2+MW2*SW2**2-2D0*CW2**2*T-
     &  2D0*CW2**2*U
        K(6) = 8D0*D2(0,0)+4D0*D3(0,0,2)+2D0*D1(2)*(KP2-T)+2D0*D2(2,
     &  2)*(KP2-T)-2D0*D2(2,3)*(T-U)-D1(3)*CW2**(-2)*(CW2**2*MW2-
     &  2D0*CW2*MW2*SW2+MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*U)
        K(7) = -(8D0*D2(0,0))-4D0*D3(0,0,2)-2D0*D1(2)*(KM2-T)-2D0*
     &  D2(2,2)*(KM2-T)+2D0*D2(1,2)*(T-U)+D1(1)*CW2**(-2)*(CW2**2*
     &  MW2-2D0*CW2*MW2*SW2+MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*U)
        K(8) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,2,
     &  2)+D3(2,2,2)+D3(2,2,3)
        K(9) = 2D0*D2(0,0)-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,0,
     &  3)+D1(3)*KM2+D1(1)*KP2+D1(2)*T+D0*T
        K(10) = 2D0*D2(0,0)+2D0*D3(0,0,1)+D1(3)*KM2+D1(1)*KP2+D1(2)*
     &  T+D0*T+D2(1,2)*(KP2-T)-D2(1,3)*(T-U)
        K(11) = 2D0*D2(0,0)+2D0*D3(0,0,3)+D1(3)*KM2+D1(1)*KP2+D1(2)*
     &  T+D0*T+D2(2,3)*(KM2-T)-D2(1,3)*(T-U)

        DF(1,-1) = K(1)**2*(-(16D0*D2(0,0))+D1(1)*K(5)*CW2**(-2)+
     &  D1(2)*K(5)*CW2**(-2)+D1(3)*K(5)*CW2**(-2)+K(5)*CW2**(-2)*
     &  D0)*SW2**(-2)
        DF(1,1) = 4D0*(-(16D0*D2(0,0))+D1(1)*K(5)*CW2**(-2)+D1(2)*
     &  K(5)*CW2**(-2)+D1(3)*K(5)*CW2**(-2)+K(5)*CW2**(-2)*D0)
        DF(2,-1) = -(4D0*K(1)**2*K(9)*SW2**(-2))
        DF(2,1) = -(16D0*K(9))
        DF(3,-1) = 4D0*K(1)**2*K(10)*SW2**(-2)
        DF(3,1) = 16D0*K(10)
        DF(4,-1) = 4D0*K(1)**2*K(11)*SW2**(-2)
        DF(4,1) = 16D0*K(11)
        DF(5,-1) = 2D0*K(1)**2*K(6)*SW2**(-2)
        DF(5,1) = 8D0*K(6)
        DF(6,-1) = -(2D0*K(1)**2*K(7)*SW2**(-2))
        DF(6,1) = -(8D0*K(7))
        DF(7,-1) = 8D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))*
     &  K(1)**2*SW2**(-2)
        DF(7,1) = 32D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))
        DF(8,-1) = -(8D0*K(1)**2*K(8)*SW2**(-2))
        DF(8,1) = -(32D0*K(8))
        DF(9,-1) = -(8D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3))*K(1)**2*SW2**(-2))
        DF(9,1) = -(32D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3)))
        DF(10,-1) = -(8D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))*K(1)**2*SW2**(-2))
        DF(10,1) = -(32D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3)))

        PRE = (ALPHA)/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T3,N21)+G(T3,N23)

        CALL D01234_(KP2,0D0,0D0,KM2,S,T,MZ,MW,0D0,MW,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = MW2-2D0*CW2*T-2D0*CW2*U

        DF(1,-1) = D1(1)*K(1)+D1(2)*K(1)+D1(3)*K(1)-16D0*D2(0,0)*
     &  CW2+K(1)*D0
        DF(2,-1) = -(4D0*CW2*(2D0*D2(0,0)-2D0*D3(0,0,1)-2D0*D3(0,0,
     &  2)-2D0*D3(0,0,3)+D1(3)*KM2+D1(1)*KP2+D1(2)*T+D0*T))
        DF(3,-1) = 4D0*CW2*(2D0*D2(0,0)+2D0*D3(0,0,1)+D1(3)*KM2+
     &  D1(1)*KP2+D1(2)*T+D0*T+D2(1,2)*(KP2-T)-D2(1,3)*(T-U))
        DF(4,-1) = 4D0*CW2*(2D0*D2(0,0)+2D0*D3(0,0,3)+D1(3)*KM2+
     &  D1(1)*KP2+D1(2)*T+D0*T+D2(2,3)*(KM2-T)-D2(1,3)*(T-U))
        DF(5,-1) = -(2D0*(D1(3)*K(1)-8D0*D2(0,0)*CW2-4D0*D3(0,0,2)*
     &  CW2-2D0*D1(2)*CW2*(KP2-T)-2D0*D2(2,2)*CW2*(KP2-T)+2D0*D2(2,
     &  3)*CW2*(T-U)))
        DF(6,-1) = -(2D0*(D1(1)*K(1)-8D0*D2(0,0)*CW2-4D0*D3(0,0,2)*
     &  CW2-2D0*D1(2)*CW2*(KM2-T)-2D0*D2(2,2)*CW2*(KM2-T)+2D0*D2(1,
     &  2)*CW2*(T-U)))
        DF(7,-1) = 8D0*(D2(1,3)-D3(1,1,3)-D3(1,2,3)-D3(1,3,3))*CW2
        DF(8,-1) = -(8D0*(2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,2,2)+D3(2,2,2)+D3(2,2,3))*CW2)
        DF(9,-1) = -(8D0*(2D0*D2(1,3)+D2(2,3)+D3(1,2,3)+D3(2,2,3)+
     &  D3(2,3,3))*CW2)
        DF(10,-1) = -(8D0*(D2(1,2)+2D0*D2(1,3)+D3(1,1,2)+D3(1,2,2)+
     &  D3(1,2,3))*CW2)

        PRE = (ALPHA*SW2**(-2))/(8D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T4,N24)

        CALL D01234_(KP2,0d0,0d0,KM2,S,U,0d0,0D0,MW,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,0)

        K(1) = T+U

        DF(1,-1) = 4D0*D2(0,0)+6D0*D3(0,0,1)+6D0*D3(0,0,2)+6D0*D3(0,
     &  0,3)+2D0*D2(1,3)*K(1)+D1(3)*KM2+2D0*D2(3,3)*KM2+D3(3,3,3)*
     &  KM2+D3(1,3,3)*(K(1)+KM2)+D1(1)*KP2+2D0*D2(1,1)*KP2+D3(1,1,
     &  1)*KP2+D3(1,1,3)*(K(1)+KP2)+2D0*D2(2,2)*U+D3(2,2,2)*U+2D0*
     &  D2(2,3)*(KM2+U)+D3(2,3,3)*(2D0*KM2+U)+2D0*D2(1,2)*(KP2+U)+
     &  D3(1,1,2)*(2D0*KP2+U)-D1(2)*(S-U)+D3(2,2,3)*(KM2+2D0*U)+
     &  D3(1,2,2)*(KP2+2D0*U)+D3(1,2,3)*(KM2+KP2+T+3D0*U)
        DF(2,-1) = -(2D0*(-(2D0*D3(0,0,1))-2D0*D3(0,0,2)-2D0*D3(0,0,
     &  3)+D1(2)*S))
        DF(3,-1) = 2D0*(-(4D0*D3(0,0,1))-4D0*D3(0,0,2)-D3(1,1,3)*
     &  K(1)-D3(1,3,3)*KM2-D3(2,3,3)*KM2-D2(1,1)*KP2-2D0*D2(1,2)*
     &  KP2-D2(2,2)*KP2-D3(1,1,1)*KP2-D2(1,3)*T-D2(2,3)*T+D1(2)*
     &  (KM2-T)-D3(2,2,2)*U-D3(2,2,3)*(KM2+U)-D3(1,1,2)*(2D0*KP2+U)-
     &  D3(1,2,2)*(KP2+2D0*U)-D3(1,2,3)*(KM2+T+2D0*U))
        DF(4,-1) = 2D0*(-(4D0*D3(0,0,2))-4D0*D3(0,0,3)-D3(1,3,3)*
     &  K(1)-D2(2,2)*KM2-2D0*D2(2,3)*KM2-D2(3,3)*KM2-D3(3,3,3)*KM2-
     &  D3(1,1,2)*KP2-D3(1,1,3)*KP2-D2(1,2)*T-D2(1,3)*T+D1(2)*(KP2-
     &  T)-D3(2,2,2)*U-D3(2,3,3)*(2D0*KM2+U)-D3(1,2,2)*(KP2+U)-D3(2,
     &  2,3)*(KM2+2D0*U)-D3(1,2,3)*(KP2+T+2D0*U))
        DF(5,-1) = -(2D0*(2D0*D2(0,0)+2D0*D3(0,0,2)+6D0*D3(0,0,3)+
     &  D2(1,3)*K(1)+D3(1,3,3)*K(1)+D2(3,3)*KM2+D3(3,3,3)*KM2+D1(1)*
     &  KP2+D2(1,1)*KP2+D3(1,1,3)*KP2+D2(1,2)*U+D3(2,2,3)*U+D3(2,3,
     &  3)*(KM2+U)+D3(1,2,3)*(KP2+U)+D2(2,3)*(KM2-T+U)-D1(2)*(KP2-
     &  U)-D2(2,2)*(KP2-U)))
        DF(6,-1) = 2D0*(-(2D0*D2(0,0))-6D0*D3(0,0,1)-2D0*D3(0,0,2)-
     &  D2(1,3)*K(1)-D3(1,1,3)*K(1)-D1(3)*KM2-D2(3,3)*KM2-D3(1,3,3)*
     &  KM2-D2(1,1)*KP2-D3(1,1,1)*KP2-D2(2,3)*U-D3(1,2,2)*U-D3(1,2,
     &  3)*(KM2+U)-D3(1,1,2)*(KP2+U)-D2(1,2)*(KP2-T+U)+D1(2)*(KM2-
     &  U)+D2(2,2)*(KM2-U))
        DF(7,-1) = 4D0*(D1(2)-D3(1,1,2)-D3(1,1,3)-2D0*D3(1,2,2)-3D0*
     &  D3(1,2,3)-D3(1,3,3)-D3(2,2,2)-2D0*D3(2,2,3)-D3(2,3,3))
        DF(8,-1) = 4D0*(D1(2)+D2(1,2)+D2(2,3)-D3(1,2,2)-D3(2,2,2)-
     &  D3(2,2,3))
        DF(9,-1) = -(4D0*(D1(2)+D2(1,2)+D2(1,3)-D3(1,2,2)-D3(1,2,3)-
     &  D3(2,2,2)-2D0*D3(2,2,3)-D3(2,3,3)))
        DF(10,-1) = -(4D0*(D1(2)+D2(1,3)+D2(2,3)-D3(1,1,2)-2D0*D3(1,
     &  2,2)-D3(1,2,3)-D3(2,2,2)-D3(2,2,3)))

        PRE = (ALPHA*SW2**(-2))/(8D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE

C       G(T4,N25)+G(T4,N26)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,U,MW,0D0,mxe,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,403)

        K(1) = KM2-U
        K(2) = KP2-U
        K(3) = 2D0*MW2-T
        K(4) = T-U
        K(5) = KM2-2D0*MW2-S
        K(6) = KP2-2D0*MW2-S
        K(7) = 2D0*MW2-T-U
        K(8) = 4D0*D2(0,0)+2D0*D3(0,0,2)+D1(2)*(KP2-U)+D2(2,2)*(KP2-
     &  U)+D2(2,3)*(T-U)-D1(3)*(2D0*MW2-T-U)
        K(9) = -(4D0*D2(0,0))-2D0*D3(0,0,2)-D1(2)*(KM2-U)-D2(2,2)*
     &  (KM2-U)-D2(1,2)*(T-U)+D1(1)*(2D0*MW2-T-U)
        K(10) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(11) = -(6D0*D2(0,0))-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,
     &  0,3)-D1(1)*(KM2-2D0*MW2-S)-D1(3)*(KP2-2D0*MW2-S)+D1(2)*(2D0*
     &  MW2-T)+D0*(2D0*MW2-T)
        K(12) = 2D0*D1(2)+2D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,
     &  3)
        K(13) = 2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,1,2)+2D0*D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,
     &  3)
        K(14) = -(2D0*D2(0,0))+2D0*D3(0,0,2)+2D0*D3(0,0,3)+D1(1)*
     &  KP2-D1(2)*(KP2-2D0*MW2-S)-D1(3)*(KP2-2D0*MW2-S)+D0*(2D0*MW2-
     &  T)+D2(2,2)*(KM2-U)+D2(2,3)*(KM2-U)+D2(1,2)*(T-U)+D2(1,3)*(T-
     &  U)
        K(15) = -(2D0*D2(0,0))+2D0*D3(0,0,1)+2D0*D3(0,0,2)+D1(3)*
     &  KM2-D1(1)*(KM2-2D0*MW2-S)-D1(2)*(KM2-2D0*MW2-S)+D0*(2D0*MW2-
     &  T)+D2(1,2)*(KP2-U)+D2(2,2)*(KP2-U)+D2(1,3)*(T-U)+D2(2,3)*(T-
     &  U)
        K(16) = 2D0*D1(2)+3D0*D2(1,2)+3D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)

        DF(1,-1) = -(8D0*D2(0,0))+D1(1)*(K(3)-U)+D1(2)*(K(3)-U)+
     &  D1(3)*(K(3)-U)+D0*(K(3)-U)
        DF(1,1) = DF(1,-1)
        DF(2,-1) = 2D0*K(11)
        DF(2,1) = DF(2,-1)
        DF(3,-1) = -(2D0*K(15))
        DF(3,1) = DF(3,-1)
        DF(4,-1) = -(2D0*K(14))
        DF(4,1) = DF(4,-1)
        DF(5,-1) = 2D0*K(8)
        DF(5,1) = DF(5,-1)
        DF(6,-1) = -(2D0*K(9))
        DF(6,1) = DF(6,-1)
        DF(7,-1) = 4D0*K(16)
        DF(7,1) = DF(7,-1)
        DF(8,-1) = 4D0*K(10)
        DF(8,1) = DF(8,-1)
        DF(9,-1) = -(4D0*K(12))
        DF(9,1) = DF(9,-1)
        DF(10,-1) = -(4D0*K(13))
        DF(10,1) = DF(10,-1)

        PRE = (ALPHA)/(2D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T4,N29)+G(T4,N30)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,U,MW,0D0,mxe,MZ,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,404)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2-U
        K(3) = KP2-U
        K(4) = T-U
        K(5) = CW2*MW2-MW2*SW2-CW2*T
        K(6) = CW2*KM2-CW2*MW2-CW2*S+MW2*SW2
        K(7) = CW2*KP2-CW2*MW2-CW2*S+MW2*SW2
        K(8) = CW2*MW2-MW2*SW2-CW2*T-CW2*U
        K(9) = 4D0*D2(0,0)+2D0*D3(0,0,2)+D1(2)*(KP2-U)+D2(2,2)*(KP2-
     &  U)+D2(2,3)*(T-U)-(D1(3)*(CW2*MW2-MW2*SW2-CW2*T-CW2*U))/CW2
        K(10) = -(4D0*D2(0,0))-2D0*D3(0,0,2)-D1(2)*(KM2-U)-D2(2,2)*
     &  (KM2-U)-D2(1,2)*(T-U)+(D1(1)*(CW2*MW2-MW2*SW2-CW2*T-CW2*U))/
     &  CW2
        K(11) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(12) = -(6D0*D2(0,0))-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,
     &  0,3)-(D1(1)*(CW2*KM2-CW2*MW2-CW2*S+MW2*SW2))/CW2-(D1(3)*
     &  (CW2*KP2-CW2*MW2-CW2*S+MW2*SW2))/CW2+(D1(2)*(CW2*MW2-MW2*
     &  SW2-CW2*T))/CW2+(D0*(CW2*MW2-MW2*SW2-CW2*T))/CW2
        K(13) = 2D0*D1(2)+2D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,
     &  3)
        K(14) = 2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,1,2)+2D0*D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,
     &  3)
        K(15) = -(2D0*D2(0,0))+2D0*D3(0,0,2)+2D0*D3(0,0,3)+D1(1)*
     &  KP2-(D1(2)*(CW2*KP2-CW2*MW2-CW2*S+MW2*SW2))/CW2-(D1(3)*(CW2*
     &  KP2-CW2*MW2-CW2*S+MW2*SW2))/CW2+(D0*(CW2*MW2-MW2*SW2-CW2*
     &  T))/CW2+D2(2,2)*(KM2-U)+D2(2,3)*(KM2-U)+D2(1,2)*(T-U)+D2(1,
     &  3)*(T-U)
        K(16) = -(2D0*D2(0,0))+2D0*D3(0,0,1)+2D0*D3(0,0,2)+D1(3)*
     &  KM2-(D1(1)*(CW2*KM2-CW2*MW2-CW2*S+MW2*SW2))/CW2-(D1(2)*(CW2*
     &  KM2-CW2*MW2-CW2*S+MW2*SW2))/CW2+(D0*(CW2*MW2-MW2*SW2-CW2*
     &  T))/CW2+D2(1,2)*(KP2-U)+D2(2,2)*(KP2-U)+D2(1,3)*(T-U)+D2(2,
     &  3)*(T-U)
        K(17) = 2D0*D1(2)+3D0*D2(1,2)+3D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)

        DF(1,-1) = (K(1)*(-(8D0*D2(0,0))+(D1(1)*(K(5)-CW2*U))/CW2+
     &  (D1(2)*(K(5)-CW2*U))/CW2+(D1(3)*(K(5)-CW2*U))/CW2+(D0*(K(5)-
     &  CW2*U))/CW2))/SW2
        DF(1,1) = -(2D0*(-(8D0*D2(0,0))+(D1(1)*(K(5)-CW2*U))/CW2+
     &  (D1(2)*(K(5)-CW2*U))/CW2+(D1(3)*(K(5)-CW2*U))/CW2+(D0*(K(5)-
     &  CW2*U))/CW2))
        DF(2,-1) = (2D0*K(1)*K(12))/SW2
        DF(2,1) = -(4D0*K(12))
        DF(3,-1) = -((2D0*K(1)*K(16))/SW2)
        DF(3,1) = 4D0*K(16)
        DF(4,-1) = -((2D0*K(1)*K(15))/SW2)
        DF(4,1) = 4D0*K(15)
        DF(5,-1) = (2D0*K(1)*K(9))/SW2
        DF(5,1) = -(4D0*K(9))
        DF(6,-1) = -((2D0*K(1)*K(10))/SW2)
        DF(6,1) = 4D0*K(10)
        DF(7,-1) = (4D0*K(1)*K(17))/SW2
        DF(7,1) = -(8D0*K(17))
        DF(8,-1) = (4D0*K(1)*K(11))/SW2
        DF(8,1) = -(8D0*K(11))
        DF(9,-1) = -((4D0*K(1)*K(13))/SW2)
        DF(9,1) = 8D0*K(13)
        DF(10,-1) = -((4D0*K(1)*K(14))/SW2)
        DF(10,1) = 8D0*K(14)

        PRE = (ALPHA)/(4D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T4,N27)+G(T4,N28)

        CALL D01234_(KP2,mxe2,mxe2,KM2,S,U,MW,MZ,mxe,0D0,D0,D1,D2,D3,
     &  D4,C0,C1,C2,C3,3,404)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2-U
        K(3) = KP2-U
        K(4) = T-U
        K(5) = CW2*MW2-MW2*SW2-CW2*T
        K(6) = CW2*KM2-CW2*MW2-CW2*S+MW2*SW2
        K(7) = CW2*KP2-CW2*MW2-CW2*S+MW2*SW2
        K(8) = CW2*MW2-MW2*SW2-CW2*T-CW2*U
        K(9) = 4D0*D2(0,0)+2D0*D3(0,0,2)+D1(2)*(KP2-U)+D2(2,2)*(KP2-
     &  U)+D2(2,3)*(T-U)-(D1(3)*(CW2*MW2-MW2*SW2-CW2*T-CW2*U))/CW2
        K(10) = -(4D0*D2(0,0))-2D0*D3(0,0,2)-D1(2)*(KM2-U)-D2(2,2)*
     &  (KM2-U)-D2(1,2)*(T-U)+(D1(1)*(CW2*MW2-MW2*SW2-CW2*T-CW2*U))/
     &  CW2
        K(11) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(12) = -(6D0*D2(0,0))-2D0*D3(0,0,1)-2D0*D3(0,0,2)-2D0*D3(0,
     &  0,3)-(D1(1)*(CW2*KM2-CW2*MW2-CW2*S+MW2*SW2))/CW2-(D1(3)*
     &  (CW2*KP2-CW2*MW2-CW2*S+MW2*SW2))/CW2+(D1(2)*(CW2*MW2-MW2*
     &  SW2-CW2*T))/CW2+(D0*(CW2*MW2-MW2*SW2-CW2*T))/CW2
        K(13) = 2D0*D1(2)+2D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,
     &  3)
        K(14) = 2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,1,2)+2D0*D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,
     &  3)
        K(15) = -(2D0*D2(0,0))+2D0*D3(0,0,2)+2D0*D3(0,0,3)+D1(1)*
     &  KP2-(D1(2)*(CW2*KP2-CW2*MW2-CW2*S+MW2*SW2))/CW2-(D1(3)*(CW2*
     &  KP2-CW2*MW2-CW2*S+MW2*SW2))/CW2+(D0*(CW2*MW2-MW2*SW2-CW2*
     &  T))/CW2+D2(2,2)*(KM2-U)+D2(2,3)*(KM2-U)+D2(1,2)*(T-U)+D2(1,
     &  3)*(T-U)
        K(16) = -(2D0*D2(0,0))+2D0*D3(0,0,1)+2D0*D3(0,0,2)+D1(3)*
     &  KM2-(D1(1)*(CW2*KM2-CW2*MW2-CW2*S+MW2*SW2))/CW2-(D1(2)*(CW2*
     &  KM2-CW2*MW2-CW2*S+MW2*SW2))/CW2+(D0*(CW2*MW2-MW2*SW2-CW2*
     &  T))/CW2+D2(1,2)*(KP2-U)+D2(2,2)*(KP2-U)+D2(1,3)*(T-U)+D2(2,
     &  3)*(T-U)
        K(17) = 2D0*D1(2)+3D0*D2(1,2)+3D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)

        DF(1,-1) = (K(1)*(-(8D0*D2(0,0))+(D1(1)*(K(5)-CW2*U))/CW2+
     &  (D1(2)*(K(5)-CW2*U))/CW2+(D1(3)*(K(5)-CW2*U))/CW2+(D0*(K(5)-
     &  CW2*U))/CW2))/SW2
        DF(1,1) = -(2D0*(-(8D0*D2(0,0))+(D1(1)*(K(5)-CW2*U))/CW2+
     &  (D1(2)*(K(5)-CW2*U))/CW2+(D1(3)*(K(5)-CW2*U))/CW2+(D0*(K(5)-
     &  CW2*U))/CW2))
        DF(2,-1) = (2D0*K(1)*K(12))/SW2
        DF(2,1) = -(4D0*K(12))
        DF(3,-1) = -((2D0*K(1)*K(16))/SW2)
        DF(3,1) = 4D0*K(16)
        DF(4,-1) = -((2D0*K(1)*K(15))/SW2)
        DF(4,1) = 4D0*K(15)
        DF(5,-1) = (2D0*K(1)*K(9))/SW2
        DF(5,1) = -(4D0*K(9))
        DF(6,-1) = -((2D0*K(1)*K(10))/SW2)
        DF(6,1) = 4D0*K(10)
        DF(7,-1) = (4D0*K(1)*K(17))/SW2
        DF(7,1) = -(8D0*K(17))
        DF(8,-1) = (4D0*K(1)*K(11))/SW2
        DF(8,1) = -(8D0*K(11))
        DF(9,-1) = -((4D0*K(1)*K(13))/SW2)
        DF(9,1) = 8D0*K(13)
        DF(10,-1) = -((4D0*K(1)*K(14))/SW2)
        DF(10,1) = 8D0*K(14)

        PRE = (ALPHA)/(4D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T4,N31)+G(T4,N32)

        CALL D01234_(KP2,0d0,0d0,KM2,S,U,MW,MZ,0d0,MZ,D0,D1,D2,D3,D4,
     &  C0,C1,C2,C3,3,0)

        K(1) = 1D0-2D0*SW2
        K(2) = KM2-U
        K(3) = KP2-U
        K(4) = T-U
        K(5) = CW2**2*MW2-2D0*CW2*MW2*SW2+MW2*SW2**2-2D0*CW2**2*T
        K(6) = -(2D0*CW2**2*KM2)+CW2**2*MW2+2D0*CW2**2*S-2D0*CW2*
     &  MW2*SW2+MW2*SW2**2
        K(7) = -(2D0*CW2**2*KP2)+CW2**2*MW2+2D0*CW2**2*S-2D0*CW2*
     &  MW2*SW2+MW2*SW2**2
        K(8) = CW2**2*MW2-2D0*CW2*MW2*SW2+MW2*SW2**2-2D0*CW2**2*T-
     &  2D0*CW2**2*U
        K(9) = 8D0*D2(0,0)+4D0*D3(0,0,2)+2D0*D1(2)*(KP2-U)+2D0*D2(2,
     &  2)*(KP2-U)+2D0*D2(2,3)*(T-U)-D1(3)*CW2**(-2)*(CW2**2*MW2-
     &  2D0*CW2*MW2*SW2+MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*U)
        K(10) = -(8D0*D2(0,0))-4D0*D3(0,0,2)-2D0*D1(2)*(KM2-U)-2D0*
     &  D2(2,2)*(KM2-U)-2D0*D2(1,2)*(T-U)+D1(1)*CW2**(-2)*(CW2**2*
     &  MW2-2D0*CW2*MW2*SW2+MW2*SW2**2-2D0*CW2**2*T-2D0*CW2**2*U)
        K(11) = 2D0*D1(2)+2D0*D2(1,2)+3D0*D2(2,2)+2D0*D2(2,3)+D3(1,
     &  2,2)+D3(2,2,2)+D3(2,2,3)
        K(12) = -(12D0*D2(0,0))-4D0*D3(0,0,1)-4D0*D3(0,0,2)-4D0*
     &  D3(0,0,3)+D1(1)*CW2**(-2)*(-(2D0*CW2**2*KM2)+CW2**2*MW2+2D0*
     &  CW2**2*S-2D0*CW2*MW2*SW2+MW2*SW2**2)+D1(3)*CW2**(-2)*(-(2D0*
     &  CW2**2*KP2)+CW2**2*MW2+2D0*CW2**2*S-2D0*CW2*MW2*SW2+MW2*
     &  SW2**2)+D1(2)*CW2**(-2)*(CW2**2*MW2-2D0*CW2*MW2*SW2+MW2*
     &  SW2**2-2D0*CW2**2*T)+CW2**(-2)*D0*(CW2**2*MW2-2D0*CW2*MW2*
     &  SW2+MW2*SW2**2-2D0*CW2**2*T)
        K(13) = 2D0*D1(2)+2D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,
     &  3)
        K(14) = 2D0*D1(2)+3D0*D2(1,2)+2D0*D2(1,3)+3D0*D2(2,2)+2D0*
     &  D2(2,3)+D3(1,1,2)+2D0*D3(1,2,2)+D3(1,2,3)+D3(2,2,2)+D3(2,2,
     &  3)
        K(15) = -(4D0*D2(0,0))+4D0*D3(0,0,2)+4D0*D3(0,0,3)+2D0*
     &  D1(1)*KP2+D1(2)*CW2**(-2)*(-(2D0*CW2**2*KP2)+CW2**2*MW2+2D0*
     &  CW2**2*S-2D0*CW2*MW2*SW2+MW2*SW2**2)+D1(3)*CW2**(-2)*(-(2D0*
     &  CW2**2*KP2)+CW2**2*MW2+2D0*CW2**2*S-2D0*CW2*MW2*SW2+MW2*
     &  SW2**2)+CW2**(-2)*D0*(CW2**2*MW2-2D0*CW2*MW2*SW2+MW2*SW2**2-
     &  2D0*CW2**2*T)+2D0*D2(2,2)*(KM2-U)+2D0*D2(2,3)*(KM2-U)+2D0*
     &  D2(1,2)*(T-U)+2D0*D2(1,3)*(T-U)
        K(16) = -(4D0*D2(0,0))+4D0*D3(0,0,1)+4D0*D3(0,0,2)+2D0*
     &  D1(3)*KM2+D1(1)*CW2**(-2)*(-(2D0*CW2**2*KM2)+CW2**2*MW2+2D0*
     &  CW2**2*S-2D0*CW2*MW2*SW2+MW2*SW2**2)+D1(2)*CW2**(-2)*(-(2D0*
     &  CW2**2*KM2)+CW2**2*MW2+2D0*CW2**2*S-2D0*CW2*MW2*SW2+MW2*
     &  SW2**2)+CW2**(-2)*D0*(CW2**2*MW2-2D0*CW2*MW2*SW2+MW2*SW2**2-
     &  2D0*CW2**2*T)+2D0*D2(1,2)*(KP2-U)+2D0*D2(2,2)*(KP2-U)+2D0*
     &  D2(1,3)*(T-U)+2D0*D2(2,3)*(T-U)
        K(17) = 2D0*D1(2)+3D0*D2(1,2)+3D0*D2(1,3)+3D0*D2(2,2)+3D0*
     &  D2(2,3)+D3(1,1,2)+D3(1,1,3)+2D0*D3(1,2,2)+3D0*D3(1,2,3)+
     &  D3(1,3,3)+D3(2,2,2)+2D0*D3(2,2,3)+D3(2,3,3)

        DF(1,-1) = K(1)**2*SW2**(-2)*(-(16D0*D2(0,0))+D1(1)*CW2**(-
     &  2)*(K(5)-2D0*CW2**2*U)+D1(2)*CW2**(-2)*(K(5)-2D0*CW2**2*U)+
     &  D1(3)*CW2**(-2)*(K(5)-2D0*CW2**2*U)+CW2**(-2)*D0*(K(5)-2D0*
     &  CW2**2*U))
        DF(1,1) = 4D0*(-(16D0*D2(0,0))+D1(1)*CW2**(-2)*(K(5)-2D0*
     &  CW2**2*U)+D1(2)*CW2**(-2)*(K(5)-2D0*CW2**2*U)+D1(3)*CW2**(-
     &  2)*(K(5)-2D0*CW2**2*U)+CW2**(-2)*D0*(K(5)-2D0*CW2**2*U))
        DF(2,-1) = 2D0*K(1)**2*K(12)*SW2**(-2)
        DF(2,1) = 8D0*K(12)
        DF(3,-1) = -(2D0*K(1)**2*K(16)*SW2**(-2))
        DF(3,1) = -(8D0*K(16))
        DF(4,-1) = -(2D0*K(1)**2*K(15)*SW2**(-2))
        DF(4,1) = -(8D0*K(15))
        DF(5,-1) = 2D0*K(1)**2*K(9)*SW2**(-2)
        DF(5,1) = 8D0*K(9)
        DF(6,-1) = -(2D0*K(1)**2*K(10)*SW2**(-2))
        DF(6,1) = -(8D0*K(10))
        DF(7,-1) = 8D0*K(1)**2*K(17)*SW2**(-2)
        DF(7,1) = 32D0*K(17)
        DF(8,-1) = 8D0*K(1)**2*K(11)*SW2**(-2)
        DF(8,1) = 32D0*K(11)
        DF(9,-1) = -(8D0*K(1)**2*K(13)*SW2**(-2))
        DF(9,1) = -(32D0*K(13))
        DF(10,-1) = -(8D0*K(1)**2*K(14)*SW2**(-2))
        DF(10,1) = -(32D0*K(14))

        PRE = (ALPHA)/(16D0*PI)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(8,-1) = F(8,-1)+DF(8,-1)*PRE
        F(8,1) = F(8,1)+DF(8,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

	end

************************************************************************
        subroutine add_Vself_ferm(F)
************************************************************************
*	fermion-loop contributions of AA, AZ, and ZZ self-energies
*-----------------------------------------------------------------------
*       2.6.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        COMPLEX*16 K(6)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer i,qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc_ferm/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                 dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

	do 10 i=1,3

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N12)

        CALL B012_(S,0D0,0D0,B0,B1,B2,2,0)

        DF(2,-1) = (1D0-2D0*CW2)/SW2
        DF(2,1) = 2D0
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = -((ALPHA*(MZ2-S)**(-2)*(B2(0,0)*12D0+S+B1(1)*6D0*S+
     &  B2(1,1)*6D0*S))/(48D0*CW2*PI*SW2))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N11)+G(T1,N2)+G(T1,N5)+G(T1,N8)

        CALL B012_(S,MQP(I),MQP(I),B0,B1,B2,2,0)

        K(1) = S-6D0*MQP2(I)
        K(2) = 32D0*CW2*MZ2**2-32D0*CW2**2*MZ2**2-20D0*MZ2*S+8D0*
     &  CW2*MZ2*S+3D0*S**2
        K(3) = -(64D0*CW2*MZ2**2)+128D0*CW2**2*MZ2**2-64D0*CW2**3*
     &  MZ2**2+20D0*MZ2*S-4D0*CW2*MZ2*S-16D0*CW2**2*MZ2*S-3D0*S**2-
     &  6D0*CW2*S**2
        K(4) = 64D0*CW2*MZ2**2-128D0*CW2**2*MZ2**2+64D0*CW2**3*MZ2**
     &  2-20D0*MZ2*S+4D0*CW2*MZ2*S+16D0*CW2**2*MZ2*S+3D0*S**2+6D0*
     &  CW2*S**2

        DF(2,-1) = (B1(1)*K(4)*6D0)/SW2+(B2(1,1)*K(4)*6D0)/SW2+
     &  (K(1)*K(4))/(S*SW2)+(B2(0,0)*K(4)*12D0)/(S*SW2)+(24D0*B0*(-
     &  (16D0*CW2*MZ2**2)+16D0*CW2**2*MZ2**2+5D0*MZ2*S+4D0*CW2*MZ2*
     &  S-3D0*S**2)*MQP2(I))/S
        DF(2,1) = 2D0*(B1(1)*K(2)*6D0+B2(1,1)*K(2)*6D0+(K(1)*K(2))/
     &  S+(B2(0,0)*K(2)*12D0)/S+(24D0*B0*(-(8D0*CW2*MZ2**2)+8D0*
     &  CW2**2*MZ2**2+5D0*MZ2*S-2D0*CW2*MZ2*S-3D0*S**2)*MQP2(I))/S)
        DF(3,-1) = -((B1(1)*K(4)*6D0)/SW2)-(B2(1,1)*K(4)*6D0)/SW2-
     &  (K(1)*K(4))/(S*SW2)+(B2(0,0)*K(3)*12D0)/(S*SW2)+(24D0*B0*
     &  (16D0*CW2*MZ2**2-16D0*CW2**2*MZ2**2-5D0*MZ2*S-4D0*CW2*MZ2*S+
     &  3D0*S**2)*MQP2(I))/S
        DF(3,1) = -(2D0*(B1(1)*K(2)*6D0+B2(1,1)*K(2)*6D0+(K(1)*
     &  K(2))/S-(B2(0,0)*12D0*(-(32D0*CW2*MZ2**2)+32D0*CW2**2*MZ2**
     &  2+20D0*MZ2*S-8D0*CW2*MZ2*S-3D0*S**2))/S-(24D0*B0*(8D0*CW2*
     &  MZ2**2-8D0*CW2**2*MZ2**2-5D0*MZ2*S+2D0*CW2*MZ2*S+3D0*S**2)*
     &  MQP2(I))/S))
        DF(4,-1) = DF(3,-1)
        DF(4,1) = DF(3,1)

        PRE = (ALPHA*(MZ2-S)**(-2))/(144D0*CW2*PI*S*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N13)+G(T1,N3)+G(T1,N6)+G(T1,N9)

        CALL B012_(S,ML(I),ML(I),B0,B1,B2,2,0)

        K(1) = S-6D0*ML2(I)
        K(2) = 8D0*CW2*MZ2**2-8D0*CW2**2*MZ2**2-6D0*MZ2*S+4D0*CW2*
     &  MZ2*S+S**2
        K(3) = -(16D0*CW2*MZ2**2)+32D0*CW2**2*MZ2**2-16D0*CW2**3*
     &  MZ2**2+6D0*MZ2*S-6D0*CW2*MZ2*S-S**2
        K(4) = 16D0*CW2*MZ2**2-32D0*CW2**2*MZ2**2+16D0*CW2**3*MZ2**
     &  2-6D0*MZ2*S+6D0*CW2*MZ2*S+S**2

        DF(2,-1) = (B1(1)*K(4)*6D0)/SW2+(B2(1,1)*K(4)*6D0)/SW2+
     &  (K(1)*K(4))/(S*SW2)+(B2(0,0)*K(4)*12D0)/(S*SW2)+(12D0*B0*(-
     &  (8D0*CW2*MZ2**2)+8D0*CW2**2*MZ2**2+3D0*MZ2*S-S**2)*ML2(I))/S
        DF(2,1) = 2D0*(B1(1)*K(2)*6D0+B2(1,1)*K(2)*6D0+(K(1)*K(2))/
     &  S+(B2(0,0)*K(2)*12D0)/S+(12D0*B0*(-(4D0*CW2*MZ2**2)+4D0*
     &  CW2**2*MZ2**2+3D0*MZ2*S-2D0*CW2*MZ2*S-S**2)*ML2(I))/S)
        DF(3,-1) = -((B1(1)*K(4)*6D0)/SW2)-(B2(1,1)*K(4)*6D0)/SW2-
     &  (K(1)*K(4))/(S*SW2)+(B2(0,0)*K(3)*12D0)/(S*SW2)+(12D0*B0*
     &  (8D0*CW2*MZ2**2-8D0*CW2**2*MZ2**2-3D0*MZ2*S+S**2)*ML2(I))/S
        DF(3,1) = -(2D0*(B1(1)*K(2)*6D0+B2(1,1)*K(2)*6D0+(K(1)*
     &  K(2))/S-(B2(0,0)*12D0*(-(8D0*CW2*MZ2**2)+8D0*CW2**2*MZ2**2+
     &  6D0*MZ2*S-4D0*CW2*MZ2*S-S**2))/S-(12D0*B0*(4D0*CW2*MZ2**2-
     &  4D0*CW2**2*MZ2**2-3D0*MZ2*S+2D0*CW2*MZ2*S+S**2)*ML2(I))/S))
        DF(4,-1) = DF(3,-1)
        DF(4,1) = DF(3,1)

        PRE = (ALPHA*(MZ2-S)**(-2))/(48D0*CW2*PI*S*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

C       G(T1,N1)+G(T1,N10)+G(T1,N4)+G(T1,N7)

        CALL B012_(S,MQM(I),MQM(I),B0,B1,B2,2,0)

        K(1) = S-6D0*MQM2(I)
        K(2) = -(8D0*CW2*MZ2**2)+8D0*CW2**2*MZ2**2+2D0*MZ2*S+4D0*
     &  CW2*MZ2*S+3D0*S**2
        K(3) = -(16D0*CW2*MZ2**2)+32D0*CW2**2*MZ2**2-16D0*CW2**3*
     &  MZ2**2+2D0*MZ2*S+14D0*CW2*MZ2*S-16D0*CW2**2*MZ2*S+3D0*S**2-
     &  12D0*CW2*S**2
        K(4) = 16D0*CW2*MZ2**2-32D0*CW2**2*MZ2**2+16D0*CW2**3*MZ2**
     &  2-2D0*MZ2*S-14D0*CW2*MZ2*S+16D0*CW2**2*MZ2*S-3D0*S**2+12D0*
     &  CW2*S**2

        DF(2,-1) = (B1(1)*K(4)*6D0)/SW2+(B2(1,1)*K(4)*6D0)/SW2+
     &  (K(1)*K(4))/(S*SW2)+(B2(0,0)*K(4)*12D0)/(S*SW2)+(12D0*B0*(-
     &  (8D0*CW2*MZ2**2)+8D0*CW2**2*MZ2**2+MZ2*S+8D0*CW2*MZ2*S-3D0*
     &  S**2)*MQM2(I))/S
        DF(2,1) = -(2D0*(B1(1)*K(2)*6D0+B2(1,1)*K(2)*6D0+(K(1)*
     &  K(2))/S-(B2(0,0)*12D0*(8D0*CW2*MZ2**2-8D0*CW2**2*MZ2**2-2D0*
     &  MZ2*S-4D0*CW2*MZ2*S-3D0*S**2))/S-(12D0*B0*(-(4D0*CW2*MZ2**
     &  2)+4D0*CW2**2*MZ2**2+MZ2*S+2D0*CW2*MZ2*S-3D0*S**2)*MQM2(I))/
     &  S))
        DF(3,-1) = -((B1(1)*K(4)*6D0)/SW2)-(B2(1,1)*K(4)*6D0)/SW2-
     &  (K(1)*K(4))/(S*SW2)+(B2(0,0)*K(3)*12D0)/(S*SW2)+(12D0*B0*
     &  (8D0*CW2*MZ2**2-8D0*CW2**2*MZ2**2-MZ2*S-8D0*CW2*MZ2*S+3D0*
     &  S**2)*MQM2(I))/S
        DF(3,1) = 2D0*(B1(1)*K(2)*6D0+B2(1,1)*K(2)*6D0+(K(1)*K(2))/
     &  S+(B2(0,0)*K(2)*12D0)/S+(12D0*B0*(4D0*CW2*MZ2**2-4D0*CW2**2*
     &  MZ2**2-MZ2*S-2D0*CW2*MZ2*S+3D0*S**2)*MQM2(I))/S)
        DF(4,-1) = DF(3,-1)
        DF(4,1) = DF(3,1)

        PRE = (ALPHA*(MZ2-S)**(-2))/(144D0*CW2*PI*S*SW2)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

10	continue

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       2D0*G(T2,N2)+2D0*G(T2,N3)

        K(1) = DMZ2+DZZZ*(MZ2-S)
        K(2) = DZAZ+DZZA
        K(3) = MZ2-S
        K(4) = -((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0
        K(5) = CW2*MZ2*S*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)-
     &  CW2*S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)+CW*DZAA*
     &  MZ2**2*S*SW-2D0*CW*DZAA*MZ2*S**2*SW+CW*DZAA*S**3*SW+CW*S**2*
     &  (DMZ2+DZZZ*(MZ2-S))*SW-MZ2*S*(-((DZZA*MZ2)/2D0)+((DZAZ+
     &  DZZA)*S)/2D0)*SW2+S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/
     &  2D0)*SW2
        K(6) = -(CW*S**2*(DMZ2+DZZZ*(MZ2-S)))+MZ2*S*(-((DZZA*MZ2)/
     &  2D0)+((DZAZ+DZZA)*S)/2D0)*SW+2D0*CW2*MZ2*S*(-((DZZA*MZ2)/
     &  2D0)+((DZAZ+DZZA)*S)/2D0)*SW-S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+
     &  DZZA)*S)/2D0)*SW-2D0*CW2*S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+
     &  DZZA)*S)/2D0)*SW+2D0*CW*DZAA*MZ2**2*S*SW2-4D0*CW*DZAA*MZ2*
     &  S**2*SW2+2D0*CW*DZAA*S**3*SW2+2D0*CW*S**2*(DMZ2+DZZZ*(MZ2-
     &  S))*SW2-2D0*MZ2*S*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)*
     &  SW*SW2+2D0*S**2*(-((DZZA*MZ2)/2D0)+((DZAZ+DZZA)*S)/2D0)*SW*
     &  SW2

        DF(2,-1) = -((K(6))/SW)
        DF(2,1) = -(K(5)*2D0)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = (S**(-2)*(MZ2-S)**(-2))/(CW*SW)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

	end

************************************************************************
        subroutine add_VVVvert_ferm(F)
************************************************************************
*	fermion-loop contributions of AWW and ZWW vertex corrections
*-----------------------------------------------------------------------
*       2.6.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 B0,B1(1),B2(0:2,0:2) 
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        COMPLEX*16 K(11)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer i,qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc_ferm/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                 dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

	qeps = 1d0

	do 10 i=1,3

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N2)+G(T1,N5)

        CALL C0123_(KP2,S,KM2,0D0,ML(I),ML(I),C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = 1D0-QEPS
        K(2) = 1D0+QEPS
        K(3) = T+U
        K(4) = KM2+T+U
        K(5) = KP2+T+U
        K(6) = CW2*MZ2-CW2*S-S*SW2
        K(7) = -(2D0*CW2*MZ2)-S+2D0*CW2*S+2D0*S*SW2
        K(8) = 2D0*CW2*MZ2+S-2D0*CW2*S-2D0*S*SW2
        K(9) = S-4D0*CW2*MZ2*SW2-4D0*S*SW2+4D0*CW2*S*SW2+4D0*S*SW2**
     &  2

        DF(1,-1) = (3D0*QEPS*(4D0*C2(0,0)*K(9)+6D0*C3(0,0,1)*K(9)+
     &  6D0*C3(0,0,2)*K(9)+2D0*C2(1,2)*K(3)*K(9)+2D0*C2(2,2)*K(9)*
     &  KM2+C3(2,2,2)*K(9)*KM2+C3(1,2,2)*K(9)*(K(3)+KM2)+2D0*C2(1,
     &  1)*K(9)*KP2+C3(1,1,1)*K(9)*KP2+C3(1,1,2)*K(9)*(K(3)+KP2)+
     &  C1(2)*(KM2*S-4D0*CW2*KM2*MZ2*SW2-4D0*KM2*S*SW2+4D0*CW2*KM2*
     &  S*SW2+4D0*KM2*S*SW2**2+4D0*CW2*MZ2*SW2*ML2(I)+2D0*S*SW2*
     &  ML2(I)-4D0*CW2*S*SW2*ML2(I)-4D0*S*SW2**2*ML2(I))+C1(1)*(KP2*
     &  S-4D0*CW2*KP2*MZ2*SW2-4D0*KP2*S*SW2+4D0*CW2*KP2*S*SW2+4D0*
     &  KP2*S*SW2**2+4D0*CW2*MZ2*SW2*ML2(I)+2D0*S*SW2*ML2(I)-4D0*
     &  CW2*S*SW2*ML2(I)-4D0*S*SW2**2*ML2(I))))/SW2
        DF(1,1) = -(6D0*QEPS*(4D0*C2(0,0)*K(7)+6D0*C3(0,0,1)*K(7)+
     &  6D0*C3(0,0,2)*K(7)-2D0*C2(1,2)*K(3)*K(8)+2D0*C2(2,2)*K(7)*
     &  KM2+C3(2,2,2)*K(7)*KM2+C3(1,2,2)*K(7)*(K(3)+KM2)+2D0*C2(1,
     &  1)*K(7)*KP2+C3(1,1,1)*K(7)*KP2+C3(1,1,2)*K(7)*(K(3)+KP2)+
     &  C1(2)*(-(2D0*CW2*KM2*MZ2)-KM2*S+2D0*CW2*KM2*S+2D0*KM2*S*SW2+
     &  2D0*CW2*MZ2*ML2(I)-2D0*CW2*S*ML2(I)-2D0*S*SW2*ML2(I))+C1(1)*
     &  (-(2D0*CW2*KP2*MZ2)-KP2*S+2D0*CW2*KP2*S+2D0*KP2*S*SW2+2D0*
     &  CW2*MZ2*ML2(I)-2D0*CW2*S*ML2(I)-2D0*S*SW2*ML2(I))))
        DF(2,-1) = (-(2D0*K(9))+12D0*C2(0,0)*K(2)*K(9)+6D0*C2(1,2)*
     &  K(2)*K(3)*K(9)+6D0*C2(2,2)*K(2)*K(9)*KM2+3D0*C3(2,2,2)*K(2)*
     &  K(9)*KM2+3D0*C3(1,2,2)*K(2)*K(9)*(K(3)+KM2)+6D0*C2(1,1)*
     &  K(2)*K(9)*KP2+3D0*C3(1,1,1)*K(2)*K(9)*KP2+3D0*C3(1,1,2)*
     &  K(2)*K(9)*(K(3)+KP2)+6D0*C3(0,0,1)*K(9)*(1D0+3D0*QEPS)+6D0*
     &  C3(0,0,2)*K(9)*(1D0+3D0*QEPS)+3D0*C1(2)*(KM2*S+KM2*QEPS*S-
     &  4D0*CW2*KM2*MZ2*SW2-4D0*CW2*KM2*MZ2*QEPS*SW2-4D0*KM2*S*SW2+
     &  4D0*CW2*KM2*S*SW2-4D0*KM2*QEPS*S*SW2+4D0*CW2*KM2*QEPS*S*SW2+
     &  4D0*KM2*S*SW2**2+4D0*KM2*QEPS*S*SW2**2-4D0*CW2*MZ2*SW2*
     &  ML2(I)+4D0*CW2*MZ2*QEPS*SW2*ML2(I)-2D0*S*SW2*ML2(I)+4D0*CW2*
     &  S*SW2*ML2(I)+2D0*QEPS*S*SW2*ML2(I)-4D0*CW2*QEPS*S*SW2*
     &  ML2(I)+4D0*S*SW2**2*ML2(I)-4D0*QEPS*S*SW2**2*ML2(I))+3D0*
     &  C1(1)*(KP2*S+KP2*QEPS*S-4D0*CW2*KP2*MZ2*SW2-4D0*CW2*KP2*MZ2*
     &  QEPS*SW2-4D0*KP2*S*SW2+4D0*CW2*KP2*S*SW2-4D0*KP2*QEPS*S*SW2+
     &  4D0*CW2*KP2*QEPS*S*SW2+4D0*KP2*S*SW2**2+4D0*KP2*QEPS*S*SW2**
     &  2-4D0*CW2*MZ2*SW2*ML2(I)+4D0*CW2*MZ2*QEPS*SW2*ML2(I)-2D0*S*
     &  SW2*ML2(I)+4D0*CW2*S*SW2*ML2(I)+2D0*QEPS*S*SW2*ML2(I)-4D0*
     &  CW2*QEPS*S*SW2*ML2(I)+4D0*S*SW2**2*ML2(I)-4D0*QEPS*S*SW2**2*
     &  ML2(I)))/SW2
        DF(2,1) = 2D0*(-(2D0*K(7))+12D0*C2(0,0)*K(1)*K(7)-6D0*C2(1,
     &  2)*K(1)*K(3)*K(8)+6D0*C2(2,2)*K(1)*K(7)*KM2+3D0*C3(2,2,2)*
     &  K(1)*K(7)*KM2+3D0*C3(1,2,2)*K(1)*K(7)*(K(3)+KM2)+6D0*C2(1,
     &  1)*K(1)*K(7)*KP2+3D0*C3(1,1,1)*K(1)*K(7)*KP2+3D0*C3(1,1,2)*
     &  K(1)*K(7)*(K(3)+KP2)+6D0*C3(0,0,1)*K(7)*(1D0-3D0*QEPS)+6D0*
     &  C3(0,0,2)*K(7)*(1D0-3D0*QEPS)-3D0*C1(2)*(2D0*CW2*KM2*MZ2-
     &  2D0*CW2*KM2*MZ2*QEPS+KM2*S-2D0*CW2*KM2*S-KM2*QEPS*S+2D0*CW2*
     &  KM2*QEPS*S-2D0*KM2*S*SW2+2D0*KM2*QEPS*S*SW2+2D0*CW2*MZ2*
     &  ML2(I)+2D0*CW2*MZ2*QEPS*ML2(I)-2D0*CW2*S*ML2(I)-2D0*CW2*
     &  QEPS*S*ML2(I)-2D0*S*SW2*ML2(I)-2D0*QEPS*S*SW2*ML2(I))-3D0*
     &  C1(1)*(2D0*CW2*KP2*MZ2-2D0*CW2*KP2*MZ2*QEPS+KP2*S-2D0*CW2*
     &  KP2*S-KP2*QEPS*S+2D0*CW2*KP2*QEPS*S-2D0*KP2*S*SW2+2D0*KP2*
     &  QEPS*S*SW2+2D0*CW2*MZ2*ML2(I)+2D0*CW2*MZ2*QEPS*ML2(I)-2D0*
     &  CW2*S*ML2(I)-2D0*CW2*QEPS*S*ML2(I)-2D0*S*SW2*ML2(I)-2D0*
     &  QEPS*S*SW2*ML2(I)))
        DF(3,-1) = (2D0*K(9))/SW2-(6D0*C2(0,0)*K(2)*K(9))/SW2+(3D0*
     &  C3(1,1,2)*K(1)*K(3)*K(9))/SW2+(3D0*C3(1,2,2)*K(1)*K(9)*KM2)/
     &  SW2-(3D0*C1(2)*K(2)*K(9)*KM2)/SW2-(3D0*C2(2,2)*K(2)*K(9)*
     &  KM2)/SW2+(3D0*C2(1,1)*K(1)*K(9)*KP2)/SW2+(3D0*C3(1,1,1)*
     &  K(1)*K(9)*KP2)/SW2+(6D0*C3(0,0,1)*K(9)*(1D0-3D0*QEPS))/SW2-
     &  (6D0*C2(1,2)*K(9)*QEPS*T)/SW2-6D0*C1(1)*K(1)*K(7)*ML2(I)
        DF(3,1) = -(2D0*(-(2D0*K(7))+6D0*C2(0,0)*K(1)*K(7)+3D0*C3(1,
     &  1,2)*K(2)*K(3)*K(8)+3D0*C1(2)*K(1)*K(7)*KM2+3D0*C2(2,2)*
     &  K(1)*K(7)*KM2-3D0*C3(1,2,2)*K(2)*K(7)*KM2-3D0*C2(1,1)*K(2)*
     &  K(7)*KP2-3D0*C3(1,1,1)*K(2)*K(7)*KP2-6D0*C3(0,0,1)*K(7)*
     &  (1D0+3D0*QEPS)+6D0*C2(1,2)*K(8)*QEPS*T-6D0*C1(1)*K(2)*K(6)*
     &  ML2(I)))
        DF(4,-1) = (2D0*K(9))/SW2-(6D0*C2(0,0)*K(2)*K(9))/SW2+(3D0*
     &  C3(1,2,2)*K(1)*K(3)*K(9))/SW2+(3D0*C2(2,2)*K(1)*K(9)*KM2)/
     &  SW2+(3D0*C3(2,2,2)*K(1)*K(9)*KM2)/SW2+(3D0*C3(1,1,2)*K(1)*
     &  K(9)*KP2)/SW2-(3D0*C1(1)*K(2)*K(9)*KP2)/SW2-(3D0*C2(1,1)*
     &  K(2)*K(9)*KP2)/SW2+(6D0*C3(0,0,2)*K(9)*(1D0-3D0*QEPS))/SW2-
     &  (6D0*C2(1,2)*K(9)*QEPS*T)/SW2-6D0*C1(2)*K(1)*K(7)*ML2(I)
        DF(4,1) = -(2D0*(-(2D0*K(7))+6D0*C2(0,0)*K(1)*K(7)+3D0*C3(1,
     &  2,2)*K(2)*K(3)*K(8)-3D0*C2(2,2)*K(2)*K(7)*KM2-3D0*C3(2,2,2)*
     &  K(2)*K(7)*KM2+3D0*C1(1)*K(1)*K(7)*KP2+3D0*C2(1,1)*K(1)*K(7)*
     &  KP2-3D0*C3(1,1,2)*K(2)*K(7)*KP2-6D0*C3(0,0,2)*K(7)*(1D0+3D0*
     &  QEPS)+6D0*C2(1,2)*K(8)*QEPS*T-6D0*C1(2)*K(2)*K(6)*ML2(I)))
        DF(5,-1) = -(6D0*QEPS*((2D0*C2(0,0)*K(9))/SW2+(6D0*C3(0,0,
     &  2)*K(9))/SW2+(C2(1,2)*K(3)*K(9))/SW2+(C3(1,2,2)*K(3)*K(9))/
     &  SW2+(C2(2,2)*K(9)*KM2)/SW2+(C3(2,2,2)*K(9)*KM2)/SW2+(C1(1)*
     &  K(9)*KP2)/SW2+(C2(1,1)*K(9)*KP2)/SW2+(C3(1,1,2)*K(9)*KP2)/
     &  SW2-2D0*C1(2)*K(7)*ML2(I)))
        DF(5,1) = 12D0*QEPS*(2D0*C2(0,0)*K(7)+6D0*C3(0,0,2)*K(7)-
     &  C2(1,2)*K(3)*K(8)-C3(1,2,2)*K(3)*K(8)+C2(2,2)*K(7)*KM2+C3(2,
     &  2,2)*K(7)*KM2+C1(1)*K(7)*KP2+C2(1,1)*K(7)*KP2+C3(1,1,2)*
     &  K(7)*KP2+2D0*C1(2)*K(6)*ML2(I))
        DF(6,-1) = -(6D0*QEPS*((2D0*C2(0,0)*K(9))/SW2+(6D0*C3(0,0,
     &  1)*K(9))/SW2+(C2(1,2)*K(3)*K(9))/SW2+(C3(1,1,2)*K(3)*K(9))/
     &  SW2+(C1(2)*K(9)*KM2)/SW2+(C2(2,2)*K(9)*KM2)/SW2+(C3(1,2,2)*
     &  K(9)*KM2)/SW2+(C2(1,1)*K(9)*KP2)/SW2+(C3(1,1,1)*K(9)*KP2)/
     &  SW2-2D0*C1(1)*K(7)*ML2(I)))
        DF(6,1) = 12D0*QEPS*(2D0*C2(0,0)*K(7)+6D0*C3(0,0,1)*K(7)-
     &  C2(1,2)*K(3)*K(8)-C3(1,1,2)*K(3)*K(8)+C1(2)*K(7)*KM2+C2(2,
     &  2)*K(7)*KM2+C3(1,2,2)*K(7)*KM2+C2(1,1)*K(7)*KP2+C3(1,1,1)*
     &  K(7)*KP2+2D0*C1(1)*K(6)*ML2(I))
        DF(7,-1) = (12D0*(C3(1,1,2)+C3(1,2,2)+C2(1,2)*K(2))*K(9))/
     &  SW2
        DF(7,1) = 24D0*(C3(1,1,2)+C3(1,2,2)+C2(1,2)*K(1))*K(7)
        DF(9,-1) = -((12D0*C2(1,2)*K(9)*QEPS)/SW2)
        DF(9,1) = 24D0*C2(1,2)*K(7)*QEPS
        DF(10,-1) = DF(9,-1)
        DF(10,1) = DF(9,1)

        PRE = -((ALPHA)/(48D0*CW2*PI*S*(MZ2-S)*SW2))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T1,N1)+G(T1,N4)

        CALL C0123_(KP2,S,KM2,MQP(I),MQM(I),MQM(I),C0,C1,C2,C3,B0,B1,B2,
     &  3,0)

        K(1) = 1D0-QEPS
        K(2) = 1D0+QEPS
        K(3) = T+U
        K(4) = KM2+T+U
        K(5) = KP2+T+U
        K(6) = CW2*MZ2-CW2*S-S*SW2
        K(7) = -(2D0*CW2*MZ2)-3D0*S+2D0*CW2*S+2D0*S*SW2
        K(8) = -(2D0*CW2*MZ2)-S+2D0*CW2*S+2D0*S*SW2
        K(9) = 2D0*CW2*MZ2+3D0*S-2D0*CW2*S-2D0*S*SW2
        K(10) = -(3D0*S)+4D0*CW2*MZ2*SW2+8D0*S*SW2-4D0*CW2*S*SW2-
     &  4D0*S*SW2**2
        K(11) = 3D0*S-4D0*CW2*MZ2*SW2-8D0*S*SW2+4D0*CW2*S*SW2+4D0*S*
     &  SW2**2

        DF(1,-1) = (3D0*QEPS*(4D0*C2(0,0)*K(11)+6D0*C3(0,0,1)*K(11)+
     &  6D0*C3(0,0,2)*K(11)-2D0*C2(1,2)*K(10)*K(3)+2D0*C2(2,2)*
     &  K(11)*KM2+C3(2,2,2)*K(11)*KM2+C3(1,2,2)*K(11)*(K(3)+KM2)+
     &  2D0*C2(1,1)*K(11)*KP2+C3(1,1,1)*K(11)*KP2+C3(1,1,2)*K(11)*
     &  (K(3)+KP2)+C1(2)*(3D0*KM2*S-4D0*CW2*KM2*MZ2*SW2-8D0*KM2*S*
     &  SW2+4D0*CW2*KM2*S*SW2+4D0*KM2*S*SW2**2+4D0*CW2*MZ2*SW2*
     &  MQM2(I)+2D0*S*SW2*MQM2(I)-4D0*CW2*S*SW2*MQM2(I)-4D0*S*SW2**
     &  2*MQM2(I))+C1(1)*(3D0*KP2*S-4D0*CW2*KP2*MZ2*SW2-8D0*KP2*S*
     &  SW2+4D0*CW2*KP2*S*SW2+4D0*KP2*S*SW2**2+4D0*CW2*MZ2*SW2*
     &  MQM2(I)+2D0*S*SW2*MQM2(I)-4D0*CW2*S*SW2*MQM2(I)-4D0*S*SW2**
     &  2*MQM2(I))))/SW2
        DF(1,1) = -(6D0*QEPS*(4D0*C2(0,0)*K(7)+6D0*C3(0,0,1)*K(7)+
     &  6D0*C3(0,0,2)*K(7)-2D0*C2(1,2)*K(3)*K(9)+2D0*C2(2,2)*K(7)*
     &  KM2+C3(2,2,2)*K(7)*KM2+C3(1,2,2)*K(7)*(K(3)+KM2)+2D0*C2(1,
     &  1)*K(7)*KP2+C3(1,1,1)*K(7)*KP2+C3(1,1,2)*K(7)*(K(3)+KP2)+
     &  C1(2)*(-(2D0*CW2*KM2*MZ2)-3D0*KM2*S+2D0*CW2*KM2*S+2D0*KM2*S*
     &  SW2+2D0*CW2*MZ2*MQM2(I)-2D0*CW2*S*MQM2(I)-2D0*S*SW2*
     &  MQM2(I))+C1(1)*(-(2D0*CW2*KP2*MZ2)-3D0*KP2*S+2D0*CW2*KP2*S+
     &  2D0*KP2*S*SW2+2D0*CW2*MZ2*MQM2(I)-2D0*CW2*S*MQM2(I)-2D0*S*
     &  SW2*MQM2(I))))
        DF(2,-1) = -(((2D0*K(11)-12D0*C2(0,0)*K(11)*K(2)+6D0*C2(1,
     &  2)*K(10)*K(2)*K(3)-6D0*C2(2,2)*K(11)*K(2)*KM2-3D0*C3(2,2,2)*
     &  K(11)*K(2)*KM2-3D0*C3(1,2,2)*K(11)*K(2)*(K(3)+KM2)-6D0*C2(1,
     &  1)*K(11)*K(2)*KP2-3D0*C3(1,1,1)*K(11)*K(2)*KP2-3D0*C3(1,1,
     &  2)*K(11)*K(2)*(K(3)+KP2)-6D0*C3(0,0,1)*K(11)*(1D0+3D0*QEPS)-
     &  6D0*C3(0,0,2)*K(11)*(1D0+3D0*QEPS)-3D0*C1(2)*(3D0*KM2*S+3D0*
     &  KM2*QEPS*S-4D0*CW2*KM2*MZ2*SW2-4D0*CW2*KM2*MZ2*QEPS*SW2-8D0*
     &  KM2*S*SW2+4D0*CW2*KM2*S*SW2-8D0*KM2*QEPS*S*SW2+4D0*CW2*KM2*
     &  QEPS*S*SW2+4D0*KM2*S*SW2**2+4D0*KM2*QEPS*S*SW2**2-4D0*CW2*
     &  MZ2*SW2*MQM2(I)+4D0*CW2*MZ2*QEPS*SW2*MQM2(I)-2D0*S*SW2*
     &  MQM2(I)+4D0*CW2*S*SW2*MQM2(I)+2D0*QEPS*S*SW2*MQM2(I)-4D0*
     &  CW2*QEPS*S*SW2*MQM2(I)+4D0*S*SW2**2*MQM2(I)-4D0*QEPS*S*SW2**
     &  2*MQM2(I))-3D0*C1(1)*(3D0*KP2*S+3D0*KP2*QEPS*S-4D0*CW2*KP2*
     &  MZ2*SW2-4D0*CW2*KP2*MZ2*QEPS*SW2-8D0*KP2*S*SW2+4D0*CW2*KP2*
     &  S*SW2-8D0*KP2*QEPS*S*SW2+4D0*CW2*KP2*QEPS*S*SW2+4D0*KP2*S*
     &  SW2**2+4D0*KP2*QEPS*S*SW2**2-4D0*CW2*MZ2*SW2*MQM2(I)+4D0*
     &  CW2*MZ2*QEPS*SW2*MQM2(I)-2D0*S*SW2*MQM2(I)+4D0*CW2*S*SW2*
     &  MQM2(I)+2D0*QEPS*S*SW2*MQM2(I)-4D0*CW2*QEPS*S*SW2*MQM2(I)+
     &  4D0*S*SW2**2*MQM2(I)-4D0*QEPS*S*SW2**2*MQM2(I))))/SW2)
        DF(2,1) = 2D0*(-(2D0*K(7))+12D0*C2(0,0)*K(1)*K(7)-6D0*C2(1,
     &  2)*K(1)*K(3)*K(9)+6D0*C2(2,2)*K(1)*K(7)*KM2+3D0*C3(2,2,2)*
     &  K(1)*K(7)*KM2+3D0*C3(1,2,2)*K(1)*K(7)*(K(3)+KM2)+6D0*C2(1,
     &  1)*K(1)*K(7)*KP2+3D0*C3(1,1,1)*K(1)*K(7)*KP2+3D0*C3(1,1,2)*
     &  K(1)*K(7)*(K(3)+KP2)+6D0*C3(0,0,1)*K(7)*(1D0-3D0*QEPS)+6D0*
     &  C3(0,0,2)*K(7)*(1D0-3D0*QEPS)-3D0*C1(2)*(2D0*CW2*KM2*MZ2-
     &  2D0*CW2*KM2*MZ2*QEPS+3D0*KM2*S-2D0*CW2*KM2*S-3D0*KM2*QEPS*S+
     &  2D0*CW2*KM2*QEPS*S-2D0*KM2*S*SW2+2D0*KM2*QEPS*S*SW2+2D0*CW2*
     &  MZ2*MQM2(I)+2D0*CW2*MZ2*QEPS*MQM2(I)-2D0*CW2*S*MQM2(I)-2D0*
     &  CW2*QEPS*S*MQM2(I)-2D0*S*SW2*MQM2(I)-2D0*QEPS*S*SW2*
     &  MQM2(I))-3D0*C1(1)*(2D0*CW2*KP2*MZ2-2D0*CW2*KP2*MZ2*QEPS+
     &  3D0*KP2*S-2D0*CW2*KP2*S-3D0*KP2*QEPS*S+2D0*CW2*KP2*QEPS*S-
     &  2D0*KP2*S*SW2+2D0*KP2*QEPS*S*SW2+2D0*CW2*MZ2*MQM2(I)+2D0*
     &  CW2*MZ2*QEPS*MQM2(I)-2D0*CW2*S*MQM2(I)-2D0*CW2*QEPS*S*
     &  MQM2(I)-2D0*S*SW2*MQM2(I)-2D0*QEPS*S*SW2*MQM2(I)))
        DF(3,-1) = (2D0*K(11))/SW2-(6D0*C2(0,0)*K(11)*K(2))/SW2-
     &  (3D0*C3(1,1,2)*K(1)*K(10)*K(3))/SW2+(3D0*C3(1,2,2)*K(1)*
     &  K(11)*KM2)/SW2-(3D0*C1(2)*K(11)*K(2)*KM2)/SW2-(3D0*C2(2,2)*
     &  K(11)*K(2)*KM2)/SW2+(3D0*C2(1,1)*K(1)*K(11)*KP2)/SW2+(3D0*
     &  C3(1,1,1)*K(1)*K(11)*KP2)/SW2+(6D0*C3(0,0,1)*K(11)*(1D0-3D0*
     &  QEPS))/SW2+(6D0*C2(1,2)*K(10)*QEPS*T)/SW2-6D0*C1(1)*K(1)*
     &  K(8)*MQM2(I)
        DF(3,1) = -(2D0*(-(2D0*K(7))+6D0*C2(0,0)*K(1)*K(7)+3D0*C3(1,
     &  1,2)*K(2)*K(3)*K(9)+3D0*C1(2)*K(1)*K(7)*KM2+3D0*C2(2,2)*
     &  K(1)*K(7)*KM2-3D0*C3(1,2,2)*K(2)*K(7)*KM2-3D0*C2(1,1)*K(2)*
     &  K(7)*KP2-3D0*C3(1,1,1)*K(2)*K(7)*KP2-6D0*C3(0,0,1)*K(7)*
     &  (1D0+3D0*QEPS)+6D0*C2(1,2)*K(9)*QEPS*T-6D0*C1(1)*K(2)*K(6)*
     &  MQM2(I)))
        DF(4,-1) = (2D0*K(11))/SW2-(6D0*C2(0,0)*K(11)*K(2))/SW2-
     &  (3D0*C3(1,2,2)*K(1)*K(10)*K(3))/SW2+(3D0*C2(2,2)*K(1)*K(11)*
     &  KM2)/SW2+(3D0*C3(2,2,2)*K(1)*K(11)*KM2)/SW2+(3D0*C3(1,1,2)*
     &  K(1)*K(11)*KP2)/SW2-(3D0*C1(1)*K(11)*K(2)*KP2)/SW2-(3D0*
     &  C2(1,1)*K(11)*K(2)*KP2)/SW2+(6D0*C3(0,0,2)*K(11)*(1D0-3D0*
     &  QEPS))/SW2+(6D0*C2(1,2)*K(10)*QEPS*T)/SW2-6D0*C1(2)*K(1)*
     &  K(8)*MQM2(I)
        DF(4,1) = -(2D0*(-(2D0*K(7))+6D0*C2(0,0)*K(1)*K(7)+3D0*C3(1,
     &  2,2)*K(2)*K(3)*K(9)-3D0*C2(2,2)*K(2)*K(7)*KM2-3D0*C3(2,2,2)*
     &  K(2)*K(7)*KM2+3D0*C1(1)*K(1)*K(7)*KP2+3D0*C2(1,1)*K(1)*K(7)*
     &  KP2-3D0*C3(1,1,2)*K(2)*K(7)*KP2-6D0*C3(0,0,2)*K(7)*(1D0+3D0*
     &  QEPS)+6D0*C2(1,2)*K(9)*QEPS*T-6D0*C1(2)*K(2)*K(6)*MQM2(I)))
        DF(5,-1) = -(6D0*QEPS*((2D0*C2(0,0)*K(11))/SW2+(6D0*C3(0,0,
     &  2)*K(11))/SW2-(C2(1,2)*K(10)*K(3))/SW2-(C3(1,2,2)*K(10)*
     &  K(3))/SW2+(C2(2,2)*K(11)*KM2)/SW2+(C3(2,2,2)*K(11)*KM2)/SW2+
     &  (C1(1)*K(11)*KP2)/SW2+(C2(1,1)*K(11)*KP2)/SW2+(C3(1,1,2)*
     &  K(11)*KP2)/SW2-2D0*C1(2)*K(8)*MQM2(I)))
        DF(5,1) = 12D0*QEPS*(2D0*C2(0,0)*K(7)+6D0*C3(0,0,2)*K(7)-
     &  C2(1,2)*K(3)*K(9)-C3(1,2,2)*K(3)*K(9)+C2(2,2)*K(7)*KM2+C3(2,
     &  2,2)*K(7)*KM2+C1(1)*K(7)*KP2+C2(1,1)*K(7)*KP2+C3(1,1,2)*
     &  K(7)*KP2+2D0*C1(2)*K(6)*MQM2(I))
        DF(6,-1) = -(6D0*QEPS*((2D0*C2(0,0)*K(11))/SW2+(6D0*C3(0,0,
     &  1)*K(11))/SW2-(C2(1,2)*K(10)*K(3))/SW2-(C3(1,1,2)*K(10)*
     &  K(3))/SW2+(C1(2)*K(11)*KM2)/SW2+(C2(2,2)*K(11)*KM2)/SW2+
     &  (C3(1,2,2)*K(11)*KM2)/SW2+(C2(1,1)*K(11)*KP2)/SW2+(C3(1,1,
     &  1)*K(11)*KP2)/SW2-2D0*C1(1)*K(8)*MQM2(I)))
        DF(6,1) = 12D0*QEPS*(2D0*C2(0,0)*K(7)+6D0*C3(0,0,1)*K(7)-
     &  C2(1,2)*K(3)*K(9)-C3(1,1,2)*K(3)*K(9)+C1(2)*K(7)*KM2+C2(2,
     &  2)*K(7)*KM2+C3(1,2,2)*K(7)*KM2+C2(1,1)*K(7)*KP2+C3(1,1,1)*
     &  K(7)*KP2+2D0*C1(1)*K(6)*MQM2(I))
        DF(7,-1) = (12D0*K(11)*(C3(1,1,2)+C3(1,2,2)+C2(1,2)*K(2)))/
     &  SW2
        DF(7,1) = 24D0*(C3(1,1,2)+C3(1,2,2)+C2(1,2)*K(1))*K(7)
        DF(9,-1) = -((12D0*C2(1,2)*K(11)*QEPS)/SW2)
        DF(9,1) = 24D0*C2(1,2)*K(7)*QEPS
        DF(10,-1) = DF(9,-1)
        DF(10,1) = DF(9,1)

        PRE = -((ALPHA)/(48D0*CW2*PI*S*(MZ2-S)*SW2))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T1,N6)

        CALL C0123_(KP2,S,KM2,ML(I),0D0,0D0,C0,C1,C2,C3,B0,B1,B2,3,0)

        K(1) = 1D0-QEPS
        K(2) = 1D0+QEPS
        K(3) = 1D0-2D0*SW2
        K(4) = T+U
        K(5) = KM2+T+U
        K(6) = KP2+T+U
        K(7) = 2D0*C2(0,0)+6D0*C3(0,0,2)+C2(2,2)*KM2+C3(2,2,2)*KM2+
     &  C1(1)*KP2+C2(1,1)*KP2+C3(1,1,2)*KP2+C2(1,2)*(T+U)+C3(1,2,2)*
     &  (T+U)
        K(8) = 2D0*C2(0,0)+6D0*C3(0,0,1)+C1(2)*KM2+C2(2,2)*KM2+C3(1,
     &  2,2)*KM2+C2(1,1)*KP2+C3(1,1,1)*KP2+C2(1,2)*(T+U)+C3(1,1,2)*
     &  (T+U)
        K(9) = 4D0*C2(0,0)+6D0*C3(0,0,1)+6D0*C3(0,0,2)+C1(2)*KM2+
     &  2D0*C2(2,2)*KM2+C3(2,2,2)*KM2+C1(1)*KP2+2D0*C2(1,1)*KP2+
     &  C3(1,1,1)*KP2+2D0*C2(1,2)*(T+U)+C3(1,2,2)*(KM2+T+U)+C3(1,1,
     &  2)*(KP2+T+U)

        DF(1,-1) = -((3D0*K(3)*K(9)*QEPS)/SW2)
        DF(1,1) = -(6D0*K(9)*QEPS)
        DF(2,-1) = -((K(3)*(2D0-12D0*C2(0,0)*K(1)-6D0*C2(1,2)*K(1)*
     &  K(4)-3D0*C1(2)*K(1)*KM2-6D0*C2(2,2)*K(1)*KM2-3D0*C3(2,2,2)*
     &  K(1)*KM2-3D0*C3(1,2,2)*K(1)*(K(4)+KM2)-3D0*C1(1)*K(1)*KP2-
     &  6D0*C2(1,1)*K(1)*KP2-3D0*C3(1,1,1)*K(1)*KP2-3D0*C3(1,1,2)*
     &  K(1)*(K(4)+KP2)-6D0*C3(0,0,1)*(1D0-3D0*QEPS)-6D0*C3(0,0,2)*
     &  (1D0-3D0*QEPS)))/SW2)
        DF(2,1) = 2D0*(2D0-12D0*C2(0,0)*K(2)-6D0*C2(1,2)*K(2)*K(4)-
     &  3D0*C1(2)*K(2)*KM2-6D0*C2(2,2)*K(2)*KM2-3D0*C3(2,2,2)*K(2)*
     &  KM2-3D0*C3(1,2,2)*K(2)*(K(4)+KM2)-3D0*C1(1)*K(2)*KP2-6D0*
     &  C2(1,1)*K(2)*KP2-3D0*C3(1,1,1)*K(2)*KP2-3D0*C3(1,1,2)*K(2)*
     &  (K(4)+KP2)-6D0*C3(0,0,1)*(1D0+3D0*QEPS)-6D0*C3(0,0,2)*(1D0+
     &  3D0*QEPS))
        DF(3,-1) = (K(3)*(2D0-6D0*C2(0,0)*K(1)+3D0*C3(1,1,2)*K(2)*
     &  K(4)-3D0*C1(2)*K(1)*KM2-3D0*C2(2,2)*K(1)*KM2+3D0*C3(1,2,2)*
     &  K(2)*KM2+3D0*C2(1,1)*K(2)*KP2+3D0*C3(1,1,1)*K(2)*KP2+6D0*
     &  C3(0,0,1)*(1D0+3D0*QEPS)+6D0*C2(1,2)*QEPS*T))/SW2
        DF(3,1) = -(2D0*(2D0-6D0*C2(0,0)*K(2)+3D0*C3(1,1,2)*K(1)*
     &  K(4)+3D0*C3(1,2,2)*K(1)*KM2-3D0*C1(2)*K(2)*KM2-3D0*C2(2,2)*
     &  K(2)*KM2+3D0*C2(1,1)*K(1)*KP2+3D0*C3(1,1,1)*K(1)*KP2+6D0*
     &  C3(0,0,1)*(1D0-3D0*QEPS)-6D0*C2(1,2)*QEPS*T))
        DF(4,-1) = (K(3)*(2D0-6D0*C2(0,0)*K(1)+3D0*C3(1,2,2)*K(2)*
     &  K(4)+3D0*C2(2,2)*K(2)*KM2+3D0*C3(2,2,2)*K(2)*KM2-3D0*C1(1)*
     &  K(1)*KP2-3D0*C2(1,1)*K(1)*KP2+3D0*C3(1,1,2)*K(2)*KP2+6D0*
     &  C3(0,0,2)*(1D0+3D0*QEPS)+6D0*C2(1,2)*QEPS*T))/SW2
        DF(4,1) = -(2D0*(2D0-6D0*C2(0,0)*K(2)+3D0*C3(1,2,2)*K(1)*
     &  K(4)+3D0*C2(2,2)*K(1)*KM2+3D0*C3(2,2,2)*K(1)*KM2+3D0*C3(1,1,
     &  2)*K(1)*KP2-3D0*C1(1)*K(2)*KP2-3D0*C2(1,1)*K(2)*KP2+6D0*
     &  C3(0,0,2)*(1D0-3D0*QEPS)-6D0*C2(1,2)*QEPS*T))
        DF(5,-1) = (6D0*K(3)*K(7)*QEPS)/SW2
        DF(5,1) = 12D0*K(7)*QEPS
        DF(6,-1) = (6D0*K(3)*K(8)*QEPS)/SW2
        DF(6,1) = 12D0*K(8)*QEPS
        DF(7,-1) = (12D0*(C3(1,1,2)+C3(1,2,2)+C2(1,2)*K(1))*K(3))/
     &  SW2
        DF(7,1) = -(24D0*(C3(1,1,2)+C3(1,2,2)+C2(1,2)*K(2)))
        DF(9,-1) = (12D0*C2(1,2)*K(3)*QEPS)/SW2
        DF(9,1) = 24D0*C2(1,2)*QEPS
        DF(10,-1) = DF(9,-1)
        DF(10,1) = DF(9,1)

        PRE = -((ALPHA)/(48D0*CW2*PI*(MZ2-S)*SW2))

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

C       G(T1,N3)+G(T1,N7)

        CALL C0123_(KP2,S,KM2,MQM(I),MQP(I),MQP(I),C0,C1,C2,C3,B0,B1,B2,
     &  3,0)

        K(1) = 1D0-QEPS
        K(2) = 1D0+QEPS
        K(3) = T+U
        K(4) = KM2+T+U
        K(5) = KP2+T+U
        K(6) = CW2*MZ2-CW2*S-S*SW2
        K(7) = -(4D0*CW2*MZ2)-3D0*S+4D0*CW2*S+4D0*S*SW2
        K(8) = -(2D0*CW2*MZ2)-S+2D0*CW2*S+2D0*S*SW2
        K(9) = 4D0*CW2*MZ2+3D0*S-4D0*CW2*S-4D0*S*SW2
        K(10) = -(3D0*S)+8D0*CW2*MZ2*SW2+10D0*S*SW2-8D0*CW2*S*SW2-
     &  8D0*S*SW2**2
        K(11) = 3D0*S-8D0*CW2*MZ2*SW2-10D0*S*SW2+8D0*CW2*S*SW2+8D0*
     &  S*SW2**2

        DF(1,-1) = (3D0*QEPS*(4D0*C2(0,0)*K(11)+6D0*C3(0,0,1)*K(11)+
     &  6D0*C3(0,0,2)*K(11)-2D0*C2(1,2)*K(10)*K(3)+2D0*C2(2,2)*
     &  K(11)*KM2+C3(2,2,2)*K(11)*KM2+C3(1,2,2)*K(11)*(K(3)+KM2)+
     &  2D0*C2(1,1)*K(11)*KP2+C3(1,1,1)*K(11)*KP2+C3(1,1,2)*K(11)*
     &  (K(3)+KP2)+C1(2)*(3D0*KM2*S-8D0*CW2*KM2*MZ2*SW2-10D0*KM2*S*
     &  SW2+8D0*CW2*KM2*S*SW2+8D0*KM2*S*SW2**2+8D0*CW2*MZ2*SW2*
     &  MQP2(I)+4D0*S*SW2*MQP2(I)-8D0*CW2*S*SW2*MQP2(I)-8D0*S*SW2**
     &  2*MQP2(I))+C1(1)*(3D0*KP2*S-8D0*CW2*KP2*MZ2*SW2-10D0*KP2*S*
     &  SW2+8D0*CW2*KP2*S*SW2+8D0*KP2*S*SW2**2+8D0*CW2*MZ2*SW2*
     &  MQP2(I)+4D0*S*SW2*MQP2(I)-8D0*CW2*S*SW2*MQP2(I)-8D0*S*SW2**
     &  2*MQP2(I))))/SW2
        DF(1,1) = -(6D0*QEPS*(4D0*C2(0,0)*K(7)+6D0*C3(0,0,1)*K(7)+
     &  6D0*C3(0,0,2)*K(7)-2D0*C2(1,2)*K(3)*K(9)+2D0*C2(2,2)*K(7)*
     &  KM2+C3(2,2,2)*K(7)*KM2+C3(1,2,2)*K(7)*(K(3)+KM2)+2D0*C2(1,
     &  1)*K(7)*KP2+C3(1,1,1)*K(7)*KP2+C3(1,1,2)*K(7)*(K(3)+KP2)+
     &  C1(2)*(-(4D0*CW2*KM2*MZ2)-3D0*KM2*S+4D0*CW2*KM2*S+4D0*KM2*S*
     &  SW2+4D0*CW2*MZ2*MQP2(I)-4D0*CW2*S*MQP2(I)-4D0*S*SW2*
     &  MQP2(I))+C1(1)*(-(4D0*CW2*KP2*MZ2)-3D0*KP2*S+4D0*CW2*KP2*S+
     &  4D0*KP2*S*SW2+4D0*CW2*MZ2*MQP2(I)-4D0*CW2*S*MQP2(I)-4D0*S*
     &  SW2*MQP2(I))))
        DF(2,-1) = (2D0*K(11)-12D0*C2(0,0)*K(1)*K(11)+6D0*C2(1,2)*
     &  K(1)*K(10)*K(3)-6D0*C2(2,2)*K(1)*K(11)*KM2-3D0*C3(2,2,2)*
     &  K(1)*K(11)*KM2-3D0*C3(1,2,2)*K(1)*K(11)*(K(3)+KM2)-6D0*C2(1,
     &  1)*K(1)*K(11)*KP2-3D0*C3(1,1,1)*K(1)*K(11)*KP2-3D0*C3(1,1,
     &  2)*K(1)*K(11)*(K(3)+KP2)-6D0*C3(0,0,1)*K(11)*(1D0-3D0*QEPS)-
     &  6D0*C3(0,0,2)*K(11)*(1D0-3D0*QEPS)+3D0*C1(2)*(-(3D0*KM2*S)+
     &  3D0*KM2*QEPS*S+8D0*CW2*KM2*MZ2*SW2-8D0*CW2*KM2*MZ2*QEPS*SW2+
     &  10D0*KM2*S*SW2-8D0*CW2*KM2*S*SW2-10D0*KM2*QEPS*S*SW2+8D0*
     &  CW2*KM2*QEPS*S*SW2-8D0*KM2*S*SW2**2+8D0*KM2*QEPS*S*SW2**2+
     &  8D0*CW2*MZ2*SW2*MQP2(I)+8D0*CW2*MZ2*QEPS*SW2*MQP2(I)+4D0*S*
     &  SW2*MQP2(I)-8D0*CW2*S*SW2*MQP2(I)+4D0*QEPS*S*SW2*MQP2(I)-
     &  8D0*CW2*QEPS*S*SW2*MQP2(I)-8D0*S*SW2**2*MQP2(I)-8D0*QEPS*S*
     &  SW2**2*MQP2(I))+3D0*C1(1)*(-(3D0*KP2*S)+3D0*KP2*QEPS*S+8D0*
     &  CW2*KP2*MZ2*SW2-8D0*CW2*KP2*MZ2*QEPS*SW2+10D0*KP2*S*SW2-8D0*
     &  CW2*KP2*S*SW2-10D0*KP2*QEPS*S*SW2+8D0*CW2*KP2*QEPS*S*SW2-
     &  8D0*KP2*S*SW2**2+8D0*KP2*QEPS*S*SW2**2+8D0*CW2*MZ2*SW2*
     &  MQP2(I)+8D0*CW2*MZ2*QEPS*SW2*MQP2(I)+4D0*S*SW2*MQP2(I)-8D0*
     &  CW2*S*SW2*MQP2(I)+4D0*QEPS*S*SW2*MQP2(I)-8D0*CW2*QEPS*S*SW2*
     &  MQP2(I)-8D0*S*SW2**2*MQP2(I)-8D0*QEPS*S*SW2**2*MQP2(I)))/SW2
        DF(2,1) = -(2D0*(-(2D0*K(7))+12D0*C2(0,0)*K(2)*K(7)-6D0*
     &  C2(1,2)*K(2)*K(3)*K(9)+6D0*C2(2,2)*K(2)*K(7)*KM2+3D0*C3(2,2,
     &  2)*K(2)*K(7)*KM2+3D0*C3(1,2,2)*K(2)*K(7)*(K(3)+KM2)+6D0*
     &  C2(1,1)*K(2)*K(7)*KP2+3D0*C3(1,1,1)*K(2)*K(7)*KP2+3D0*C3(1,
     &  1,2)*K(2)*K(7)*(K(3)+KP2)+6D0*C3(0,0,1)*K(7)*(1D0+3D0*QEPS)+
     &  6D0*C3(0,0,2)*K(7)*(1D0+3D0*QEPS)+3D0*C1(2)*(-(4D0*CW2*KM2*
     &  MZ2)-4D0*CW2*KM2*MZ2*QEPS-3D0*KM2*S+4D0*CW2*KM2*S-3D0*KM2*
     &  QEPS*S+4D0*CW2*KM2*QEPS*S+4D0*KM2*S*SW2+4D0*KM2*QEPS*S*SW2-
     &  4D0*CW2*MZ2*MQP2(I)+4D0*CW2*MZ2*QEPS*MQP2(I)+4D0*CW2*S*
     &  MQP2(I)-4D0*CW2*QEPS*S*MQP2(I)+4D0*S*SW2*MQP2(I)-4D0*QEPS*S*
     &  SW2*MQP2(I))+3D0*C1(1)*(-(4D0*CW2*KP2*MZ2)-4D0*CW2*KP2*MZ2*
     &  QEPS-3D0*KP2*S+4D0*CW2*KP2*S-3D0*KP2*QEPS*S+4D0*CW2*KP2*
     &  QEPS*S+4D0*KP2*S*SW2+4D0*KP2*QEPS*S*SW2-4D0*CW2*MZ2*MQP2(I)+
     &  4D0*CW2*MZ2*QEPS*MQP2(I)+4D0*CW2*S*MQP2(I)-4D0*CW2*QEPS*S*
     &  MQP2(I)+4D0*S*SW2*MQP2(I)-4D0*QEPS*S*SW2*MQP2(I))))
        DF(3,-1) = -((2D0*K(11))/SW2)+(6D0*C2(0,0)*K(1)*K(11))/SW2+
     &  (3D0*C3(1,1,2)*K(10)*K(2)*K(3))/SW2+(3D0*C1(2)*K(1)*K(11)*
     &  KM2)/SW2+(3D0*C2(2,2)*K(1)*K(11)*KM2)/SW2-(3D0*C3(1,2,2)*
     &  K(11)*K(2)*KM2)/SW2-(3D0*C2(1,1)*K(11)*K(2)*KP2)/SW2-(3D0*
     &  C3(1,1,1)*K(11)*K(2)*KP2)/SW2-(6D0*C3(0,0,1)*K(11)*(1D0+3D0*
     &  QEPS))/SW2+(6D0*C2(1,2)*K(10)*QEPS*T)/SW2+12D0*C1(1)*K(2)*
     &  K(8)*MQP2(I)
        DF(3,1) = -(2D0*(2D0*K(7)-6D0*C2(0,0)*K(2)*K(7)-3D0*C3(1,1,
     &  2)*K(1)*K(3)*K(9)+3D0*C3(1,2,2)*K(1)*K(7)*KM2-3D0*C1(2)*
     &  K(2)*K(7)*KM2-3D0*C2(2,2)*K(2)*K(7)*KM2+3D0*C2(1,1)*K(1)*
     &  K(7)*KP2+3D0*C3(1,1,1)*K(1)*K(7)*KP2+6D0*C3(0,0,1)*K(7)*
     &  (1D0-3D0*QEPS)+6D0*C2(1,2)*K(9)*QEPS*T+12D0*C1(1)*K(1)*K(6)*
     &  MQP2(I)))
        DF(4,-1) = -((2D0*K(11))/SW2)+(6D0*C2(0,0)*K(1)*K(11))/SW2+
     &  (3D0*C3(1,2,2)*K(10)*K(2)*K(3))/SW2-(3D0*C2(2,2)*K(11)*K(2)*
     &  KM2)/SW2-(3D0*C3(2,2,2)*K(11)*K(2)*KM2)/SW2+(3D0*C1(1)*K(1)*
     &  K(11)*KP2)/SW2+(3D0*C2(1,1)*K(1)*K(11)*KP2)/SW2-(3D0*C3(1,1,
     &  2)*K(11)*K(2)*KP2)/SW2-(6D0*C3(0,0,2)*K(11)*(1D0+3D0*QEPS))/
     &  SW2+(6D0*C2(1,2)*K(10)*QEPS*T)/SW2+12D0*C1(2)*K(2)*K(8)*
     &  MQP2(I)
        DF(4,1) = -(2D0*(2D0*K(7)-6D0*C2(0,0)*K(2)*K(7)-3D0*C3(1,2,
     &  2)*K(1)*K(3)*K(9)+3D0*C2(2,2)*K(1)*K(7)*KM2+3D0*C3(2,2,2)*
     &  K(1)*K(7)*KM2+3D0*C3(1,1,2)*K(1)*K(7)*KP2-3D0*C1(1)*K(2)*
     &  K(7)*KP2-3D0*C2(1,1)*K(2)*K(7)*KP2+6D0*C3(0,0,2)*K(7)*(1D0-
     &  3D0*QEPS)+6D0*C2(1,2)*K(9)*QEPS*T+12D0*C1(2)*K(1)*K(6)*
     &  MQP2(I)))
        DF(5,-1) = -(6D0*QEPS*((2D0*C2(0,0)*K(11))/SW2+(6D0*C3(0,0,
     &  2)*K(11))/SW2-(C2(1,2)*K(10)*K(3))/SW2-(C3(1,2,2)*K(10)*
     &  K(3))/SW2+(C2(2,2)*K(11)*KM2)/SW2+(C3(2,2,2)*K(11)*KM2)/SW2+
     &  (C1(1)*K(11)*KP2)/SW2+(C2(1,1)*K(11)*KP2)/SW2+(C3(1,1,2)*
     &  K(11)*KP2)/SW2-4D0*C1(2)*K(8)*MQP2(I)))
        DF(5,1) = 12D0*QEPS*(2D0*C2(0,0)*K(7)+6D0*C3(0,0,2)*K(7)-
     &  C2(1,2)*K(3)*K(9)-C3(1,2,2)*K(3)*K(9)+C2(2,2)*K(7)*KM2+C3(2,
     &  2,2)*K(7)*KM2+C1(1)*K(7)*KP2+C2(1,1)*K(7)*KP2+C3(1,1,2)*
     &  K(7)*KP2+4D0*C1(2)*K(6)*MQP2(I))
        DF(6,-1) = -(6D0*QEPS*((2D0*C2(0,0)*K(11))/SW2+(6D0*C3(0,0,
     &  1)*K(11))/SW2-(C2(1,2)*K(10)*K(3))/SW2-(C3(1,1,2)*K(10)*
     &  K(3))/SW2+(C1(2)*K(11)*KM2)/SW2+(C2(2,2)*K(11)*KM2)/SW2+
     &  (C3(1,2,2)*K(11)*KM2)/SW2+(C2(1,1)*K(11)*KP2)/SW2+(C3(1,1,
     &  1)*K(11)*KP2)/SW2-4D0*C1(1)*K(8)*MQP2(I)))
        DF(6,1) = 12D0*QEPS*(2D0*C2(0,0)*K(7)+6D0*C3(0,0,1)*K(7)-
     &  C2(1,2)*K(3)*K(9)-C3(1,1,2)*K(3)*K(9)+C1(2)*K(7)*KM2+C2(2,
     &  2)*K(7)*KM2+C3(1,2,2)*K(7)*KM2+C2(1,1)*K(7)*KP2+C3(1,1,1)*
     &  K(7)*KP2+4D0*C1(1)*K(6)*MQP2(I))
        DF(7,-1) = -((12D0*(C3(1,1,2)+C3(1,2,2)+C2(1,2)*K(1))*
     &  K(11))/SW2)
        DF(7,1) = -(24D0*(C3(1,1,2)+C3(1,2,2)+C2(1,2)*K(2))*K(7))
        DF(9,-1) = -((12D0*C2(1,2)*K(11)*QEPS)/SW2)
        DF(9,1) = 24D0*C2(1,2)*K(7)*QEPS
        DF(10,-1) = DF(9,-1)
        DF(10,1) = DF(9,1)

        PRE = (ALPHA)/(48D0*CW2*PI*S*(MZ2-S)*SW2)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE
        F(1,1) = F(1,1)+DF(1,1)*PRE
        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE
        F(5,-1) = F(5,-1)+DF(5,-1)*PRE
        F(5,1) = F(5,1)+DF(5,1)*PRE
        F(6,-1) = F(6,-1)+DF(6,-1)*PRE
        F(6,1) = F(6,1)+DF(6,1)*PRE
        F(7,-1) = F(7,-1)+DF(7,-1)*PRE
        F(7,1) = F(7,1)+DF(7,1)*PRE
        F(9,-1) = F(9,-1)+DF(9,-1)*PRE
        F(9,1) = F(9,1)+DF(9,1)*PRE
        F(10,-1) = F(10,-1)+DF(10,-1)*PRE
        F(10,1) = F(10,1)+DF(10,1)*PRE

10	continue

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T2,N2)+G(T2,N3)

        K(1) = CW*CW2*DZZA*MZ2+2D0*DSW*S-CW*CW2*DZZA*S-CW2*DZAA*MZ2*
     &  SW-2D0*CW2*DZE*MZ2*SW-2D0*CW2*DZW*MZ2*SW+CW2*DZAA*S*SW-CW2*
     &  DZZZ*S*SW+CW*DZAZ*S*SW2
        K(2) = 2D0*DSW*S-2D0*CW2*DZE*S*SW-2D0*CW2*DZW*S*SW-CW2*DZZZ*
     &  S*SW-2D0*CW*CW2*DZZA*MZ2*SW2-4D0*DSW*S*SW2+CW*DZAZ*S*SW2+
     &  2D0*CW*CW2*DZZA*S*SW2+2D0*CW2*DZAA*MZ2*SW*SW2+4D0*CW2*DZE*
     &  MZ2*SW*SW2+4D0*CW2*DZW*MZ2*SW*SW2-2D0*CW2*DZAA*S*SW*SW2+2D0*
     &  CW2*DZZZ*S*SW*SW2-2D0*CW*DZAZ*S*SW2**2

        DF(2,-1) = K(2)/SW2
        DF(2,1) = -(K(1)*2D0)
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = 1D0/(2D0*CW2*S*(MZ2-S)*SW)

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

	end

************************************************************************
        subroutine add_eeVvert_ferm(F)
************************************************************************
*	fermion-loop contributions to eeA- and eeZ-vertex counterterms
*-----------------------------------------------------------------------
*       2.6.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        COMPLEX*16 K(2)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc_ferm/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                 dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T2,N2)+G(T2,N3)

        K(1) = -(2D0*DSW*S)-CW*CW2*DZAZ*S-CW2*DZAA*MZ2*SW-2D0*CW2*
     &  DZE*MZ2*SW-2D0*CW2*DZEER*MZ2*SW+CW2*DZAA*S*SW-CW2*DZZZ*S*SW-
     &  CW*DZZA*MZ2*SW2+CW*DZZA*S*SW2
        K(2) = -(2D0*DSW*S)+2D0*CW2*DZE*S*SW+2D0*CW2*DZEEL*S*SW+CW2*
     &  DZZZ*S*SW+CW*DZZA*MZ2*SW2-2D0*CW*CW2*DZAZ*S*SW2-CW*DZZA*S*
     &  SW2-2D0*CW2*DZAA*MZ2*SW*SW2-4D0*CW2*DZE*MZ2*SW*SW2-4D0*CW2*
     &  DZEEL*MZ2*SW*SW2+2D0*CW2*DZAA*S*SW*SW2-2D0*CW2*DZZZ*S*SW*
     &  SW2-2D0*CW*DZZA*MZ2*SW2**2+2D0*CW*DZZA*S*SW2**2

        DF(2,-1) = K(2)/SW2
        DF(2,1) = K(1)*2D0
        DF(3,-1) = -(DF(2,-1))
        DF(3,1) = -(DF(2,1))
        DF(4,-1) = -(DF(2,-1))
        DF(4,1) = -(DF(2,1))

        PRE = -(1D0/(2D0*CW2*S*(MZ2-S)*SW))

        F(2,-1) = F(2,-1)+DF(2,-1)*PRE
        F(2,1) = F(2,1)+DF(2,1)*PRE
        F(3,-1) = F(3,-1)+DF(3,-1)*PRE
        F(3,1) = F(3,1)+DF(3,1)*PRE
        F(4,-1) = F(4,-1)+DF(4,-1)*PRE
        F(4,1) = F(4,1)+DF(4,1)*PRE

	end

************************************************************************
        subroutine add_enWvert_ferm(F)
************************************************************************
*	fermion-loop contributions to enW-vertex counterterms
*-----------------------------------------------------------------------
*       2.6.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        COMPLEX*16 PRE,DF(10,-1:1),F(10,-1:1)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc_ferm/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                 dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

c*** counterterms

C       Process: aF1F1aV3V3,   Model: SM

C       G(T1,N1)

        DF(1,-1) = 1D0

        PRE = ((2D0*DSW-2D0*DZE*SW-DZEEL*SW-DZNNL*SW-DZW*SW))/(2D0*
     &  SW*SW2*T)

        F(1,-1) = F(1,-1)+DF(1,-1)*PRE

	end

**********************************************************************
	function FWf1f2virt(kw2,Q1,Q2,m1,m2)
**********************************************************************
*	Form factor for O(alpha) correction to  W-f1-f2bar vertex
*	conventional formalism
*--------------------------------------------------------------------- 
*	W boson:  invariant mass = kw2
*	fermions: on shell, masses m1 and m2 as regulators
*--------------------------------------------------------------------- 
*	8.6.99 Stefan Dittmaier
**********************************************************************
	implicit real*8 (a-z)
	complex*16 BW,BZ,BW1,BZ1,BK,BKW,BKWZ,B1,B2
	complex*16 CK1W,CK2W,CKWZ,CKZ,CK12
	complex*16 dZf1m,dZf2m,ct,corr,FWf1f2virt
	complex*16 B0_,B1_,C0_
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl

	common/renc/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	            dZeer,dZeel,dZnnl
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
        COMMON/UV/MUDIM2
        COMMON/IR/LAMBDA,LAMBDA2

	m12 = m1**2
	m22 = m2**2

	BW   = B0_(0D0,0D0, MW,0)
	BZ   = B0_(0D0,0D0, MZ,0)
	BW1  = B1_(0D0,0D0, MW,0)
	BZ1  = B1_(0D0,0D0, MZ,0)
	BK   = B0_(KW2,0D0,0D0,0)
	BKW  = B0_(KW2,0D0, MW,0)
	BKWZ = B0_(KW2, MW, MZ,0)
	CKWZ = C0_(0D0,KW2,0D0,0D0, MW, MZ,0)
	CKZ  = C0_(0D0,KW2,0D0, MZ,0D0,0D0,0)

c*** Mass- and IR-singular scalar integrals
	if (q1.eq.0d0) then
	  B1   = 0d0
	  CK1W = 0d0
      	  fac1 = 0d0
	else
	  B1   = B0_(m12,0d0,m1,0)
	  CK1W = C0_(M12,KW2,0D0,M1,0D0,MW,306)
      	  fac1 = log(mudim2/m12)+4d0+4d0*log(lambda/m1)
	endif
	if (q2.eq.0d0) then
	  B2   = 0d0
	  CK2W = 0d0
      	  fac2 = 0d0
	else
	  B2   = B0_(m22,0d0,m2,0)
	  CK2W = C0_(M22,KW2,0D0,M2,0D0,MW,306)
      	  fac2 = log(mudim2/m22)+4d0+4d0*log(lambda/m2)
 	endif
	if (q1*q2.eq.0d0) then
	  CK12 = 0d0
	else
	  CK12 = C0_(M12,KW2,M22,0D0, M1, M2,307)
	endif

c*** one-loop EW correction
	corr = -ALPHA/16D0/PI*(
     &	-2D0/CW2/SW2*(1D0-2D0*SW2-4D0*Q1*Q2*SW2)
     &  +4D0*BKW*(KW2+MW2)/KW2
     &	-4D0*BW*(2D0*KW2+MW2)/SW2/KW2
     &	+4D0*BKWZ*CW2*(KW2+MW2+MZ2)/SW2/KW2
     &	-2D0*BZ/CW2*(2D0*KW2+MZ2)/SW2
     &  	*(1D0-2D0*SW2+2D0*Q1**2*SW**4+2D0*Q2**2*SW**4)/KW2
     &	+BK/CW2/SW2*(-2D0*MZ2*(Q1-Q2-2D0*Q1*SW2)*(Q1-Q2+2D0*Q2*SW2)
     &		-3D0*KW2*(1D0-2D0*SW2-4D0*Q1*Q2*SW2))/KW2	   
     &	-8D0*B1*Q1**2
     &	-8D0*B2*Q2**2
     &	+8D0*CK12*KW2*Q1*Q2
     &	-8D0*CK1W*MW2*Q1*(Q1-Q2)
     &	+8D0*CK2W*MW2*Q2*(Q1-Q2)
     &	-8D0*CKWZ*CW2*(KW2*MW2+KW2*MZ2+MW2*MZ2)/SW2/KW2
     &	-2D0*CKZ/CW2*(KW2+MZ2)**2/SW2
     &		*(Q1-Q2-2D0*Q1*SW2)*(Q1-Q2+2D0*Q2*SW2)/KW2	    )

c*** counterterms
	dZf1m = ALPHA/4D0/PI*(
     &	  	  - Q1**2*fac1
     &		  + ((Q1-Q2)/2d0/sw/cw-sw/cw*Q1)**2*(2D0*BZ1+1D0)
     &		  + (2D0*BW1+1D0)/2D0/SW2		)
	dZf2m = ALPHA/4D0/PI*(
     &		  - Q2**2*fac2
     &		  + ((Q2-Q1)/2d0/sw/cw-sw/cw*Q2)**2*(2D0*BZ1+1D0)
     &		  + (2D0*BW1+1D0)/2D0/SW2		)
	ct    = dZe - dsw/sw + (dZW+dZf1m+dZf2m)/2d0

	FWf1f2virt = corr + ct 

	end

**********************************************************************
	function FWf1f2virt_QCD(kw2,Q1,Q2,m1,m2)
**********************************************************************
*	Form factor for one-loop QCD correction to  W-f1-f2bar vertex
*--------------------------------------------------------------------- 
*	W boson:  invariant mass = kw2
*	fermions: on shell, masses m1 and m2 as regulators
*--------------------------------------------------------------------- 
*	8.6.99 Stefan Dittmaier
**********************************************************************
	implicit real*8 (a-z)
	complex*16 BK,B1,B2,CK12,FWf1f2virt_QCD,B0_,C0_

	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
        COMMON/UV/MUDIM2
        COMMON/IR/LAMBDA,LAMBDA2

	if (q1*q2.ne.0d0) then
	  m12  = m1**2
	  m22  = m2**2
	  BK   = B0_(KW2,0D0,0D0,0)
	  B1   = B0_(m12,0d0,m1,0)
      	  fac1 = log(mudim2/m12)+4d0+4d0*log(lambda/m1)
	  B2   = B0_(m22,0d0,m2,0)
      	  fac2 = log(mudim2/m22)+4d0+4d0*log(lambda/m2)
	  CK12 = C0_(M12,KW2,M22,0D0, M1, M2,307)
	  FWf1f2virt_QCD = ALPHAS/3D0/PI*(-2D0+2D0*B1+2D0*B2-3D0*BK
     &		                -2D0*KW2*CK12-fac1/2d0-fac2/2d0)
	else
	  FWf1f2virt_QCD = 0d0
	endif

	end

************************************************************************
        subroutine add_Wdec(F)
************************************************************************
*	contributions of Wf1f2 and Wf3f4 vertices
*	conventional formalism
*-----------------------------------------------------------------------
*	8.6.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        COMPLEX*16 F(10,-1:1),fb(10,-1:1),FWp,FWm,FWf1f2virt
	integer i,j,dir(7)
        integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/fstate/qf(4),mf(4),nc4f,dir
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

	FWp = FWf1f2virt(kp2,qf(1),qf(2),mf(1),mf(2))
	FWm = FWf1f2virt(km2,qf(3),qf(4),mf(3),mf(4))

	fb(1,-1) = -1d0/2d0/sw2/t
	fb(2,-1) = (s-2d0*mz2*sw2)/sw2/s/(s-mz2)
	fb(3,-1) = -fb(2,-1)
	fb(4,-1) = -fb(2,-1)
	fb(1,+1) = 0d0
	fb(2,+1) = -2d0*mz2/s/(s-mz2)
	fb(3,+1) = -fb(2,+1)
	fb(4,+1) = -fb(2,+1)

	do 10 i=1,4
	do 10 j=-1,1,2
          F(i,j) = F(i,j)+FB(i,j)*(FWp+FWm)
10	continue

	end

************************************************************************
        subroutine add_Wdec_QCD(F)
************************************************************************
*	contributions of Wf1f2 and Wf3f4 vertices
*	conventional formalism
*-----------------------------------------------------------------------
*	8.6.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        COMPLEX*16 F(10,-1:1),fb(10,-1:1),FWp,FWm,FWf1f2virt_QCD
	integer i,j,dir(7)
        integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/fstate/qf(4),mf(4),nc4f,dir
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

	FWp = FWf1f2virt_QCD(kp2,qf(1),qf(2),mf(1),mf(2))
	FWm = FWf1f2virt_QCD(km2,qf(3),qf(4),mf(3),mf(4))

	fb(1,-1) = -1d0/2d0/sw2/t
	fb(2,-1) = (s-2d0*mz2*sw2)/sw2/s/(s-mz2)
	fb(3,-1) = -fb(2,-1)
	fb(4,-1) = -fb(2,-1)
	fb(1,+1) = 0d0
	fb(2,+1) = -2d0*mz2/s/(s-mz2)
	fb(3,+1) = -fb(2,+1)
	fb(4,+1) = -fb(2,+1)

	do 10 i=1,4
	do 10 j=-1,1,2
          F(i,j) = F(i,j)+FB(i,j)*(FWp+FWm)
10	continue

	end

**********************************************************************
	function FWf1f2virt_bgf(kw2,Q1,Q2,m1,m2)
**********************************************************************
*	Form factor for one-loop correction to  W-f1-f2bar vertex
*	background-field formalism
*--------------------------------------------------------------------- 
*	W boson:  invariant mass = kw2
*	fermions: on shell, masses m1 and m2 as regulators
*--------------------------------------------------------------------- 
*	8.6.99 Stefan Dittmaier
**********************************************************************
	implicit real*8 (a-z)
	complex*16 BW,BZ,BW1,BZ1,BK,BKW,BKWZ,B1,B2
	complex*16 CK1W,CK2W,CKWZ,CKZ,CK12
	complex*16 dZf1m,dZf2m,ct,corr,FWf1f2virt_bgf
	complex*16 B0_,B1_,C0_
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl

	common/renc_bgf/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                dZeer,dZeel,dZnnl
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
        COMMON/UV/MUDIM2
        COMMON/IR/LAMBDA,LAMBDA2

	m12 = m1**2
	m22 = m2**2

	BW   = B0_(0D0,0D0, MW,0)
	BZ   = B0_(0D0,0D0, MZ,0)
	BW1  = B1_(0D0,0D0, MW,0)
	BZ1  = B1_(0D0,0D0, MZ,0)
	BK   = B0_(KW2,0D0,0D0,0)
	BKW  = B0_(KW2,0D0, MW,0)
	BKWZ = B0_(KW2, MW, MZ,0)
	CKWZ = C0_(0D0,KW2,0D0,0D0, MW, MZ,0)
	CKZ  = C0_(0D0,KW2,0D0, MZ,0D0,0D0,0)

c*** Mass- and IR-singular scalar integrals
	if (q1.eq.0d0) then
	  B1   = 0d0
	  CK1W = 0d0
      	  fac1 = 0d0
	else
	  B1   = B0_(m12,0d0,m1,0)
	  CK1W = C0_(M12,KW2,0D0,M1,0D0,MW,306)
      	  fac1 = log(mudim2/m12)+4d0+4d0*log(lambda/m1)
	endif
	if (q2.eq.0d0) then
	  B2   = 0d0
	  CK2W = 0d0
      	  fac2 = 0d0
	else
	  B2   = B0_(m22,0d0,m2,0)
	  CK2W = C0_(M22,KW2,0D0,M2,0D0,MW,306)
      	  fac2 = log(mudim2/m22)+4d0+4d0*log(lambda/m2)
 	endif
	if (q1*q2.eq.0d0) then
	  CK12 = 0d0
	else
	  CK12 = C0_(M12,KW2,M22,0D0, M1, M2,307)
	endif

c*** one-loop EW correction
	corr = -ALPHA/16D0/PI*(
     &  +4D0*BKW*(3D0*KW2+MW2)/KW2
     &  +4D0*BKWZ*CW2*(3D0*KW2+MW2+MZ2)/SW2/KW2
     &	-4D0*BW*(2D0*KW2+MW2)/SW2/KW2
     &	-2D0*BZ/CW2*(2D0*KW2+MZ2)/SW2
     &		*(1D0-2D0*SW2+2D0*Q1**2*SW**4+2D0*Q2**2*SW**4)/KW2
     &	+BK/CW2/SW2*(-2D0*MZ2*(Q1-Q2-2D0*Q1*SW2)*(Q1-Q2+2D0*Q2*SW2)
     &		-3D0*KW2*(1D0-2D0*SW2-4D0*Q1*Q2*SW2))/KW2
     &	-8D0*B1*Q1**2
     &	-8D0*B2*Q2**2
     &	+8D0*CK12*KW2*Q1*Q2
     &	-8D0*CK1W*MW2*Q1*(Q1-Q2)
     &	+8D0*CK2W*MW2*Q2*(Q1-Q2)
     &	-8D0*CKWZ*CW2*(KW2*MW2+KW2*MZ2+MW2*MZ2)/SW2/KW2
     &	-2D0*CKZ/CW2*(KW2+MZ2)**2/SW2
     &		*(Q1-Q2-2D0*Q1*SW2)*(Q1-Q2+2D0*Q2*SW2)/KW2
     &	-2D0/CW2/SW2*(1D0-2D0*SW2-4D0*Q1*Q2*SW2)		)

c*** counterterms
	dZf1m = ALPHA/4D0/PI*(
     &	  	  - Q1**2*fac1
     &		  + ((Q1-Q2)/2d0/sw/cw-sw/cw*Q1)**2*(2D0*BZ1+1D0)
     &		  + (2D0*BW1+1D0)/2D0/SW2		)
	dZf2m = ALPHA/4D0/PI*(
     &		  - Q2**2*fac2
     &		  + ((Q2-Q1)/2d0/sw/cw-sw/cw*Q2)**2*(2D0*BZ1+1D0)
     &		  + (2D0*BW1+1D0)/2D0/SW2		)
	ct    = dZe - dsw/sw + (dZW+dZf1m+dZf2m)/2d0

	FWf1f2virt_bgf = corr + ct 

	end

************************************************************************
        subroutine add_Wdec_bgf(F)
************************************************************************
*	contributions of Wf1f2 and Wf3f4 vertices
*	background-field formalism
*-----------------------------------------------------------------------
*       8.6.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        COMPLEX*16 F(10,-1:1),fb(10,-1:1),FWp,FWm,FWf1f2virt_bgf
	integer i,j,dir(7)
        integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/fstate/qf(4),mf(4),nc4f,dir
	common/mandel/s,t,u,kp2,km2
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

	FWp =   FWf1f2virt_bgf(kp2,qf(1),qf(2),mf(1),mf(2))
	FWm =   FWf1f2virt_bgf(km2,qf(3),qf(4),mf(3),mf(4))

	fb(1,-1) = -1d0/2d0/sw2/t
	fb(2,-1) = (s-2d0*mz2*sw2)/sw2/s/(s-mz2)
	fb(3,-1) = -fb(2,-1)
	fb(4,-1) = -fb(2,-1)
	fb(1,+1) = 0d0
	fb(2,+1) = -2d0*mz2/s/(s-mz2)
	fb(3,+1) = -fb(2,+1)
	fb(4,+1) = -fb(2,+1)

	do 10 i=1,4
	do 10 j=-1,1,2
          F(i,j) = F(i,j)+FB(i,j)*(FWp+FWm)
10	continue

	end

************************************************************************
        subroutine add_Wdec_ferm(F)
************************************************************************
*	fermion-loop contributions to Wf1f2 and Wf3f4 vertices
*-----------------------------------------------------------------------
*       8.6.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
        COMPLEX*16 F(10,-1:1),fb(10,-1:1)
	integer i,j
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast

	common/renc_ferm/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                 dZeer,dZeel,dZnnl
	common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
	common/mandel/s,t,u,kp2,km2

	fb(1,-1) = -1d0/2d0/sw2/t
	fb(2,-1) = (s-2d0*mz2*sw2)/sw2/s/(s-mz2)
	fb(3,-1) = -fb(2,-1)
	fb(4,-1) = -fb(2,-1)
	fb(1,+1) = 0d0
	fb(2,+1) = -2d0*mz2/s/(s-mz2)
	fb(3,+1) = -fb(2,+1)
	fb(4,+1) = -fb(2,+1)

c*** counterterms
	do 10 i=1,4
	do 10 j=-1,1,2
          F(i,j) = F(i,j)+FB(i,j)*(2d0*dZe - 2d0*dsw/sw + dZW)
10	continue

	end

************************************************************************
        function pWwidth(q1,q2)
************************************************************************
*	partial decay width for W --> f1 f2bar    (f1,f2 massless)
*-----------------------------------------------------------------------
*       30.7.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)
	complex*16 dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw
	complex*16 dZeer,dZeel,dZnnl
	complex*16 FWf1f2virt,FWf1f2virt_bgf,FWf1f2virt_QCD
	integer qgwborn,qgwqcd,qgwbos,qgwferm

	common/gwrcoptions/qgwborn,qgwqcd,qgwbos,qgwferm
	common/renc_ferm/dmw2,dmz2,dsw,dZe,dZaa,dZaz,dZza,dZzz,dZw,
     &	                 dZeer,dZeel,dZnnl
	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)
        COMMON/IR/LAMBDA,LAMBDA2

	if ((q1.eq.-1d0).or.(q1.eq.0d0)) ncf = 1d0
	if ((q1.eq.-1d0/3d0).or.(q1.eq.2d0/3d0)) ncf = 3d0

	m1 = 1d0
	m2 = 1d0

	FWreal = -alpha/pi*( 
     &	  q1**2*( log(mw2/lambda2)*log(m1/mw)+log(m1/mw)**2+pi**2/3d0 
     &	         +log(mw2/lambda2)-log(m1/mw)/2d0-77d0/24d0 )
     &	 +q2**2*( log(mw2/lambda2)*log(m2/mw)+log(m2/mw)**2+pi**2/3d0 
     &	         +log(mw2/lambda2)-log(m2/mw)/2d0-77d0/24d0 )
     &	 +q1*q2*( log(lambda2/mw2)+22d0/6d0 )		       )

	if (q1*q2.ne.0d0) then
	  FWrealQCD = -alphas/pi*4d0/3d0*( 
     &	   log(mw2/lambda2)*log(m1*m2/mw2)+log(m1/mw)**2+log(m2/mw)**2
     &	  +2d0*pi**2/3d0+log(mw2/lambda2)-log(m1*m2/mw2)/2d0-33d0/12d0 ) 
	else
	  FWrealQCD = 0d0
	endif

	corr   = 0d0

	if (qgwbos.eq.1) then
	  corr = corr + 2d0*FWf1f2virt(MW2,q1,q2,m1,m2) + FWreal
	elseif (qgwbos.eq.2) then
	  corr = corr + 2d0*FWf1f2virt_bgf(MW2,q1,q2,m1,m2) + FWreal
	endif

	if (qgwferm.eq.1) corr = corr + 2d0*dZe - 2d0*dsw/sw + dZW

	if (qgwqcd.eq.1) 
     &	  corr = corr + 2d0*FWf1f2virt_QCD(MW2,q1,q2,m1,m2) + FWrealQCD

	pWwidth = ncf*alpha/12d0/sw2*mw*(qgwborn+corr)

	end

************************************************************************
        function Wwidth()
************************************************************************
*	total decay width for W --> f1 f2bar    (f1,f2 massless)
*-----------------------------------------------------------------------
*       30.7.99 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)

	common/param/pi,el,alpha,alpha0,alphaz,alphas,GF,cw,cw2,sw,sw2,
     &		     mw,mw2,gw,mz,mz2,gz,mh,mh2,
     &		     mxe,mxe2,ml(3),ml2(3),mqp(3),mqp2(3),mqm(3),mqm2(3)

	Wwidth = 3d0*pWwidth(-1d0,0d0) + 2d0*pWwidth(-1d0/3d0,2d0/3d0)

c	write(*,*) 'BR lept ',pWwidth(-1d0,0d0)/Wwidth
c	write(*,*) 'BR hadr ',pWwidth(-1d0/3d0,2d0/3d0)/Wwidth
c	write(*,*) 'Wwidth  ',Wwidth

	end

